/********************************************************************************
** Form generated from reading UI file 'MusicSearchWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICSEARCHWIDGET_H
#define UI_MUSICSEARCHWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/MyCheckBox.h>
#include "ui/small_widgets/MyLineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_MusicSearchWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QComboBox *comboScraper;
    QSpacerItem *horizontalSpacer_2;
    MyLineEdit *searchString;
    QHBoxLayout *horizontalLayout_3;
    QTableWidget *results;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_2;
    QSpacerItem *verticalSpacer_2;
    MyCheckBox *chkName;
    MyCheckBox *chkBorn;
    MyCheckBox *chkFormed;
    MyCheckBox *chkYearsActive;
    MyCheckBox *chkDisbanded;
    MyCheckBox *chkDied;
    MyCheckBox *chkBiography;
    MyCheckBox *chkDiscography;
    MyCheckBox *chkArtist;
    MyCheckBox *chkLabel;
    MyCheckBox *chkReview;
    MyCheckBox *chkYear;
    MyCheckBox *chkRating;
    MyCheckBox *chkReleaseDate;
    MyCheckBox *chkTitle;
    MyCheckBox *chkGenres;
    MyCheckBox *chkStyles;
    MyCheckBox *chkMoods;
    MyCheckBox *chkThumb;
    MyCheckBox *chkFanart;
    MyCheckBox *chkExtraFanarts;
    MyCheckBox *chkLogo;
    MyCheckBox *chkCover;
    MyCheckBox *chkCdArt;
    QFrame *line;
    QCheckBox *chkUnCheckAll;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *MusicSearchWidget)
    {
        if (MusicSearchWidget->objectName().isEmpty())
            MusicSearchWidget->setObjectName("MusicSearchWidget");
        MusicSearchWidget->resize(680, 640);
        MusicSearchWidget->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(MusicSearchWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(MusicSearchWidget);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        comboScraper = new QComboBox(MusicSearchWidget);
        comboScraper->setObjectName("comboScraper");

        horizontalLayout_2->addWidget(comboScraper);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_2);

        searchString = new MyLineEdit(MusicSearchWidget);
        searchString->setObjectName("searchString");

        verticalLayout->addWidget(searchString);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        results = new QTableWidget(MusicSearchWidget);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);

        horizontalLayout_3->addWidget(results);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(MusicSearchWidget);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");
        QFont font;
        font.setBold(true);
        label_2->setFont(font);

        verticalLayout_4->addWidget(label_2);

        verticalSpacer_2 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_2);

        chkName = new MyCheckBox(groupBox);
        chkName->setObjectName("chkName");
        chkName->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkName);

        chkBorn = new MyCheckBox(groupBox);
        chkBorn->setObjectName("chkBorn");
        chkBorn->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkBorn);

        chkFormed = new MyCheckBox(groupBox);
        chkFormed->setObjectName("chkFormed");
        chkFormed->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkFormed);

        chkYearsActive = new MyCheckBox(groupBox);
        chkYearsActive->setObjectName("chkYearsActive");
        chkYearsActive->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkYearsActive);

        chkDisbanded = new MyCheckBox(groupBox);
        chkDisbanded->setObjectName("chkDisbanded");
        chkDisbanded->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkDisbanded);

        chkDied = new MyCheckBox(groupBox);
        chkDied->setObjectName("chkDied");
        chkDied->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkDied);

        chkBiography = new MyCheckBox(groupBox);
        chkBiography->setObjectName("chkBiography");
        chkBiography->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkBiography);

        chkDiscography = new MyCheckBox(groupBox);
        chkDiscography->setObjectName("chkDiscography");
        chkDiscography->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkDiscography);

        chkArtist = new MyCheckBox(groupBox);
        chkArtist->setObjectName("chkArtist");
        chkArtist->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkArtist);

        chkLabel = new MyCheckBox(groupBox);
        chkLabel->setObjectName("chkLabel");
        chkLabel->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkLabel);

        chkReview = new MyCheckBox(groupBox);
        chkReview->setObjectName("chkReview");
        chkReview->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkReview);

        chkYear = new MyCheckBox(groupBox);
        chkYear->setObjectName("chkYear");
        chkYear->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkYear);

        chkRating = new MyCheckBox(groupBox);
        chkRating->setObjectName("chkRating");
        chkRating->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkRating);

        chkReleaseDate = new MyCheckBox(groupBox);
        chkReleaseDate->setObjectName("chkReleaseDate");
        chkReleaseDate->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkReleaseDate);

        chkTitle = new MyCheckBox(groupBox);
        chkTitle->setObjectName("chkTitle");
        chkTitle->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkTitle);

        chkGenres = new MyCheckBox(groupBox);
        chkGenres->setObjectName("chkGenres");
        chkGenres->setProperty("type", QVariant(QString::fromUtf8("both")));

        verticalLayout_4->addWidget(chkGenres);

        chkStyles = new MyCheckBox(groupBox);
        chkStyles->setObjectName("chkStyles");
        chkStyles->setProperty("type", QVariant(QString::fromUtf8("both")));

        verticalLayout_4->addWidget(chkStyles);

        chkMoods = new MyCheckBox(groupBox);
        chkMoods->setObjectName("chkMoods");
        chkMoods->setProperty("type", QVariant(QString::fromUtf8("both")));

        verticalLayout_4->addWidget(chkMoods);

        chkThumb = new MyCheckBox(groupBox);
        chkThumb->setObjectName("chkThumb");
        chkThumb->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkThumb);

        chkFanart = new MyCheckBox(groupBox);
        chkFanart->setObjectName("chkFanart");
        chkFanart->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkFanart);

        chkExtraFanarts = new MyCheckBox(groupBox);
        chkExtraFanarts->setObjectName("chkExtraFanarts");
        chkExtraFanarts->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkExtraFanarts);

        chkLogo = new MyCheckBox(groupBox);
        chkLogo->setObjectName("chkLogo");
        chkLogo->setProperty("type", QVariant(QString::fromUtf8("artist")));

        verticalLayout_4->addWidget(chkLogo);

        chkCover = new MyCheckBox(groupBox);
        chkCover->setObjectName("chkCover");
        chkCover->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkCover);

        chkCdArt = new MyCheckBox(groupBox);
        chkCdArt->setObjectName("chkCdArt");
        chkCdArt->setProperty("type", QVariant(QString::fromUtf8("album")));

        verticalLayout_4->addWidget(chkCdArt);

        line = new QFrame(groupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        chkUnCheckAll = new QCheckBox(groupBox);
        chkUnCheckAll->setObjectName("chkUnCheckAll");

        verticalLayout_4->addWidget(chkUnCheckAll);


        verticalLayout_2->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout_3->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout_3);


        retranslateUi(MusicSearchWidget);

        QMetaObject::connectSlotsByName(MusicSearchWidget);
    } // setupUi

    void retranslateUi(QWidget *MusicSearchWidget)
    {
        label->setText(QCoreApplication::translate("MusicSearchWidget", "Scraper", nullptr));
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("MusicSearchWidget", "Result", nullptr));
        groupBox->setTitle(QString());
        label_2->setText(QCoreApplication::translate("MusicSearchWidget", "Infos to load", nullptr));
        chkName->setText(QCoreApplication::translate("MusicSearchWidget", "Name", nullptr));
        chkBorn->setText(QCoreApplication::translate("MusicSearchWidget", "Born", nullptr));
        chkFormed->setText(QCoreApplication::translate("MusicSearchWidget", "Formed", nullptr));
        chkYearsActive->setText(QCoreApplication::translate("MusicSearchWidget", "Years Active", nullptr));
        chkDisbanded->setText(QCoreApplication::translate("MusicSearchWidget", "Disbanded", nullptr));
        chkDied->setText(QCoreApplication::translate("MusicSearchWidget", "Died", nullptr));
        chkBiography->setText(QCoreApplication::translate("MusicSearchWidget", "Biography", nullptr));
        chkDiscography->setText(QCoreApplication::translate("MusicSearchWidget", "Discography", nullptr));
        chkArtist->setText(QCoreApplication::translate("MusicSearchWidget", "Artist", nullptr));
        chkLabel->setText(QCoreApplication::translate("MusicSearchWidget", "Label", nullptr));
        chkReview->setText(QCoreApplication::translate("MusicSearchWidget", "Review", nullptr));
        chkYear->setText(QCoreApplication::translate("MusicSearchWidget", "Year", nullptr));
        chkRating->setText(QCoreApplication::translate("MusicSearchWidget", "Rating", nullptr));
        chkReleaseDate->setText(QCoreApplication::translate("MusicSearchWidget", "Release Date", nullptr));
        chkTitle->setText(QCoreApplication::translate("MusicSearchWidget", "Title", nullptr));
        chkGenres->setText(QCoreApplication::translate("MusicSearchWidget", "Genres", nullptr));
        chkStyles->setText(QCoreApplication::translate("MusicSearchWidget", "Styles", nullptr));
        chkMoods->setText(QCoreApplication::translate("MusicSearchWidget", "Moods", nullptr));
        chkThumb->setText(QCoreApplication::translate("MusicSearchWidget", "Thumbnail", nullptr));
        chkFanart->setText(QCoreApplication::translate("MusicSearchWidget", "Fanart", nullptr));
        chkExtraFanarts->setText(QCoreApplication::translate("MusicSearchWidget", "Extra Fanarts", nullptr));
        chkLogo->setText(QCoreApplication::translate("MusicSearchWidget", "Logo", nullptr));
        chkCover->setText(QCoreApplication::translate("MusicSearchWidget", "Cover", nullptr));
        chkCdArt->setText(QCoreApplication::translate("MusicSearchWidget", "CD Art", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("MusicSearchWidget", "(Un)Check all", nullptr));
        (void)MusicSearchWidget;
    } // retranslateUi

};

namespace Ui {
    class MusicSearchWidget: public Ui_MusicSearchWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICSEARCHWIDGET_H
