/********************************************************************************
** Form generated from reading UI file 'SetsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETSWIDGET_H
#define UI_SETSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyLabel.h"
#include "ui/small_widgets/MySplitter.h"

QT_BEGIN_NAMESPACE

class Ui_SetsWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    MySplitter *splitter;
    QTableWidget *sets;
    QWidget *verticalLayoutWidget;
    QVBoxLayout *verticalLayout;
    QLabel *setName;
    QGroupBox *movieSetGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *movies;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonAddMovie;
    QPushButton *buttonRemoveMovie;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_13;
    MyLabel *poster;
    QHBoxLayout *horizontalLayout;
    QToolButton *buttonPreviewPoster;
    QLabel *posterResolution;
    QLabel *label_14;
    MyLabel *backdrop;
    QHBoxLayout *horizontalLayout_7;
    QToolButton *buttonPreviewBackdrop;
    QLabel *backdropResolution;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SetsWidget)
    {
        if (SetsWidget->objectName().isEmpty())
            SetsWidget->setObjectName("SetsWidget");
        SetsWidget->resize(1072, 778);
        verticalLayout_2 = new QVBoxLayout(SetsWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        splitter = new MySplitter(SetsWidget);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        splitter->setHandleWidth(1);
        sets = new QTableWidget(splitter);
        if (sets->columnCount() < 1)
            sets->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        sets->setHorizontalHeaderItem(0, __qtablewidgetitem);
        sets->setObjectName("sets");
        sets->setAlternatingRowColors(true);
        sets->setSelectionMode(QAbstractItemView::SingleSelection);
        sets->setSelectionBehavior(QAbstractItemView::SelectRows);
        sets->setShowGrid(false);
        sets->setSortingEnabled(true);
        splitter->addWidget(sets);
        sets->horizontalHeader()->setVisible(false);
        sets->horizontalHeader()->setStretchLastSection(true);
        sets->verticalHeader()->setVisible(false);
        verticalLayoutWidget = new QWidget(splitter);
        verticalLayoutWidget->setObjectName("verticalLayoutWidget");
        verticalLayout = new QVBoxLayout(verticalLayoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 21);
        setName = new QLabel(verticalLayoutWidget);
        setName->setObjectName("setName");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        setName->setFont(font);
        setName->setIndent(20);
        setName->setProperty("isHeader", QVariant(true));

        verticalLayout->addWidget(setName);

        movieSetGroupBox = new QGroupBox(verticalLayoutWidget);
        movieSetGroupBox->setObjectName("movieSetGroupBox");
        horizontalLayout_2 = new QHBoxLayout(movieSetGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        movies = new QTableWidget(movieSetGroupBox);
        if (movies->columnCount() < 2)
            movies->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        movies->setHorizontalHeaderItem(0, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        movies->setHorizontalHeaderItem(1, __qtablewidgetitem2);
        movies->setObjectName("movies");
        movies->setAlternatingRowColors(true);
        movies->setShowGrid(false);
        movies->setSortingEnabled(false);
        movies->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(movies);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        buttonAddMovie = new QPushButton(movieSetGroupBox);
        buttonAddMovie->setObjectName("buttonAddMovie");

        horizontalLayout_4->addWidget(buttonAddMovie);

        buttonRemoveMovie = new QPushButton(movieSetGroupBox);
        buttonRemoveMovie->setObjectName("buttonRemoveMovie");

        horizontalLayout_4->addWidget(buttonRemoveMovie);


        verticalLayout_3->addLayout(horizontalLayout_4);

        verticalLayout_3->setStretch(0, 1);

        horizontalLayout_2->addLayout(verticalLayout_3);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(10, 0, -1, -1);
        label_13 = new QLabel(movieSetGroupBox);
        label_13->setObjectName("label_13");

        verticalLayout_4->addWidget(label_13);

        poster = new MyLabel(movieSetGroupBox);
        poster->setObjectName("poster");
        poster->setMinimumSize(QSize(200, 300));
        poster->setMaximumSize(QSize(200, 300));
        poster->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        poster->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/poster.png")));
        poster->setAlignment(Qt::AlignCenter);

        verticalLayout_4->addWidget(poster);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        buttonPreviewPoster = new QToolButton(movieSetGroupBox);
        buttonPreviewPoster->setObjectName("buttonPreviewPoster");
        buttonPreviewPoster->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/zoom_in.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonPreviewPoster->setIcon(icon);

        horizontalLayout->addWidget(buttonPreviewPoster);

        posterResolution = new QLabel(movieSetGroupBox);
        posterResolution->setObjectName("posterResolution");
        posterResolution->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(posterResolution);


        verticalLayout_4->addLayout(horizontalLayout);

        label_14 = new QLabel(movieSetGroupBox);
        label_14->setObjectName("label_14");

        verticalLayout_4->addWidget(label_14);

        backdrop = new MyLabel(movieSetGroupBox);
        backdrop->setObjectName("backdrop");
        backdrop->setMinimumSize(QSize(200, 112));
        backdrop->setMaximumSize(QSize(200, 112));
        backdrop->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        backdrop->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/fanart.png")));
        backdrop->setScaledContents(false);
        backdrop->setAlignment(Qt::AlignCenter);

        verticalLayout_4->addWidget(backdrop);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        buttonPreviewBackdrop = new QToolButton(movieSetGroupBox);
        buttonPreviewBackdrop->setObjectName("buttonPreviewBackdrop");
        buttonPreviewBackdrop->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonPreviewBackdrop->setIcon(icon);

        horizontalLayout_7->addWidget(buttonPreviewBackdrop);

        backdropResolution = new QLabel(movieSetGroupBox);
        backdropResolution->setObjectName("backdropResolution");
        backdropResolution->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_7->addWidget(backdropResolution);


        verticalLayout_4->addLayout(horizontalLayout_7);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout_4);


        verticalLayout->addWidget(movieSetGroupBox);

        verticalLayout->setStretch(1, 1);
        splitter->addWidget(verticalLayoutWidget);

        verticalLayout_2->addWidget(splitter);


        retranslateUi(SetsWidget);

        QMetaObject::connectSlotsByName(SetsWidget);
    } // setupUi

    void retranslateUi(QWidget *SetsWidget)
    {
        QTableWidgetItem *___qtablewidgetitem = sets->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("SetsWidget", "Set", nullptr));
        setName->setText(QCoreApplication::translate("SetsWidget", "Set Name", nullptr));
        movieSetGroupBox->setTitle(QString());
        QTableWidgetItem *___qtablewidgetitem1 = movies->horizontalHeaderItem(0);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("SetsWidget", "Movie", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = movies->horizontalHeaderItem(1);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("SetsWidget", "Sorttitle", nullptr));
#if QT_CONFIG(tooltip)
        buttonAddMovie->setToolTip(QCoreApplication::translate("SetsWidget", "Add movie to set", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddMovie->setText(QCoreApplication::translate("SetsWidget", "Add Movie", nullptr));
#if QT_CONFIG(tooltip)
        buttonRemoveMovie->setToolTip(QCoreApplication::translate("SetsWidget", "Remove selected movie from set", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveMovie->setText(QCoreApplication::translate("SetsWidget", "Remove Movie", nullptr));
        label_13->setText(QCoreApplication::translate("SetsWidget", "Poster", nullptr));
#if QT_CONFIG(tooltip)
        poster->setToolTip(QCoreApplication::translate("SetsWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        poster->setText(QString());
#if QT_CONFIG(tooltip)
        buttonPreviewPoster->setToolTip(QCoreApplication::translate("SetsWidget", "Full preview", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPreviewPoster->setText(QString());
#if QT_CONFIG(tooltip)
        posterResolution->setToolTip(QCoreApplication::translate("SetsWidget", "Resolution", nullptr));
#endif // QT_CONFIG(tooltip)
        posterResolution->setText(QCoreApplication::translate("SetsWidget", "Resolution", nullptr));
        label_14->setText(QCoreApplication::translate("SetsWidget", "Backdrop", nullptr));
#if QT_CONFIG(tooltip)
        backdrop->setToolTip(QCoreApplication::translate("SetsWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        backdrop->setText(QString());
#if QT_CONFIG(tooltip)
        buttonPreviewBackdrop->setToolTip(QCoreApplication::translate("SetsWidget", "Full preview", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPreviewBackdrop->setText(QString());
#if QT_CONFIG(tooltip)
        backdropResolution->setToolTip(QCoreApplication::translate("SetsWidget", "Resolution", nullptr));
#endif // QT_CONFIG(tooltip)
        backdropResolution->setText(QCoreApplication::translate("SetsWidget", "Resolution", nullptr));
        (void)SetsWidget;
    } // retranslateUi

};

namespace Ui {
    class SetsWidget: public Ui_SetsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETSWIDGET_H
