/********************************************************************************
** Form generated from reading UI file 'KodiSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KODISETTINGSWIDGET_H
#define UI_KODISETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_KodiSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *lblKodiFormDescription;
    QFormLayout *kodiForm;
    QLabel *lblKodiVersion;
    QComboBox *kodiVersion;
    QFrame *line;
    QLabel *lblWebserverDescription;
    QFormLayout *formLayout;
    QLabel *lblHost;
    QLineEdit *xbmcHost;
    QLabel *lblPort;
    QLineEdit *xbmcPort;
    QLabel *lblUser;
    QLabel *lblPassword;
    QLineEdit *xbmcUser;
    QLineEdit *xbmcPassword;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KodiSettingsWidget)
    {
        if (KodiSettingsWidget->objectName().isEmpty())
            KodiSettingsWidget->setObjectName("KodiSettingsWidget");
        KodiSettingsWidget->resize(970, 625);
        verticalLayout_2 = new QVBoxLayout(KodiSettingsWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        lblKodiFormDescription = new QLabel(KodiSettingsWidget);
        lblKodiFormDescription->setObjectName("lblKodiFormDescription");

        verticalLayout_2->addWidget(lblKodiFormDescription);

        kodiForm = new QFormLayout();
        kodiForm->setObjectName("kodiForm");
        kodiForm->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        lblKodiVersion = new QLabel(KodiSettingsWidget);
        lblKodiVersion->setObjectName("lblKodiVersion");

        kodiForm->setWidget(0, QFormLayout::LabelRole, lblKodiVersion);

        kodiVersion = new QComboBox(KodiSettingsWidget);
        kodiVersion->setObjectName("kodiVersion");

        kodiForm->setWidget(0, QFormLayout::FieldRole, kodiVersion);


        verticalLayout_2->addLayout(kodiForm);

        line = new QFrame(KodiSettingsWidget);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        lblWebserverDescription = new QLabel(KodiSettingsWidget);
        lblWebserverDescription->setObjectName("lblWebserverDescription");
        lblWebserverDescription->setWordWrap(true);

        verticalLayout_2->addWidget(lblWebserverDescription);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        lblHost = new QLabel(KodiSettingsWidget);
        lblHost->setObjectName("lblHost");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblHost);

        xbmcHost = new QLineEdit(KodiSettingsWidget);
        xbmcHost->setObjectName("xbmcHost");

        formLayout->setWidget(0, QFormLayout::FieldRole, xbmcHost);

        lblPort = new QLabel(KodiSettingsWidget);
        lblPort->setObjectName("lblPort");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblPort);

        xbmcPort = new QLineEdit(KodiSettingsWidget);
        xbmcPort->setObjectName("xbmcPort");

        formLayout->setWidget(1, QFormLayout::FieldRole, xbmcPort);

        lblUser = new QLabel(KodiSettingsWidget);
        lblUser->setObjectName("lblUser");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblUser);

        lblPassword = new QLabel(KodiSettingsWidget);
        lblPassword->setObjectName("lblPassword");

        formLayout->setWidget(3, QFormLayout::LabelRole, lblPassword);

        xbmcUser = new QLineEdit(KodiSettingsWidget);
        xbmcUser->setObjectName("xbmcUser");

        formLayout->setWidget(2, QFormLayout::FieldRole, xbmcUser);

        xbmcPassword = new QLineEdit(KodiSettingsWidget);
        xbmcPassword->setObjectName("xbmcPassword");
        xbmcPassword->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(3, QFormLayout::FieldRole, xbmcPassword);


        verticalLayout_2->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        verticalLayout_2->setStretch(4, 1);

        retranslateUi(KodiSettingsWidget);

        QMetaObject::connectSlotsByName(KodiSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *KodiSettingsWidget)
    {
        lblKodiFormDescription->setText(QCoreApplication::translate("KodiSettingsWidget", "General Kodi Settings. MediaElch uses the Kodi version below for writing NFO files.", nullptr));
        lblKodiVersion->setText(QCoreApplication::translate("KodiSettingsWidget", "Kodi Version", nullptr));
        lblWebserverDescription->setText(QCoreApplication::translate("KodiSettingsWidget", "If you want to use the synchronization feature you need to enable the webserver within Kodi (Settings -> Services -> Webserver). Enter the port of the webserver here (usually 80 or 8080).", nullptr));
        lblHost->setText(QCoreApplication::translate("KodiSettingsWidget", "Host", nullptr));
        xbmcHost->setPlaceholderText(QCoreApplication::translate("KodiSettingsWidget", "127.0.0.1", nullptr));
        lblPort->setText(QCoreApplication::translate("KodiSettingsWidget", "Port", nullptr));
        xbmcPort->setPlaceholderText(QCoreApplication::translate("KodiSettingsWidget", "8080", nullptr));
        lblUser->setText(QCoreApplication::translate("KodiSettingsWidget", "User", nullptr));
        lblPassword->setText(QCoreApplication::translate("KodiSettingsWidget", "Password", nullptr));
        (void)KodiSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class KodiSettingsWidget: public Ui_KodiSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_KODISETTINGSWIDGET_H
