/********************************************************************************
** Form generated from reading UI file 'KodiSync.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KODISYNC_H
#define UI_KODISYNC_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_KodiSync
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *lblIntro;
    QRadioButton *radioUpdateContents;
    QLabel *lblContents;
    QRadioButton *radioClean;
    QLabel *lblClean;
    QRadioButton *radioGetWatched;
    QLabel *lblWatched;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *laytouStatus;
    QLabel *lblStatus;
    QLabel *status;
    QProgressBar *progressBar;
    QHBoxLayout *layoutButtons;
    QPushButton *buttonClose;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonSync;

    void setupUi(QDialog *KodiSync)
    {
        if (KodiSync->objectName().isEmpty())
            KodiSync->setObjectName("KodiSync");
        KodiSync->setWindowModality(Qt::WindowModal);
        KodiSync->resize(680, 420);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KodiSync->sizePolicy().hasHeightForWidth());
        KodiSync->setSizePolicy(sizePolicy);
        KodiSync->setModal(true);
        verticalLayout = new QVBoxLayout(KodiSync);
        verticalLayout->setObjectName("verticalLayout");
        lblIntro = new QLabel(KodiSync);
        lblIntro->setObjectName("lblIntro");
        lblIntro->setWordWrap(true);

        verticalLayout->addWidget(lblIntro);

        radioUpdateContents = new QRadioButton(KodiSync);
        radioUpdateContents->setObjectName("radioUpdateContents");
        radioUpdateContents->setChecked(true);

        verticalLayout->addWidget(radioUpdateContents);

        lblContents = new QLabel(KodiSync);
        lblContents->setObjectName("lblContents");
        lblContents->setWordWrap(true);

        verticalLayout->addWidget(lblContents);

        radioClean = new QRadioButton(KodiSync);
        radioClean->setObjectName("radioClean");

        verticalLayout->addWidget(radioClean);

        lblClean = new QLabel(KodiSync);
        lblClean->setObjectName("lblClean");

        verticalLayout->addWidget(lblClean);

        radioGetWatched = new QRadioButton(KodiSync);
        radioGetWatched->setObjectName("radioGetWatched");

        verticalLayout->addWidget(radioGetWatched);

        lblWatched = new QLabel(KodiSync);
        lblWatched->setObjectName("lblWatched");
        lblWatched->setWordWrap(true);

        verticalLayout->addWidget(lblWatched);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        laytouStatus = new QHBoxLayout();
        laytouStatus->setObjectName("laytouStatus");
        lblStatus = new QLabel(KodiSync);
        lblStatus->setObjectName("lblStatus");
        QFont font;
        font.setBold(true);
        lblStatus->setFont(font);

        laytouStatus->addWidget(lblStatus);

        status = new QLabel(KodiSync);
        status->setObjectName("status");

        laytouStatus->addWidget(status);

        laytouStatus->setStretch(1, 1);

        verticalLayout->addLayout(laytouStatus);

        progressBar = new QProgressBar(KodiSync);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        verticalLayout->addWidget(progressBar);

        layoutButtons = new QHBoxLayout();
        layoutButtons->setObjectName("layoutButtons");
        buttonClose = new QPushButton(KodiSync);
        buttonClose->setObjectName("buttonClose");
        buttonClose->setAutoDefault(false);

        layoutButtons->addWidget(buttonClose);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutButtons->addItem(horizontalSpacer);

        buttonSync = new QPushButton(KodiSync);
        buttonSync->setObjectName("buttonSync");

        layoutButtons->addWidget(buttonSync);


        verticalLayout->addLayout(layoutButtons);


        retranslateUi(KodiSync);

        QMetaObject::connectSlotsByName(KodiSync);
    } // setupUi

    void retranslateUi(QDialog *KodiSync)
    {
        KodiSync->setWindowTitle(QCoreApplication::translate("KodiSync", "Kodi Synchronization", nullptr));
        lblIntro->setText(QCoreApplication::translate("KodiSync", "Please make sure you have setup your sources in Kodi and that the webserver is enabled (Kodi -> Settings -> Services -> Webserver).", nullptr));
        radioUpdateContents->setText(QCoreApplication::translate("KodiSync", "Update contents", nullptr));
        lblContents->setText(QCoreApplication::translate("KodiSync", "This will tell Kodi to remove the changed movies, concerts or shows. Afterwards a Kodi library update is triggered and the removed items will be picked up again.", nullptr));
        radioClean->setText(QCoreApplication::translate("KodiSync", "Remove non-existent items from Kodis database (Clean library)", nullptr));
        lblClean->setText(QCoreApplication::translate("KodiSync", "Kodi will remove not longer existing items from your database.", nullptr));
        radioGetWatched->setText(QCoreApplication::translate("KodiSync", "Retrieve watched status", nullptr));
        lblWatched->setText(QCoreApplication::translate("KodiSync", "The fields last played and playcount will be retrieved from Kodi.", nullptr));
        lblStatus->setText(QCoreApplication::translate("KodiSync", "Status", nullptr));
        status->setText(QCoreApplication::translate("KodiSync", "Status", nullptr));
        buttonClose->setText(QCoreApplication::translate("KodiSync", "Close", nullptr));
        buttonSync->setText(QCoreApplication::translate("KodiSync", "Start", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KodiSync: public Ui_KodiSync {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_KODISYNC_H
