/********************************************************************************
** Form generated from reading UI file 'ExportTemplateWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTTEMPLATEWIDGET_H
#define UI_EXPORTTEMPLATEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ExportTemplateWidget
{
public:
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *name;
    QLabel *version;
    QLabel *authors;
    QLabel *website;
    QLabel *description;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnInstall;

    void setupUi(QWidget *ExportTemplateWidget)
    {
        if (ExportTemplateWidget->objectName().isEmpty())
            ExportTemplateWidget->setObjectName("ExportTemplateWidget");
        ExportTemplateWidget->resize(670, 94);
        horizontalLayout_2 = new QHBoxLayout(ExportTemplateWidget);
        horizontalLayout_2->setSpacing(4);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setSizeConstraint(QLayout::SetMinimumSize);
        horizontalLayout_2->setContentsMargins(0, -1, 0, -1);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        name = new QLabel(ExportTemplateWidget);
        name->setObjectName("name");
        QFont font;
        font.setBold(true);
        name->setFont(font);
        name->setText(QString::fromUtf8("Theme Name"));

        horizontalLayout->addWidget(name);

        version = new QLabel(ExportTemplateWidget);
        version->setObjectName("version");
        version->setText(QString::fromUtf8("Version"));
        version->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout->addWidget(version);

        authors = new QLabel(ExportTemplateWidget);
        authors->setObjectName("authors");
        authors->setText(QString::fromUtf8("Authors"));

        horizontalLayout->addWidget(authors);

        horizontalLayout->setStretch(2, 1);

        verticalLayout_2->addLayout(horizontalLayout);

        website = new QLabel(ExportTemplateWidget);
        website->setObjectName("website");
        website->setText(QString::fromUtf8("Website"));

        verticalLayout_2->addWidget(website);

        description = new QLabel(ExportTemplateWidget);
        description->setObjectName("description");
        description->setText(QString::fromUtf8("Description"));
        description->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        description->setWordWrap(true);

        verticalLayout_2->addWidget(description);


        horizontalLayout_2->addLayout(verticalLayout_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        btnInstall = new QPushButton(ExportTemplateWidget);
        btnInstall->setObjectName("btnInstall");

        horizontalLayout_2->addWidget(btnInstall);

        horizontalLayout_2->setStretch(0, 1);

        retranslateUi(ExportTemplateWidget);
        QObject::connect(btnInstall, SIGNAL(clicked()), ExportTemplateWidget, SLOT(onBtnInstall()));

        QMetaObject::connectSlotsByName(ExportTemplateWidget);
    } // setupUi

    void retranslateUi(QWidget *ExportTemplateWidget)
    {
        btnInstall->setText(QCoreApplication::translate("ExportTemplateWidget", "Install", nullptr));
        (void)ExportTemplateWidget;
    } // retranslateUi

};

namespace Ui {
    class ExportTemplateWidget: public Ui_ExportTemplateWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTTEMPLATEWIDGET_H
