/****************************************************************************
** Meta object code from reading C++ file 'MusicSearchWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/music/MusicSearchWidget.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MusicSearchWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17MusicSearchWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN17MusicSearchWidgetE = QtMocHelpers::stringData(
    "MusicSearchWidget",
    "sigResultClicked",
    "",
    "search",
    "searchString",
    "scraperNo",
    "scraperId",
    "scraperId2",
    "infosToLoad",
    "QSet<MusicScraperInfo>",
    "setType",
    "type",
    "setArtistName",
    "artistName",
    "startSearch",
    "startSearchWithIndex",
    "index",
    "onAlbumSearchFinished",
    "mediaelch::scraper::AlbumSearchJob*",
    "searchJob",
    "onArtistSearchFinished",
    "mediaelch::scraper::ArtistSearchJob*",
    "resultClicked",
    "QTableWidgetItem*",
    "item",
    "resultClickedRow",
    "row",
    "chkToggled",
    "chkAllToggled",
    "toggled"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN17MusicSearchWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  110,    2, 0x06,    1 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       3,    1,  111,    2, 0x0a,    2 /* Public */,
       5,    0,  114,    2, 0x10a,    4 /* Public | MethodIsConst  */,
       6,    0,  115,    2, 0x0a,    5 /* Public */,
       7,    0,  116,    2, 0x0a,    6 /* Public */,
       8,    0,  117,    2, 0x0a,    7 /* Public */,
      10,    1,  118,    2, 0x0a,    8 /* Public */,
      12,    1,  121,    2, 0x0a,   10 /* Public */,
      14,    0,  124,    2, 0x08,   12 /* Private */,
      15,    1,  125,    2, 0x08,   13 /* Private */,
      17,    1,  128,    2, 0x08,   15 /* Private */,
      20,    1,  131,    2, 0x08,   17 /* Private */,
      22,    1,  134,    2, 0x08,   19 /* Private */,
      25,    1,  137,    2, 0x08,   21 /* Private */,
      27,    0,  140,    2, 0x08,   23 /* Private */,
      28,    1,  141,    2, 0x08,   24 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Int,
    QMetaType::QString,
    QMetaType::QString,
    0x80000000 | 9,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void, 0x80000000 | 21,   19,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void, QMetaType::Int,   26,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   29,

       0        // eod
};

Q_CONSTINIT const QMetaObject MusicSearchWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN17MusicSearchWidgetE.offsetsAndSizes,
    qt_meta_data_ZN17MusicSearchWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN17MusicSearchWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MusicSearchWidget, std::true_type>,
        // method 'sigResultClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'search'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'scraperNo'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'scraperId'
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'scraperId2'
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'infosToLoad'
        QtPrivate::TypeAndForceComplete<QSet<MusicScraperInfo>, std::false_type>,
        // method 'setType'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'setArtistName'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'startSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'startSearchWithIndex'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onAlbumSearchFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::AlbumSearchJob *, std::false_type>,
        // method 'onArtistSearchFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::ArtistSearchJob *, std::false_type>,
        // method 'resultClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'resultClickedRow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'chkToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'chkAllToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>
    >,
    nullptr
} };

void MusicSearchWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MusicSearchWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigResultClicked(); break;
        case 1: _t->search((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { int _r = _t->scraperNo();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->scraperId();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->scraperId2();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QSet<MusicScraperInfo> _r = _t->infosToLoad();
            if (_a[0]) *reinterpret_cast< QSet<MusicScraperInfo>*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setType((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setArtistName((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->startSearch(); break;
        case 9: _t->startSearchWithIndex((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->onAlbumSearchFinished((*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::AlbumSearchJob*>>(_a[1]))); break;
        case 11: _t->onArtistSearchFinished((*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::ArtistSearchJob*>>(_a[1]))); break;
        case 12: _t->resultClicked((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 13: _t->resultClickedRow((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->chkToggled(); break;
        case 15: _t->chkAllToggled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::AlbumSearchJob* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::ArtistSearchJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (MusicSearchWidget::*)();
            if (_q_method_type _q_method = &MusicSearchWidget::sigResultClicked; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject *MusicSearchWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MusicSearchWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN17MusicSearchWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MusicSearchWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void MusicSearchWidget::sigResultClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
