/********************************************************************************
** Form generated from reading UI file 'ScrapePreview.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRAPEPREVIEW_H
#define UI_SCRAPEPREVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/WebImageLabel.h"

QT_BEGIN_NAMESPACE

class Ui_ScrapePreview
{
public:
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *vboxPoster;
    QLabel *lblPreview;
    WebImageLabel *poster;
    QSpacerItem *spacerPoster;
    QVBoxLayout *vboxOverview;
    QLabel *lblDescription;
    QPlainTextEdit *lblDescriptionContent;
    QSpacerItem *spacerOverview;

    void setupUi(QWidget *ScrapePreview)
    {
        if (ScrapePreview->objectName().isEmpty())
            ScrapePreview->setObjectName("ScrapePreview");
        ScrapePreview->resize(420, 200);
        ScrapePreview->setMinimumSize(QSize(0, 160));
        ScrapePreview->setMaximumSize(QSize(16777215, 300));
        horizontalLayout = new QHBoxLayout(ScrapePreview);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        vboxPoster = new QVBoxLayout();
        vboxPoster->setObjectName("vboxPoster");
        lblPreview = new QLabel(ScrapePreview);
        lblPreview->setObjectName("lblPreview");

        vboxPoster->addWidget(lblPreview);

        poster = new WebImageLabel(ScrapePreview);
        poster->setObjectName("poster");
        poster->setMinimumSize(QSize(100, 120));
        poster->setMaximumSize(QSize(100, 180));

        vboxPoster->addWidget(poster);

        spacerPoster = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxPoster->addItem(spacerPoster);

        vboxPoster->setStretch(1, 1);

        horizontalLayout->addLayout(vboxPoster);

        vboxOverview = new QVBoxLayout();
        vboxOverview->setObjectName("vboxOverview");
        lblDescription = new QLabel(ScrapePreview);
        lblDescription->setObjectName("lblDescription");
        lblDescription->setTextFormat(Qt::RichText);

        vboxOverview->addWidget(lblDescription);

        lblDescriptionContent = new QPlainTextEdit(ScrapePreview);
        lblDescriptionContent->setObjectName("lblDescriptionContent");
        lblDescriptionContent->setMinimumSize(QSize(0, 100));
        lblDescriptionContent->setReadOnly(true);

        vboxOverview->addWidget(lblDescriptionContent);

        spacerOverview = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxOverview->addItem(spacerOverview);

        vboxOverview->setStretch(1, 1);

        horizontalLayout->addLayout(vboxOverview);

        horizontalLayout->setStretch(1, 1);

        retranslateUi(ScrapePreview);

        QMetaObject::connectSlotsByName(ScrapePreview);
    } // setupUi

    void retranslateUi(QWidget *ScrapePreview)
    {
        lblPreview->setText(QCoreApplication::translate("ScrapePreview", "<b>Preview</p>", nullptr));
        lblDescription->setText(QCoreApplication::translate("ScrapePreview", "<html><head/><body><p><span style=\" font-weight:700;\">Description</span></p></body></html>", nullptr));
        (void)ScrapePreview;
    } // retranslateUi

};

namespace Ui {
    class ScrapePreview: public Ui_ScrapePreview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCRAPEPREVIEW_H
