/****************************************************************************
** Meta object code from reading C++ file 'ImportDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/import/ImportDialog.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ImportDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ImportDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN12ImportDialogE = QtMocHelpers::stringData(
    "ImportDialog",
    "exec",
    "",
    "execMovie",
    "searchString",
    "execTvShow",
    "TvShow*",
    "tvShow",
    "execConcert",
    "reject",
    "accept",
    "onMovieChosen",
    "onMovieLoadDone",
    "Movie*",
    "movie",
    "onConcertChosen",
    "onConcertLoadDone",
    "Concert*",
    "concert",
    "onTvShowChosen",
    "onEpisodeLoadDone",
    "TvShowEpisode*",
    "episode",
    "onImport",
    "onFileWatcherTimeout",
    "onMovingFilesFinished",
    "onEpisodeDownloadFinished",
    "DownloadManagerElement",
    "elem"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN12ImportDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  110,    2, 0x0a,    1 /* Public */,
       3,    1,  111,    2, 0x0a,    2 /* Public */,
       5,    2,  114,    2, 0x0a,    4 /* Public */,
       8,    1,  119,    2, 0x0a,    7 /* Public */,
       9,    0,  122,    2, 0x0a,    9 /* Public */,
      10,    0,  123,    2, 0x0a,   10 /* Public */,
      11,    0,  124,    2, 0x08,   11 /* Private */,
      12,    1,  125,    2, 0x08,   12 /* Private */,
      15,    0,  128,    2, 0x08,   14 /* Private */,
      16,    1,  129,    2, 0x08,   15 /* Private */,
      19,    0,  132,    2, 0x08,   17 /* Private */,
      20,    1,  133,    2, 0x08,   18 /* Private */,
      23,    0,  136,    2, 0x08,   20 /* Private */,
      24,    0,  137,    2, 0x08,   21 /* Private */,
      25,    0,  138,    2, 0x08,   22 /* Private */,
      26,    1,  139,    2, 0x08,   23 /* Private */,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Int, QMetaType::QString,    4,
    QMetaType::Int, QMetaType::QString, 0x80000000 | 6,    4,    7,
    QMetaType::Int, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 27,   28,

       0        // eod
};

Q_CONSTINIT const QMetaObject ImportDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN12ImportDialogE.offsetsAndSizes,
    qt_meta_data_ZN12ImportDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN12ImportDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<ImportDialog, std::true_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'execMovie'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'execTvShow'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        // method 'execConcert'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'accept'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onMovieChosen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onMovieLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onConcertChosen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onConcertLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        // method 'onTvShowChosen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onEpisodeLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShowEpisode *, std::false_type>,
        // method 'onImport'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onFileWatcherTimeout'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onMovingFilesFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onEpisodeDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>
    >,
    nullptr
} };

void ImportDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImportDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: { int _r = _t->execMovie((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->execTvShow((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 3: { int _r = _t->execConcert((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->reject(); break;
        case 5: _t->accept(); break;
        case 6: _t->onMovieChosen(); break;
        case 7: _t->onMovieLoadDone((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 8: _t->onConcertChosen(); break;
        case 9: _t->onConcertLoadDone((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1]))); break;
        case 10: _t->onTvShowChosen(); break;
        case 11: _t->onEpisodeLoadDone((*reinterpret_cast< std::add_pointer_t<TvShowEpisode*>>(_a[1]))); break;
        case 12: _t->onImport(); break;
        case 13: _t->onFileWatcherTimeout(); break;
        case 14: _t->onMovingFilesFinished(); break;
        case 15: _t->onEpisodeDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShowEpisode* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        }
    }
}

const QMetaObject *ImportDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImportDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN12ImportDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int ImportDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
