/****************************************************************************
** Meta object code from reading C++ file 'SlidingStackedWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/small_widgets/SlidingStackedWidget.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SlidingStackedWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20SlidingStackedWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN20SlidingStackedWidgetE = QtMocHelpers::stringData(
    "SlidingStackedWidget",
    "animationFinished",
    "",
    "setSpeed",
    "speed",
    "setAnimation",
    "QEasingCurve::Type",
    "animationType",
    "setVerticalMode",
    "vertical",
    "setWrap",
    "wrap",
    "slideInNext",
    "slideInPrev",
    "slideInIdx",
    "idx",
    "SlidingStackedWidget::direction",
    "_direction",
    "expandToOne",
    "collapse",
    "animationDoneSlot"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN20SlidingStackedWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   92,    2, 0x06,    1 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       3,    1,   93,    2, 0x0a,    2 /* Public */,
       5,    1,   96,    2, 0x0a,    4 /* Public */,
       8,    1,   99,    2, 0x0a,    6 /* Public */,
       8,    0,  102,    2, 0x2a,    8 /* Public | MethodCloned */,
      10,    1,  103,    2, 0x0a,    9 /* Public */,
      12,    0,  106,    2, 0x0a,   11 /* Public */,
      13,    0,  107,    2, 0x0a,   12 /* Public */,
      14,    2,  108,    2, 0x0a,   13 /* Public */,
      14,    1,  113,    2, 0x2a,   16 /* Public | MethodCloned */,
      18,    0,  116,    2, 0x0a,   18 /* Public */,
      19,    0,  117,    2, 0x0a,   19 /* Public */,
      20,    0,  118,    2, 0x09,   20 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 16,   15,   17,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject SlidingStackedWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN20SlidingStackedWidgetE.offsetsAndSizes,
    qt_meta_data_ZN20SlidingStackedWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN20SlidingStackedWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<SlidingStackedWidget, std::true_type>,
        // method 'animationFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setSpeed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'setAnimation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QEasingCurve::Type, std::false_type>,
        // method 'setVerticalMode'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setVerticalMode'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setWrap'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'slideInNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'slideInPrev'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'slideInIdx'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<SlidingStackedWidget::direction, std::false_type>,
        // method 'slideInIdx'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'expandToOne'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'collapse'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'animationDoneSlot'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void SlidingStackedWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SlidingStackedWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->animationFinished(); break;
        case 1: _t->setSpeed((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setAnimation((*reinterpret_cast< std::add_pointer_t<QEasingCurve::Type>>(_a[1]))); break;
        case 3: _t->setVerticalMode((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setVerticalMode(); break;
        case 5: _t->setWrap((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->slideInNext(); break;
        case 7: _t->slideInPrev(); break;
        case 8: _t->slideInIdx((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<SlidingStackedWidget::direction>>(_a[2]))); break;
        case 9: _t->slideInIdx((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->expandToOne(); break;
        case 11: _t->collapse(); break;
        case 12: _t->animationDoneSlot(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (SlidingStackedWidget::*)();
            if (_q_method_type _q_method = &SlidingStackedWidget::animationFinished; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject *SlidingStackedWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SlidingStackedWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN20SlidingStackedWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int SlidingStackedWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void SlidingStackedWidget::animationFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
