/****************************************************************************
** Meta object code from reading C++ file 'TvShowWidgetTvShow.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowWidgetTvShow.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowWidgetTvShow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TvShowWidgetTvShowE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN18TvShowWidgetTvShowE = QtMocHelpers::stringData(
    "TvShowWidgetTvShow",
    "sigSetActionSearchEnabled",
    "",
    "MainWidgets",
    "sigSetActionSaveEnabled",
    "sigDownloadsStarted",
    "sigDownloadsProgress",
    "sigDownloadsFinished",
    "onSetEnabled",
    "enabled",
    "onClear",
    "onSaveInformation",
    "onStartScraperSearch",
    "setBigWindow",
    "bigWindow",
    "onAddGenre",
    "genre",
    "onRemoveGenre",
    "removeGenre",
    "onAddTag",
    "tag",
    "onRemoveTag",
    "onAddActor",
    "onRemoveActor",
    "onInfoLoadDone",
    "TvShow*",
    "show",
    "QSet<ShowScraperInfo>",
    "details",
    "mediaelch::Locale",
    "locale",
    "onLoadDone",
    "QMap<ImageType,QList<Poster>>",
    "posters",
    "onChooseImage",
    "onDeleteImage",
    "onImageDropped",
    "ImageType",
    "imageType",
    "imageUrl",
    "onPosterDownloadFinished",
    "DownloadManagerElement",
    "elem",
    "onDownloadsFinished",
    "onDownloadsLeft",
    "left",
    "onActorChanged",
    "onChangeActorImage",
    "onRevertChanges",
    "onArtPageOne",
    "onArtPageTwo",
    "onDownloadTune",
    "onTitleChange",
    "text",
    "onSortTitleChange",
    "onOriginalTitleChange",
    "onImdbIdChange",
    "onTmdbIdChange",
    "onTvdbIdChange",
    "onTvMazeIdChange",
    "onStatusChange",
    "index",
    "onCertificationChange",
    "onUserRatingChange",
    "value",
    "onTop250Change",
    "onFirstAiredChange",
    "date",
    "onStudioChange",
    "studios",
    "onOverviewChange",
    "onActorEdited",
    "QTableWidgetItem*",
    "item",
    "onRuntimeChange",
    "runtime",
    "onRemoveExtraFanart",
    "file",
    "image",
    "onAddExtraFanart",
    "onExtraFanartDropped",
    "QList<QUrl>",
    "imageUrls",
    "onImdbIdOpen",
    "onTmdbIdOpen",
    "onTvMazeIdOpen",
    "onShowScraperProgress",
    "current",
    "max"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN18TvShowWidgetTvShowE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      54,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,  338,    2, 0x06,    1 /* Public */,
       4,    2,  343,    2, 0x06,    4 /* Public */,
       5,    2,  348,    2, 0x06,    7 /* Public */,
       6,    3,  353,    2, 0x06,   10 /* Public */,
       7,    1,  360,    2, 0x06,   14 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       8,    1,  363,    2, 0x0a,   16 /* Public */,
      10,    0,  366,    2, 0x0a,   18 /* Public */,
      11,    0,  367,    2, 0x0a,   19 /* Public */,
      12,    0,  368,    2, 0x0a,   20 /* Public */,
      13,    1,  369,    2, 0x0a,   21 /* Public */,
      15,    1,  372,    2, 0x08,   23 /* Private */,
      17,    1,  375,    2, 0x08,   25 /* Private */,
      19,    1,  378,    2, 0x08,   27 /* Private */,
      21,    1,  381,    2, 0x08,   29 /* Private */,
      22,    0,  384,    2, 0x08,   31 /* Private */,
      23,    0,  385,    2, 0x08,   32 /* Private */,
      24,    3,  386,    2, 0x08,   33 /* Private */,
      31,    2,  393,    2, 0x08,   37 /* Private */,
      34,    0,  398,    2, 0x08,   40 /* Private */,
      35,    0,  399,    2, 0x08,   41 /* Private */,
      36,    2,  400,    2, 0x08,   42 /* Private */,
      40,    1,  405,    2, 0x08,   45 /* Private */,
      43,    1,  408,    2, 0x08,   47 /* Private */,
      44,    2,  411,    2, 0x08,   49 /* Private */,
      46,    0,  416,    2, 0x08,   52 /* Private */,
      47,    0,  417,    2, 0x08,   53 /* Private */,
      48,    0,  418,    2, 0x08,   54 /* Private */,
      49,    0,  419,    2, 0x08,   55 /* Private */,
      50,    0,  420,    2, 0x08,   56 /* Private */,
      51,    0,  421,    2, 0x08,   57 /* Private */,
      52,    1,  422,    2, 0x08,   58 /* Private */,
      54,    1,  425,    2, 0x08,   60 /* Private */,
      55,    1,  428,    2, 0x08,   62 /* Private */,
      56,    1,  431,    2, 0x08,   64 /* Private */,
      57,    1,  434,    2, 0x08,   66 /* Private */,
      58,    1,  437,    2, 0x08,   68 /* Private */,
      59,    1,  440,    2, 0x08,   70 /* Private */,
      60,    1,  443,    2, 0x08,   72 /* Private */,
      62,    1,  446,    2, 0x08,   74 /* Private */,
      63,    1,  449,    2, 0x08,   76 /* Private */,
      65,    1,  452,    2, 0x08,   78 /* Private */,
      66,    1,  455,    2, 0x08,   80 /* Private */,
      68,    1,  458,    2, 0x08,   82 /* Private */,
      70,    0,  461,    2, 0x08,   84 /* Private */,
      71,    1,  462,    2, 0x08,   85 /* Private */,
      74,    1,  465,    2, 0x08,   87 /* Private */,
      76,    1,  468,    2, 0x08,   89 /* Private */,
      76,    1,  471,    2, 0x08,   91 /* Private */,
      79,    0,  474,    2, 0x08,   93 /* Private */,
      80,    1,  475,    2, 0x08,   94 /* Private */,
      83,    0,  478,    2, 0x08,   96 /* Private */,
      84,    0,  479,    2, 0x08,   97 /* Private */,
      85,    0,  480,    2, 0x08,   98 /* Private */,
      86,    3,  481,    2, 0x08,   99 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 25, 0x80000000 | 27, 0x80000000 | 29,   26,   28,   30,
    QMetaType::Void, 0x80000000 | 25, 0x80000000 | 32,   26,   33,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 37, QMetaType::QUrl,   38,   39,
    QMetaType::Void, 0x80000000 | 41,   42,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 41,   45,   42,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::Int,   61,
    QMetaType::Void, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::Double,   64,
    QMetaType::Void, QMetaType::Int,   64,
    QMetaType::Void, QMetaType::QDate,   67,
    QMetaType::Void, QMetaType::QString,   69,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 72,   73,
    QMetaType::Void, QMetaType::Int,   75,
    QMetaType::Void, QMetaType::QString,   77,
    QMetaType::Void, QMetaType::QByteArray,   78,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 81,   82,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 25, QMetaType::Int, QMetaType::Int,   26,   87,   88,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowWidgetTvShow::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN18TvShowWidgetTvShowE.offsetsAndSizes,
    qt_meta_data_ZN18TvShowWidgetTvShowE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN18TvShowWidgetTvShowE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowWidgetTvShow, std::true_type>,
        // method 'sigSetActionSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigSetActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigDownloadsStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'sigDownloadsProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'sigDownloadsFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onSetEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onClear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartScraperSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setBigWindow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onAddGenre'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRemoveGenre'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onAddTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRemoveTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onAddActor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveActor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onInfoLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QSet<ShowScraperInfo>, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::Locale, std::false_type>,
        // method 'onLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'onChooseImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDeleteImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImageDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QUrl, std::false_type>,
        // method 'onPosterDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>,
        // method 'onDownloadsFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        // method 'onDownloadsLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>,
        // method 'onActorChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChangeActorImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRevertChanges'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtPageOne'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtPageTwo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDownloadTune'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTitleChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onSortTitleChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onOriginalTitleChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onImdbIdChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTmdbIdChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTvdbIdChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTvMazeIdChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onStatusChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onCertificationChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onUserRatingChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<double, std::false_type>,
        // method 'onTop250Change'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onFirstAiredChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QDate, std::false_type>,
        // method 'onStudioChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onOverviewChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActorEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onRuntimeChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onRemoveExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRemoveExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'onAddExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onExtraFanartDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<QUrl>, std::false_type>,
        // method 'onImdbIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTmdbIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTvMazeIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onShowScraperProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>
    >,
    nullptr
} };

void TvShowWidgetTvShow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowWidgetTvShow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigSetActionSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->sigSetActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 2: _t->sigDownloadsStarted((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->sigDownloadsProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->sigDownloadsFinished((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->onSetEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->onClear(); break;
        case 7: _t->onSaveInformation(); break;
        case 8: _t->onStartScraperSearch(); break;
        case 9: _t->setBigWindow((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->onAddGenre((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->onRemoveGenre((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->onAddTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->onRemoveTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->onAddActor(); break;
        case 15: _t->onRemoveActor(); break;
        case 16: _t->onInfoLoadDone((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QSet<ShowScraperInfo>>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<mediaelch::Locale>>(_a[3]))); break;
        case 17: _t->onLoadDone((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 18: _t->onChooseImage(); break;
        case 19: _t->onDeleteImage(); break;
        case 20: _t->onImageDropped((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 21: _t->onPosterDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        case 22: _t->onDownloadsFinished((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1]))); break;
        case 23: _t->onDownloadsLeft((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[2]))); break;
        case 24: _t->onActorChanged(); break;
        case 25: _t->onChangeActorImage(); break;
        case 26: _t->onRevertChanges(); break;
        case 27: _t->onArtPageOne(); break;
        case 28: _t->onArtPageTwo(); break;
        case 29: _t->onDownloadTune(); break;
        case 30: _t->onTitleChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->onSortTitleChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->onOriginalTitleChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->onImdbIdChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->onTmdbIdChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->onTvdbIdChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->onTvMazeIdChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->onStatusChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->onCertificationChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->onUserRatingChange((*reinterpret_cast< std::add_pointer_t<double>>(_a[1]))); break;
        case 40: _t->onTop250Change((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 41: _t->onFirstAiredChange((*reinterpret_cast< std::add_pointer_t<QDate>>(_a[1]))); break;
        case 42: _t->onStudioChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->onOverviewChange(); break;
        case 44: _t->onActorEdited((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 45: _t->onRuntimeChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 46: _t->onRemoveExtraFanart((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 47: _t->onRemoveExtraFanart((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 48: _t->onAddExtraFanart(); break;
        case 49: _t->onExtraFanartDropped((*reinterpret_cast< std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 50: _t->onImdbIdOpen(); break;
        case 51: _t->onTmdbIdOpen(); break;
        case 52: _t->onTvMazeIdOpen(); break;
        case 53: _t->onShowScraperProgress((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        case 49:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QUrl> >(); break;
            }
            break;
        case 53:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (TvShowWidgetTvShow::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidgetTvShow::sigSetActionSearchEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidgetTvShow::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidgetTvShow::sigSetActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidgetTvShow::*)(QString , int );
            if (_q_method_type _q_method = &TvShowWidgetTvShow::sigDownloadsStarted; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidgetTvShow::*)(int , int , int );
            if (_q_method_type _q_method = &TvShowWidgetTvShow::sigDownloadsProgress; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidgetTvShow::*)(int );
            if (_q_method_type _q_method = &TvShowWidgetTvShow::sigDownloadsFinished; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
    }
}

const QMetaObject *TvShowWidgetTvShow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowWidgetTvShow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN18TvShowWidgetTvShowE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TvShowWidgetTvShow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 54)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 54;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 54)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 54;
    }
    return _id;
}

// SIGNAL 0
void TvShowWidgetTvShow::sigSetActionSearchEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TvShowWidgetTvShow::sigSetActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TvShowWidgetTvShow::sigDownloadsStarted(QString _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TvShowWidgetTvShow::sigDownloadsProgress(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TvShowWidgetTvShow::sigDownloadsFinished(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
