/********************************************************************************
** Form generated from reading UI file 'MediaFlags.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MEDIAFLAGS_H
#define UI_MEDIAFLAGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MediaFlags
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *mediaFlagResolution;
    QLabel *mediaFlagAspect;
    QLabel *mediaFlagCodec;
    QLabel *mediaFlagAudio;
    QLabel *mediaFlagChannels;

    void setupUi(QWidget *MediaFlags)
    {
        if (MediaFlags->objectName().isEmpty())
            MediaFlags->setObjectName("MediaFlags");
        MediaFlags->resize(400, 300);
        horizontalLayout = new QHBoxLayout(MediaFlags);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 6, 0);
        mediaFlagResolution = new QLabel(MediaFlags);
        mediaFlagResolution->setObjectName("mediaFlagResolution");

        horizontalLayout->addWidget(mediaFlagResolution);

        mediaFlagAspect = new QLabel(MediaFlags);
        mediaFlagAspect->setObjectName("mediaFlagAspect");

        horizontalLayout->addWidget(mediaFlagAspect);

        mediaFlagCodec = new QLabel(MediaFlags);
        mediaFlagCodec->setObjectName("mediaFlagCodec");

        horizontalLayout->addWidget(mediaFlagCodec);

        mediaFlagAudio = new QLabel(MediaFlags);
        mediaFlagAudio->setObjectName("mediaFlagAudio");

        horizontalLayout->addWidget(mediaFlagAudio);

        mediaFlagChannels = new QLabel(MediaFlags);
        mediaFlagChannels->setObjectName("mediaFlagChannels");

        horizontalLayout->addWidget(mediaFlagChannels);


        retranslateUi(MediaFlags);

        QMetaObject::connectSlotsByName(MediaFlags);
    } // setupUi

    void retranslateUi(QWidget *MediaFlags)
    {
        mediaFlagResolution->setText(QString());
        mediaFlagAspect->setText(QString());
        mediaFlagCodec->setText(QString());
        mediaFlagAudio->setText(QString());
        mediaFlagChannels->setText(QString());
        (void)MediaFlags;
    } // retranslateUi

};

namespace Ui {
    class MediaFlags: public Ui_MediaFlags {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MEDIAFLAGS_H
