/****************************************************************************
** Meta object code from reading C++ file 'ImageDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/image/ImageDialog.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ImageDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11ImageDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN11ImageDialogE = QtMocHelpers::stringData(
    "ImageDialog",
    "accept",
    "",
    "reject",
    "execWithType",
    "ImageType",
    "type",
    "downloadFinished",
    "startNextDownload",
    "imageClicked",
    "row",
    "col",
    "chooseLocalImage",
    "onImageDropped",
    "url",
    "onPreviewSizeChange",
    "value",
    "onZoomIn",
    "onZoomOut",
    "onSearch",
    "onlyFirstResult",
    "onSearchWithAllResults",
    "onProviderChanged",
    "index",
    "onLanguageChanged",
    "onSearchFinished",
    "QList<ScraperSearchResult>",
    "results",
    "mediaelch::ScraperError",
    "error",
    "onResultClicked",
    "QTableWidgetItem*",
    "item",
    "onProviderImagesLoaded",
    "QList<Poster>",
    "images",
    "onImageClosed",
    "updateSourceLink"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN11ImageDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  140,    2, 0x0a,    1 /* Public */,
       3,    0,  141,    2, 0x0a,    2 /* Public */,
       4,    1,  142,    2, 0x0a,    3 /* Public */,
       7,    0,  145,    2, 0x08,    5 /* Private */,
       8,    0,  146,    2, 0x08,    6 /* Private */,
       9,    2,  147,    2, 0x08,    7 /* Private */,
      12,    0,  152,    2, 0x08,   10 /* Private */,
      13,    1,  153,    2, 0x08,   11 /* Private */,
      15,    1,  156,    2, 0x08,   13 /* Private */,
      17,    0,  159,    2, 0x08,   15 /* Private */,
      18,    0,  160,    2, 0x08,   16 /* Private */,
      19,    1,  161,    2, 0x08,   17 /* Private */,
      19,    0,  164,    2, 0x28,   19 /* Private | MethodCloned */,
      21,    0,  165,    2, 0x08,   20 /* Private */,
      22,    1,  166,    2, 0x08,   21 /* Private */,
      24,    1,  169,    2, 0x08,   23 /* Private */,
      25,    2,  172,    2, 0x08,   25 /* Private */,
      30,    1,  177,    2, 0x08,   28 /* Private */,
      33,    2,  180,    2, 0x08,   30 /* Private */,
      36,    1,  185,    2, 0x08,   33 /* Private */,
      37,    0,  188,    2, 0x08,   35 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Int, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   14,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, 0x80000000 | 26, 0x80000000 | 28,   27,   29,
    QMetaType::Void, 0x80000000 | 31,   32,
    QMetaType::Void, 0x80000000 | 34, 0x80000000 | 28,   35,   29,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject ImageDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN11ImageDialogE.offsetsAndSizes,
    qt_meta_data_ZN11ImageDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN11ImageDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<ImageDialog, std::true_type>,
        // method 'accept'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'execWithType'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        // method 'downloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'startNextDownload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'imageClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'chooseLocalImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImageDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QUrl, std::false_type>,
        // method 'onPreviewSizeChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onZoomIn'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onZoomOut'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSearchWithAllResults'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onProviderChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onLanguageChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onSearchFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<ScraperSearchResult>, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::ScraperError, std::false_type>,
        // method 'onResultClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onProviderImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<Poster>, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::ScraperError, std::false_type>,
        // method 'onImageClosed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'updateSourceLink'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void ImageDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImageDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->reject(); break;
        case 2: { int _r = _t->execWithType((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->downloadFinished(); break;
        case 4: _t->startNextDownload(); break;
        case 5: _t->imageClicked((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->chooseLocalImage(); break;
        case 7: _t->onImageDropped((*reinterpret_cast< std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->onPreviewSizeChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->onZoomIn(); break;
        case 10: _t->onZoomOut(); break;
        case 11: _t->onSearch((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->onSearch(); break;
        case 13: _t->onSearchWithAllResults(); break;
        case 14: _t->onProviderChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->onLanguageChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->onSearchFinished((*reinterpret_cast< std::add_pointer_t<QList<ScraperSearchResult>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::ScraperError>>(_a[2]))); break;
        case 17: _t->onResultClicked((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 18: _t->onProviderImagesLoaded((*reinterpret_cast< std::add_pointer_t<QList<Poster>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::ScraperError>>(_a[2]))); break;
        case 19: _t->onImageClosed((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->updateSourceLink(); break;
        default: ;
        }
    }
}

const QMetaObject *ImageDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImageDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN11ImageDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int ImageDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
