/****************************************************************************
** Meta object code from reading C++ file 'Navbar.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/main/Navbar.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Navbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6NavbarE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN6NavbarE = QtMocHelpers::stringData(
    "Navbar",
    "sigSearch",
    "",
    "sigSave",
    "sigSaveAll",
    "sigReload",
    "sigRename",
    "sigSettings",
    "sigSync",
    "sigExport",
    "sigCsvExport",
    "sigAbout",
    "sigLike",
    "sigFilterChanged",
    "QList<Filter*>",
    "setActiveWidget",
    "MainWidgets",
    "widget",
    "setFilterWidgetEnabled",
    "enabled",
    "onDonated",
    "donated"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN6NavbarE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  104,    2, 0x06,    1 /* Public */,
       3,    0,  105,    2, 0x06,    2 /* Public */,
       4,    0,  106,    2, 0x06,    3 /* Public */,
       5,    0,  107,    2, 0x06,    4 /* Public */,
       6,    0,  108,    2, 0x06,    5 /* Public */,
       7,    0,  109,    2, 0x06,    6 /* Public */,
       8,    0,  110,    2, 0x06,    7 /* Public */,
       9,    0,  111,    2, 0x06,    8 /* Public */,
      10,    0,  112,    2, 0x06,    9 /* Public */,
      11,    0,  113,    2, 0x06,   10 /* Public */,
      12,    0,  114,    2, 0x06,   11 /* Public */,
      13,    2,  115,    2, 0x06,   12 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      15,    1,  120,    2, 0x0a,   15 /* Public */,
      18,    1,  123,    2, 0x0a,   17 /* Public */,
      20,    1,  126,    2, 0x08,   19 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14, QMetaType::QString,    2,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Bool,   21,

       0        // eod
};

Q_CONSTINIT const QMetaObject Navbar::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN6NavbarE.offsetsAndSizes,
    qt_meta_data_ZN6NavbarE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN6NavbarE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<Navbar, std::true_type>,
        // method 'sigSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigSave'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigSaveAll'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigReload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigRename'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigSettings'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigSync'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigExport'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigCsvExport'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigAbout'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigLike'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigFilterChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<Filter*>, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'setActiveWidget'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'setFilterWidgetEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onDonated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>
    >,
    nullptr
} };

void Navbar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Navbar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigSearch(); break;
        case 1: _t->sigSave(); break;
        case 2: _t->sigSaveAll(); break;
        case 3: _t->sigReload(); break;
        case 4: _t->sigRename(); break;
        case 5: _t->sigSettings(); break;
        case 6: _t->sigSync(); break;
        case 7: _t->sigExport(); break;
        case 8: _t->sigCsvExport(); break;
        case 9: _t->sigAbout(); break;
        case 10: _t->sigLike(); break;
        case 11: _t->sigFilterChanged((*reinterpret_cast< std::add_pointer_t<QList<Filter*>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->setActiveWidget((*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[1]))); break;
        case 13: _t->setFilterWidgetEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->onDonated((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Filter*> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigSearch; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigSave; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigSaveAll; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigReload; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigRename; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigSettings; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 5;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigSync; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 6;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigExport; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 7;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigCsvExport; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 8;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigAbout; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 9;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)();
            if (_q_method_type _q_method = &Navbar::sigLike; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 10;
                return;
            }
        }
        {
            using _q_method_type = void (Navbar::*)(QVector<Filter*> , QString );
            if (_q_method_type _q_method = &Navbar::sigFilterChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 11;
                return;
            }
        }
    }
}

const QMetaObject *Navbar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Navbar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN6NavbarE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Navbar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Navbar::sigSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Navbar::sigSave()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Navbar::sigSaveAll()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Navbar::sigReload()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Navbar::sigRename()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Navbar::sigSettings()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Navbar::sigSync()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Navbar::sigExport()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Navbar::sigCsvExport()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Navbar::sigAbout()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Navbar::sigLike()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Navbar::sigFilterChanged(QVector<Filter*> _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}
QT_WARNING_POP
