/****************************************************************************
** Meta object code from reading C++ file 'KodiSync.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/ui/media_center/KodiSync.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KodiSync.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8KodiSyncE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8KodiSyncE = QtMocHelpers::stringData(
    "KodiSync",
    "sigTriggerReload",
    "",
    "sigFinished",
    "exec",
    "reject",
    "startSync",
    "onMovieListFinished",
    "onConcertListFinished",
    "onTvShowListFinished",
    "onEpisodeListFinished",
    "onRemoveFinished",
    "onScanFinished",
    "onCleanFinished",
    "onRadioContents",
    "onRadioClean",
    "onRadioWatched",
    "onButtonClose",
    "triggerReload",
    "triggerClean",
    "onAuthRequired",
    "QNetworkReply*",
    "reply",
    "QAuthenticator*",
    "authenticator"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8KodiSyncE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  128,    2, 0x06,    1 /* Public */,
       3,    0,  129,    2, 0x06,    2 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       4,    0,  130,    2, 0x0a,    3 /* Public */,
       5,    0,  131,    2, 0x0a,    4 /* Public */,
       6,    0,  132,    2, 0x08,    5 /* Private */,
       7,    0,  133,    2, 0x08,    6 /* Private */,
       8,    0,  134,    2, 0x08,    7 /* Private */,
       9,    0,  135,    2, 0x08,    8 /* Private */,
      10,    0,  136,    2, 0x08,    9 /* Private */,
      11,    0,  137,    2, 0x08,   10 /* Private */,
      12,    0,  138,    2, 0x08,   11 /* Private */,
      13,    0,  139,    2, 0x08,   12 /* Private */,
      14,    0,  140,    2, 0x08,   13 /* Private */,
      15,    0,  141,    2, 0x08,   14 /* Private */,
      16,    0,  142,    2, 0x08,   15 /* Private */,
      17,    0,  143,    2, 0x08,   16 /* Private */,
      18,    0,  144,    2, 0x08,   17 /* Private */,
      19,    0,  145,    2, 0x08,   18 /* Private */,
      20,    2,  146,    2, 0x08,   19 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21, 0x80000000 | 23,   22,   24,

       0        // eod
};

Q_CONSTINIT const QMetaObject KodiSync::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN8KodiSyncE.offsetsAndSizes,
    qt_meta_data_ZN8KodiSyncE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8KodiSyncE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<KodiSync, std::true_type>,
        // method 'sigTriggerReload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'startSync'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onMovieListFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onConcertListFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTvShowListFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onEpisodeListFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onScanFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onCleanFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRadioContents'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRadioClean'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRadioWatched'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onButtonClose'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'triggerReload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'triggerClean'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAuthRequired'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QNetworkReply *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QAuthenticator *, std::false_type>
    >,
    nullptr
} };

void KodiSync::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KodiSync *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigTriggerReload(); break;
        case 1: _t->sigFinished(); break;
        case 2: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->reject(); break;
        case 4: _t->startSync(); break;
        case 5: _t->onMovieListFinished(); break;
        case 6: _t->onConcertListFinished(); break;
        case 7: _t->onTvShowListFinished(); break;
        case 8: _t->onEpisodeListFinished(); break;
        case 9: _t->onRemoveFinished(); break;
        case 10: _t->onScanFinished(); break;
        case 11: _t->onCleanFinished(); break;
        case 12: _t->onRadioContents(); break;
        case 13: _t->onRadioClean(); break;
        case 14: _t->onRadioWatched(); break;
        case 15: _t->onButtonClose(); break;
        case 16: _t->triggerReload(); break;
        case 17: _t->triggerClean(); break;
        case 18: _t->onAuthRequired((*reinterpret_cast< std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QAuthenticator*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (KodiSync::*)();
            if (_q_method_type _q_method = &KodiSync::sigTriggerReload; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (KodiSync::*)();
            if (_q_method_type _q_method = &KodiSync::sigFinished; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *KodiSync::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KodiSync::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8KodiSyncE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int KodiSync::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void KodiSync::sigTriggerReload()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KodiSync::sigFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
