/********************************************************************************
** Form generated from reading UI file 'ImportActions.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTACTIONS_H
#define UI_IMPORTACTIONS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ImportActions
{
public:
    QHBoxLayout *horizontalLayout;
    QPushButton *btnImport;
    QPushButton *btnDelete;

    void setupUi(QWidget *ImportActions)
    {
        if (ImportActions->objectName().isEmpty())
            ImportActions->setObjectName("ImportActions");
        ImportActions->resize(400, 60);
        horizontalLayout = new QHBoxLayout(ImportActions);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        btnImport = new QPushButton(ImportActions);
        btnImport->setObjectName("btnImport");

        horizontalLayout->addWidget(btnImport);

        btnDelete = new QPushButton(ImportActions);
        btnDelete->setObjectName("btnDelete");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/trash_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnDelete->setIcon(icon);
        btnDelete->setIconSize(QSize(12, 12));

        horizontalLayout->addWidget(btnDelete);

        horizontalLayout->setStretch(0, 1);

        retranslateUi(ImportActions);

        QMetaObject::connectSlotsByName(ImportActions);
    } // setupUi

    void retranslateUi(QWidget *ImportActions)
    {
        btnImport->setText(QCoreApplication::translate("ImportActions", "Import", nullptr));
#if QT_CONFIG(tooltip)
        btnDelete->setToolTip(QCoreApplication::translate("ImportActions", "Delete", nullptr));
#endif // QT_CONFIG(tooltip)
        btnDelete->setText(QString());
        (void)ImportActions;
    } // retranslateUi

};

namespace Ui {
    class ImportActions: public Ui_ImportActions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMPORTACTIONS_H
