/********************************************************************************
** Form generated from reading UI file 'TvShowWidgetEpisode.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWWIDGETEPISODE_H
#define UI_TVSHOWWIDGETEPISODE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/ClosableImage.h"
#include "ui/small_widgets/MediaFlags.h"
#include "ui/small_widgets/MyLabel.h"
#include "ui/small_widgets/MySpinBox.h"
#include "ui/small_widgets/RatingsWidget.h"
#include "ui/small_widgets/StereoModeComboBox.h"
#include "ui/small_widgets/TagCloud.h"

QT_BEGIN_NAMESPACE

class Ui_TvShowWidgetEpisode
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer_6;
    QToolButton *buttonRevert;
    QLabel *episodeName;
    QSpacerItem *horizontalSpacer_3;
    MediaFlags *mediaFlags;
    QLabel *missingLabel;
    QSpacerItem *horizontalSpacer_16;
    QToolButton *buttonPlay;
    QSpacerItem *hSpacerRight;
    QGroupBox *episodeGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tabInformation;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *lblFiles;
    QLineEdit *files;
    QLabel *lblIds;
    QGridLayout *gridLayout;
    QLabel *lblImdbId;
    QLineEdit *tmdbId;
    QSpacerItem *horizontalSpacer_13;
    QLabel *lblTmdbId;
    QLineEdit *tvdbId;
    QLabel *lblTheTvDbId;
    QLabel *lblTvmazeId;
    QPushButton *btnImdb;
    QPushButton *btnTvmaze;
    QLineEdit *tvmazeId;
    QSpacerItem *horizontalSpacer_12;
    QLineEdit *imdbId;
    QSpacerItem *horizontalSpacer_11;
    QSpacerItem *horizontalSpacer_14;
    QLabel *lblName;
    QLineEdit *name;
    QLabel *lblShowTitle;
    QLineEdit *showTitle;
    QLabel *lblSeason;
    QHBoxLayout *horizontalLayout_6;
    QSpinBox *season;
    QSpacerItem *horizontalSpacer_4;
    QLabel *lblEpisode;
    QSpinBox *episode;
    QLabel *lblDisplaySeason;
    QHBoxLayout *horizontalLayout_7;
    QSpinBox *displaySeason;
    QSpacerItem *horizontalSpacer_5;
    QLabel *lblDisplayEpisode;
    QSpinBox *displayEpisode;
    QLabel *lblRating;
    QVBoxLayout *ratingsLayout;
    QHBoxLayout *horizontalLayout_13;
    QLabel *lblTop250;
    MySpinBox *top250;
    QSpacerItem *horizontalSpacer_8;
    RatingsWidget *ratings;
    QLabel *lblCertification;
    QComboBox *certification;
    QLabel *lblFirstAired;
    QLabel *lblPlayCount;
    QHBoxLayout *horizontalLayout;
    QSpinBox *playCount;
    QLabel *lblLastPlayed;
    QDateTimeEdit *lastPlayed;
    QLabel *labelBookmark;
    QTimeEdit *epBookmark;
    QLabel *lblStudio;
    QLineEdit *studio;
    QLabel *lblOverview;
    QHBoxLayout *layoutFirstAired;
    QDateEdit *firstAired;
    QLabel *lblMissingFirstAired;
    QSpacerItem *horizontalSpacer_15;
    QPlainTextEdit *overview;
    QWidget *tabExtended;
    QFormLayout *formLayout_2;
    QLabel *lblWriters;
    QHBoxLayout *horizontalLayout_4;
    QTableWidget *writers;
    QVBoxLayout *verticalLayout_3;
    QPushButton *buttonAddWriter;
    QPushButton *buttonRemoveWriter;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *horizontalSpacer;
    QLabel *lblDirectors;
    QHBoxLayout *horizontalLayout_5;
    QTableWidget *directors;
    QVBoxLayout *verticalLayout_7;
    QPushButton *buttonAddDirector;
    QPushButton *buttonRemoveDirector;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *horizontalSpacer_2;
    QLabel *lblActors;
    QHBoxLayout *horizontalLayout_8;
    QTableWidget *actors;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout_9;
    QPushButton *buttonRemoveActor;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *buttonAddActor;
    QSpacerItem *verticalSpacer_4;
    MyLabel *actor;
    MyLabel *actorResolution;
    QSpacerItem *verticalSpacer_5;
    TagCloud *tagCloud;
    QLabel *lblTags;
    QWidget *tabStreamdetails;
    QVBoxLayout *verticalLayout_6;
    QGridLayout *streamDetails;
    QLabel *lblScanType;
    QTimeEdit *videoDuration;
    QLineEdit *videoScantype;
    QLabel *labelStreamDetailsAudio;
    QLabel *lblAspectRatio;
    QHBoxLayout *horizontalLayout_16;
    QSpinBox *videoWidth;
    QLabel *lblResolutionTimes;
    QSpinBox *videoHeight;
    QSpacerItem *horizontalSpacer_10;
    QLabel *lblDuration;
    QDoubleSpinBox *videoAspectRatio;
    QLineEdit *videoCodec;
    QLabel *lblResolution;
    QLabel *lblVideo;
    QLabel *lblCodec;
    QLabel *lblStereoMode;
    StereoModeComboBox *stereoMode;
    QHBoxLayout *horizontalLayout_17;
    QPushButton *buttonReloadStreamDetails;
    QLabel *lblReloadStreamDetailsError;
    QSpacerItem *horizontalSpacer_9;
    QSpacerItem *verticalSpacer_8;
    QFrame *line;
    QVBoxLayout *verticalLayout_sidebar;
    QLabel *labelThumbnail;
    ClosableImage *thumbnail;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *TvShowWidgetEpisode)
    {
        if (TvShowWidgetEpisode->objectName().isEmpty())
            TvShowWidgetEpisode->setObjectName("TvShowWidgetEpisode");
        TvShowWidgetEpisode->resize(924, 812);
        verticalLayout = new QVBoxLayout(TvShowWidgetEpisode);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 21);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setSpacing(0);
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_6);

        buttonRevert = new QToolButton(TvShowWidgetEpisode);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);

        horizontalLayout_10->addWidget(buttonRevert);

        episodeName = new QLabel(TvShowWidgetEpisode);
        episodeName->setObjectName("episodeName");
        QFont font;
        font.setPointSize(22);
        episodeName->setFont(font);
        episodeName->setIndent(4);
        episodeName->setProperty("isHeader", QVariant(true));

        horizontalLayout_10->addWidget(episodeName);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_3);

        mediaFlags = new MediaFlags(TvShowWidgetEpisode);
        mediaFlags->setObjectName("mediaFlags");

        horizontalLayout_10->addWidget(mediaFlags);

        missingLabel = new QLabel(TvShowWidgetEpisode);
        missingLabel->setObjectName("missingLabel");

        horizontalLayout_10->addWidget(missingLabel);

        horizontalSpacer_16 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_16);

        buttonPlay = new QToolButton(TvShowWidgetEpisode);
        buttonPlay->setObjectName("buttonPlay");
        buttonPlay->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/actions/32/media-playback-start.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonPlay->setIcon(icon1);

        horizontalLayout_10->addWidget(buttonPlay);

        hSpacerRight = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(hSpacerRight);


        verticalLayout->addLayout(horizontalLayout_10);

        episodeGroupBox = new QGroupBox(TvShowWidgetEpisode);
        episodeGroupBox->setObjectName("episodeGroupBox");
        horizontalLayout_2 = new QHBoxLayout(episodeGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(episodeGroupBox);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        tabInformation = new QWidget();
        tabInformation->setObjectName("tabInformation");
        verticalLayout_2 = new QVBoxLayout(tabInformation);
        verticalLayout_2->setObjectName("verticalLayout_2");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setRowWrapPolicy(QFormLayout::RowWrapPolicy::WrapLongRows);
        formLayout->setLabelAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        lblFiles = new QLabel(tabInformation);
        lblFiles->setObjectName("lblFiles");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblFiles);

        files = new QLineEdit(tabInformation);
        files->setObjectName("files");
        files->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, files);

        lblIds = new QLabel(tabInformation);
        lblIds->setObjectName("lblIds");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblIds);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        lblImdbId = new QLabel(tabInformation);
        lblImdbId->setObjectName("lblImdbId");

        gridLayout->addWidget(lblImdbId, 0, 3, 1, 1);

        tmdbId = new QLineEdit(tabInformation);
        tmdbId->setObjectName("tmdbId");

        gridLayout->addWidget(tmdbId, 1, 1, 1, 1);

        horizontalSpacer_13 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_13, 1, 2, 1, 1);

        lblTmdbId = new QLabel(tabInformation);
        lblTmdbId->setObjectName("lblTmdbId");

        gridLayout->addWidget(lblTmdbId, 1, 0, 1, 1);

        tvdbId = new QLineEdit(tabInformation);
        tvdbId->setObjectName("tvdbId");
        tvdbId->setMaximumSize(QSize(150, 16777215));

        gridLayout->addWidget(tvdbId, 0, 1, 1, 1);

        lblTheTvDbId = new QLabel(tabInformation);
        lblTheTvDbId->setObjectName("lblTheTvDbId");

        gridLayout->addWidget(lblTheTvDbId, 0, 0, 1, 1);

        lblTvmazeId = new QLabel(tabInformation);
        lblTvmazeId->setObjectName("lblTvmazeId");

        gridLayout->addWidget(lblTvmazeId, 1, 3, 1, 1);

        btnImdb = new QPushButton(tabInformation);
        btnImdb->setObjectName("btnImdb");

        gridLayout->addWidget(btnImdb, 0, 5, 1, 1);

        btnTvmaze = new QPushButton(tabInformation);
        btnTvmaze->setObjectName("btnTvmaze");

        gridLayout->addWidget(btnTvmaze, 1, 5, 1, 1);

        tvmazeId = new QLineEdit(tabInformation);
        tvmazeId->setObjectName("tvmazeId");
        tvmazeId->setMaximumSize(QSize(150, 16777215));

        gridLayout->addWidget(tvmazeId, 1, 4, 1, 1);

        horizontalSpacer_12 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_12, 0, 2, 1, 1);

        imdbId = new QLineEdit(tabInformation);
        imdbId->setObjectName("imdbId");
        imdbId->setMaximumSize(QSize(150, 16777215));

        gridLayout->addWidget(imdbId, 0, 4, 1, 1);

        horizontalSpacer_11 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_11, 0, 6, 1, 1);

        horizontalSpacer_14 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_14, 1, 6, 1, 1);


        formLayout->setLayout(1, QFormLayout::FieldRole, gridLayout);

        lblName = new QLabel(tabInformation);
        lblName->setObjectName("lblName");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblName);

        name = new QLineEdit(tabInformation);
        name->setObjectName("name");

        formLayout->setWidget(2, QFormLayout::FieldRole, name);

        lblShowTitle = new QLabel(tabInformation);
        lblShowTitle->setObjectName("lblShowTitle");

        formLayout->setWidget(3, QFormLayout::LabelRole, lblShowTitle);

        showTitle = new QLineEdit(tabInformation);
        showTitle->setObjectName("showTitle");

        formLayout->setWidget(3, QFormLayout::FieldRole, showTitle);

        lblSeason = new QLabel(tabInformation);
        lblSeason->setObjectName("lblSeason");

        formLayout->setWidget(4, QFormLayout::LabelRole, lblSeason);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        season = new QSpinBox(tabInformation);
        season->setObjectName("season");
        season->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        season->setMaximum(99999);

        horizontalLayout_6->addWidget(season);

        horizontalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_4);

        lblEpisode = new QLabel(tabInformation);
        lblEpisode->setObjectName("lblEpisode");

        horizontalLayout_6->addWidget(lblEpisode);

        episode = new QSpinBox(tabInformation);
        episode->setObjectName("episode");
        episode->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        episode->setMaximum(99999);

        horizontalLayout_6->addWidget(episode);


        formLayout->setLayout(4, QFormLayout::FieldRole, horizontalLayout_6);

        lblDisplaySeason = new QLabel(tabInformation);
        lblDisplaySeason->setObjectName("lblDisplaySeason");

        formLayout->setWidget(5, QFormLayout::LabelRole, lblDisplaySeason);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        displaySeason = new QSpinBox(tabInformation);
        displaySeason->setObjectName("displaySeason");
        displaySeason->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        displaySeason->setMinimum(-1);
        displaySeason->setMaximum(99999);

        horizontalLayout_7->addWidget(displaySeason);

        horizontalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_5);

        lblDisplayEpisode = new QLabel(tabInformation);
        lblDisplayEpisode->setObjectName("lblDisplayEpisode");

        horizontalLayout_7->addWidget(lblDisplayEpisode);

        displayEpisode = new QSpinBox(tabInformation);
        displayEpisode->setObjectName("displayEpisode");
        displayEpisode->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        displayEpisode->setMinimum(-1);
        displayEpisode->setMaximum(99999);

        horizontalLayout_7->addWidget(displayEpisode);


        formLayout->setLayout(5, QFormLayout::FieldRole, horizontalLayout_7);

        lblRating = new QLabel(tabInformation);
        lblRating->setObjectName("lblRating");

        formLayout->setWidget(6, QFormLayout::LabelRole, lblRating);

        ratingsLayout = new QVBoxLayout();
        ratingsLayout->setObjectName("ratingsLayout");
        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        lblTop250 = new QLabel(tabInformation);
        lblTop250->setObjectName("lblTop250");

        horizontalLayout_13->addWidget(lblTop250);

        top250 = new MySpinBox(tabInformation);
        top250->setObjectName("top250");
        top250->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        top250->setMaximum(9999999);

        horizontalLayout_13->addWidget(top250);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_8);


        ratingsLayout->addLayout(horizontalLayout_13);

        ratings = new RatingsWidget(tabInformation);
        ratings->setObjectName("ratings");

        ratingsLayout->addWidget(ratings);


        formLayout->setLayout(6, QFormLayout::FieldRole, ratingsLayout);

        lblCertification = new QLabel(tabInformation);
        lblCertification->setObjectName("lblCertification");

        formLayout->setWidget(7, QFormLayout::LabelRole, lblCertification);

        certification = new QComboBox(tabInformation);
        certification->setObjectName("certification");
        certification->setMinimumSize(QSize(140, 0));
        certification->setEditable(true);
        certification->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);

        formLayout->setWidget(7, QFormLayout::FieldRole, certification);

        lblFirstAired = new QLabel(tabInformation);
        lblFirstAired->setObjectName("lblFirstAired");

        formLayout->setWidget(8, QFormLayout::LabelRole, lblFirstAired);

        lblPlayCount = new QLabel(tabInformation);
        lblPlayCount->setObjectName("lblPlayCount");

        formLayout->setWidget(9, QFormLayout::LabelRole, lblPlayCount);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        playCount = new QSpinBox(tabInformation);
        playCount->setObjectName("playCount");
        playCount->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        horizontalLayout->addWidget(playCount);

        lblLastPlayed = new QLabel(tabInformation);
        lblLastPlayed->setObjectName("lblLastPlayed");

        horizontalLayout->addWidget(lblLastPlayed);

        lastPlayed = new QDateTimeEdit(tabInformation);
        lastPlayed->setObjectName("lastPlayed");
        lastPlayed->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        horizontalLayout->addWidget(lastPlayed);


        formLayout->setLayout(9, QFormLayout::FieldRole, horizontalLayout);

        labelBookmark = new QLabel(tabInformation);
        labelBookmark->setObjectName("labelBookmark");

        formLayout->setWidget(10, QFormLayout::LabelRole, labelBookmark);

        epBookmark = new QTimeEdit(tabInformation);
        epBookmark->setObjectName("epBookmark");
        epBookmark->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        formLayout->setWidget(10, QFormLayout::FieldRole, epBookmark);

        lblStudio = new QLabel(tabInformation);
        lblStudio->setObjectName("lblStudio");

        formLayout->setWidget(11, QFormLayout::LabelRole, lblStudio);

        studio = new QLineEdit(tabInformation);
        studio->setObjectName("studio");

        formLayout->setWidget(11, QFormLayout::FieldRole, studio);

        lblOverview = new QLabel(tabInformation);
        lblOverview->setObjectName("lblOverview");

        formLayout->setWidget(12, QFormLayout::LabelRole, lblOverview);

        layoutFirstAired = new QHBoxLayout();
        layoutFirstAired->setObjectName("layoutFirstAired");
        firstAired = new QDateEdit(tabInformation);
        firstAired->setObjectName("firstAired");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(firstAired->sizePolicy().hasHeightForWidth());
        firstAired->setSizePolicy(sizePolicy);
        firstAired->setMinimumSize(QSize(140, 0));
        firstAired->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        layoutFirstAired->addWidget(firstAired);

        lblMissingFirstAired = new QLabel(tabInformation);
        lblMissingFirstAired->setObjectName("lblMissingFirstAired");

        layoutFirstAired->addWidget(lblMissingFirstAired);

        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutFirstAired->addItem(horizontalSpacer_15);

        layoutFirstAired->setStretch(2, 1);

        formLayout->setLayout(8, QFormLayout::FieldRole, layoutFirstAired);

        overview = new QPlainTextEdit(tabInformation);
        overview->setObjectName("overview");

        formLayout->setWidget(12, QFormLayout::FieldRole, overview);


        verticalLayout_2->addLayout(formLayout);

        tabWidget->addTab(tabInformation, QString());
        tabExtended = new QWidget();
        tabExtended->setObjectName("tabExtended");
        formLayout_2 = new QFormLayout(tabExtended);
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        lblWriters = new QLabel(tabExtended);
        lblWriters->setObjectName("lblWriters");

        formLayout_2->setWidget(1, QFormLayout::LabelRole, lblWriters);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        writers = new QTableWidget(tabExtended);
        if (writers->columnCount() < 1)
            writers->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        writers->setHorizontalHeaderItem(0, __qtablewidgetitem);
        writers->setObjectName("writers");
        writers->setMaximumSize(QSize(300, 100));
        writers->setEditTriggers(QAbstractItemView::EditTrigger::DoubleClicked);
        writers->setShowGrid(false);
        writers->horizontalHeader()->setStretchLastSection(true);
        writers->verticalHeader()->setVisible(false);

        horizontalLayout_4->addWidget(writers);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        buttonAddWriter = new QPushButton(tabExtended);
        buttonAddWriter->setObjectName("buttonAddWriter");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/img/plus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonAddWriter->setIcon(icon2);

        verticalLayout_3->addWidget(buttonAddWriter);

        buttonRemoveWriter = new QPushButton(tabExtended);
        buttonRemoveWriter->setObjectName("buttonRemoveWriter");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/img/minus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRemoveWriter->setIcon(icon3);

        verticalLayout_3->addWidget(buttonRemoveWriter);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);


        horizontalLayout_4->addLayout(verticalLayout_3);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        horizontalLayout_4->setStretch(0, 1);

        formLayout_2->setLayout(1, QFormLayout::FieldRole, horizontalLayout_4);

        lblDirectors = new QLabel(tabExtended);
        lblDirectors->setObjectName("lblDirectors");

        formLayout_2->setWidget(2, QFormLayout::LabelRole, lblDirectors);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        directors = new QTableWidget(tabExtended);
        if (directors->columnCount() < 1)
            directors->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        directors->setHorizontalHeaderItem(0, __qtablewidgetitem1);
        directors->setObjectName("directors");
        directors->setMaximumSize(QSize(300, 100));
        directors->setEditTriggers(QAbstractItemView::EditTrigger::DoubleClicked);
        directors->setShowGrid(false);
        directors->horizontalHeader()->setStretchLastSection(true);
        directors->verticalHeader()->setVisible(false);

        horizontalLayout_5->addWidget(directors);

        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName("verticalLayout_7");
        buttonAddDirector = new QPushButton(tabExtended);
        buttonAddDirector->setObjectName("buttonAddDirector");
        buttonAddDirector->setIcon(icon2);

        verticalLayout_7->addWidget(buttonAddDirector);

        buttonRemoveDirector = new QPushButton(tabExtended);
        buttonRemoveDirector->setObjectName("buttonRemoveDirector");
        buttonRemoveDirector->setIcon(icon3);

        verticalLayout_7->addWidget(buttonRemoveDirector);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_3);


        horizontalLayout_5->addLayout(verticalLayout_7);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);

        horizontalLayout_5->setStretch(0, 1);

        formLayout_2->setLayout(2, QFormLayout::FieldRole, horizontalLayout_5);

        lblActors = new QLabel(tabExtended);
        lblActors->setObjectName("lblActors");

        formLayout_2->setWidget(0, QFormLayout::LabelRole, lblActors);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        actors = new QTableWidget(tabExtended);
        if (actors->columnCount() < 2)
            actors->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        actors->setHorizontalHeaderItem(0, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        actors->setHorizontalHeaderItem(1, __qtablewidgetitem3);
        actors->setObjectName("actors");
        actors->setEditTriggers(QAbstractItemView::EditTrigger::DoubleClicked);
        actors->setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
        actors->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        actors->setShowGrid(false);
        actors->verticalHeader()->setVisible(false);

        horizontalLayout_8->addWidget(actors);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setSpacing(0);
        verticalLayout_9->setObjectName("verticalLayout_9");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        buttonRemoveActor = new QPushButton(tabExtended);
        buttonRemoveActor->setObjectName("buttonRemoveActor");
        buttonRemoveActor->setIcon(icon3);

        horizontalLayout_9->addWidget(buttonRemoveActor);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_7);

        buttonAddActor = new QPushButton(tabExtended);
        buttonAddActor->setObjectName("buttonAddActor");
        buttonAddActor->setIcon(icon2);

        horizontalLayout_9->addWidget(buttonAddActor);


        verticalLayout_9->addLayout(horizontalLayout_9);

        verticalSpacer_4 = new QSpacerItem(20, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_9->addItem(verticalSpacer_4);

        actor = new MyLabel(tabExtended);
        actor->setObjectName("actor");
        actor->setMinimumSize(QSize(120, 180));
        actor->setMaximumSize(QSize(120, 180));
        actor->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        actor->setPixmap(QPixmap(QString::fromUtf8(":/img/man.png")));
        actor->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout_9->addWidget(actor);

        actorResolution = new MyLabel(tabExtended);
        actorResolution->setObjectName("actorResolution");
        actorResolution->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        verticalLayout_9->addWidget(actorResolution);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_5);


        horizontalLayout_8->addLayout(verticalLayout_9);

        horizontalLayout_8->setStretch(0, 1);

        formLayout_2->setLayout(0, QFormLayout::FieldRole, horizontalLayout_8);

        tagCloud = new TagCloud(tabExtended);
        tagCloud->setObjectName("tagCloud");

        formLayout_2->setWidget(3, QFormLayout::FieldRole, tagCloud);

        lblTags = new QLabel(tabExtended);
        lblTags->setObjectName("lblTags");

        formLayout_2->setWidget(3, QFormLayout::LabelRole, lblTags);

        tabWidget->addTab(tabExtended, QString());
        tabStreamdetails = new QWidget();
        tabStreamdetails->setObjectName("tabStreamdetails");
        verticalLayout_6 = new QVBoxLayout(tabStreamdetails);
        verticalLayout_6->setObjectName("verticalLayout_6");
        streamDetails = new QGridLayout();
        streamDetails->setObjectName("streamDetails");
        lblScanType = new QLabel(tabStreamdetails);
        lblScanType->setObjectName("lblScanType");
        lblScanType->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblScanType, 4, 0, 1, 1);

        videoDuration = new QTimeEdit(tabStreamdetails);
        videoDuration->setObjectName("videoDuration");
        videoDuration->setMaximumSize(QSize(200, 16777215));
        videoDuration->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        streamDetails->addWidget(videoDuration, 6, 1, 1, 1);

        videoScantype = new QLineEdit(tabStreamdetails);
        videoScantype->setObjectName("videoScantype");
        videoScantype->setMaximumSize(QSize(200, 16777215));

        streamDetails->addWidget(videoScantype, 4, 1, 1, 1);

        labelStreamDetailsAudio = new QLabel(tabStreamdetails);
        labelStreamDetailsAudio->setObjectName("labelStreamDetailsAudio");
        QFont font1;
        font1.setBold(true);
        labelStreamDetailsAudio->setFont(font1);
        labelStreamDetailsAudio->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(labelStreamDetailsAudio, 7, 0, 1, 1);

        lblAspectRatio = new QLabel(tabStreamdetails);
        lblAspectRatio->setObjectName("lblAspectRatio");
        lblAspectRatio->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblAspectRatio, 3, 0, 1, 1);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        videoWidth = new QSpinBox(tabStreamdetails);
        videoWidth->setObjectName("videoWidth");
        videoWidth->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        videoWidth->setMaximum(9999);

        horizontalLayout_16->addWidget(videoWidth);

        lblResolutionTimes = new QLabel(tabStreamdetails);
        lblResolutionTimes->setObjectName("lblResolutionTimes");
        lblResolutionTimes->setText(QString::fromUtf8("x"));

        horizontalLayout_16->addWidget(lblResolutionTimes);

        videoHeight = new QSpinBox(tabStreamdetails);
        videoHeight->setObjectName("videoHeight");
        videoHeight->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        videoHeight->setMaximum(99999);

        horizontalLayout_16->addWidget(videoHeight);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_10);


        streamDetails->addLayout(horizontalLayout_16, 2, 1, 1, 1);

        lblDuration = new QLabel(tabStreamdetails);
        lblDuration->setObjectName("lblDuration");
        lblDuration->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblDuration, 6, 0, 1, 1);

        videoAspectRatio = new QDoubleSpinBox(tabStreamdetails);
        videoAspectRatio->setObjectName("videoAspectRatio");
        videoAspectRatio->setMaximumSize(QSize(100, 16777215));
        videoAspectRatio->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        videoAspectRatio->setDecimals(3);

        streamDetails->addWidget(videoAspectRatio, 3, 1, 1, 1);

        videoCodec = new QLineEdit(tabStreamdetails);
        videoCodec->setObjectName("videoCodec");
        videoCodec->setMaximumSize(QSize(100, 16777215));

        streamDetails->addWidget(videoCodec, 1, 1, 1, 1);

        lblResolution = new QLabel(tabStreamdetails);
        lblResolution->setObjectName("lblResolution");
        lblResolution->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblResolution, 2, 0, 1, 1);

        lblVideo = new QLabel(tabStreamdetails);
        lblVideo->setObjectName("lblVideo");
        lblVideo->setFont(font1);
        lblVideo->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblVideo, 0, 0, 1, 1);

        lblCodec = new QLabel(tabStreamdetails);
        lblCodec->setObjectName("lblCodec");
        lblCodec->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblCodec, 1, 0, 1, 1);

        lblStereoMode = new QLabel(tabStreamdetails);
        lblStereoMode->setObjectName("lblStereoMode");
        lblStereoMode->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblStereoMode, 5, 0, 1, 1);

        stereoMode = new StereoModeComboBox(tabStreamdetails);
        stereoMode->setObjectName("stereoMode");
        stereoMode->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);

        streamDetails->addWidget(stereoMode, 5, 1, 1, 1);


        verticalLayout_6->addLayout(streamDetails);

        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        buttonReloadStreamDetails = new QPushButton(tabStreamdetails);
        buttonReloadStreamDetails->setObjectName("buttonReloadStreamDetails");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/img/arrow_circle_right_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonReloadStreamDetails->setIcon(icon4);

        horizontalLayout_17->addWidget(buttonReloadStreamDetails);

        lblReloadStreamDetailsError = new QLabel(tabStreamdetails);
        lblReloadStreamDetailsError->setObjectName("lblReloadStreamDetailsError");
        lblReloadStreamDetailsError->setText(QString::fromUtf8("Error"));

        horizontalLayout_17->addWidget(lblReloadStreamDetailsError);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_9);


        verticalLayout_6->addLayout(horizontalLayout_17);

        verticalSpacer_8 = new QSpacerItem(20, 278, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_8);

        tabWidget->addTab(tabStreamdetails, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(episodeGroupBox);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Shadow::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(0);
        line->setFrameShape(QFrame::Shape::VLine);

        horizontalLayout_2->addWidget(line);

        verticalLayout_sidebar = new QVBoxLayout();
        verticalLayout_sidebar->setObjectName("verticalLayout_sidebar");
        verticalLayout_sidebar->setContentsMargins(10, 0, -1, -1);
        labelThumbnail = new QLabel(episodeGroupBox);
        labelThumbnail->setObjectName("labelThumbnail");

        verticalLayout_sidebar->addWidget(labelThumbnail);

        thumbnail = new ClosableImage(episodeGroupBox);
        thumbnail->setObjectName("thumbnail");
        thumbnail->setMinimumSize(QSize(180, 120));
        thumbnail->setMaximumSize(QSize(180, 120));
        thumbnail->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        thumbnail->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/thumb.png")));
        thumbnail->setAlignment(Qt::AlignmentFlag::AlignCenter);
        thumbnail->setProperty("clickable", QVariant(true));
        thumbnail->setProperty("myFixedHeight", QVariant(120));

        verticalLayout_sidebar->addWidget(thumbnail);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_sidebar->addItem(verticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout_sidebar);


        verticalLayout->addWidget(episodeGroupBox);

        verticalLayout->setStretch(1, 1);
#if QT_CONFIG(shortcut)
        lblImdbId->setBuddy(imdbId);
        lblTmdbId->setBuddy(tmdbId);
        lblTheTvDbId->setBuddy(tvdbId);
        lblTvmazeId->setBuddy(tvmazeId);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TvShowWidgetEpisode);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(TvShowWidgetEpisode);
    } // setupUi

    void retranslateUi(QWidget *TvShowWidgetEpisode)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Episode has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        episodeName->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Episode Title", nullptr));
        missingLabel->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Episode missing", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlay->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Play movie", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlay->setText(QCoreApplication::translate("TvShowWidgetEpisode", "...", nullptr));
        episodeGroupBox->setTitle(QString());
        lblFiles->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Files", nullptr));
        lblIds->setText(QCoreApplication::translate("TvShowWidgetEpisode", "IDs", nullptr));
        lblImdbId->setText(QCoreApplication::translate("TvShowWidgetEpisode", "IMDb ID", nullptr));
        lblTmdbId->setText(QCoreApplication::translate("TvShowWidgetEpisode", "TMDb ID", nullptr));
        lblTheTvDbId->setText(QCoreApplication::translate("TvShowWidgetEpisode", "TheTVDB ID", nullptr));
        lblTvmazeId->setText(QCoreApplication::translate("TvShowWidgetEpisode", "TVmaze ID", nullptr));
#if QT_CONFIG(tooltip)
        btnImdb->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Open episode on IMDb.com", nullptr));
#endif // QT_CONFIG(tooltip)
        btnImdb->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Go", nullptr));
#if QT_CONFIG(tooltip)
        btnTvmaze->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Open episode on TVmaze.com", nullptr));
#endif // QT_CONFIG(tooltip)
        btnTvmaze->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Go", nullptr));
        lblName->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Name", nullptr));
        lblShowTitle->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Show Title", nullptr));
        lblSeason->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Season", nullptr));
        lblEpisode->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Episode", nullptr));
        lblDisplaySeason->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Display Season", nullptr));
        lblDisplayEpisode->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Display Episode", nullptr));
        lblRating->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Rating", nullptr));
        lblTop250->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Top 250", nullptr));
        lblCertification->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Certification", nullptr));
        lblFirstAired->setText(QCoreApplication::translate("TvShowWidgetEpisode", "First Aired", nullptr));
        lblPlayCount->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Play Count", nullptr));
        lblLastPlayed->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Last Played", nullptr));
        lastPlayed->setDisplayFormat(QCoreApplication::translate("TvShowWidgetEpisode", "dd.MM.yyyy HH:mm", nullptr));
        labelBookmark->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Bookmark", nullptr));
        epBookmark->setDisplayFormat(QCoreApplication::translate("TvShowWidgetEpisode", "HH:mm:ss", nullptr));
        lblStudio->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Studio", nullptr));
        lblOverview->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Overview", nullptr));
        firstAired->setDisplayFormat(QCoreApplication::translate("TvShowWidgetEpisode", "yyyy-MM-dd", nullptr));
        lblMissingFirstAired->setText(QCoreApplication::translate("TvShowWidgetEpisode", "missing", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabInformation), QCoreApplication::translate("TvShowWidgetEpisode", "Information", nullptr));
        lblWriters->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Writers", nullptr));
        QTableWidgetItem *___qtablewidgetitem = writers->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Writer", nullptr));
#if QT_CONFIG(tooltip)
        buttonAddWriter->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Add Writer", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddWriter->setText(QString());
#if QT_CONFIG(tooltip)
        buttonRemoveWriter->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Remove Writer", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveWriter->setText(QString());
        lblDirectors->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Directors", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = directors->horizontalHeaderItem(0);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Director", nullptr));
#if QT_CONFIG(tooltip)
        buttonAddDirector->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Add Director", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddDirector->setText(QString());
#if QT_CONFIG(tooltip)
        buttonRemoveDirector->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Remove Director", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveDirector->setText(QString());
        lblActors->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Actors", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = actors->horizontalHeaderItem(0);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Actor", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = actors->horizontalHeaderItem(1);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Role", nullptr));
#if QT_CONFIG(tooltip)
        buttonRemoveActor->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Remove Actor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveActor->setText(QString());
#if QT_CONFIG(tooltip)
        buttonAddActor->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Add Actor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddActor->setText(QString());
#if QT_CONFIG(tooltip)
        actor->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Click to change", nullptr));
#endif // QT_CONFIG(tooltip)
        actor->setText(QString());
        actorResolution->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Resolution", nullptr));
        lblTags->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Tags", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtended), QCoreApplication::translate("TvShowWidgetEpisode", "Extended", nullptr));
        lblScanType->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Scantype", nullptr));
        videoDuration->setDisplayFormat(QCoreApplication::translate("TvShowWidgetEpisode", "HH:mm:ss", nullptr));
        labelStreamDetailsAudio->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Audio", nullptr));
        lblAspectRatio->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Aspect Ratio", nullptr));
        lblDuration->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Duration", nullptr));
        lblResolution->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Resolution", nullptr));
        lblVideo->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Video", nullptr));
        lblCodec->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Codec", nullptr));
        lblStereoMode->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Stereo Mode", nullptr));
        buttonReloadStreamDetails->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Reload from File", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabStreamdetails), QCoreApplication::translate("TvShowWidgetEpisode", "Streamdetails", nullptr));
        labelThumbnail->setText(QCoreApplication::translate("TvShowWidgetEpisode", "Thumbnail", nullptr));
#if QT_CONFIG(tooltip)
        thumbnail->setToolTip(QCoreApplication::translate("TvShowWidgetEpisode", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        thumbnail->setText(QString());
        (void)TvShowWidgetEpisode;
    } // retranslateUi

};

namespace Ui {
    class TvShowWidgetEpisode: public Ui_TvShowWidgetEpisode {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWWIDGETEPISODE_H
