/********************************************************************************
** Form generated from reading UI file 'TagCloud.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TAGCLOUD_H
#define UI_TAGCLOUD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyWidget.h"

QT_BEGIN_NAMESPACE

class Ui_TagCloud
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *lblTag;
    QLineEdit *lineEdit;
    QScrollArea *tagScrollArea;
    MyWidget *scrollAreaWidgetContents;

    void setupUi(QWidget *TagCloud)
    {
        if (TagCloud->objectName().isEmpty())
            TagCloud->setObjectName("TagCloud");
        TagCloud->resize(400, 300);
        verticalLayout = new QVBoxLayout(TagCloud);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 20);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        lblTag = new QLabel(TagCloud);
        lblTag->setObjectName("lblTag");

        horizontalLayout->addWidget(lblTag);

        lineEdit = new QLineEdit(TagCloud);
        lineEdit->setObjectName("lineEdit");

        horizontalLayout->addWidget(lineEdit);


        verticalLayout->addLayout(horizontalLayout);

        tagScrollArea = new QScrollArea(TagCloud);
        tagScrollArea->setObjectName("tagScrollArea");
        tagScrollArea->setFrameShape(QFrame::NoFrame);
        tagScrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new MyWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 400, 248));
        tagScrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(tagScrollArea);

        verticalLayout->setStretch(1, 1);

        retranslateUi(TagCloud);

        QMetaObject::connectSlotsByName(TagCloud);
    } // setupUi

    void retranslateUi(QWidget *TagCloud)
    {
        lblTag->setText(QCoreApplication::translate("TagCloud", "Tag", nullptr));
        (void)TagCloud;
    } // retranslateUi

};

namespace Ui {
    class TagCloud: public Ui_TagCloud {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TAGCLOUD_H
