/********************************************************************************
** Form generated from reading UI file 'MovieSearchWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIESEARCHWIDGET_H
#define UI_MOVIESEARCHWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/LanguageCombo.h"
#include "ui/small_widgets/MyCheckBox.h"
#include "ui/small_widgets/MyLineEdit.h"
#include "ui/small_widgets/ScrapePreview.h"

QT_BEGIN_NAMESPACE

class Ui_MovieSearchWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QComboBox *comboScraper;
    QLabel *lblLanguage;
    LanguageCombo *comboLanguage;
    QSpacerItem *horizontalSpacer_2;
    MyLineEdit *searchString;
    QHBoxLayout *horizontalLayout_4;
    QLabel *lblSuccessMessage;
    QLabel *lblErrorMessage;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *vboxResults;
    QLabel *lblCustomScraperInfo;
    QSplitter *splitter;
    QTableWidget *results;
    ScrapePreview *preview;
    QVBoxLayout *vboxDetails;
    QGroupBox *detailsGroupBox;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *verticalLayout_41;
    QVBoxLayout *verticalLayout_42;
    MyCheckBox *chkTitle;
    MyCheckBox *chkSet;
    MyCheckBox *chkTagline;
    MyCheckBox *chkRating;
    MyCheckBox *chkReleased;
    MyCheckBox *chkRuntime;
    MyCheckBox *chkDirector;
    MyCheckBox *chkWriter;
    MyCheckBox *chkCertification;
    MyCheckBox *chkTrailer;
    MyCheckBox *chkOverview;
    MyCheckBox *chkPoster;
    QSpacerItem *verticalSpacer_31;
    QVBoxLayout *verticalLayout_43;
    MyCheckBox *chkBackdrop;
    MyCheckBox *chkActors;
    MyCheckBox *chkGenres;
    MyCheckBox *chkStudios;
    MyCheckBox *chkCountries;
    MyCheckBox *chkTags;
    MyCheckBox *chkLogo;
    MyCheckBox *chkClearArt;
    MyCheckBox *chkCdArt;
    MyCheckBox *chkBanner;
    MyCheckBox *chkThumb;
    QSpacerItem *verticalSpacer_32;
    QFrame *line;
    QCheckBox *chkUnCheckAll;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *MovieSearchWidget)
    {
        if (MovieSearchWidget->objectName().isEmpty())
            MovieSearchWidget->setObjectName("MovieSearchWidget");
        MovieSearchWidget->resize(720, 661);
        verticalLayout = new QVBoxLayout(MovieSearchWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(MovieSearchWidget);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        comboScraper = new QComboBox(MovieSearchWidget);
        comboScraper->setObjectName("comboScraper");

        horizontalLayout_2->addWidget(comboScraper);

        lblLanguage = new QLabel(MovieSearchWidget);
        lblLanguage->setObjectName("lblLanguage");

        horizontalLayout_2->addWidget(lblLanguage);

        comboLanguage = new LanguageCombo(MovieSearchWidget);
        comboLanguage->setObjectName("comboLanguage");
        comboLanguage->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout_2->addWidget(comboLanguage);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_2);

        searchString = new MyLineEdit(MovieSearchWidget);
        searchString->setObjectName("searchString");

        verticalLayout->addWidget(searchString);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        lblSuccessMessage = new QLabel(MovieSearchWidget);
        lblSuccessMessage->setObjectName("lblSuccessMessage");

        horizontalLayout_4->addWidget(lblSuccessMessage);

        lblErrorMessage = new QLabel(MovieSearchWidget);
        lblErrorMessage->setObjectName("lblErrorMessage");
        lblErrorMessage->setText(QString::fromUtf8(""));
        lblErrorMessage->setTextFormat(Qt::PlainText);

        horizontalLayout_4->addWidget(lblErrorMessage);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        vboxResults = new QVBoxLayout();
        vboxResults->setObjectName("vboxResults");
        lblCustomScraperInfo = new QLabel(MovieSearchWidget);
        lblCustomScraperInfo->setObjectName("lblCustomScraperInfo");
        lblCustomScraperInfo->setText(QString::fromUtf8("Custom Movie Scraper Infos"));

        vboxResults->addWidget(lblCustomScraperInfo);

        splitter = new QSplitter(MovieSearchWidget);
        splitter->setObjectName("splitter");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(splitter->sizePolicy().hasHeightForWidth());
        splitter->setSizePolicy(sizePolicy);
        splitter->setOrientation(Qt::Vertical);
        splitter->setOpaqueResize(false);
        splitter->setChildrenCollapsible(false);
        results = new QTableWidget(splitter);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(results->sizePolicy().hasHeightForWidth());
        results->setSizePolicy(sizePolicy1);
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        splitter->addWidget(results);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);
        preview = new ScrapePreview(splitter);
        preview->setObjectName("preview");
        splitter->addWidget(preview);

        vboxResults->addWidget(splitter);


        horizontalLayout_3->addLayout(vboxResults);

        vboxDetails = new QVBoxLayout();
        vboxDetails->setObjectName("vboxDetails");
        detailsGroupBox = new QGroupBox(MovieSearchWidget);
        detailsGroupBox->setObjectName("detailsGroupBox");
        verticalLayout_4 = new QVBoxLayout(detailsGroupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalSpacer_2 = new QSpacerItem(20, 6, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_2);

        verticalLayout_41 = new QHBoxLayout();
        verticalLayout_41->setObjectName("verticalLayout_41");
        verticalLayout_42 = new QVBoxLayout();
        verticalLayout_42->setObjectName("verticalLayout_42");
        chkTitle = new MyCheckBox(detailsGroupBox);
        chkTitle->setObjectName("chkTitle");

        verticalLayout_42->addWidget(chkTitle);

        chkSet = new MyCheckBox(detailsGroupBox);
        chkSet->setObjectName("chkSet");

        verticalLayout_42->addWidget(chkSet);

        chkTagline = new MyCheckBox(detailsGroupBox);
        chkTagline->setObjectName("chkTagline");

        verticalLayout_42->addWidget(chkTagline);

        chkRating = new MyCheckBox(detailsGroupBox);
        chkRating->setObjectName("chkRating");

        verticalLayout_42->addWidget(chkRating);

        chkReleased = new MyCheckBox(detailsGroupBox);
        chkReleased->setObjectName("chkReleased");

        verticalLayout_42->addWidget(chkReleased);

        chkRuntime = new MyCheckBox(detailsGroupBox);
        chkRuntime->setObjectName("chkRuntime");

        verticalLayout_42->addWidget(chkRuntime);

        chkDirector = new MyCheckBox(detailsGroupBox);
        chkDirector->setObjectName("chkDirector");

        verticalLayout_42->addWidget(chkDirector);

        chkWriter = new MyCheckBox(detailsGroupBox);
        chkWriter->setObjectName("chkWriter");

        verticalLayout_42->addWidget(chkWriter);

        chkCertification = new MyCheckBox(detailsGroupBox);
        chkCertification->setObjectName("chkCertification");

        verticalLayout_42->addWidget(chkCertification);

        chkTrailer = new MyCheckBox(detailsGroupBox);
        chkTrailer->setObjectName("chkTrailer");

        verticalLayout_42->addWidget(chkTrailer);

        chkOverview = new MyCheckBox(detailsGroupBox);
        chkOverview->setObjectName("chkOverview");

        verticalLayout_42->addWidget(chkOverview);

        chkPoster = new MyCheckBox(detailsGroupBox);
        chkPoster->setObjectName("chkPoster");

        verticalLayout_42->addWidget(chkPoster);

        verticalSpacer_31 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_42->addItem(verticalSpacer_31);


        verticalLayout_41->addLayout(verticalLayout_42);

        verticalLayout_43 = new QVBoxLayout();
        verticalLayout_43->setObjectName("verticalLayout_43");
        chkBackdrop = new MyCheckBox(detailsGroupBox);
        chkBackdrop->setObjectName("chkBackdrop");

        verticalLayout_43->addWidget(chkBackdrop);

        chkActors = new MyCheckBox(detailsGroupBox);
        chkActors->setObjectName("chkActors");

        verticalLayout_43->addWidget(chkActors);

        chkGenres = new MyCheckBox(detailsGroupBox);
        chkGenres->setObjectName("chkGenres");

        verticalLayout_43->addWidget(chkGenres);

        chkStudios = new MyCheckBox(detailsGroupBox);
        chkStudios->setObjectName("chkStudios");

        verticalLayout_43->addWidget(chkStudios);

        chkCountries = new MyCheckBox(detailsGroupBox);
        chkCountries->setObjectName("chkCountries");

        verticalLayout_43->addWidget(chkCountries);

        chkTags = new MyCheckBox(detailsGroupBox);
        chkTags->setObjectName("chkTags");

        verticalLayout_43->addWidget(chkTags);

        chkLogo = new MyCheckBox(detailsGroupBox);
        chkLogo->setObjectName("chkLogo");

        verticalLayout_43->addWidget(chkLogo);

        chkClearArt = new MyCheckBox(detailsGroupBox);
        chkClearArt->setObjectName("chkClearArt");

        verticalLayout_43->addWidget(chkClearArt);

        chkCdArt = new MyCheckBox(detailsGroupBox);
        chkCdArt->setObjectName("chkCdArt");

        verticalLayout_43->addWidget(chkCdArt);

        chkBanner = new MyCheckBox(detailsGroupBox);
        chkBanner->setObjectName("chkBanner");

        verticalLayout_43->addWidget(chkBanner);

        chkThumb = new MyCheckBox(detailsGroupBox);
        chkThumb->setObjectName("chkThumb");

        verticalLayout_43->addWidget(chkThumb);

        verticalSpacer_32 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_43->addItem(verticalSpacer_32);


        verticalLayout_41->addLayout(verticalLayout_43);


        verticalLayout_4->addLayout(verticalLayout_41);

        line = new QFrame(detailsGroupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        chkUnCheckAll = new QCheckBox(detailsGroupBox);
        chkUnCheckAll->setObjectName("chkUnCheckAll");

        verticalLayout_4->addWidget(chkUnCheckAll);


        vboxDetails->addWidget(detailsGroupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxDetails->addItem(verticalSpacer);


        horizontalLayout_3->addLayout(vboxDetails);

        horizontalLayout_3->setStretch(0, 1);

        verticalLayout->addLayout(horizontalLayout_3);

        verticalLayout->setStretch(3, 1);

        retranslateUi(MovieSearchWidget);

        QMetaObject::connectSlotsByName(MovieSearchWidget);
    } // setupUi

    void retranslateUi(QWidget *MovieSearchWidget)
    {
        label->setText(QCoreApplication::translate("MovieSearchWidget", "Scraper", nullptr));
        lblLanguage->setText(QCoreApplication::translate("MovieSearchWidget", "Language", nullptr));
#if QT_CONFIG(tooltip)
        searchString->setToolTip(QCoreApplication::translate("MovieSearchWidget", "When using IMDb you can also use the IMDb id as search query.\n"
"If you want to search by an TMDb id please prefix it with \"id\".", nullptr));
#endif // QT_CONFIG(tooltip)
        lblSuccessMessage->setText(QString());
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("MovieSearchWidget", "Result", nullptr));
        chkTitle->setText(QCoreApplication::translate("MovieSearchWidget", "Title", nullptr));
        chkSet->setText(QCoreApplication::translate("MovieSearchWidget", "Set", nullptr));
        chkTagline->setText(QCoreApplication::translate("MovieSearchWidget", "Tagline", nullptr));
        chkRating->setText(QCoreApplication::translate("MovieSearchWidget", "Rating", nullptr));
        chkReleased->setText(QCoreApplication::translate("MovieSearchWidget", "Released", nullptr));
        chkRuntime->setText(QCoreApplication::translate("MovieSearchWidget", "Runtime", nullptr));
        chkDirector->setText(QCoreApplication::translate("MovieSearchWidget", "Director", nullptr));
        chkWriter->setText(QCoreApplication::translate("MovieSearchWidget", "Writer", nullptr));
        chkCertification->setText(QCoreApplication::translate("MovieSearchWidget", "Certification", nullptr));
        chkTrailer->setText(QCoreApplication::translate("MovieSearchWidget", "Trailer", nullptr));
        chkOverview->setText(QCoreApplication::translate("MovieSearchWidget", "Overview", nullptr));
        chkPoster->setText(QCoreApplication::translate("MovieSearchWidget", "Poster", nullptr));
        chkBackdrop->setText(QCoreApplication::translate("MovieSearchWidget", "Backdrop", nullptr));
        chkActors->setText(QCoreApplication::translate("MovieSearchWidget", "Actors", nullptr));
        chkGenres->setText(QCoreApplication::translate("MovieSearchWidget", "Genres", nullptr));
        chkStudios->setText(QCoreApplication::translate("MovieSearchWidget", "Studios", nullptr));
        chkCountries->setText(QCoreApplication::translate("MovieSearchWidget", "Countries", nullptr));
        chkTags->setText(QCoreApplication::translate("MovieSearchWidget", "Tags", nullptr));
        chkLogo->setText(QCoreApplication::translate("MovieSearchWidget", "Logo", nullptr));
        chkClearArt->setText(QCoreApplication::translate("MovieSearchWidget", "Clear Art", nullptr));
        chkCdArt->setText(QCoreApplication::translate("MovieSearchWidget", "CD Art", nullptr));
        chkBanner->setText(QCoreApplication::translate("MovieSearchWidget", "Banner", nullptr));
        chkThumb->setText(QCoreApplication::translate("MovieSearchWidget", "Thumb", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("MovieSearchWidget", "(Un)Check all", nullptr));
        (void)MovieSearchWidget;
    } // retranslateUi

};

namespace Ui {
    class MovieSearchWidget: public Ui_MovieSearchWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIESEARCHWIDGET_H
