/****************************************************************************
** Meta object code from reading C++ file 'FernsehserienDe.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../src/scrapers/tv_show/fernsehserien_de/FernsehserienDe.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FernsehserienDe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper15FernsehserienDeE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper15FernsehserienDeE = QtMocHelpers::stringData(
    "mediaelch::scraper::FernsehserienDe"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper15FernsehserienDeE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::FernsehserienDe::staticMetaObject = { {
    QMetaObject::SuperData::link<TvScraper::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper15FernsehserienDeE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper15FernsehserienDeE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper15FernsehserienDeE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FernsehserienDe, std::true_type>
    >,
    nullptr
} };

void mediaelch::scraper::FernsehserienDe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FernsehserienDe *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *mediaelch::scraper::FernsehserienDe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::FernsehserienDe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper15FernsehserienDeE.stringdata0))
        return static_cast<void*>(this);
    return TvScraper::qt_metacast(_clname);
}

int mediaelch::scraper::FernsehserienDe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TvScraper::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::FernsehserienDeShowSearchJob",
    "onSearchPageLoaded",
    ""
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   20,    2, 0x08,    1 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::FernsehserienDeShowSearchJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ShowSearchJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FernsehserienDeShowSearchJob, std::true_type>,
        // method 'onSearchPageLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::FernsehserienDeShowSearchJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FernsehserienDeShowSearchJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onSearchPageLoaded(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *mediaelch::scraper::FernsehserienDeShowSearchJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::FernsehserienDeShowSearchJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper28FernsehserienDeShowSearchJobE.stringdata0))
        return static_cast<void*>(this);
    return ShowSearchJob::qt_metacast(_clname);
}

int mediaelch::scraper::FernsehserienDeShowSearchJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ShowSearchJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::FernsehserienDeShowScrapeJob"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::FernsehserienDeShowScrapeJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ShowScrapeJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FernsehserienDeShowScrapeJob, std::true_type>
    >,
    nullptr
} };

void mediaelch::scraper::FernsehserienDeShowScrapeJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FernsehserienDeShowScrapeJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *mediaelch::scraper::FernsehserienDeShowScrapeJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::FernsehserienDeShowScrapeJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper28FernsehserienDeShowScrapeJobE.stringdata0))
        return static_cast<void*>(this);
    return ShowScrapeJob::qt_metacast(_clname);
}

int mediaelch::scraper::FernsehserienDeShowScrapeJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ShowScrapeJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::FernsehserienDeSeasonScrapeJob",
    "onSeasonPageLoaded",
    ""
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   20,    2, 0x08,    1 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::FernsehserienDeSeasonScrapeJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SeasonScrapeJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FernsehserienDeSeasonScrapeJob, std::true_type>,
        // method 'onSeasonPageLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::FernsehserienDeSeasonScrapeJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FernsehserienDeSeasonScrapeJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onSeasonPageLoaded(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *mediaelch::scraper::FernsehserienDeSeasonScrapeJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::FernsehserienDeSeasonScrapeJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper30FernsehserienDeSeasonScrapeJobE.stringdata0))
        return static_cast<void*>(this);
    return SeasonScrapeJob::qt_metacast(_clname);
}

int mediaelch::scraper::FernsehserienDeSeasonScrapeJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SeasonScrapeJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::FernsehserienDeEpisodeScrapeJob",
    "onSeasonPageLoaded",
    ""
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   20,    2, 0x08,    1 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::FernsehserienDeEpisodeScrapeJob::staticMetaObject = { {
    QMetaObject::SuperData::link<EpisodeScrapeJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FernsehserienDeEpisodeScrapeJob, std::true_type>,
        // method 'onSeasonPageLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::FernsehserienDeEpisodeScrapeJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FernsehserienDeEpisodeScrapeJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onSeasonPageLoaded(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *mediaelch::scraper::FernsehserienDeEpisodeScrapeJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::FernsehserienDeEpisodeScrapeJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper31FernsehserienDeEpisodeScrapeJobE.stringdata0))
        return static_cast<void*>(this);
    return EpisodeScrapeJob::qt_metacast(_clname);
}

int mediaelch::scraper::FernsehserienDeEpisodeScrapeJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EpisodeScrapeJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
