/********************************************************************************
** Form generated from reading UI file 'MovieDuplicateItem.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIEDUPLICATEITEM_H
#define UI_MOVIEDUPLICATEITEM_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MovieDuplicateItem
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *widget;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelMovieTitle;
    QLabel *labelFiles;
    QWidget *widget_2;
    QGridLayout *gridLayout_2;
    QLabel *iconTitle;
    QLabel *iconTmdbId;
    QLabel *label_6;
    QLabel *iconImdbId;
    QLabel *label;
    QLabel *label_3;

    void setupUi(QWidget *MovieDuplicateItem)
    {
        if (MovieDuplicateItem->objectName().isEmpty())
            MovieDuplicateItem->setObjectName("MovieDuplicateItem");
        MovieDuplicateItem->resize(731, 151);
        verticalLayout = new QVBoxLayout(MovieDuplicateItem);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(MovieDuplicateItem);
        widget->setObjectName("widget");
        verticalLayout_2 = new QVBoxLayout(widget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        labelMovieTitle = new QLabel(widget);
        labelMovieTitle->setObjectName("labelMovieTitle");
        QFont font;
        font.setPointSize(16);
        labelMovieTitle->setFont(font);
        labelMovieTitle->setText(QString::fromUtf8("Title"));
        labelMovieTitle->setProperty("isHeader", QVariant(true));

        verticalLayout_2->addWidget(labelMovieTitle);

        labelFiles = new QLabel(widget);
        labelFiles->setObjectName("labelFiles");
        labelFiles->setText(QString::fromUtf8("Files"));
        labelFiles->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        verticalLayout_2->addWidget(labelFiles);

        widget_2 = new QWidget(widget);
        widget_2->setObjectName("widget_2");
        gridLayout_2 = new QGridLayout(widget_2);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        iconTitle = new QLabel(widget_2);
        iconTitle->setObjectName("iconTitle");
        iconTitle->setMinimumSize(QSize(20, 20));
        iconTitle->setMaximumSize(QSize(20, 20));
        iconTitle->setText(QString::fromUtf8("TextLabel"));
        iconTitle->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(iconTitle, 2, 0, 1, 1);

        iconTmdbId = new QLabel(widget_2);
        iconTmdbId->setObjectName("iconTmdbId");
        iconTmdbId->setMinimumSize(QSize(20, 20));
        iconTmdbId->setMaximumSize(QSize(20, 20));
        iconTmdbId->setText(QString::fromUtf8("TextLabel"));
        iconTmdbId->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(iconTmdbId, 1, 0, 1, 1);

        label_6 = new QLabel(widget_2);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 2, 1, 1, 1);

        iconImdbId = new QLabel(widget_2);
        iconImdbId->setObjectName("iconImdbId");
        iconImdbId->setMinimumSize(QSize(20, 20));
        iconImdbId->setMaximumSize(QSize(20, 20));
        iconImdbId->setText(QString::fromUtf8("TextLabel"));
        iconImdbId->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(iconImdbId, 0, 0, 1, 1);

        label = new QLabel(widget_2);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 1, 1, 1, 1);

        label_3 = new QLabel(widget_2);
        label_3->setObjectName("label_3");

        gridLayout_2->addWidget(label_3, 0, 1, 1, 1);

        gridLayout_2->setColumnStretch(1, 1);

        verticalLayout_2->addWidget(widget_2);


        verticalLayout->addWidget(widget);


        retranslateUi(MovieDuplicateItem);

        QMetaObject::connectSlotsByName(MovieDuplicateItem);
    } // setupUi

    void retranslateUi(QWidget *MovieDuplicateItem)
    {
        label_6->setText(QCoreApplication::translate("MovieDuplicateItem", "Movie title", nullptr));
        label->setText(QCoreApplication::translate("MovieDuplicateItem", "TMDb ID", nullptr));
        label_3->setText(QCoreApplication::translate("MovieDuplicateItem", "IMDb ID", nullptr));
        (void)MovieDuplicateItem;
    } // retranslateUi

};

namespace Ui {
    class MovieDuplicateItem: public Ui_MovieDuplicateItem {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIEDUPLICATEITEM_H
