/********************************************************************************
** Form generated from reading UI file 'AboutDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hLayout_title;
    QLabel *labelMediaElch;
    QSpacerItem *horizontalSpacer_8;
    QHBoxLayout *horizontalLayout;
    QLabel *icon;
    QSpacerItem *horizontalSpacer_2;
    QVBoxLayout *vLayout_infos;
    QTabWidget *tabWidget;
    QWidget *tabAbout;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *lblQt;
    QSpacerItem *horizontalSpacer_9;
    QPushButton *btnAboutQt;
    QLabel *lblIconSet;
    QLabel *lblKdeBreezeIcons;
    QLabel *lblLogoSource;
    QLabel *lblMediaInfo;
    QLabel *lblMediaElchDe;
    QLabel *lblGitHub;
    QSpacerItem *verticalSpacer_3;
    QWidget *tabDev;
    QVBoxLayout *verticalLayout_4;
    QLabel *lblDeveloper;
    QTextBrowser *txtDetails;
    QHBoxLayout *clipBoardLayout;
    QSpacerItem *clipBoardSpacer;
    QPushButton *copyToClipboard;
    QLabel *lblCollection;
    QHBoxLayout *horizontalLayout_2;
    QLabel *lblMovies;
    QLabel *numMovies;
    QSpacerItem *horizontalSpacer_3;
    QLabel *lblTvShows;
    QLabel *numShows;
    QSpacerItem *horizontalSpacer_4;
    QLabel *lblEpisodes;
    QLabel *numEpisodes;
    QSpacerItem *horizontalSpacer_5;
    QLabel *lblConcerts;
    QLabel *numConcerts;
    QSpacerItem *horizontalSpacer;
    QLabel *lblArtist;
    QLabel *numArtists;
    QSpacerItem *horizontalSpacer_7;
    QLabel *lblAlbums;
    QLabel *numAlbums;
    QSpacerItem *horizontalSpacer_6;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName("AboutDialog");
        AboutDialog->setWindowModality(Qt::WindowModal);
        AboutDialog->resize(680, 480);
        AboutDialog->setMinimumSize(QSize(680, 400));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        AboutDialog->setWindowIcon(icon1);
        AboutDialog->setModal(true);
        verticalLayout = new QVBoxLayout(AboutDialog);
        verticalLayout->setObjectName("verticalLayout");
        hLayout_title = new QHBoxLayout();
        hLayout_title->setObjectName("hLayout_title");
        labelMediaElch = new QLabel(AboutDialog);
        labelMediaElch->setObjectName("labelMediaElch");
        QFont font;
        font.setPointSize(16);
        font.setBold(true);
        labelMediaElch->setFont(font);

        hLayout_title->addWidget(labelMediaElch);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hLayout_title->addItem(horizontalSpacer_8);


        verticalLayout->addLayout(hLayout_title);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        icon = new QLabel(AboutDialog);
        icon->setObjectName("icon");
        icon->setMinimumSize(QSize(128, 128));
        icon->setMaximumSize(QSize(128, 128));
        icon->setPixmap(QPixmap(QString::fromUtf8(":/img/MediaElch.png")));

        horizontalLayout->addWidget(icon);

        horizontalSpacer_2 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        vLayout_infos = new QVBoxLayout();
        vLayout_infos->setObjectName("vLayout_infos");
        tabWidget = new QTabWidget(AboutDialog);
        tabWidget->setObjectName("tabWidget");
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        verticalLayout_2 = new QVBoxLayout(tabAbout);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        lblQt = new QLabel(tabAbout);
        lblQt->setObjectName("lblQt");
        lblQt->setWordWrap(true);
        lblQt->setOpenExternalLinks(true);

        horizontalLayout_3->addWidget(lblQt);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_9);

        btnAboutQt = new QPushButton(tabAbout);
        btnAboutQt->setObjectName("btnAboutQt");

        horizontalLayout_3->addWidget(btnAboutQt);

        horizontalLayout_3->setStretch(0, 1);

        verticalLayout_2->addLayout(horizontalLayout_3);

        lblIconSet = new QLabel(tabAbout);
        lblIconSet->setObjectName("lblIconSet");
        lblIconSet->setWordWrap(true);

        verticalLayout_2->addWidget(lblIconSet);

        lblKdeBreezeIcons = new QLabel(tabAbout);
        lblKdeBreezeIcons->setObjectName("lblKdeBreezeIcons");
        lblKdeBreezeIcons->setWordWrap(true);
        lblKdeBreezeIcons->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(lblKdeBreezeIcons);

        lblLogoSource = new QLabel(tabAbout);
        lblLogoSource->setObjectName("lblLogoSource");
        lblLogoSource->setWordWrap(true);

        verticalLayout_2->addWidget(lblLogoSource);

        lblMediaInfo = new QLabel(tabAbout);
        lblMediaInfo->setObjectName("lblMediaInfo");
        lblMediaInfo->setWordWrap(true);
        lblMediaInfo->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(lblMediaInfo);

        lblMediaElchDe = new QLabel(tabAbout);
        lblMediaElchDe->setObjectName("lblMediaElchDe");
        lblMediaElchDe->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(lblMediaElchDe);

        lblGitHub = new QLabel(tabAbout);
        lblGitHub->setObjectName("lblGitHub");
        lblGitHub->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(lblGitHub);

        verticalSpacer_3 = new QSpacerItem(20, 33, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);

        tabWidget->addTab(tabAbout, QString());
        tabDev = new QWidget();
        tabDev->setObjectName("tabDev");
        verticalLayout_4 = new QVBoxLayout(tabDev);
        verticalLayout_4->setObjectName("verticalLayout_4");
        lblDeveloper = new QLabel(tabDev);
        lblDeveloper->setObjectName("lblDeveloper");
        lblDeveloper->setWordWrap(true);

        verticalLayout_4->addWidget(lblDeveloper);

        txtDetails = new QTextBrowser(tabDev);
        txtDetails->setObjectName("txtDetails");

        verticalLayout_4->addWidget(txtDetails);

        clipBoardLayout = new QHBoxLayout();
        clipBoardLayout->setObjectName("clipBoardLayout");
        clipBoardSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        clipBoardLayout->addItem(clipBoardSpacer);

        copyToClipboard = new QPushButton(tabDev);
        copyToClipboard->setObjectName("copyToClipboard");

        clipBoardLayout->addWidget(copyToClipboard);


        verticalLayout_4->addLayout(clipBoardLayout);

        tabWidget->addTab(tabDev, QString());

        vLayout_infos->addWidget(tabWidget);


        horizontalLayout->addLayout(vLayout_infos);

        horizontalLayout->setStretch(2, 1);

        verticalLayout->addLayout(horizontalLayout);

        lblCollection = new QLabel(AboutDialog);
        lblCollection->setObjectName("lblCollection");
        lblCollection->setFont(font);

        verticalLayout->addWidget(lblCollection);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        lblMovies = new QLabel(AboutDialog);
        lblMovies->setObjectName("lblMovies");

        horizontalLayout_2->addWidget(lblMovies);

        numMovies = new QLabel(AboutDialog);
        numMovies->setObjectName("numMovies");
        numMovies->setText(QString::fromUtf8("0"));
        numMovies->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(numMovies);

        horizontalSpacer_3 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        lblTvShows = new QLabel(AboutDialog);
        lblTvShows->setObjectName("lblTvShows");

        horizontalLayout_2->addWidget(lblTvShows);

        numShows = new QLabel(AboutDialog);
        numShows->setObjectName("numShows");
        numShows->setText(QString::fromUtf8("0"));
        numShows->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(numShows);

        horizontalSpacer_4 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_4);

        lblEpisodes = new QLabel(AboutDialog);
        lblEpisodes->setObjectName("lblEpisodes");

        horizontalLayout_2->addWidget(lblEpisodes);

        numEpisodes = new QLabel(AboutDialog);
        numEpisodes->setObjectName("numEpisodes");
        numEpisodes->setText(QString::fromUtf8("0"));
        numEpisodes->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(numEpisodes);

        horizontalSpacer_5 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        lblConcerts = new QLabel(AboutDialog);
        lblConcerts->setObjectName("lblConcerts");

        horizontalLayout_2->addWidget(lblConcerts);

        numConcerts = new QLabel(AboutDialog);
        numConcerts->setObjectName("numConcerts");
        numConcerts->setText(QString::fromUtf8("0"));
        numConcerts->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(numConcerts);

        horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        lblArtist = new QLabel(AboutDialog);
        lblArtist->setObjectName("lblArtist");

        horizontalLayout_2->addWidget(lblArtist);

        numArtists = new QLabel(AboutDialog);
        numArtists->setObjectName("numArtists");
        numArtists->setText(QString::fromUtf8("0"));

        horizontalLayout_2->addWidget(numArtists);

        horizontalSpacer_7 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_7);

        lblAlbums = new QLabel(AboutDialog);
        lblAlbums->setObjectName("lblAlbums");

        horizontalLayout_2->addWidget(lblAlbums);

        numAlbums = new QLabel(AboutDialog);
        numAlbums->setObjectName("numAlbums");
        numAlbums->setText(QString::fromUtf8("0"));

        horizontalLayout_2->addWidget(numAlbums);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_6);


        verticalLayout->addLayout(horizontalLayout_2);

        buttonBox = new QDialogButtonBox(AboutDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(AboutDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, AboutDialog, qOverload<>(&QDialog::accept));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About MediaElch", nullptr));
        labelMediaElch->setText(QCoreApplication::translate("AboutDialog", "MediaElch", nullptr));
        icon->setText(QString());
        lblQt->setText(QCoreApplication::translate("AboutDialog", "MediaElch was built with <a href=\"https://www.qt.io/\">Qt</a>", nullptr));
        btnAboutQt->setText(QCoreApplication::translate("AboutDialog", "About Qt", nullptr));
        lblIconSet->setText(QCoreApplication::translate("AboutDialog", "Icon Sets: Retina by \"The Working Group\" and \"Capital Suite\" by \"capital18 (Jugal Paryani)\"", nullptr));
        lblKdeBreezeIcons->setText(QCoreApplication::translate("AboutDialog", "<a href=\"https://develop.kde.org/frameworks/breeze-icons/\" title=\"KDE Breeze Website\">Breeze icons</a> copyright KDE and licenced under the GNU LGPL version 3 or later", nullptr));
        lblLogoSource->setText(QCoreApplication::translate("AboutDialog", "MediaElch Icon by Kathrin Luckner", nullptr));
        lblMediaInfo->setText(QCoreApplication::translate("AboutDialog", "Stream Details detection with <a href=\"https://www.mediaarea.net\">Media Info</a>", nullptr));
        lblMediaElchDe->setText(QCoreApplication::translate("AboutDialog", "<a href=\"https://www.mediaelch.de/mediaelch/\">http://www.mediaelch.de</a> powered by <a href=\"https://www.kvibes.de\">k.vibes</a>", nullptr));
        lblGitHub->setText(QCoreApplication::translate("AboutDialog", "Bugs and wishes can be reported on <a href=\"https://github.com/Komet/MediaElch/issues\">GitHub</a> .", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAbout), QCoreApplication::translate("AboutDialog", "About", nullptr));
        lblDeveloper->setText(QCoreApplication::translate("AboutDialog", "The information below is important for developers. Please provide if you need help.", nullptr));
        copyToClipboard->setText(QCoreApplication::translate("AboutDialog", "Copy to clipboard", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabDev), QCoreApplication::translate("AboutDialog", "Developer", nullptr));
        lblCollection->setText(QCoreApplication::translate("AboutDialog", "Your Collection", nullptr));
        lblMovies->setText(QCoreApplication::translate("AboutDialog", "Movies", nullptr));
        lblTvShows->setText(QCoreApplication::translate("AboutDialog", "TV Shows", nullptr));
        lblEpisodes->setText(QCoreApplication::translate("AboutDialog", "Episodes", nullptr));
        lblConcerts->setText(QCoreApplication::translate("AboutDialog", "Concerts", nullptr));
        lblArtist->setText(QCoreApplication::translate("AboutDialog", "Artists", nullptr));
        lblAlbums->setText(QCoreApplication::translate("AboutDialog", "Albums", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
