/********************************************************************************
** Form generated from reading UI file 'FilterWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERWIDGET_H
#define UI_FILTERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyLineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_FilterWidget
{
public:
    QHBoxLayout *horizontalLayout;
    MyLineEdit *lineEdit;

    void setupUi(QWidget *FilterWidget)
    {
        if (FilterWidget->objectName().isEmpty())
            FilterWidget->setObjectName("FilterWidget");
        FilterWidget->resize(400, 77);
        horizontalLayout = new QHBoxLayout(FilterWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        lineEdit = new MyLineEdit(FilterWidget);
        lineEdit->setObjectName("lineEdit");

        horizontalLayout->addWidget(lineEdit);

        horizontalLayout->setStretch(0, 1);

        retranslateUi(FilterWidget);

        QMetaObject::connectSlotsByName(FilterWidget);
    } // setupUi

    void retranslateUi(QWidget *FilterWidget)
    {
        lineEdit->setPlaceholderText(QCoreApplication::translate("FilterWidget", "Filter", nullptr));
        (void)FilterWidget;
    } // retranslateUi

};

namespace Ui {
    class FilterWidget: public Ui_FilterWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERWIDGET_H
