/********************************************************************************
** Form generated from reading UI file 'SettingsWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSWINDOW_H
#define UI_SETTINGSWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/settings/ConcertSettingsWidget.h"
#include "ui/settings/ExportSettingsWidget.h"
#include "ui/settings/GlobalSettingsWidget.h"
#include "ui/settings/ImportSettingsWidget.h"
#include "ui/settings/KodiSettingsWidget.h"
#include "ui/settings/MovieSettingsWidget.h"
#include "ui/settings/MusicSettingsWidget.h"
#include "ui/settings/NetworkSettingsWidget.h"
#include "ui/settings/ScraperSettingsWidget.h"
#include "ui/settings/TvShowSettingsWidget.h"

QT_BEGIN_NAMESPACE

class Ui_SettingsWindow
{
public:
    QAction *actionMovies;
    QAction *actionTV_Shows;
    QAction *actionConcerts;
    QAction *actionGlobal;
    QAction *actionXBMC;
    QAction *actionNetwork;
    QAction *actionScraper;
    QAction *actionExport;
    QAction *actionImport;
    QAction *actionMusic;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout;
    QStackedWidget *settingsTabs;
    GlobalSettingsWidget *globalSettings;
    ScraperSettingsWidget *scraperSettings;
    MovieSettingsWidget *movieSettings;
    TvShowSettingsWidget *tvShowSettings;
    ConcertSettingsWidget *concertSettings;
    MusicSettingsWidget *musicSettings;
    KodiSettingsWidget *kodiSettings;
    NetworkSettingsWidget *networkSettings;
    ExportSettingsWidget *exportSettings;
    ImportSettingsWidget *importSettings;
    QHBoxLayout *horizontalLayoutButtons;
    QSpacerItem *horizontalSpacerButtons;
    QPushButton *btnCancel;
    QPushButton *btnSave;
    QToolBar *toolBar;

    void setupUi(QMainWindow *SettingsWindow)
    {
        if (SettingsWindow->objectName().isEmpty())
            SettingsWindow->setObjectName("SettingsWindow");
        SettingsWindow->resize(960, 640);
        SettingsWindow->setMinimumSize(QSize(600, 480));
        SettingsWindow->setUnifiedTitleAndToolBarOnMac(false);
        actionMovies = new QAction(SettingsWindow);
        actionMovies->setObjectName("actionMovies");
        actionMovies->setProperty("page", QVariant(2));
        actionMovies->setProperty("iconName", QVariant(QString::fromUtf8("film")));
        actionTV_Shows = new QAction(SettingsWindow);
        actionTV_Shows->setObjectName("actionTV_Shows");
        actionTV_Shows->setProperty("page", QVariant(3));
        actionTV_Shows->setProperty("iconName", QVariant(QString::fromUtf8("monitor")));
        actionConcerts = new QAction(SettingsWindow);
        actionConcerts->setObjectName("actionConcerts");
        actionConcerts->setProperty("page", QVariant(4));
        actionConcerts->setProperty("iconName", QVariant(QString::fromUtf8("micro")));
        actionGlobal = new QAction(SettingsWindow);
        actionGlobal->setObjectName("actionGlobal");
        actionGlobal->setProperty("page", QVariant(0));
        actionGlobal->setProperty("iconName", QVariant(QString::fromUtf8("tools")));
        actionXBMC = new QAction(SettingsWindow);
        actionXBMC->setObjectName("actionXBMC");
        actionXBMC->setProperty("page", QVariant(6));
        actionXBMC->setProperty("iconName", QVariant(QString::fromUtf8("network")));
        actionNetwork = new QAction(SettingsWindow);
        actionNetwork->setObjectName("actionNetwork");
        actionNetwork->setProperty("page", QVariant(7));
        actionNetwork->setProperty("iconName", QVariant(QString::fromUtf8("world")));
        actionScraper = new QAction(SettingsWindow);
        actionScraper->setObjectName("actionScraper");
        actionScraper->setProperty("page", QVariant(1));
        actionScraper->setProperty("iconName", QVariant(QString::fromUtf8("search")));
        actionExport = new QAction(SettingsWindow);
        actionExport->setObjectName("actionExport");
        actionExport->setProperty("page", QVariant(8));
        actionExport->setProperty("iconName", QVariant(QString::fromUtf8("cloud_upload")));
        actionImport = new QAction(SettingsWindow);
        actionImport->setObjectName("actionImport");
        actionImport->setProperty("page", QVariant(9));
        actionImport->setProperty("iconName", QVariant(QString::fromUtf8("download")));
        actionMusic = new QAction(SettingsWindow);
        actionMusic->setObjectName("actionMusic");
        actionMusic->setProperty("page", QVariant(5));
        actionMusic->setProperty("iconName", QVariant(QString::fromUtf8("music")));
        centralwidget = new QWidget(SettingsWindow);
        centralwidget->setObjectName("centralwidget");
        verticalLayout = new QVBoxLayout(centralwidget);
        verticalLayout->setObjectName("verticalLayout");
        settingsTabs = new QStackedWidget(centralwidget);
        settingsTabs->setObjectName("settingsTabs");
        globalSettings = new GlobalSettingsWidget();
        globalSettings->setObjectName("globalSettings");
        settingsTabs->addWidget(globalSettings);
        scraperSettings = new ScraperSettingsWidget();
        scraperSettings->setObjectName("scraperSettings");
        settingsTabs->addWidget(scraperSettings);
        movieSettings = new MovieSettingsWidget();
        movieSettings->setObjectName("movieSettings");
        settingsTabs->addWidget(movieSettings);
        tvShowSettings = new TvShowSettingsWidget();
        tvShowSettings->setObjectName("tvShowSettings");
        settingsTabs->addWidget(tvShowSettings);
        concertSettings = new ConcertSettingsWidget();
        concertSettings->setObjectName("concertSettings");
        settingsTabs->addWidget(concertSettings);
        musicSettings = new MusicSettingsWidget();
        musicSettings->setObjectName("musicSettings");
        settingsTabs->addWidget(musicSettings);
        kodiSettings = new KodiSettingsWidget();
        kodiSettings->setObjectName("kodiSettings");
        settingsTabs->addWidget(kodiSettings);
        networkSettings = new NetworkSettingsWidget();
        networkSettings->setObjectName("networkSettings");
        settingsTabs->addWidget(networkSettings);
        exportSettings = new ExportSettingsWidget();
        exportSettings->setObjectName("exportSettings");
        settingsTabs->addWidget(exportSettings);
        importSettings = new ImportSettingsWidget();
        importSettings->setObjectName("importSettings");
        settingsTabs->addWidget(importSettings);

        verticalLayout->addWidget(settingsTabs);

        horizontalLayoutButtons = new QHBoxLayout();
        horizontalLayoutButtons->setObjectName("horizontalLayoutButtons");
        horizontalSpacerButtons = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayoutButtons->addItem(horizontalSpacerButtons);

        btnCancel = new QPushButton(centralwidget);
        btnCancel->setObjectName("btnCancel");

        horizontalLayoutButtons->addWidget(btnCancel);

        btnSave = new QPushButton(centralwidget);
        btnSave->setObjectName("btnSave");

        horizontalLayoutButtons->addWidget(btnSave);


        verticalLayout->addLayout(horizontalLayoutButtons);

        SettingsWindow->setCentralWidget(centralwidget);
        toolBar = new QToolBar(SettingsWindow);
        toolBar->setObjectName("toolBar");
        toolBar->setMovable(false);
        toolBar->setAllowedAreas(Qt::NoToolBarArea);
        toolBar->setIconSize(QSize(32, 32));
        toolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolBar->setFloatable(false);
        SettingsWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, toolBar);

        toolBar->addAction(actionGlobal);
        toolBar->addAction(actionScraper);
        toolBar->addAction(actionMovies);
        toolBar->addAction(actionTV_Shows);
        toolBar->addAction(actionConcerts);
        toolBar->addAction(actionMusic);
        toolBar->addAction(actionXBMC);
        toolBar->addAction(actionNetwork);
        toolBar->addAction(actionExport);
        toolBar->addAction(actionImport);

        retranslateUi(SettingsWindow);
        QObject::connect(actionGlobal, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionConcerts, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionMovies, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionNetwork, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionScraper, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionTV_Shows, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionXBMC, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionExport, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionImport, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));
        QObject::connect(actionMusic, SIGNAL(triggered()), SettingsWindow, SLOT(onAction()));

        settingsTabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(SettingsWindow);
    } // setupUi

    void retranslateUi(QMainWindow *SettingsWindow)
    {
        SettingsWindow->setWindowTitle(QCoreApplication::translate("SettingsWindow", "Settings", nullptr));
        actionMovies->setText(QCoreApplication::translate("SettingsWindow", "Movies", nullptr));
        actionTV_Shows->setText(QCoreApplication::translate("SettingsWindow", "TV Shows", nullptr));
        actionConcerts->setText(QCoreApplication::translate("SettingsWindow", "Concerts", nullptr));
        actionGlobal->setText(QCoreApplication::translate("SettingsWindow", "Global", nullptr));
        actionXBMC->setText(QCoreApplication::translate("SettingsWindow", "Kodi", nullptr));
        actionNetwork->setText(QCoreApplication::translate("SettingsWindow", "Network", nullptr));
        actionScraper->setText(QCoreApplication::translate("SettingsWindow", "Scraper", nullptr));
        actionExport->setText(QCoreApplication::translate("SettingsWindow", "Export", nullptr));
        actionImport->setText(QCoreApplication::translate("SettingsWindow", "Import", nullptr));
#if QT_CONFIG(tooltip)
        actionImport->setToolTip(QCoreApplication::translate("SettingsWindow", "Import", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMusic->setText(QCoreApplication::translate("SettingsWindow", "Music", nullptr));
#if QT_CONFIG(tooltip)
        actionMusic->setToolTip(QCoreApplication::translate("SettingsWindow", "Music", nullptr));
#endif // QT_CONFIG(tooltip)
        btnCancel->setText(QCoreApplication::translate("SettingsWindow", "Cancel", nullptr));
        btnSave->setText(QCoreApplication::translate("SettingsWindow", "Save Settings", nullptr));
        toolBar->setWindowTitle(QCoreApplication::translate("SettingsWindow", "toolBar", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SettingsWindow: public Ui_SettingsWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSWINDOW_H
