#pragma once

// clang-format off

namespace Constants {
    const int MovieFileSearcherProgressMessageId   = 10000;
    const int MovieWidgetProgressMessageId         = 10001;
    const int TvShowSearcherProgressMessageId      = 10002;
    const int TvShowScrapeProgressMessageId        = 10003;
    const int TvShowWidgetSaveProgressMessageId    = 10004;
    const int ConcertFileSearcherProgressMessageId = 10005;
    const int TvShowUpdaterProgressMessageId       = 10006;
    const int MusicFileSearcherProgressMessageId   = 10007;
    const int MovieProgressMessageId               = 20000;
    const int TvShowProgressMessageId              = 40000;
    const int EpisodeProgressMessageId             = 60000;
    const int ConcertProgressMessageId             = 80000;
    const int MovieDuplicatesProgressMessageId     = 90000;
    const int MusicArtistProgressMessageId         = 100000;
    const int MusicAlbumProgressMessageId          = 200000;
    const int MusicWidgetSaveProgressMessageId     = 300000;
}

// clang-format on
