/********************************************************************************
** Form generated from reading UI file 'MainWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/concerts/ConcertFilesWidget.h"
#include "ui/concerts/ConcertWidget.h"
#include "ui/import/DownloadsWidget.h"
#include "ui/main/Navbar.h"
#include "ui/movie_sets/SetsWidget.h"
#include "ui/movies/CertificationWidget.h"
#include "ui/movies/GenreWidget.h"
#include "ui/movies/MovieDuplicates.h"
#include "ui/movies/MovieFilesWidget.h"
#include "ui/movies/MovieWidget.h"
#include "ui/music/MusicFilesWidget.h"
#include "ui/music/MusicWidget.h"
#include "ui/small_widgets/MySplitter.h"
#include "ui/tv_show/TvShowFilesWidget.h"
#include "ui/tv_show/TvShowWidget.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *actionSettings;
    QAction *actionQuit;
    QAction *actionAbout;
    QWidget *centralWidget;
    QHBoxLayout *horizontalLayout_2;
    QWidget *menuWidget;
    QVBoxLayout *verticalLayout;
    QLabel *labelMovies;
    QToolButton *buttonMovies;
    QToolButton *buttonMovieSets;
    QToolButton *buttonGenres;
    QToolButton *buttonCertifications;
    QToolButton *buttonMovieDuplicates;
    QVBoxLayout *layoutMovies;
    QLabel *labelShows;
    QToolButton *buttonTvshows;
    QVBoxLayout *layoutTvShows;
    QLabel *labelConcerts;
    QToolButton *buttonConcerts;
    QVBoxLayout *layoutConcerts;
    QLabel *labelMusic;
    QToolButton *buttonMusic;
    QVBoxLayout *layoutMusic;
    QLabel *labelDownloads;
    QToolButton *buttonDownloads;
    QVBoxLayout *layoutImport;
    QSpacerItem *menuSpacer;
    QVBoxLayout *verticalLayout_11;
    Navbar *navbar;
    QStackedWidget *stackedWidget;
    QWidget *moviePage;
    QHBoxLayout *horizontalLayout;
    MySplitter *movieSplitter;
    MovieFilesWidget *movieFilesWidget;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents_2;
    QVBoxLayout *verticalLayout_2;
    MovieWidget *movieWidget;
    QWidget *tvShowPage;
    QVBoxLayout *verticalLayout_8;
    MySplitter *tvShowSplitter;
    TvShowFilesWidget *tvShowFilesWidget;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_3;
    QVBoxLayout *verticalLayout_3;
    TvShowWidget *tvShowWidget;
    QWidget *movieSetsPage;
    QVBoxLayout *verticalLayout_5;
    SetsWidget *setsWidget;
    QWidget *concertsPage;
    QVBoxLayout *verticalLayout_9;
    MySplitter *concertSplitter;
    ConcertFilesWidget *concertFilesWidget;
    QScrollArea *scrollArea_3;
    QWidget *scrollAreaWidgetContents_4;
    QVBoxLayout *verticalLayout_4;
    ConcertWidget *concertWidget;
    QWidget *genresPage;
    QVBoxLayout *verticalLayout_6;
    GenreWidget *genreWidget;
    QWidget *certificationsPage;
    QVBoxLayout *verticalLayout_7;
    CertificationWidget *certificationWidget;
    QWidget *downloadsPage;
    QVBoxLayout *verticalLayout_10;
    DownloadsWidget *downloadsWidget;
    QWidget *musicPage;
    QVBoxLayout *verticalLayout_13;
    MySplitter *musicSplitter;
    MusicFilesWidget *musicFilesWidget;
    QScrollArea *scrollArea_4;
    QWidget *scrollAreaWidgetContents_5;
    QVBoxLayout *verticalLayout_12;
    MusicWidget *musicWidget;
    QWidget *duplicatesPage;
    QVBoxLayout *verticalLayout_14;
    MovieDuplicates *movieDuplicatesWidget;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(1062, 794);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MainWindow->setWindowIcon(icon);
        actionSettings = new QAction(MainWindow);
        actionSettings->setObjectName("actionSettings");
        actionQuit = new QAction(MainWindow);
        actionQuit->setObjectName("actionQuit");
        actionAbout = new QAction(MainWindow);
        actionAbout->setObjectName("actionAbout");
        centralWidget = new QWidget(MainWindow);
        centralWidget->setObjectName("centralWidget");
        horizontalLayout_2 = new QHBoxLayout(centralWidget);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        menuWidget = new QWidget(centralWidget);
        menuWidget->setObjectName("menuWidget");
        menuWidget->setMinimumSize(QSize(0, 0));
        menuWidget->setMaximumSize(QSize(16777215, 16777215));
        verticalLayout = new QVBoxLayout(menuWidget);
        verticalLayout->setSpacing(8);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(2, 20, 2, 0);
        labelMovies = new QLabel(menuWidget);
        labelMovies->setObjectName("labelMovies");
        QFont font;
        font.setPointSize(10);
        labelMovies->setFont(font);
        labelMovies->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelMovies);

        buttonMovies = new QToolButton(menuWidget);
        buttonMovies->setObjectName("buttonMovies");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonMovies->sizePolicy().hasHeightForWidth());
        buttonMovies->setSizePolicy(sizePolicy);
        buttonMovies->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonMovies->setText(QString::fromUtf8(""));
        buttonMovies->setIconSize(QSize(32, 32));
        buttonMovies->setProperty("page", QVariant(0));
        buttonMovies->setProperty("iconName", QVariant(QString::fromUtf8("film")));

        verticalLayout->addWidget(buttonMovies);

        buttonMovieSets = new QToolButton(menuWidget);
        buttonMovieSets->setObjectName("buttonMovieSets");
        sizePolicy.setHeightForWidth(buttonMovieSets->sizePolicy().hasHeightForWidth());
        buttonMovieSets->setSizePolicy(sizePolicy);
        buttonMovieSets->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonMovieSets->setText(QString::fromUtf8(""));
        buttonMovieSets->setIconSize(QSize(32, 32));
        buttonMovieSets->setProperty("page", QVariant(2));
        buttonMovieSets->setProperty("iconName", QVariant(QString::fromUtf8("photo_gallery")));

        verticalLayout->addWidget(buttonMovieSets);

        buttonGenres = new QToolButton(menuWidget);
        buttonGenres->setObjectName("buttonGenres");
        sizePolicy.setHeightForWidth(buttonGenres->sizePolicy().hasHeightForWidth());
        buttonGenres->setSizePolicy(sizePolicy);
        buttonGenres->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonGenres->setText(QString::fromUtf8(""));
        buttonGenres->setIconSize(QSize(32, 32));
        buttonGenres->setProperty("page", QVariant(4));
        buttonGenres->setProperty("iconName", QVariant(QString::fromUtf8("notebook")));

        verticalLayout->addWidget(buttonGenres);

        buttonCertifications = new QToolButton(menuWidget);
        buttonCertifications->setObjectName("buttonCertifications");
        sizePolicy.setHeightForWidth(buttonCertifications->sizePolicy().hasHeightForWidth());
        buttonCertifications->setSizePolicy(sizePolicy);
        buttonCertifications->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonCertifications->setText(QString::fromUtf8(""));
        buttonCertifications->setIconSize(QSize(32, 32));
        buttonCertifications->setProperty("page", QVariant(5));
        buttonCertifications->setProperty("iconName", QVariant(QString::fromUtf8("key")));

        verticalLayout->addWidget(buttonCertifications);

        buttonMovieDuplicates = new QToolButton(menuWidget);
        buttonMovieDuplicates->setObjectName("buttonMovieDuplicates");
        sizePolicy.setHeightForWidth(buttonMovieDuplicates->sizePolicy().hasHeightForWidth());
        buttonMovieDuplicates->setSizePolicy(sizePolicy);
        buttonMovieDuplicates->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonMovieDuplicates->setText(QString::fromUtf8(""));
        buttonMovieDuplicates->setIconSize(QSize(32, 32));
        buttonMovieDuplicates->setProperty("page", QVariant(8));
        buttonMovieDuplicates->setProperty("iconName", QVariant(QString::fromUtf8("copy_file")));

        verticalLayout->addWidget(buttonMovieDuplicates);

        layoutMovies = new QVBoxLayout();
        layoutMovies->setSpacing(6);
        layoutMovies->setObjectName("layoutMovies");

        verticalLayout->addLayout(layoutMovies);

        labelShows = new QLabel(menuWidget);
        labelShows->setObjectName("labelShows");
        labelShows->setFont(font);
        labelShows->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelShows);

        buttonTvshows = new QToolButton(menuWidget);
        buttonTvshows->setObjectName("buttonTvshows");
        sizePolicy.setHeightForWidth(buttonTvshows->sizePolicy().hasHeightForWidth());
        buttonTvshows->setSizePolicy(sizePolicy);
        buttonTvshows->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonTvshows->setText(QString::fromUtf8(""));
        buttonTvshows->setIconSize(QSize(32, 32));
        buttonTvshows->setProperty("page", QVariant(1));
        buttonTvshows->setProperty("iconName", QVariant(QString::fromUtf8("monitor")));

        verticalLayout->addWidget(buttonTvshows);

        layoutTvShows = new QVBoxLayout();
        layoutTvShows->setSpacing(6);
        layoutTvShows->setObjectName("layoutTvShows");

        verticalLayout->addLayout(layoutTvShows);

        labelConcerts = new QLabel(menuWidget);
        labelConcerts->setObjectName("labelConcerts");
        labelConcerts->setFont(font);
        labelConcerts->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelConcerts);

        buttonConcerts = new QToolButton(menuWidget);
        buttonConcerts->setObjectName("buttonConcerts");
        sizePolicy.setHeightForWidth(buttonConcerts->sizePolicy().hasHeightForWidth());
        buttonConcerts->setSizePolicy(sizePolicy);
        buttonConcerts->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonConcerts->setText(QString::fromUtf8(""));
        buttonConcerts->setIconSize(QSize(32, 32));
        buttonConcerts->setProperty("page", QVariant(3));
        buttonConcerts->setProperty("iconName", QVariant(QString::fromUtf8("micro")));

        verticalLayout->addWidget(buttonConcerts);

        layoutConcerts = new QVBoxLayout();
        layoutConcerts->setSpacing(6);
        layoutConcerts->setObjectName("layoutConcerts");

        verticalLayout->addLayout(layoutConcerts);

        labelMusic = new QLabel(menuWidget);
        labelMusic->setObjectName("labelMusic");
        labelMusic->setFont(font);
        labelMusic->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelMusic);

        buttonMusic = new QToolButton(menuWidget);
        buttonMusic->setObjectName("buttonMusic");
        sizePolicy.setHeightForWidth(buttonMusic->sizePolicy().hasHeightForWidth());
        buttonMusic->setSizePolicy(sizePolicy);
        buttonMusic->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonMusic->setText(QString::fromUtf8(""));
        buttonMusic->setIconSize(QSize(32, 32));
        buttonMusic->setProperty("page", QVariant(7));
        buttonMusic->setProperty("iconName", QVariant(QString::fromUtf8("music")));

        verticalLayout->addWidget(buttonMusic);

        layoutMusic = new QVBoxLayout();
        layoutMusic->setSpacing(6);
        layoutMusic->setObjectName("layoutMusic");

        verticalLayout->addLayout(layoutMusic);

        labelDownloads = new QLabel(menuWidget);
        labelDownloads->setObjectName("labelDownloads");
        labelDownloads->setFont(font);
        labelDownloads->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelDownloads);

        buttonDownloads = new QToolButton(menuWidget);
        buttonDownloads->setObjectName("buttonDownloads");
        sizePolicy.setHeightForWidth(buttonDownloads->sizePolicy().hasHeightForWidth());
        buttonDownloads->setSizePolicy(sizePolicy);
        buttonDownloads->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonDownloads->setText(QString::fromUtf8(""));
        buttonDownloads->setIconSize(QSize(32, 32));
        buttonDownloads->setProperty("page", QVariant(6));
        buttonDownloads->setProperty("iconName", QVariant(QString::fromUtf8("download")));

        verticalLayout->addWidget(buttonDownloads);

        layoutImport = new QVBoxLayout();
        layoutImport->setSpacing(6);
        layoutImport->setObjectName("layoutImport");

        verticalLayout->addLayout(layoutImport);

        menuSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(menuSpacer);


        horizontalLayout_2->addWidget(menuWidget);

        verticalLayout_11 = new QVBoxLayout();
        verticalLayout_11->setSpacing(0);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_11->setContentsMargins(-1, 0, 0, -1);
        navbar = new Navbar(centralWidget);
        navbar->setObjectName("navbar");
        navbar->setMinimumSize(QSize(0, 56));
        navbar->setMaximumSize(QSize(16777215, 56));

        verticalLayout_11->addWidget(navbar);

        stackedWidget = new QStackedWidget(centralWidget);
        stackedWidget->setObjectName("stackedWidget");
        moviePage = new QWidget();
        moviePage->setObjectName("moviePage");
        horizontalLayout = new QHBoxLayout(moviePage);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        movieSplitter = new MySplitter(moviePage);
        movieSplitter->setObjectName("movieSplitter");
        movieSplitter->setOrientation(Qt::Horizontal);
        movieSplitter->setHandleWidth(1);
        movieFilesWidget = new MovieFilesWidget(movieSplitter);
        movieFilesWidget->setObjectName("movieFilesWidget");
        movieFilesWidget->setMinimumSize(QSize(100, 0));
        movieSplitter->addWidget(movieFilesWidget);
        scrollArea = new QScrollArea(movieSplitter);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setLineWidth(0);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName("scrollAreaWidgetContents_2");
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 72, 736));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents_2);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        movieWidget = new MovieWidget(scrollAreaWidgetContents_2);
        movieWidget->setObjectName("movieWidget");

        verticalLayout_2->addWidget(movieWidget);

        scrollArea->setWidget(scrollAreaWidgetContents_2);
        movieSplitter->addWidget(scrollArea);

        horizontalLayout->addWidget(movieSplitter);

        stackedWidget->addWidget(moviePage);
        tvShowPage = new QWidget();
        tvShowPage->setObjectName("tvShowPage");
        verticalLayout_8 = new QVBoxLayout(tvShowPage);
        verticalLayout_8->setSpacing(6);
        verticalLayout_8->setContentsMargins(11, 11, 11, 11);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        tvShowSplitter = new MySplitter(tvShowPage);
        tvShowSplitter->setObjectName("tvShowSplitter");
        tvShowSplitter->setOrientation(Qt::Horizontal);
        tvShowSplitter->setHandleWidth(1);
        tvShowFilesWidget = new TvShowFilesWidget(tvShowSplitter);
        tvShowFilesWidget->setObjectName("tvShowFilesWidget");
        tvShowFilesWidget->setMinimumSize(QSize(100, 0));
        tvShowFilesWidget->setFocusPolicy(Qt::NoFocus);
        tvShowSplitter->addWidget(tvShowFilesWidget);
        scrollArea_2 = new QScrollArea(tvShowSplitter);
        scrollArea_2->setObjectName("scrollArea_2");
        scrollArea_2->setFrameShape(QFrame::NoFrame);
        scrollArea_2->setFrameShadow(QFrame::Plain);
        scrollArea_2->setLineWidth(0);
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName("scrollAreaWidgetContents_3");
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 49, 30));
        verticalLayout_3 = new QVBoxLayout(scrollAreaWidgetContents_3);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        tvShowWidget = new TvShowWidget(scrollAreaWidgetContents_3);
        tvShowWidget->setObjectName("tvShowWidget");

        verticalLayout_3->addWidget(tvShowWidget);

        scrollArea_2->setWidget(scrollAreaWidgetContents_3);
        tvShowSplitter->addWidget(scrollArea_2);

        verticalLayout_8->addWidget(tvShowSplitter);

        stackedWidget->addWidget(tvShowPage);
        movieSetsPage = new QWidget();
        movieSetsPage->setObjectName("movieSetsPage");
        verticalLayout_5 = new QVBoxLayout(movieSetsPage);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        setsWidget = new SetsWidget(movieSetsPage);
        setsWidget->setObjectName("setsWidget");

        verticalLayout_5->addWidget(setsWidget);

        stackedWidget->addWidget(movieSetsPage);
        concertsPage = new QWidget();
        concertsPage->setObjectName("concertsPage");
        verticalLayout_9 = new QVBoxLayout(concertsPage);
        verticalLayout_9->setSpacing(6);
        verticalLayout_9->setContentsMargins(11, 11, 11, 11);
        verticalLayout_9->setObjectName("verticalLayout_9");
        verticalLayout_9->setContentsMargins(0, 0, 0, 0);
        concertSplitter = new MySplitter(concertsPage);
        concertSplitter->setObjectName("concertSplitter");
        concertSplitter->setOrientation(Qt::Horizontal);
        concertSplitter->setHandleWidth(1);
        concertSplitter->setChildrenCollapsible(false);
        concertFilesWidget = new ConcertFilesWidget(concertSplitter);
        concertFilesWidget->setObjectName("concertFilesWidget");
        concertFilesWidget->setMinimumSize(QSize(100, 0));
        concertSplitter->addWidget(concertFilesWidget);
        scrollArea_3 = new QScrollArea(concertSplitter);
        scrollArea_3->setObjectName("scrollArea_3");
        scrollArea_3->setFrameShape(QFrame::NoFrame);
        scrollArea_3->setFrameShadow(QFrame::Plain);
        scrollArea_3->setLineWidth(0);
        scrollArea_3->setWidgetResizable(true);
        scrollAreaWidgetContents_4 = new QWidget();
        scrollAreaWidgetContents_4->setObjectName("scrollAreaWidgetContents_4");
        scrollAreaWidgetContents_4->setGeometry(QRect(0, 0, 49, 30));
        verticalLayout_4 = new QVBoxLayout(scrollAreaWidgetContents_4);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        concertWidget = new ConcertWidget(scrollAreaWidgetContents_4);
        concertWidget->setObjectName("concertWidget");

        verticalLayout_4->addWidget(concertWidget);

        scrollArea_3->setWidget(scrollAreaWidgetContents_4);
        concertSplitter->addWidget(scrollArea_3);

        verticalLayout_9->addWidget(concertSplitter);

        stackedWidget->addWidget(concertsPage);
        genresPage = new QWidget();
        genresPage->setObjectName("genresPage");
        verticalLayout_6 = new QVBoxLayout(genresPage);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        genreWidget = new GenreWidget(genresPage);
        genreWidget->setObjectName("genreWidget");

        verticalLayout_6->addWidget(genreWidget);

        stackedWidget->addWidget(genresPage);
        certificationsPage = new QWidget();
        certificationsPage->setObjectName("certificationsPage");
        verticalLayout_7 = new QVBoxLayout(certificationsPage);
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        certificationWidget = new CertificationWidget(certificationsPage);
        certificationWidget->setObjectName("certificationWidget");

        verticalLayout_7->addWidget(certificationWidget);

        stackedWidget->addWidget(certificationsPage);
        downloadsPage = new QWidget();
        downloadsPage->setObjectName("downloadsPage");
        verticalLayout_10 = new QVBoxLayout(downloadsPage);
        verticalLayout_10->setSpacing(6);
        verticalLayout_10->setContentsMargins(11, 11, 11, 11);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        downloadsWidget = new DownloadsWidget(downloadsPage);
        downloadsWidget->setObjectName("downloadsWidget");

        verticalLayout_10->addWidget(downloadsWidget);

        stackedWidget->addWidget(downloadsPage);
        musicPage = new QWidget();
        musicPage->setObjectName("musicPage");
        verticalLayout_13 = new QVBoxLayout(musicPage);
        verticalLayout_13->setSpacing(6);
        verticalLayout_13->setContentsMargins(11, 11, 11, 11);
        verticalLayout_13->setObjectName("verticalLayout_13");
        verticalLayout_13->setContentsMargins(0, 0, 0, 0);
        musicSplitter = new MySplitter(musicPage);
        musicSplitter->setObjectName("musicSplitter");
        musicSplitter->setOrientation(Qt::Horizontal);
        musicFilesWidget = new MusicFilesWidget(musicSplitter);
        musicFilesWidget->setObjectName("musicFilesWidget");
        musicSplitter->addWidget(musicFilesWidget);
        scrollArea_4 = new QScrollArea(musicSplitter);
        scrollArea_4->setObjectName("scrollArea_4");
        scrollArea_4->setFrameShape(QFrame::NoFrame);
        scrollArea_4->setFrameShadow(QFrame::Plain);
        scrollArea_4->setLineWidth(0);
        scrollArea_4->setWidgetResizable(true);
        scrollAreaWidgetContents_5 = new QWidget();
        scrollAreaWidgetContents_5->setObjectName("scrollAreaWidgetContents_5");
        scrollAreaWidgetContents_5->setGeometry(QRect(0, 0, 72, 30));
        verticalLayout_12 = new QVBoxLayout(scrollAreaWidgetContents_5);
        verticalLayout_12->setSpacing(6);
        verticalLayout_12->setContentsMargins(11, 11, 11, 11);
        verticalLayout_12->setObjectName("verticalLayout_12");
        verticalLayout_12->setContentsMargins(0, 0, 0, 0);
        musicWidget = new MusicWidget(scrollAreaWidgetContents_5);
        musicWidget->setObjectName("musicWidget");

        verticalLayout_12->addWidget(musicWidget);

        scrollArea_4->setWidget(scrollAreaWidgetContents_5);
        musicSplitter->addWidget(scrollArea_4);

        verticalLayout_13->addWidget(musicSplitter);

        stackedWidget->addWidget(musicPage);
        duplicatesPage = new QWidget();
        duplicatesPage->setObjectName("duplicatesPage");
        verticalLayout_14 = new QVBoxLayout(duplicatesPage);
        verticalLayout_14->setSpacing(6);
        verticalLayout_14->setContentsMargins(11, 11, 11, 11);
        verticalLayout_14->setObjectName("verticalLayout_14");
        verticalLayout_14->setContentsMargins(0, 0, 0, 0);
        movieDuplicatesWidget = new MovieDuplicates(duplicatesPage);
        movieDuplicatesWidget->setObjectName("movieDuplicatesWidget");

        verticalLayout_14->addWidget(movieDuplicatesWidget);

        stackedWidget->addWidget(duplicatesPage);

        verticalLayout_11->addWidget(stackedWidget);

        verticalLayout_11->setStretch(1, 1);

        horizontalLayout_2->addLayout(verticalLayout_11);

        MainWindow->setCentralWidget(centralWidget);

        retranslateUi(MainWindow);
        QObject::connect(movieWidget, SIGNAL(setActionSaveEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(movieWidget, SIGNAL(setActionSearchEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSearchEnabled(bool,MainWidgets)));
        QObject::connect(concertWidget, SIGNAL(setActionSaveEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(concertWidget, SIGNAL(setActionSearchEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSearchEnabled(bool,MainWidgets)));
        QObject::connect(tvShowWidget, SIGNAL(sigSetActionSaveEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(tvShowWidget, SIGNAL(sigSetActionSearchEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSearchEnabled(bool,MainWidgets)));
        QObject::connect(setsWidget, SIGNAL(setActionSaveEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(genreWidget, SIGNAL(setActionSaveEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(certificationWidget, SIGNAL(setActionSaveEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(musicWidget, SIGNAL(sigSetActionSearchEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSaveEnabled(bool,MainWidgets)));
        QObject::connect(musicWidget, SIGNAL(sigSetActionSearchEnabled(bool,MainWidgets)), MainWindow, SLOT(onSetSearchEnabled(bool,MainWidgets)));

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "MediaElch", nullptr));
        actionSettings->setText(QCoreApplication::translate("MainWindow", "Settings", nullptr));
        actionQuit->setText(QCoreApplication::translate("MainWindow", "Quit", nullptr));
        actionAbout->setText(QCoreApplication::translate("MainWindow", "About", nullptr));
        labelMovies->setText(QCoreApplication::translate("MainWindow", "Movies", nullptr));
#if QT_CONFIG(tooltip)
        buttonMovies->setToolTip(QCoreApplication::translate("MainWindow", "Movies", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonMovieSets->setToolTip(QCoreApplication::translate("MainWindow", "Movie Sets", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonGenres->setToolTip(QCoreApplication::translate("MainWindow", "Genres", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonCertifications->setToolTip(QCoreApplication::translate("MainWindow", "Certifications", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonMovieDuplicates->setToolTip(QCoreApplication::translate("MainWindow", "Duplicates", nullptr));
#endif // QT_CONFIG(tooltip)
        labelShows->setText(QCoreApplication::translate("MainWindow", "Shows", nullptr));
#if QT_CONFIG(tooltip)
        buttonTvshows->setToolTip(QCoreApplication::translate("MainWindow", "TV Shows", nullptr));
#endif // QT_CONFIG(tooltip)
        labelConcerts->setText(QCoreApplication::translate("MainWindow", "Concerts", nullptr));
#if QT_CONFIG(tooltip)
        buttonConcerts->setToolTip(QCoreApplication::translate("MainWindow", "Concerts", nullptr));
#endif // QT_CONFIG(tooltip)
        labelMusic->setText(QCoreApplication::translate("MainWindow", "Music", nullptr));
#if QT_CONFIG(tooltip)
        buttonMusic->setToolTip(QCoreApplication::translate("MainWindow", "Music", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDownloads->setText(QCoreApplication::translate("MainWindow", "Import", nullptr));
#if QT_CONFIG(tooltip)
        buttonDownloads->setToolTip(QCoreApplication::translate("MainWindow", "Import", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
