/********************************************************************************
** Form generated from reading UI file 'CsvExportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CSVEXPORTDIALOG_H
#define UI_CSVEXPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MessageLabel.h"

QT_BEGIN_NAMESPACE

class Ui_CsvExportDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *layoutSettings;
    QGroupBox *groupBoxDetails;
    QVBoxLayout *verticalLayout_10;
    QComboBox *comboDetails;
    QStackedWidget *stackedWidgetDetails;
    QWidget *tabMovieDetails;
    QVBoxLayout *verticalLayout_3;
    QListWidget *movieDetailsToExport;
    QWidget *tabTvShowDetails;
    QVBoxLayout *verticalLayout_4;
    QListWidget *tvShowDetailsToExport;
    QWidget *tabEpisodeDetails;
    QVBoxLayout *verticalLayout_6;
    QListWidget *tvEpisodeDetailsToExport;
    QWidget *tabConcertDetails;
    QVBoxLayout *verticalLayout_7;
    QListWidget *concertDetailsToExport;
    QWidget *tabArtistDetails;
    QVBoxLayout *verticalLayout_8;
    QListWidget *artistDetailsToExport;
    QWidget *tabAlbumDetails;
    QVBoxLayout *verticalLayout_9;
    QListWidget *albumDetailsToExport;
    QGroupBox *groupBoxConfig;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *formLayout;
    QLabel *lblExportTypes;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *checkMovies;
    QCheckBox *checkTvShows;
    QCheckBox *checkTvEpisodes;
    QCheckBox *checkConcerts;
    QCheckBox *checkMusicArtists;
    QCheckBox *checkMusicAlbums;
    QLabel *lblSeparator;
    QComboBox *separator;
    QLabel *lblReplacement;
    QComboBox *replacement;
    QLabel *lblNewlines;
    QLabel *lblTextReplacements;
    QSpacerItem *verticalSpacer;
    QLabel *lblTip;
    QHBoxLayout *layoutExport;
    QPushButton *btnExport;
    QProgressBar *exportProgress;
    MessageLabel *lblMessage;
    QDialogButtonBox *btnClose;

    void setupUi(QDialog *CsvExportDialog)
    {
        if (CsvExportDialog->objectName().isEmpty())
            CsvExportDialog->setObjectName("CsvExportDialog");
        CsvExportDialog->resize(500, 450);
        verticalLayout = new QVBoxLayout(CsvExportDialog);
        verticalLayout->setObjectName("verticalLayout");
        layoutSettings = new QHBoxLayout();
        layoutSettings->setObjectName("layoutSettings");
        groupBoxDetails = new QGroupBox(CsvExportDialog);
        groupBoxDetails->setObjectName("groupBoxDetails");
        verticalLayout_10 = new QVBoxLayout(groupBoxDetails);
        verticalLayout_10->setSpacing(2);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(-1, -1, -1, 0);
        comboDetails = new QComboBox(groupBoxDetails);
        comboDetails->addItem(QString());
        comboDetails->addItem(QString());
        comboDetails->addItem(QString());
        comboDetails->addItem(QString());
        comboDetails->addItem(QString());
        comboDetails->addItem(QString());
        comboDetails->setObjectName("comboDetails");

        verticalLayout_10->addWidget(comboDetails);

        stackedWidgetDetails = new QStackedWidget(groupBoxDetails);
        stackedWidgetDetails->setObjectName("stackedWidgetDetails");
        tabMovieDetails = new QWidget();
        tabMovieDetails->setObjectName("tabMovieDetails");
        verticalLayout_3 = new QVBoxLayout(tabMovieDetails);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, -1, 0, -1);
        movieDetailsToExport = new QListWidget(tabMovieDetails);
        QListWidgetItem *__qlistwidgetitem = new QListWidgetItem(movieDetailsToExport);
        __qlistwidgetitem->setText(QString::fromUtf8("Placeholder"));
        movieDetailsToExport->setObjectName("movieDetailsToExport");
        movieDetailsToExport->setMinimumSize(QSize(0, 90));
        movieDetailsToExport->setDragDropMode(QAbstractItemView::InternalMove);
        movieDetailsToExport->setAlternatingRowColors(true);
        movieDetailsToExport->setSelectionBehavior(QAbstractItemView::SelectRows);
        movieDetailsToExport->setWordWrap(true);

        verticalLayout_3->addWidget(movieDetailsToExport);

        stackedWidgetDetails->addWidget(tabMovieDetails);
        tabTvShowDetails = new QWidget();
        tabTvShowDetails->setObjectName("tabTvShowDetails");
        verticalLayout_4 = new QVBoxLayout(tabTvShowDetails);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, -1, 0, -1);
        tvShowDetailsToExport = new QListWidget(tabTvShowDetails);
        QListWidgetItem *__qlistwidgetitem1 = new QListWidgetItem(tvShowDetailsToExport);
        __qlistwidgetitem1->setText(QString::fromUtf8("Placeholder"));
        tvShowDetailsToExport->setObjectName("tvShowDetailsToExport");
        tvShowDetailsToExport->setMinimumSize(QSize(0, 90));
        tvShowDetailsToExport->setDragDropMode(QAbstractItemView::InternalMove);
        tvShowDetailsToExport->setAlternatingRowColors(true);
        tvShowDetailsToExport->setSelectionBehavior(QAbstractItemView::SelectRows);
        tvShowDetailsToExport->setWordWrap(true);

        verticalLayout_4->addWidget(tvShowDetailsToExport);

        stackedWidgetDetails->addWidget(tabTvShowDetails);
        tabEpisodeDetails = new QWidget();
        tabEpisodeDetails->setObjectName("tabEpisodeDetails");
        verticalLayout_6 = new QVBoxLayout(tabEpisodeDetails);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, -1, 0, -1);
        tvEpisodeDetailsToExport = new QListWidget(tabEpisodeDetails);
        QListWidgetItem *__qlistwidgetitem2 = new QListWidgetItem(tvEpisodeDetailsToExport);
        __qlistwidgetitem2->setText(QString::fromUtf8("Placeholder"));
        tvEpisodeDetailsToExport->setObjectName("tvEpisodeDetailsToExport");
        tvEpisodeDetailsToExport->setMinimumSize(QSize(0, 90));
        tvEpisodeDetailsToExport->setDragDropMode(QAbstractItemView::InternalMove);
        tvEpisodeDetailsToExport->setAlternatingRowColors(true);
        tvEpisodeDetailsToExport->setSelectionBehavior(QAbstractItemView::SelectRows);
        tvEpisodeDetailsToExport->setWordWrap(true);

        verticalLayout_6->addWidget(tvEpisodeDetailsToExport);

        stackedWidgetDetails->addWidget(tabEpisodeDetails);
        tabConcertDetails = new QWidget();
        tabConcertDetails->setObjectName("tabConcertDetails");
        verticalLayout_7 = new QVBoxLayout(tabConcertDetails);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, -1, 0, -1);
        concertDetailsToExport = new QListWidget(tabConcertDetails);
        concertDetailsToExport->setObjectName("concertDetailsToExport");

        verticalLayout_7->addWidget(concertDetailsToExport);

        stackedWidgetDetails->addWidget(tabConcertDetails);
        tabArtistDetails = new QWidget();
        tabArtistDetails->setObjectName("tabArtistDetails");
        verticalLayout_8 = new QVBoxLayout(tabArtistDetails);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, -1, 0, -1);
        artistDetailsToExport = new QListWidget(tabArtistDetails);
        artistDetailsToExport->setObjectName("artistDetailsToExport");

        verticalLayout_8->addWidget(artistDetailsToExport);

        stackedWidgetDetails->addWidget(tabArtistDetails);
        tabAlbumDetails = new QWidget();
        tabAlbumDetails->setObjectName("tabAlbumDetails");
        verticalLayout_9 = new QVBoxLayout(tabAlbumDetails);
        verticalLayout_9->setObjectName("verticalLayout_9");
        verticalLayout_9->setContentsMargins(0, -1, 0, -1);
        albumDetailsToExport = new QListWidget(tabAlbumDetails);
        albumDetailsToExport->setObjectName("albumDetailsToExport");

        verticalLayout_9->addWidget(albumDetailsToExport);

        stackedWidgetDetails->addWidget(tabAlbumDetails);

        verticalLayout_10->addWidget(stackedWidgetDetails);


        layoutSettings->addWidget(groupBoxDetails);

        groupBoxConfig = new QGroupBox(CsvExportDialog);
        groupBoxConfig->setObjectName("groupBoxConfig");
        verticalLayout_5 = new QVBoxLayout(groupBoxConfig);
        verticalLayout_5->setObjectName("verticalLayout_5");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        lblExportTypes = new QLabel(groupBoxConfig);
        lblExportTypes->setObjectName("lblExportTypes");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblExportTypes);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        checkMovies = new QCheckBox(groupBoxConfig);
        checkMovies->setObjectName("checkMovies");
        checkMovies->setChecked(true);

        verticalLayout_2->addWidget(checkMovies);

        checkTvShows = new QCheckBox(groupBoxConfig);
        checkTvShows->setObjectName("checkTvShows");
        checkTvShows->setChecked(true);

        verticalLayout_2->addWidget(checkTvShows);

        checkTvEpisodes = new QCheckBox(groupBoxConfig);
        checkTvEpisodes->setObjectName("checkTvEpisodes");
        checkTvEpisodes->setChecked(true);

        verticalLayout_2->addWidget(checkTvEpisodes);

        checkConcerts = new QCheckBox(groupBoxConfig);
        checkConcerts->setObjectName("checkConcerts");
        checkConcerts->setChecked(true);

        verticalLayout_2->addWidget(checkConcerts);

        checkMusicArtists = new QCheckBox(groupBoxConfig);
        checkMusicArtists->setObjectName("checkMusicArtists");
        checkMusicArtists->setChecked(true);

        verticalLayout_2->addWidget(checkMusicArtists);

        checkMusicAlbums = new QCheckBox(groupBoxConfig);
        checkMusicAlbums->setObjectName("checkMusicAlbums");
        checkMusicAlbums->setChecked(true);

        verticalLayout_2->addWidget(checkMusicAlbums);


        formLayout->setLayout(0, QFormLayout::FieldRole, verticalLayout_2);

        lblSeparator = new QLabel(groupBoxConfig);
        lblSeparator->setObjectName("lblSeparator");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblSeparator);

        separator = new QComboBox(groupBoxConfig);
        separator->setObjectName("separator");
        separator->setMinimumSize(QSize(40, 0));

        formLayout->setWidget(1, QFormLayout::FieldRole, separator);

        lblReplacement = new QLabel(groupBoxConfig);
        lblReplacement->setObjectName("lblReplacement");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblReplacement);

        replacement = new QComboBox(groupBoxConfig);
        replacement->setObjectName("replacement");
        replacement->setMinimumSize(QSize(40, 0));

        formLayout->setWidget(2, QFormLayout::FieldRole, replacement);


        verticalLayout_5->addLayout(formLayout);

        lblNewlines = new QLabel(groupBoxConfig);
        lblNewlines->setObjectName("lblNewlines");
        lblNewlines->setWordWrap(true);

        verticalLayout_5->addWidget(lblNewlines);

        lblTextReplacements = new QLabel(groupBoxConfig);
        lblTextReplacements->setObjectName("lblTextReplacements");
        lblTextReplacements->setWordWrap(true);

        verticalLayout_5->addWidget(lblTextReplacements);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        lblTip = new QLabel(groupBoxConfig);
        lblTip->setObjectName("lblTip");
        lblTip->setWordWrap(true);

        verticalLayout_5->addWidget(lblTip);


        layoutSettings->addWidget(groupBoxConfig);

        layoutSettings->setStretch(0, 3);
        layoutSettings->setStretch(1, 4);

        verticalLayout->addLayout(layoutSettings);

        layoutExport = new QHBoxLayout();
        layoutExport->setObjectName("layoutExport");
        btnExport = new QPushButton(CsvExportDialog);
        btnExport->setObjectName("btnExport");

        layoutExport->addWidget(btnExport);

        exportProgress = new QProgressBar(CsvExportDialog);
        exportProgress->setObjectName("exportProgress");
        exportProgress->setValue(24);

        layoutExport->addWidget(exportProgress);

        lblMessage = new MessageLabel(CsvExportDialog);
        lblMessage->setObjectName("lblMessage");
        lblMessage->setMinimumSize(QSize(200, 0));
        lblMessage->setText(QString::fromUtf8("Message"));
        lblMessage->setWordWrap(true);

        layoutExport->addWidget(lblMessage);


        verticalLayout->addLayout(layoutExport);

        btnClose = new QDialogButtonBox(CsvExportDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setOrientation(Qt::Horizontal);
        btnClose->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(btnClose);


        retranslateUi(CsvExportDialog);
        QObject::connect(btnClose, &QDialogButtonBox::rejected, CsvExportDialog, qOverload<>(&QDialog::reject));
        QObject::connect(btnClose, &QDialogButtonBox::accepted, CsvExportDialog, qOverload<>(&QDialog::accept));

        btnExport->setDefault(true);


        QMetaObject::connectSlotsByName(CsvExportDialog);
    } // setupUi

    void retranslateUi(QDialog *CsvExportDialog)
    {
        CsvExportDialog->setWindowTitle(QCoreApplication::translate("CsvExportDialog", "CSV Export", nullptr));
        groupBoxDetails->setTitle(QCoreApplication::translate("CsvExportDialog", "CSV Columns", nullptr));
        comboDetails->setItemText(0, QCoreApplication::translate("CsvExportDialog", "Movies", nullptr));
        comboDetails->setItemText(1, QCoreApplication::translate("CsvExportDialog", "TV Shows", nullptr));
        comboDetails->setItemText(2, QCoreApplication::translate("CsvExportDialog", "TV Episodes", nullptr));
        comboDetails->setItemText(3, QCoreApplication::translate("CsvExportDialog", "Concerts", nullptr));
        comboDetails->setItemText(4, QCoreApplication::translate("CsvExportDialog", "Music Artists", nullptr));
        comboDetails->setItemText(5, QCoreApplication::translate("CsvExportDialog", "Music Albums", nullptr));


        const bool __sortingEnabled = movieDetailsToExport->isSortingEnabled();
        movieDetailsToExport->setSortingEnabled(false);
        movieDetailsToExport->setSortingEnabled(__sortingEnabled);


        const bool __sortingEnabled1 = tvShowDetailsToExport->isSortingEnabled();
        tvShowDetailsToExport->setSortingEnabled(false);
        tvShowDetailsToExport->setSortingEnabled(__sortingEnabled1);


        const bool __sortingEnabled2 = tvEpisodeDetailsToExport->isSortingEnabled();
        tvEpisodeDetailsToExport->setSortingEnabled(false);
        tvEpisodeDetailsToExport->setSortingEnabled(__sortingEnabled2);

        groupBoxConfig->setTitle(QCoreApplication::translate("CsvExportDialog", "Configuration", nullptr));
        lblExportTypes->setText(QCoreApplication::translate("CsvExportDialog", "Export", nullptr));
        checkMovies->setText(QCoreApplication::translate("CsvExportDialog", "Movies", nullptr));
        checkTvShows->setText(QCoreApplication::translate("CsvExportDialog", "TV Shows", nullptr));
        checkTvEpisodes->setText(QCoreApplication::translate("CsvExportDialog", "TV Episodes", nullptr));
        checkConcerts->setText(QCoreApplication::translate("CsvExportDialog", "Concerts", nullptr));
        checkMusicArtists->setText(QCoreApplication::translate("CsvExportDialog", "Music Artists", nullptr));
        checkMusicAlbums->setText(QCoreApplication::translate("CsvExportDialog", "Music Albums", nullptr));
        lblSeparator->setText(QCoreApplication::translate("CsvExportDialog", "Separator", nullptr));
        lblReplacement->setText(QCoreApplication::translate("CsvExportDialog", "Replacement", nullptr));
        lblNewlines->setText(QCoreApplication::translate("CsvExportDialog", "Linebreaks will be replaced by <code>\\n</code>. ", nullptr));
        lblTextReplacements->setText(QCoreApplication::translate("CsvExportDialog", "If any text contains the separator, it will be replaced by the replacement set above.", nullptr));
        lblTip->setText(QCoreApplication::translate("CsvExportDialog", "<b>Tip:</b> You can sort the items by Drag &amp; Drop.", nullptr));
        btnExport->setText(QCoreApplication::translate("CsvExportDialog", "Export as CSV", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CsvExportDialog: public Ui_CsvExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CSVEXPORTDIALOG_H
