/********************************************************************************
** Form generated from reading UI file 'CustomTvScraperSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CUSTOMTVSCRAPERSETTINGSWIDGET_H
#define UI_CUSTOMTVSCRAPERSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CustomTvScraperSettingsWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *lblTvScraperSettingsHelp;
    QTabWidget *tabDetails;
    QWidget *tabShow;
    QVBoxLayout *verticalLayout_2;
    QTableWidget *customTvScraperShowDetails;
    QWidget *tabEpisode;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *customTvScraperEpisodeDetails;

    void setupUi(QWidget *CustomTvScraperSettingsWidget)
    {
        if (CustomTvScraperSettingsWidget->objectName().isEmpty())
            CustomTvScraperSettingsWidget->setObjectName("CustomTvScraperSettingsWidget");
        CustomTvScraperSettingsWidget->resize(600, 400);
        verticalLayout = new QVBoxLayout(CustomTvScraperSettingsWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        lblTvScraperSettingsHelp = new QLabel(CustomTvScraperSettingsWidget);
        lblTvScraperSettingsHelp->setObjectName("lblTvScraperSettingsHelp");
        lblTvScraperSettingsHelp->setWordWrap(true);

        verticalLayout->addWidget(lblTvScraperSettingsHelp);

        tabDetails = new QTabWidget(CustomTvScraperSettingsWidget);
        tabDetails->setObjectName("tabDetails");
        tabShow = new QWidget();
        tabShow->setObjectName("tabShow");
        verticalLayout_2 = new QVBoxLayout(tabShow);
        verticalLayout_2->setObjectName("verticalLayout_2");
        customTvScraperShowDetails = new QTableWidget(tabShow);
        if (customTvScraperShowDetails->columnCount() < 2)
            customTvScraperShowDetails->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        customTvScraperShowDetails->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        customTvScraperShowDetails->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        customTvScraperShowDetails->setObjectName("customTvScraperShowDetails");
        customTvScraperShowDetails->setEditTriggers(QAbstractItemView::NoEditTriggers);
        customTvScraperShowDetails->setAlternatingRowColors(true);
        customTvScraperShowDetails->setSelectionMode(QAbstractItemView::NoSelection);
        customTvScraperShowDetails->setShowGrid(false);
        customTvScraperShowDetails->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(customTvScraperShowDetails);

        tabDetails->addTab(tabShow, QString());
        tabEpisode = new QWidget();
        tabEpisode->setObjectName("tabEpisode");
        verticalLayout_3 = new QVBoxLayout(tabEpisode);
        verticalLayout_3->setObjectName("verticalLayout_3");
        customTvScraperEpisodeDetails = new QTableWidget(tabEpisode);
        if (customTvScraperEpisodeDetails->columnCount() < 2)
            customTvScraperEpisodeDetails->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        customTvScraperEpisodeDetails->setHorizontalHeaderItem(0, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        customTvScraperEpisodeDetails->setHorizontalHeaderItem(1, __qtablewidgetitem3);
        customTvScraperEpisodeDetails->setObjectName("customTvScraperEpisodeDetails");
        customTvScraperEpisodeDetails->setEditTriggers(QAbstractItemView::NoEditTriggers);
        customTvScraperEpisodeDetails->setAlternatingRowColors(true);
        customTvScraperEpisodeDetails->setSelectionMode(QAbstractItemView::NoSelection);
        customTvScraperEpisodeDetails->setShowGrid(false);
        customTvScraperEpisodeDetails->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(customTvScraperEpisodeDetails);

        tabDetails->addTab(tabEpisode, QString());

        verticalLayout->addWidget(tabDetails);

        verticalLayout->setStretch(1, 1);

        retranslateUi(CustomTvScraperSettingsWidget);

        tabDetails->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(CustomTvScraperSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *CustomTvScraperSettingsWidget)
    {
        lblTvScraperSettingsHelp->setText(QCoreApplication::translate("CustomTvScraperSettingsWidget", "Select which site you prefer for each element of a TV show and episode.", nullptr));
        QTableWidgetItem *___qtablewidgetitem = customTvScraperShowDetails->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("CustomTvScraperSettingsWidget", "Item", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = customTvScraperShowDetails->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("CustomTvScraperSettingsWidget", "Site", nullptr));
        tabDetails->setTabText(tabDetails->indexOf(tabShow), QCoreApplication::translate("CustomTvScraperSettingsWidget", "TV show details", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = customTvScraperEpisodeDetails->horizontalHeaderItem(0);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("CustomTvScraperSettingsWidget", "Item", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = customTvScraperEpisodeDetails->horizontalHeaderItem(1);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("CustomTvScraperSettingsWidget", "Site", nullptr));
        tabDetails->setTabText(tabDetails->indexOf(tabEpisode), QCoreApplication::translate("CustomTvScraperSettingsWidget", "Episode details", nullptr));
        (void)CustomTvScraperSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class CustomTvScraperSettingsWidget: public Ui_CustomTvScraperSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CUSTOMTVSCRAPERSETTINGSWIDGET_H
