/********************************************************************************
** Form generated from reading UI file 'GenreWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GENREWIDGET_H
#define UI_GENREWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MySplitter.h"

QT_BEGIN_NAMESPACE

class Ui_GenreWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    MySplitter *splitter;
    QTableWidget *genres;
    QWidget *horizontalLayoutWidget;
    QVBoxLayout *verticalLayout_3;
    QLabel *genreName;
    QGroupBox *infoGroupBox;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_4;
    QTableWidget *movies;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *buttonAddMovie;
    QPushButton *buttonRemoveMovie;
    QVBoxLayout *verticalLayout;
    QLabel *label_3;
    QLabel *label_2;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *GenreWidget)
    {
        if (GenreWidget->objectName().isEmpty())
            GenreWidget->setObjectName("GenreWidget");
        GenreWidget->resize(984, 702);
        verticalLayout_2 = new QVBoxLayout(GenreWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        splitter = new MySplitter(GenreWidget);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        splitter->setHandleWidth(1);
        genres = new QTableWidget(splitter);
        if (genres->columnCount() < 1)
            genres->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        genres->setHorizontalHeaderItem(0, __qtablewidgetitem);
        genres->setObjectName("genres");
        genres->setAlternatingRowColors(true);
        genres->setShowGrid(false);
        splitter->addWidget(genres);
        genres->horizontalHeader()->setVisible(false);
        genres->horizontalHeader()->setStretchLastSection(true);
        genres->verticalHeader()->setVisible(false);
        horizontalLayoutWidget = new QWidget(splitter);
        horizontalLayoutWidget->setObjectName("horizontalLayoutWidget");
        verticalLayout_3 = new QVBoxLayout(horizontalLayoutWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 20, 0, 21);
        genreName = new QLabel(horizontalLayoutWidget);
        genreName->setObjectName("genreName");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        genreName->setFont(font);
        genreName->setIndent(20);
        genreName->setProperty("isHeader", QVariant(true));

        verticalLayout_3->addWidget(genreName);

        infoGroupBox = new QGroupBox(horizontalLayoutWidget);
        infoGroupBox->setObjectName("infoGroupBox");
        horizontalLayout = new QHBoxLayout(infoGroupBox);
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        movies = new QTableWidget(infoGroupBox);
        if (movies->columnCount() < 1)
            movies->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        movies->setHorizontalHeaderItem(0, __qtablewidgetitem1);
        movies->setObjectName("movies");
        movies->setEditTriggers(QAbstractItemView::NoEditTriggers);
        movies->setAlternatingRowColors(true);
        movies->setShowGrid(false);
        movies->horizontalHeader()->setStretchLastSection(true);
        movies->verticalHeader()->setVisible(false);

        verticalLayout_4->addWidget(movies);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        buttonAddMovie = new QPushButton(infoGroupBox);
        buttonAddMovie->setObjectName("buttonAddMovie");

        horizontalLayout_2->addWidget(buttonAddMovie);

        buttonRemoveMovie = new QPushButton(infoGroupBox);
        buttonRemoveMovie->setObjectName("buttonRemoveMovie");

        horizontalLayout_2->addWidget(buttonRemoveMovie);


        verticalLayout_4->addLayout(horizontalLayout_2);


        horizontalLayout->addLayout(verticalLayout_4);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label_3 = new QLabel(infoGroupBox);
        label_3->setObjectName("label_3");
        label_3->setWordWrap(true);

        verticalLayout->addWidget(label_3);

        label_2 = new QLabel(infoGroupBox);
        label_2->setObjectName("label_2");
        label_2->setWordWrap(true);

        verticalLayout->addWidget(label_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        horizontalLayout->setStretch(0, 1);

        verticalLayout_3->addWidget(infoGroupBox);

        verticalLayout_3->setStretch(1, 1);
        splitter->addWidget(horizontalLayoutWidget);

        verticalLayout_2->addWidget(splitter);


        retranslateUi(GenreWidget);
        QObject::connect(buttonAddMovie, SIGNAL(clicked()), GenreWidget, SLOT(addMovie()));
        QObject::connect(buttonRemoveMovie, SIGNAL(clicked()), GenreWidget, SLOT(removeMovie()));

        QMetaObject::connectSlotsByName(GenreWidget);
    } // setupUi

    void retranslateUi(QWidget *GenreWidget)
    {
        QTableWidgetItem *___qtablewidgetitem = genres->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("GenreWidget", "Genre", nullptr));
        genreName->setText(QCoreApplication::translate("GenreWidget", "TextLabel", nullptr));
        infoGroupBox->setTitle(QString());
        QTableWidgetItem *___qtablewidgetitem1 = movies->horizontalHeaderItem(0);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("GenreWidget", "Movie", nullptr));
#if QT_CONFIG(tooltip)
        buttonAddMovie->setToolTip(QCoreApplication::translate("GenreWidget", "Add Movie", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddMovie->setText(QCoreApplication::translate("GenreWidget", "Add Movie", nullptr));
#if QT_CONFIG(tooltip)
        buttonRemoveMovie->setToolTip(QCoreApplication::translate("GenreWidget", "Remove Current Movie", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveMovie->setText(QCoreApplication::translate("GenreWidget", "Remove Movie", nullptr));
        label_3->setText(QCoreApplication::translate("GenreWidget", "Double click a genre to rename it, right click to delete. If you want to merge two genres just give them the same name.", nullptr));
        label_2->setText(QCoreApplication::translate("GenreWidget", "Please keep in mind that the changes you make here (renaming or deleting genres) will be made for every movie.", nullptr));
        (void)GenreWidget;
    } // retranslateUi

};

namespace Ui {
    class GenreWidget: public Ui_GenreWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GENREWIDGET_H
