/********************************************************************************
** Form generated from reading UI file 'NotificationBox.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NOTIFICATIONBOX_H
#define UI_NOTIFICATIONBOX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_NotificationBox
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *widget;
    QVBoxLayout *layoutMessages;

    void setupUi(QWidget *NotificationBox)
    {
        if (NotificationBox->objectName().isEmpty())
            NotificationBox->setObjectName("NotificationBox");
        NotificationBox->resize(284, 100);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(NotificationBox->sizePolicy().hasHeightForWidth());
        NotificationBox->setSizePolicy(sizePolicy);
        NotificationBox->setMinimumSize(QSize(284, 0));
        NotificationBox->setMaximumSize(QSize(284, 16777215));
        verticalLayout = new QVBoxLayout(NotificationBox);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(NotificationBox);
        widget->setObjectName("widget");
        layoutMessages = new QVBoxLayout(widget);
        layoutMessages->setSpacing(5);
        layoutMessages->setObjectName("layoutMessages");
        layoutMessages->setContentsMargins(12, 12, 12, 12);

        verticalLayout->addWidget(widget);


        retranslateUi(NotificationBox);

        QMetaObject::connectSlotsByName(NotificationBox);
    } // setupUi

    void retranslateUi(QWidget *NotificationBox)
    {
        (void)NotificationBox;
    } // retranslateUi

};

namespace Ui {
    class NotificationBox: public Ui_NotificationBox {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NOTIFICATIONBOX_H
