/********************************************************************************
** Form generated from reading UI file 'ImageLabel.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGELABEL_H
#define UI_IMAGELABEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ImageLabel
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *image;
    QLabel *resolution;

    void setupUi(QWidget *ImageLabel)
    {
        if (ImageLabel->objectName().isEmpty())
            ImageLabel->setObjectName("ImageLabel");
        ImageLabel->resize(400, 300);
        verticalLayout = new QVBoxLayout(ImageLabel);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 8);
        image = new QLabel(ImageLabel);
        image->setObjectName("image");
        image->setAlignment(Qt::AlignHCenter|Qt::AlignTop);

        verticalLayout->addWidget(image);

        resolution = new QLabel(ImageLabel);
        resolution->setObjectName("resolution");
        resolution->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        resolution->setWordWrap(true);

        verticalLayout->addWidget(resolution);

        verticalLayout->setStretch(0, 1);

        retranslateUi(ImageLabel);

        QMetaObject::connectSlotsByName(ImageLabel);
    } // setupUi

    void retranslateUi(QWidget *ImageLabel)
    {
        image->setText(QString());
        resolution->setText(QCoreApplication::translate("ImageLabel", "Resolution", nullptr));
        (void)ImageLabel;
    } // retranslateUi

};

namespace Ui {
    class ImageLabel: public Ui_ImageLabel {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGELABEL_H
