/****************************************************************************
** Meta object code from reading C++ file 'TvShowSearchWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowSearchWidget.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowSearchWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TvShowSearchWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN18TvShowSearchWidgetE = QtMocHelpers::stringData(
    "TvShowSearchWidget",
    "sigResultClicked",
    "",
    "search",
    "searchString",
    "initializeAndStartSearch",
    "startSearch",
    "onScraperInitialized",
    "wasSuccessful",
    "mediaelch::scraper::TvScraper*",
    "scraper",
    "onShowResults",
    "mediaelch::scraper::ShowSearchJob*",
    "searchJob",
    "onResultChanged",
    "QTableWidgetItem*",
    "current",
    "previous",
    "onResultDoubleClicked",
    "item",
    "onShowInfoToggled",
    "onEpisodeInfoToggled",
    "onChkAllShowInfosToggled",
    "onChkAllEpisodeInfosToggled",
    "onUpdateTypeChanged",
    "index",
    "onSeasonOrderChanged",
    "onScraperChanged",
    "onLanguageChanged"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN18TvShowSearchWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  110,    2, 0x06,    1 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       3,    1,  111,    2, 0x0a,    2 /* Public */,
       5,    0,  114,    2, 0x08,    4 /* Private */,
       6,    0,  115,    2, 0x08,    5 /* Private */,
       7,    2,  116,    2, 0x08,    6 /* Private */,
      11,    1,  121,    2, 0x08,    9 /* Private */,
      14,    2,  124,    2, 0x08,   11 /* Private */,
      18,    1,  129,    2, 0x08,   14 /* Private */,
      20,    0,  132,    2, 0x08,   16 /* Private */,
      21,    0,  133,    2, 0x08,   17 /* Private */,
      22,    0,  134,    2, 0x08,   18 /* Private */,
      23,    0,  135,    2, 0x08,   19 /* Private */,
      24,    1,  136,    2, 0x08,   20 /* Private */,
      26,    1,  139,    2, 0x08,   22 /* Private */,
      27,    1,  142,    2, 0x08,   24 /* Private */,
      28,    0,  145,    2, 0x08,   26 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 9,    8,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 15,   16,   17,
    QMetaType::Void, 0x80000000 | 15,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowSearchWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN18TvShowSearchWidgetE.offsetsAndSizes,
    qt_meta_data_ZN18TvShowSearchWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN18TvShowSearchWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowSearchWidget, std::true_type>,
        // method 'sigResultClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'search'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'initializeAndStartSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'startSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onScraperInitialized'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::TvScraper *, std::false_type>,
        // method 'onShowResults'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::ShowSearchJob *, std::false_type>,
        // method 'onResultChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onResultDoubleClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onShowInfoToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onEpisodeInfoToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChkAllShowInfosToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChkAllEpisodeInfosToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onUpdateTypeChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onSeasonOrderChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onScraperChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onLanguageChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void TvShowSearchWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowSearchWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigResultClicked(); break;
        case 1: _t->search((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->initializeAndStartSearch(); break;
        case 3: _t->startSearch(); break;
        case 4: _t->onScraperInitialized((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::TvScraper*>>(_a[2]))); break;
        case 5: _t->onShowResults((*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::ShowSearchJob*>>(_a[1]))); break;
        case 6: _t->onResultChanged((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[2]))); break;
        case 7: _t->onResultDoubleClicked((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 8: _t->onShowInfoToggled(); break;
        case 9: _t->onEpisodeInfoToggled(); break;
        case 10: _t->onChkAllShowInfosToggled(); break;
        case 11: _t->onChkAllEpisodeInfosToggled(); break;
        case 12: _t->onUpdateTypeChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->onSeasonOrderChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->onScraperChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->onLanguageChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::TvScraper* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::ShowSearchJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (TvShowSearchWidget::*)();
            if (_q_method_type _q_method = &TvShowSearchWidget::sigResultClicked; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject *TvShowSearchWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowSearchWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN18TvShowSearchWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TvShowSearchWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void TvShowSearchWidget::sigResultClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
