/********************************************************************************
** Form generated from reading UI file 'MovieMultiScrapeDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIEMULTISCRAPEDIALOG_H
#define UI_MOVIEMULTISCRAPEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <ui/small_widgets/LanguageCombo.h>
#include <ui/small_widgets/MyCheckBox.h>

QT_BEGIN_NAMESPACE

class Ui_MovieMultiScrapeDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *lblDescription;
    QHBoxLayout *horizontalLayout_3;
    QLabel *lblScraper;
    QComboBox *comboScraper;
    QSpacerItem *horizontalSpacer_3;
    QLabel *lblLanguage;
    LanguageCombo *comboLanguage;
    QSpacerItem *horizontalSpacer_2;
    QGroupBox *detailsGroupBox;
    QVBoxLayout *verticalLayout_3;
    QLabel *lblDetails;
    QGridLayout *gridLayout_2;
    MyCheckBox *chkSet;
    MyCheckBox *chkTitle;
    MyCheckBox *chkRating;
    MyCheckBox *chkOverview;
    MyCheckBox *chkPoster;
    MyCheckBox *chkCountries;
    MyCheckBox *chkRuntime;
    MyCheckBox *chkActors;
    MyCheckBox *chkCertification;
    MyCheckBox *chkTrailer;
    MyCheckBox *chkTagline;
    MyCheckBox *chkReleased;
    MyCheckBox *chkWriter;
    MyCheckBox *chkDirector;
    MyCheckBox *chkStudios;
    MyCheckBox *chkGenres;
    MyCheckBox *chkBackdrop;
    MyCheckBox *chkThumb;
    MyCheckBox *chkBanner;
    MyCheckBox *chkCdArt;
    MyCheckBox *chkClearArt;
    MyCheckBox *chkLogo;
    MyCheckBox *chkTags;
    QFrame *line;
    QCheckBox *chkUnCheckAll;
    QSpacerItem *verticalSpacer;
    QCheckBox *chkAutoSave;
    QCheckBox *chkOnlyImdb;
    QHBoxLayout *horizontalLayout_2;
    QLabel *movieCounter;
    QLabel *movie;
    QProgressBar *progressMovie;
    QProgressBar *progressAll;
    QLabel *lblError;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnCancel;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnClose;
    QPushButton *btnStartScraping;

    void setupUi(QDialog *MovieMultiScrapeDialog)
    {
        if (MovieMultiScrapeDialog->objectName().isEmpty())
            MovieMultiScrapeDialog->setObjectName("MovieMultiScrapeDialog");
        MovieMultiScrapeDialog->setWindowModality(Qt::WindowModal);
        MovieMultiScrapeDialog->resize(634, 556);
        MovieMultiScrapeDialog->setMinimumSize(QSize(600, 0));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MovieMultiScrapeDialog->setWindowIcon(icon);
        MovieMultiScrapeDialog->setModal(true);
        verticalLayout_2 = new QVBoxLayout(MovieMultiScrapeDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        lblDescription = new QLabel(MovieMultiScrapeDialog);
        lblDescription->setObjectName("lblDescription");
        lblDescription->setWordWrap(true);

        verticalLayout_2->addWidget(lblDescription);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        lblScraper = new QLabel(MovieMultiScrapeDialog);
        lblScraper->setObjectName("lblScraper");

        horizontalLayout_3->addWidget(lblScraper);

        comboScraper = new QComboBox(MovieMultiScrapeDialog);
        comboScraper->setObjectName("comboScraper");

        horizontalLayout_3->addWidget(comboScraper);

        horizontalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        lblLanguage = new QLabel(MovieMultiScrapeDialog);
        lblLanguage->setObjectName("lblLanguage");

        horizontalLayout_3->addWidget(lblLanguage);

        comboLanguage = new LanguageCombo(MovieMultiScrapeDialog);
        comboLanguage->setObjectName("comboLanguage");

        horizontalLayout_3->addWidget(comboLanguage);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout_3);

        detailsGroupBox = new QGroupBox(MovieMultiScrapeDialog);
        detailsGroupBox->setObjectName("detailsGroupBox");
        verticalLayout_3 = new QVBoxLayout(detailsGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        lblDetails = new QLabel(detailsGroupBox);
        lblDetails->setObjectName("lblDetails");
        QFont font;
        font.setBold(true);
        lblDetails->setFont(font);

        verticalLayout_3->addWidget(lblDetails);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        chkSet = new MyCheckBox(detailsGroupBox);
        chkSet->setObjectName("chkSet");
        chkSet->setChecked(true);

        gridLayout_2->addWidget(chkSet, 0, 1, 1, 1);

        chkTitle = new MyCheckBox(detailsGroupBox);
        chkTitle->setObjectName("chkTitle");
        chkTitle->setChecked(true);

        gridLayout_2->addWidget(chkTitle, 0, 0, 1, 1);

        chkRating = new MyCheckBox(detailsGroupBox);
        chkRating->setObjectName("chkRating");
        chkRating->setChecked(true);

        gridLayout_2->addWidget(chkRating, 1, 0, 1, 1);

        chkOverview = new MyCheckBox(detailsGroupBox);
        chkOverview->setObjectName("chkOverview");
        chkOverview->setChecked(true);

        gridLayout_2->addWidget(chkOverview, 3, 1, 1, 1);

        chkPoster = new MyCheckBox(detailsGroupBox);
        chkPoster->setObjectName("chkPoster");
        chkPoster->setChecked(true);

        gridLayout_2->addWidget(chkPoster, 3, 2, 1, 1);

        chkCountries = new MyCheckBox(detailsGroupBox);
        chkCountries->setObjectName("chkCountries");
        chkCountries->setChecked(true);

        gridLayout_2->addWidget(chkCountries, 5, 1, 1, 1);

        chkRuntime = new MyCheckBox(detailsGroupBox);
        chkRuntime->setObjectName("chkRuntime");
        chkRuntime->setChecked(true);

        gridLayout_2->addWidget(chkRuntime, 1, 2, 1, 1);

        chkActors = new MyCheckBox(detailsGroupBox);
        chkActors->setObjectName("chkActors");
        chkActors->setChecked(true);

        gridLayout_2->addWidget(chkActors, 4, 1, 1, 1);

        chkCertification = new MyCheckBox(detailsGroupBox);
        chkCertification->setObjectName("chkCertification");
        chkCertification->setChecked(true);

        gridLayout_2->addWidget(chkCertification, 2, 2, 1, 1);

        chkTrailer = new MyCheckBox(detailsGroupBox);
        chkTrailer->setObjectName("chkTrailer");
        chkTrailer->setChecked(true);

        gridLayout_2->addWidget(chkTrailer, 3, 0, 1, 1);

        chkTagline = new MyCheckBox(detailsGroupBox);
        chkTagline->setObjectName("chkTagline");
        chkTagline->setChecked(true);

        gridLayout_2->addWidget(chkTagline, 0, 2, 1, 1);

        chkReleased = new MyCheckBox(detailsGroupBox);
        chkReleased->setObjectName("chkReleased");
        chkReleased->setChecked(true);

        gridLayout_2->addWidget(chkReleased, 1, 1, 1, 1);

        chkWriter = new MyCheckBox(detailsGroupBox);
        chkWriter->setObjectName("chkWriter");
        chkWriter->setChecked(true);

        gridLayout_2->addWidget(chkWriter, 2, 1, 1, 1);

        chkDirector = new MyCheckBox(detailsGroupBox);
        chkDirector->setObjectName("chkDirector");
        chkDirector->setChecked(true);

        gridLayout_2->addWidget(chkDirector, 2, 0, 1, 1);

        chkStudios = new MyCheckBox(detailsGroupBox);
        chkStudios->setObjectName("chkStudios");
        chkStudios->setChecked(true);

        gridLayout_2->addWidget(chkStudios, 5, 0, 1, 1);

        chkGenres = new MyCheckBox(detailsGroupBox);
        chkGenres->setObjectName("chkGenres");
        chkGenres->setChecked(true);

        gridLayout_2->addWidget(chkGenres, 4, 2, 1, 1);

        chkBackdrop = new MyCheckBox(detailsGroupBox);
        chkBackdrop->setObjectName("chkBackdrop");
        chkBackdrop->setChecked(true);

        gridLayout_2->addWidget(chkBackdrop, 4, 0, 1, 1);

        chkThumb = new MyCheckBox(detailsGroupBox);
        chkThumb->setObjectName("chkThumb");
        chkThumb->setChecked(true);

        gridLayout_2->addWidget(chkThumb, 7, 1, 1, 1);

        chkBanner = new MyCheckBox(detailsGroupBox);
        chkBanner->setObjectName("chkBanner");
        chkBanner->setChecked(true);

        gridLayout_2->addWidget(chkBanner, 7, 0, 1, 1);

        chkCdArt = new MyCheckBox(detailsGroupBox);
        chkCdArt->setObjectName("chkCdArt");
        chkCdArt->setChecked(true);

        gridLayout_2->addWidget(chkCdArt, 6, 2, 1, 1);

        chkClearArt = new MyCheckBox(detailsGroupBox);
        chkClearArt->setObjectName("chkClearArt");
        chkClearArt->setChecked(true);

        gridLayout_2->addWidget(chkClearArt, 6, 1, 1, 1);

        chkLogo = new MyCheckBox(detailsGroupBox);
        chkLogo->setObjectName("chkLogo");
        chkLogo->setChecked(true);

        gridLayout_2->addWidget(chkLogo, 6, 0, 1, 1);

        chkTags = new MyCheckBox(detailsGroupBox);
        chkTags->setObjectName("chkTags");
        chkTags->setChecked(true);

        gridLayout_2->addWidget(chkTags, 5, 2, 1, 1);


        verticalLayout_3->addLayout(gridLayout_2);

        line = new QFrame(detailsGroupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line);

        chkUnCheckAll = new QCheckBox(detailsGroupBox);
        chkUnCheckAll->setObjectName("chkUnCheckAll");
        chkUnCheckAll->setChecked(true);

        verticalLayout_3->addWidget(chkUnCheckAll);


        verticalLayout_2->addWidget(detailsGroupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        chkAutoSave = new QCheckBox(MovieMultiScrapeDialog);
        chkAutoSave->setObjectName("chkAutoSave");

        verticalLayout_2->addWidget(chkAutoSave);

        chkOnlyImdb = new QCheckBox(MovieMultiScrapeDialog);
        chkOnlyImdb->setObjectName("chkOnlyImdb");

        verticalLayout_2->addWidget(chkOnlyImdb);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        movieCounter = new QLabel(MovieMultiScrapeDialog);
        movieCounter->setObjectName("movieCounter");
        movieCounter->setMinimumSize(QSize(30, 0));

        horizontalLayout_2->addWidget(movieCounter);

        movie = new QLabel(MovieMultiScrapeDialog);
        movie->setObjectName("movie");

        horizontalLayout_2->addWidget(movie);

        horizontalLayout_2->setStretch(1, 1);

        verticalLayout_2->addLayout(horizontalLayout_2);

        progressMovie = new QProgressBar(MovieMultiScrapeDialog);
        progressMovie->setObjectName("progressMovie");
        progressMovie->setValue(24);

        verticalLayout_2->addWidget(progressMovie);

        progressAll = new QProgressBar(MovieMultiScrapeDialog);
        progressAll->setObjectName("progressAll");
        progressAll->setValue(24);

        verticalLayout_2->addWidget(progressAll);

        lblError = new QLabel(MovieMultiScrapeDialog);
        lblError->setObjectName("lblError");
        lblError->setText(QString::fromUtf8("Error Text"));

        verticalLayout_2->addWidget(lblError);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnCancel = new QPushButton(MovieMultiScrapeDialog);
        btnCancel->setObjectName("btnCancel");
        btnCancel->setAutoDefault(false);

        horizontalLayout->addWidget(btnCancel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnClose = new QPushButton(MovieMultiScrapeDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(true);

        horizontalLayout->addWidget(btnClose);

        btnStartScraping = new QPushButton(MovieMultiScrapeDialog);
        btnStartScraping->setObjectName("btnStartScraping");

        horizontalLayout->addWidget(btnStartScraping);


        verticalLayout_2->addLayout(horizontalLayout);


        retranslateUi(MovieMultiScrapeDialog);
        QObject::connect(btnCancel, &QPushButton::clicked, MovieMultiScrapeDialog, qOverload<>(&QDialog::reject));
        QObject::connect(btnClose, &QPushButton::clicked, MovieMultiScrapeDialog, qOverload<>(&QDialog::accept));

        btnClose->setDefault(true);
        btnStartScraping->setDefault(true);


        QMetaObject::connectSlotsByName(MovieMultiScrapeDialog);
    } // setupUi

    void retranslateUi(QDialog *MovieMultiScrapeDialog)
    {
        MovieMultiScrapeDialog->setWindowTitle(QCoreApplication::translate("MovieMultiScrapeDialog", "Movie Multi Scrape", nullptr));
        lblDescription->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Please select the scraper and the infos you want to be loaded. MediaElch will use the best result for each movie you selected.", nullptr));
        lblScraper->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Scraper", nullptr));
        lblLanguage->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Language", nullptr));
        detailsGroupBox->setTitle(QString());
        lblDetails->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Infos to load", nullptr));
        chkSet->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Set", nullptr));
        chkTitle->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Title", nullptr));
        chkRating->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Rating", nullptr));
        chkOverview->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Overview", nullptr));
        chkPoster->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Poster", nullptr));
        chkCountries->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Countries", nullptr));
        chkRuntime->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Runtime", nullptr));
        chkActors->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Actors", nullptr));
        chkCertification->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Certification", nullptr));
        chkTrailer->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Trailer", nullptr));
        chkTagline->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Tagline", nullptr));
        chkReleased->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Released", nullptr));
        chkWriter->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Writer", nullptr));
        chkDirector->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Director", nullptr));
        chkStudios->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Studios", nullptr));
        chkGenres->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Genres", nullptr));
        chkBackdrop->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Backdrop", nullptr));
        chkThumb->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Thumb", nullptr));
        chkBanner->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Banner", nullptr));
        chkCdArt->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "CD Art", nullptr));
        chkClearArt->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Clear Art", nullptr));
        chkLogo->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Logo", nullptr));
        chkTags->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Tags", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "(Un)Check all", nullptr));
        chkAutoSave->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Automatically save each movie after scraping", nullptr));
        chkOnlyImdb->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Update only movies with IMDb ID/TMDb Id", nullptr));
        movieCounter->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "1/20", nullptr));
        movie->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Movie", nullptr));
        btnCancel->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Cancel", nullptr));
        btnClose->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Close", nullptr));
        btnStartScraping->setText(QCoreApplication::translate("MovieMultiScrapeDialog", "Start Scraping", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MovieMultiScrapeDialog: public Ui_MovieMultiScrapeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIEMULTISCRAPEDIALOG_H
