/********************************************************************************
** Form generated from reading UI file 'MakeMkvDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAKEMKVDIALOG_H
#define UI_MAKEMKVDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/movies/MovieSearchWidget.h"
#include "ui/small_widgets/Badge.h"
#include "ui/small_widgets/SlidingStackedWidget.h"

QT_BEGIN_NAMESPACE

class Ui_MakeMkvDialog
{
public:
    QVBoxLayout *verticalLayout;
    SlidingStackedWidget *stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QComboBox *comboDrives;
    QPushButton *btnScanDrive;
    QPlainTextEdit *messages;
    QWidget *tracksWidget;
    QHBoxLayout *horizontalLayout_4;
    QListWidget *tracks;
    QVBoxLayout *verticalLayout_3;
    QToolButton *btnImportTracks;
    QToolButton *btnImportComplete;
    QSpacerItem *verticalSpacer;
    MovieSearchWidget *movieSearchWidget;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_3;
    QLabel *loading;
    Badge *badgeSuccess;
    QLabel *labelLoading;
    QSpacerItem *horizontalSpacer_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *gridLayout_2;
    QLabel *descOriginalTitle;
    QLabel *labelOriginalTitle;
    QLabel *label_32;
    QLabel *label_3;
    QLabel *label_7;
    QLabel *labelExtension;
    QLabel *label_14;
    QLabel *label_31;
    QLabel *label_4;
    QLabel *label_8;
    QLabel *labelPart;
    QLabel *descPart;
    QFormLayout *formLayout;
    QLabel *labelDirectoryNaming;
    QLineEdit *directoryNaming;
    QLabel *labelFileNaming;
    QLineEdit *fileNaming;
    QLabel *labelMultiFileNaming;
    QLineEdit *multiFileNaming;
    QLabel *label;
    QComboBox *comboImportDir;
    QSpacerItem *verticalSpacer_2;
    QProgressBar *progressBar;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnClose;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnImport;

    void setupUi(QDialog *MakeMkvDialog)
    {
        if (MakeMkvDialog->objectName().isEmpty())
            MakeMkvDialog->setObjectName("MakeMkvDialog");
        MakeMkvDialog->setWindowModality(Qt::WindowModal);
        MakeMkvDialog->resize(741, 466);
        MakeMkvDialog->setWindowTitle(QString::fromUtf8("MediaElch"));
        MakeMkvDialog->setModal(true);
        verticalLayout = new QVBoxLayout(MakeMkvDialog);
        verticalLayout->setObjectName("verticalLayout");
        stackedWidget = new SlidingStackedWidget(MakeMkvDialog);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        comboDrives = new QComboBox(page);
        comboDrives->setObjectName("comboDrives");

        horizontalLayout_2->addWidget(comboDrives);

        btnScanDrive = new QPushButton(page);
        btnScanDrive->setObjectName("btnScanDrive");

        horizontalLayout_2->addWidget(btnScanDrive);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout_2->addLayout(horizontalLayout_2);

        messages = new QPlainTextEdit(page);
        messages->setObjectName("messages");
        messages->setReadOnly(true);

        verticalLayout_2->addWidget(messages);

        tracksWidget = new QWidget(page);
        tracksWidget->setObjectName("tracksWidget");
        horizontalLayout_4 = new QHBoxLayout(tracksWidget);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        tracks = new QListWidget(tracksWidget);
        tracks->setObjectName("tracks");

        horizontalLayout_4->addWidget(tracks);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        btnImportTracks = new QToolButton(tracksWidget);
        btnImportTracks->setObjectName("btnImportTracks");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(btnImportTracks->sizePolicy().hasHeightForWidth());
        btnImportTracks->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/inbox_64.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnImportTracks->setIcon(icon);
        btnImportTracks->setIconSize(QSize(32, 32));
        btnImportTracks->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

        verticalLayout_3->addWidget(btnImportTracks);

        btnImportComplete = new QToolButton(tracksWidget);
        btnImportComplete->setObjectName("btnImportComplete");
        sizePolicy.setHeightForWidth(btnImportComplete->sizePolicy().hasHeightForWidth());
        btnImportComplete->setSizePolicy(sizePolicy);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/disc_64.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnImportComplete->setIcon(icon1);
        btnImportComplete->setIconSize(QSize(32, 32));
        btnImportComplete->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

        verticalLayout_3->addWidget(btnImportComplete);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        horizontalLayout_4->addLayout(verticalLayout_3);


        verticalLayout_2->addWidget(tracksWidget);

        stackedWidget->addWidget(page);
        movieSearchWidget = new MovieSearchWidget();
        movieSearchWidget->setObjectName("movieSearchWidget");
        stackedWidget->addWidget(movieSearchWidget);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_5 = new QVBoxLayout(page_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        loading = new QLabel(page_2);
        loading->setObjectName("loading");

        horizontalLayout_3->addWidget(loading);

        badgeSuccess = new Badge(page_2);
        badgeSuccess->setObjectName("badgeSuccess");

        horizontalLayout_3->addWidget(badgeSuccess);

        labelLoading = new QLabel(page_2);
        labelLoading->setObjectName("labelLoading");

        horizontalLayout_3->addWidget(labelLoading);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        horizontalLayout_3->setStretch(2, 1);

        verticalLayout_5->addLayout(horizontalLayout_3);

        groupBox = new QGroupBox(page_2);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        descOriginalTitle = new QLabel(groupBox);
        descOriginalTitle->setObjectName("descOriginalTitle");
        descOriginalTitle->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(descOriginalTitle, 4, 1, 1, 1);

        labelOriginalTitle = new QLabel(groupBox);
        labelOriginalTitle->setObjectName("labelOriginalTitle");
        labelOriginalTitle->setText(QString::fromUtf8("<originalTitle>"));
        labelOriginalTitle->setTextFormat(Qt::PlainText);
        labelOriginalTitle->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(labelOriginalTitle, 4, 0, 1, 1);

        label_32 = new QLabel(groupBox);
        label_32->setObjectName("label_32");
        QFont font;
        font.setBold(true);
        label_32->setFont(font);
        label_32->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_32, 0, 1, 1, 1);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");
        label_3->setText(QString::fromUtf8("<title>"));
        label_3->setTextFormat(Qt::PlainText);
        label_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_3, 3, 0, 1, 1);

        label_7 = new QLabel(groupBox);
        label_7->setObjectName("label_7");
        label_7->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_7, 3, 1, 1, 1);

        labelExtension = new QLabel(groupBox);
        labelExtension->setObjectName("labelExtension");
        labelExtension->setText(QString::fromUtf8("<extension>"));
        labelExtension->setTextFormat(Qt::PlainText);
        labelExtension->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(labelExtension, 1, 0, 1, 1);

        label_14 = new QLabel(groupBox);
        label_14->setObjectName("label_14");
        label_14->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_14, 1, 1, 1, 1);

        label_31 = new QLabel(groupBox);
        label_31->setObjectName("label_31");
        label_31->setFont(font);
        label_31->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_31, 0, 0, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");
        label_4->setText(QString::fromUtf8("<year>"));
        label_4->setTextFormat(Qt::PlainText);
        label_4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_4, 5, 0, 1, 1);

        label_8 = new QLabel(groupBox);
        label_8->setObjectName("label_8");
        label_8->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(label_8, 5, 1, 1, 1);

        labelPart = new QLabel(groupBox);
        labelPart->setObjectName("labelPart");
        labelPart->setText(QString::fromUtf8("<partNo>"));

        gridLayout_2->addWidget(labelPart, 2, 0, 1, 1);

        descPart = new QLabel(groupBox);
        descPart->setObjectName("descPart");

        gridLayout_2->addWidget(descPart, 2, 1, 1, 1);


        verticalLayout_4->addLayout(gridLayout_2);


        verticalLayout_5->addWidget(groupBox);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        labelDirectoryNaming = new QLabel(page_2);
        labelDirectoryNaming->setObjectName("labelDirectoryNaming");

        formLayout->setWidget(0, QFormLayout::LabelRole, labelDirectoryNaming);

        directoryNaming = new QLineEdit(page_2);
        directoryNaming->setObjectName("directoryNaming");

        formLayout->setWidget(0, QFormLayout::FieldRole, directoryNaming);

        labelFileNaming = new QLabel(page_2);
        labelFileNaming->setObjectName("labelFileNaming");

        formLayout->setWidget(1, QFormLayout::LabelRole, labelFileNaming);

        fileNaming = new QLineEdit(page_2);
        fileNaming->setObjectName("fileNaming");

        formLayout->setWidget(1, QFormLayout::FieldRole, fileNaming);

        labelMultiFileNaming = new QLabel(page_2);
        labelMultiFileNaming->setObjectName("labelMultiFileNaming");

        formLayout->setWidget(2, QFormLayout::LabelRole, labelMultiFileNaming);

        multiFileNaming = new QLineEdit(page_2);
        multiFileNaming->setObjectName("multiFileNaming");

        formLayout->setWidget(2, QFormLayout::FieldRole, multiFileNaming);

        label = new QLabel(page_2);
        label->setObjectName("label");

        formLayout->setWidget(3, QFormLayout::LabelRole, label);

        comboImportDir = new QComboBox(page_2);
        comboImportDir->setObjectName("comboImportDir");

        formLayout->setWidget(3, QFormLayout::FieldRole, comboImportDir);


        verticalLayout_5->addLayout(formLayout);

        verticalSpacer_2 = new QSpacerItem(20, 27, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_2);

        progressBar = new QProgressBar(page_2);
        progressBar->setObjectName("progressBar");
        progressBar->setMinimum(0);
        progressBar->setMaximum(100);
        progressBar->setValue(0);

        verticalLayout_5->addWidget(progressBar);

        stackedWidget->addWidget(page_2);

        verticalLayout->addWidget(stackedWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnClose = new QPushButton(MakeMkvDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(false);

        horizontalLayout->addWidget(btnClose);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnImport = new QPushButton(MakeMkvDialog);
        btnImport->setObjectName("btnImport");

        horizontalLayout->addWidget(btnImport);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(MakeMkvDialog);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(MakeMkvDialog);
    } // setupUi

    void retranslateUi(QDialog *MakeMkvDialog)
    {
        btnScanDrive->setText(QCoreApplication::translate("MakeMkvDialog", "Scan", nullptr));
        btnImportTracks->setText(QCoreApplication::translate("MakeMkvDialog", "Import Tracks", nullptr));
        btnImportComplete->setText(QCoreApplication::translate("MakeMkvDialog", "Backup Disc", nullptr));
        loading->setText(QCoreApplication::translate("MakeMkvDialog", "Loading", nullptr));
        badgeSuccess->setText(QCoreApplication::translate("MakeMkvDialog", "Success", nullptr));
        labelLoading->setText(QCoreApplication::translate("MakeMkvDialog", "Loading movie...", nullptr));
        groupBox->setTitle(QCoreApplication::translate("MakeMkvDialog", "Placeholders", nullptr));
        descOriginalTitle->setText(QCoreApplication::translate("MakeMkvDialog", "Original Title", nullptr));
        label_32->setText(QCoreApplication::translate("MakeMkvDialog", "Description", nullptr));
        label_7->setText(QCoreApplication::translate("MakeMkvDialog", "Title", nullptr));
        label_14->setText(QCoreApplication::translate("MakeMkvDialog", "File extension", nullptr));
        label_31->setText(QCoreApplication::translate("MakeMkvDialog", "Placeholder", nullptr));
        label_8->setText(QCoreApplication::translate("MakeMkvDialog", "Year", nullptr));
        descPart->setText(QCoreApplication::translate("MakeMkvDialog", "Part number of the current file", nullptr));
        labelDirectoryNaming->setText(QCoreApplication::translate("MakeMkvDialog", "Directory Naming", nullptr));
        labelFileNaming->setText(QCoreApplication::translate("MakeMkvDialog", "File Naming", nullptr));
        labelMultiFileNaming->setText(QCoreApplication::translate("MakeMkvDialog", "Multi-File Naming", nullptr));
        label->setText(QCoreApplication::translate("MakeMkvDialog", "Import directory", nullptr));
        btnClose->setText(QCoreApplication::translate("MakeMkvDialog", "Close", nullptr));
        btnImport->setText(QCoreApplication::translate("MakeMkvDialog", "Import", nullptr));
        (void)MakeMkvDialog;
    } // retranslateUi

};

namespace Ui {
    class MakeMkvDialog: public Ui_MakeMkvDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAKEMKVDIALOG_H
