/********************************************************************************
** Form generated from reading UI file 'RenamerPlaceholders.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RENAMERPLACEHOLDERS_H
#define UI_RENAMERPLACEHOLDERS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RenamerPlaceholders
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QScrollArea *scrollArea;
    QWidget *placeholderItems;
    QGridLayout *placeholderLayout;
    QLabel *descDvd;
    QLabel *descShowTitle;
    QLabel *descExtension;
    QLabel *descTitle;
    QLabel *descMovieSet;
    QLabel *lblDescription;
    QLabel *labelSeason;
    QLabel *descResolution;
    QLabel *labelAlbum;
    QLabel *descAlbum;
    QLabel *descCondSeasonName;
    QLabel *labelSortTitle;
    QLabel *descBluray;
    QLabel *labelImdbId;
    QLabel *labelPartNo;
    QLabel *lblPlaceholder;
    QLabel *labelExtension;
    QLabel *labelYear;
    QLabel *label3d;
    QLabel *descAudioCodec;
    QLabel *labelAudioCodec;
    QLabel *labelDirector;
    QLabel *descVideoCodec;
    QLabel *labelShowTitle;
    QLabel *descImdbId;
    QLabel *descSubtitleLanguage;
    QLabel *labelArtist;
    QLabel *labelOriginalTitle;
    QLabel *descOriginalTitle;
    QLabel *desc3d;
    QLabel *descSeasonName;
    QLabel *descDirector;
    QLabel *labelResolution;
    QLabel *descEpisode;
    QLabel *labelCondSeasonName;
    QLabel *labelSeasonName;
    QLabel *descAudioChannels;
    QLabel *labelMovieSet;
    QLabel *descSortTitle;
    QLabel *labelDvd;
    QLabel *labelChannels;
    QLabel *descAudioLanguage;
    QLabel *descArtist;
    QLabel *labelTitle;
    QLabel *descStudio;
    QLabel *labelStudio;
    QLabel *descPartNo;
    QLabel *labelSubtitleLanguage;
    QLabel *labelEpisode;
    QLabel *labelVideoCodec;
    QLabel *labelBluray;
    QLabel *descSeason;
    QLabel *labelAudioLanguage;
    QLabel *descYear;
    QLabel *labelTmdbId;
    QLabel *descTmdbId;

    void setupUi(QWidget *RenamerPlaceholders)
    {
        if (RenamerPlaceholders->objectName().isEmpty())
            RenamerPlaceholders->setObjectName("RenamerPlaceholders");
        RenamerPlaceholders->resize(480, 200);
        RenamerPlaceholders->setMinimumSize(QSize(0, 200));
        verticalLayout = new QVBoxLayout(RenamerPlaceholders);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 6, 0, 0);
        groupBox = new QGroupBox(RenamerPlaceholders);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(groupBox);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        placeholderItems = new QWidget();
        placeholderItems->setObjectName("placeholderItems");
        placeholderItems->setGeometry(QRect(0, -530, 524, 688));
        placeholderLayout = new QGridLayout(placeholderItems);
        placeholderLayout->setObjectName("placeholderLayout");
        descDvd = new QLabel(placeholderItems);
        descDvd->setObjectName("descDvd");
        descDvd->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descDvd, 24, 1, 1, 1);

        descShowTitle = new QLabel(placeholderItems);
        descShowTitle->setObjectName("descShowTitle");
        descShowTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(descShowTitle, 12, 1, 1, 1);

        descExtension = new QLabel(placeholderItems);
        descExtension->setObjectName("descExtension");
        descExtension->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descExtension, 2, 1, 1, 1);

        descTitle = new QLabel(placeholderItems);
        descTitle->setObjectName("descTitle");
        descTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descTitle, 4, 1, 1, 1);

        descMovieSet = new QLabel(placeholderItems);
        descMovieSet->setObjectName("descMovieSet");
        descMovieSet->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descMovieSet, 27, 1, 1, 1);

        lblDescription = new QLabel(placeholderItems);
        lblDescription->setObjectName("lblDescription");
        QFont font;
        font.setBold(true);
        lblDescription->setFont(font);
        lblDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        placeholderLayout->addWidget(lblDescription, 0, 1, 1, 1);

        labelSeason = new QLabel(placeholderItems);
        labelSeason->setObjectName("labelSeason");
        labelSeason->setText(QString::fromUtf8("<season>"));
        labelSeason->setTextFormat(Qt::PlainText);
        labelSeason->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(labelSeason, 10, 0, 1, 1);

        descResolution = new QLabel(placeholderItems);
        descResolution->setObjectName("descResolution");
        descResolution->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descResolution, 17, 1, 1, 1);

        labelAlbum = new QLabel(placeholderItems);
        labelAlbum->setObjectName("labelAlbum");
        labelAlbum->setText(QString::fromUtf8("<album>"));
        labelAlbum->setTextFormat(Qt::PlainText);
        labelAlbum->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelAlbum, 16, 0, 1, 1);

        descAlbum = new QLabel(placeholderItems);
        descAlbum->setObjectName("descAlbum");
        descAlbum->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descAlbum, 16, 1, 1, 1);

        descCondSeasonName = new QLabel(placeholderItems);
        descCondSeasonName->setObjectName("descCondSeasonName");
        descCondSeasonName->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(descCondSeasonName, 25, 1, 1, 1);

        labelSortTitle = new QLabel(placeholderItems);
        labelSortTitle->setObjectName("labelSortTitle");
        labelSortTitle->setText(QString::fromUtf8("<sortTitle>"));
        labelSortTitle->setTextFormat(Qt::PlainText);
        labelSortTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelSortTitle, 6, 0, 1, 1);

        descBluray = new QLabel(placeholderItems);
        descBluray->setObjectName("descBluray");
        descBluray->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descBluray, 23, 1, 1, 1);

        labelImdbId = new QLabel(placeholderItems);
        labelImdbId->setObjectName("labelImdbId");
        labelImdbId->setText(QString::fromUtf8("{imdbId}...<imdbId>...{/imdbId}"));
        labelImdbId->setTextFormat(Qt::PlainText);
        labelImdbId->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelImdbId, 28, 0, 1, 1);

        labelPartNo = new QLabel(placeholderItems);
        labelPartNo->setObjectName("labelPartNo");
        labelPartNo->setText(QString::fromUtf8("<partNo>"));
        labelPartNo->setTextFormat(Qt::PlainText);
        labelPartNo->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelPartNo, 3, 0, 1, 1);

        lblPlaceholder = new QLabel(placeholderItems);
        lblPlaceholder->setObjectName("lblPlaceholder");
        lblPlaceholder->setFont(font);
        lblPlaceholder->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        placeholderLayout->addWidget(lblPlaceholder, 0, 0, 1, 1);

        labelExtension = new QLabel(placeholderItems);
        labelExtension->setObjectName("labelExtension");
        labelExtension->setText(QString::fromUtf8("<extension>"));
        labelExtension->setTextFormat(Qt::PlainText);
        labelExtension->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelExtension, 2, 0, 1, 1);

        labelYear = new QLabel(placeholderItems);
        labelYear->setObjectName("labelYear");
        labelYear->setText(QString::fromUtf8("<year>"));
        labelYear->setTextFormat(Qt::PlainText);
        labelYear->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelYear, 7, 0, 1, 1);

        label3d = new QLabel(placeholderItems);
        label3d->setObjectName("label3d");
        label3d->setText(QString::fromUtf8("{3D}...{/3D}"));
        label3d->setTextFormat(Qt::PlainText);
        label3d->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(label3d, 26, 0, 1, 1);

        descAudioCodec = new QLabel(placeholderItems);
        descAudioCodec->setObjectName("descAudioCodec");
        descAudioCodec->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descAudioCodec, 19, 1, 1, 1);

        labelAudioCodec = new QLabel(placeholderItems);
        labelAudioCodec->setObjectName("labelAudioCodec");
        labelAudioCodec->setText(QString::fromUtf8("<audioCodec>"));
        labelAudioCodec->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelAudioCodec, 19, 0, 1, 1);

        labelDirector = new QLabel(placeholderItems);
        labelDirector->setObjectName("labelDirector");
        labelDirector->setText(QString::fromUtf8("<director>"));
        labelDirector->setTextFormat(Qt::PlainText);
        labelDirector->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelDirector, 13, 0, 1, 1);

        descVideoCodec = new QLabel(placeholderItems);
        descVideoCodec->setObjectName("descVideoCodec");
        descVideoCodec->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descVideoCodec, 18, 1, 1, 1);

        labelShowTitle = new QLabel(placeholderItems);
        labelShowTitle->setObjectName("labelShowTitle");
        labelShowTitle->setText(QString::fromUtf8("<showTitle>"));
        labelShowTitle->setTextFormat(Qt::PlainText);
        labelShowTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(labelShowTitle, 12, 0, 1, 1);

        descImdbId = new QLabel(placeholderItems);
        descImdbId->setObjectName("descImdbId");
        descImdbId->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descImdbId, 28, 1, 1, 1);

        descSubtitleLanguage = new QLabel(placeholderItems);
        descSubtitleLanguage->setObjectName("descSubtitleLanguage");

        placeholderLayout->addWidget(descSubtitleLanguage, 21, 1, 1, 1);

        labelArtist = new QLabel(placeholderItems);
        labelArtist->setObjectName("labelArtist");
        labelArtist->setText(QString::fromUtf8("<artist>"));
        labelArtist->setTextFormat(Qt::PlainText);
        labelArtist->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelArtist, 15, 0, 1, 1);

        labelOriginalTitle = new QLabel(placeholderItems);
        labelOriginalTitle->setObjectName("labelOriginalTitle");
        labelOriginalTitle->setText(QString::fromUtf8("<originalTitle>"));
        labelOriginalTitle->setTextFormat(Qt::PlainText);
        labelOriginalTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelOriginalTitle, 5, 0, 1, 1);

        descOriginalTitle = new QLabel(placeholderItems);
        descOriginalTitle->setObjectName("descOriginalTitle");
        descOriginalTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descOriginalTitle, 5, 1, 1, 1);

        desc3d = new QLabel(placeholderItems);
        desc3d->setObjectName("desc3d");
        desc3d->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(desc3d, 26, 1, 1, 1);

        descSeasonName = new QLabel(placeholderItems);
        descSeasonName->setObjectName("descSeasonName");
        descSeasonName->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(descSeasonName, 11, 1, 1, 1);

        descDirector = new QLabel(placeholderItems);
        descDirector->setObjectName("descDirector");
        descDirector->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descDirector, 13, 1, 1, 1);

        labelResolution = new QLabel(placeholderItems);
        labelResolution->setObjectName("labelResolution");
        labelResolution->setText(QString::fromUtf8("<resolution>"));
        labelResolution->setTextFormat(Qt::PlainText);
        labelResolution->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelResolution, 17, 0, 1, 1);

        descEpisode = new QLabel(placeholderItems);
        descEpisode->setObjectName("descEpisode");
        descEpisode->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(descEpisode, 8, 1, 1, 1);

        labelCondSeasonName = new QLabel(placeholderItems);
        labelCondSeasonName->setObjectName("labelCondSeasonName");
        labelCondSeasonName->setText(QString::fromUtf8("{seasonName}...{/seasonName}"));
        labelCondSeasonName->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(labelCondSeasonName, 25, 0, 1, 1);

        labelSeasonName = new QLabel(placeholderItems);
        labelSeasonName->setObjectName("labelSeasonName");
        labelSeasonName->setText(QString::fromUtf8("<seasonName>"));
        labelSeasonName->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(labelSeasonName, 11, 0, 1, 1);

        descAudioChannels = new QLabel(placeholderItems);
        descAudioChannels->setObjectName("descAudioChannels");
        descAudioChannels->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descAudioChannels, 22, 1, 1, 1);

        labelMovieSet = new QLabel(placeholderItems);
        labelMovieSet->setObjectName("labelMovieSet");
        labelMovieSet->setText(QString::fromUtf8("{movieset}...<movieset>...{/movieset}"));
        labelMovieSet->setTextFormat(Qt::PlainText);
        labelMovieSet->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelMovieSet, 27, 0, 1, 1);

        descSortTitle = new QLabel(placeholderItems);
        descSortTitle->setObjectName("descSortTitle");
        descSortTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descSortTitle, 6, 1, 1, 1);

        labelDvd = new QLabel(placeholderItems);
        labelDvd->setObjectName("labelDvd");
        labelDvd->setText(QString::fromUtf8("{dvd}...{/dvd}"));
        labelDvd->setTextFormat(Qt::PlainText);
        labelDvd->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelDvd, 24, 0, 1, 1);

        labelChannels = new QLabel(placeholderItems);
        labelChannels->setObjectName("labelChannels");
        labelChannels->setText(QString::fromUtf8("<channels>"));
        labelChannels->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelChannels, 22, 0, 1, 1);

        descAudioLanguage = new QLabel(placeholderItems);
        descAudioLanguage->setObjectName("descAudioLanguage");

        placeholderLayout->addWidget(descAudioLanguage, 20, 1, 1, 1);

        descArtist = new QLabel(placeholderItems);
        descArtist->setObjectName("descArtist");
        descArtist->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descArtist, 15, 1, 1, 1);

        labelTitle = new QLabel(placeholderItems);
        labelTitle->setObjectName("labelTitle");
        labelTitle->setText(QString::fromUtf8("<title>"));
        labelTitle->setTextFormat(Qt::PlainText);
        labelTitle->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelTitle, 4, 0, 1, 1);

        descStudio = new QLabel(placeholderItems);
        descStudio->setObjectName("descStudio");
        descStudio->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descStudio, 14, 1, 1, 1);

        labelStudio = new QLabel(placeholderItems);
        labelStudio->setObjectName("labelStudio");
        labelStudio->setText(QString::fromUtf8("<studio>"));
        labelStudio->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelStudio, 14, 0, 1, 1);

        descPartNo = new QLabel(placeholderItems);
        descPartNo->setObjectName("descPartNo");
        descPartNo->setWordWrap(true);
        descPartNo->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descPartNo, 3, 1, 1, 1);

        labelSubtitleLanguage = new QLabel(placeholderItems);
        labelSubtitleLanguage->setObjectName("labelSubtitleLanguage");
        labelSubtitleLanguage->setText(QString::fromUtf8("<subtitleLanguage>"));
        labelSubtitleLanguage->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelSubtitleLanguage, 21, 0, 1, 1);

        labelEpisode = new QLabel(placeholderItems);
        labelEpisode->setObjectName("labelEpisode");
        labelEpisode->setText(QString::fromUtf8("<episode>"));
        labelEpisode->setTextFormat(Qt::PlainText);
        labelEpisode->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(labelEpisode, 8, 0, 1, 1);

        labelVideoCodec = new QLabel(placeholderItems);
        labelVideoCodec->setObjectName("labelVideoCodec");
        labelVideoCodec->setText(QString::fromUtf8("<videoCodec>"));
        labelVideoCodec->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelVideoCodec, 18, 0, 1, 1);

        labelBluray = new QLabel(placeholderItems);
        labelBluray->setObjectName("labelBluray");
        labelBluray->setText(QString::fromUtf8("{bluray}...{/bluray}"));
        labelBluray->setTextFormat(Qt::PlainText);
        labelBluray->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelBluray, 23, 0, 1, 1);

        descSeason = new QLabel(placeholderItems);
        descSeason->setObjectName("descSeason");
        descSeason->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("tvshow")}));

        placeholderLayout->addWidget(descSeason, 10, 1, 1, 1);

        labelAudioLanguage = new QLabel(placeholderItems);
        labelAudioLanguage->setObjectName("labelAudioLanguage");
        labelAudioLanguage->setText(QString::fromUtf8("<audioLanguage>"));
        labelAudioLanguage->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(labelAudioLanguage, 20, 0, 1, 1);

        descYear = new QLabel(placeholderItems);
        descYear->setObjectName("descYear");
        descYear->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie"), QString::fromUtf8("tvshow"), QString::fromUtf8("concert")}));

        placeholderLayout->addWidget(descYear, 7, 1, 1, 1);

        labelTmdbId = new QLabel(placeholderItems);
        labelTmdbId->setObjectName("labelTmdbId");
        labelTmdbId->setText(QString::fromUtf8("{tmdbId}...<tmdbId>...{/tmdbId}"));
        labelTmdbId->setTextFormat(Qt::PlainText);
        labelTmdbId->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(labelTmdbId, 29, 0, 1, 1);

        descTmdbId = new QLabel(placeholderItems);
        descTmdbId->setObjectName("descTmdbId");
        descTmdbId->setProperty("itemTypes", QVariant(QStringList{QString::fromUtf8("movie")}));

        placeholderLayout->addWidget(descTmdbId, 29, 1, 1, 1);

        placeholderLayout->setColumnStretch(1, 1);
        scrollArea->setWidget(placeholderItems);

        verticalLayout_2->addWidget(scrollArea);


        verticalLayout->addWidget(groupBox);


        retranslateUi(RenamerPlaceholders);

        QMetaObject::connectSlotsByName(RenamerPlaceholders);
    } // setupUi

    void retranslateUi(QWidget *RenamerPlaceholders)
    {
        groupBox->setTitle(QCoreApplication::translate("RenamerPlaceholders", "Placeholders", nullptr));
        descDvd->setText(QCoreApplication::translate("RenamerPlaceholders", "File/directory is DVD", nullptr));
        descShowTitle->setText(QCoreApplication::translate("RenamerPlaceholders", "Title of the show", nullptr));
        descExtension->setText(QCoreApplication::translate("RenamerPlaceholders", "File extension", nullptr));
        descTitle->setText(QCoreApplication::translate("RenamerPlaceholders", "Title", nullptr));
        descMovieSet->setText(QCoreApplication::translate("RenamerPlaceholders", "Movie set name", nullptr));
        lblDescription->setText(QCoreApplication::translate("RenamerPlaceholders", "Description", nullptr));
        descResolution->setText(QCoreApplication::translate("RenamerPlaceholders", "Resolution (1080p, 720p, ...)", nullptr));
        descAlbum->setText(QCoreApplication::translate("RenamerPlaceholders", "Album", nullptr));
        descCondSeasonName->setText(QCoreApplication::translate("RenamerPlaceholders", "Episode has a season name", nullptr));
        descBluray->setText(QCoreApplication::translate("RenamerPlaceholders", "File/directory is BluRay", nullptr));
        lblPlaceholder->setText(QCoreApplication::translate("RenamerPlaceholders", "Placeholder", nullptr));
        descAudioCodec->setText(QCoreApplication::translate("RenamerPlaceholders", "Audio Codec", nullptr));
        descVideoCodec->setText(QCoreApplication::translate("RenamerPlaceholders", "Video Codec", nullptr));
        descImdbId->setText(QCoreApplication::translate("RenamerPlaceholders", "IMDb ID", nullptr));
        descSubtitleLanguage->setText(QCoreApplication::translate("RenamerPlaceholders", "Subtitle Language(s) (separated by a minus)", nullptr));
        descOriginalTitle->setText(QCoreApplication::translate("RenamerPlaceholders", "Original Title", nullptr));
        desc3d->setText(QCoreApplication::translate("RenamerPlaceholders", "File is 3D", nullptr));
        descSeasonName->setText(QCoreApplication::translate("RenamerPlaceholders", "Season Name", nullptr));
        descDirector->setText(QCoreApplication::translate("RenamerPlaceholders", "Director(s)", nullptr));
        descEpisode->setText(QCoreApplication::translate("RenamerPlaceholders", "Episode Number", nullptr));
        descAudioChannels->setText(QCoreApplication::translate("RenamerPlaceholders", "Number of audio channels", nullptr));
        descSortTitle->setText(QCoreApplication::translate("RenamerPlaceholders", "Sort Title", nullptr));
        descAudioLanguage->setText(QCoreApplication::translate("RenamerPlaceholders", "Audio Language(s) (separated by a minus)", nullptr));
        descArtist->setText(QCoreApplication::translate("RenamerPlaceholders", "Artist", nullptr));
        descStudio->setText(QCoreApplication::translate("RenamerPlaceholders", "Studio(s) (separated by a comma)", nullptr));
        descPartNo->setText(QCoreApplication::translate("RenamerPlaceholders", "Part number of the current file", nullptr));
        descSeason->setText(QCoreApplication::translate("RenamerPlaceholders", "Season Number", nullptr));
        descYear->setText(QCoreApplication::translate("RenamerPlaceholders", "Year", nullptr));
        descTmdbId->setText(QCoreApplication::translate("RenamerPlaceholders", "TMDb ID", nullptr));
        (void)RenamerPlaceholders;
    } // retranslateUi

};

namespace Ui {
    class RenamerPlaceholders: public Ui_RenamerPlaceholders {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RENAMERPLACEHOLDERS_H
