/****************************************************************************
** Meta object code from reading C++ file 'ArtistController.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/data/music/ArtistController.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ArtistController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ArtistControllerE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN16ArtistControllerE = QtMocHelpers::stringData(
    "ArtistController",
    "sigInfoLoadDone",
    "",
    "Artist*",
    "sigLoadingImages",
    "QSet<ImageType>",
    "sigLoadDone",
    "sigImage",
    "ImageType",
    "sigLoadImagesStarted",
    "sigDownloadProgress",
    "sigSaved",
    "onAllDownloadsFinished",
    "onDownloadFinished",
    "DownloadManagerElement",
    "elem",
    "onFanartLoadDone",
    "artist",
    "QMap<ImageType,QList<Poster>>",
    "posters"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN16ArtistControllerE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,   74,    2, 0x06,    1 /* Public */,
       4,    2,   77,    2, 0x06,    3 /* Public */,
       6,    1,   82,    2, 0x06,    6 /* Public */,
       7,    3,   85,    2, 0x06,    8 /* Public */,
       9,    1,   92,    2, 0x06,   12 /* Public */,
      10,    3,   95,    2, 0x06,   14 /* Public */,
      11,    1,  102,    2, 0x06,   18 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      12,    0,  105,    2, 0x08,   20 /* Private */,
      13,    1,  106,    2, 0x08,   21 /* Private */,
      16,    2,  109,    2, 0x08,   23 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 8, QMetaType::QByteArray,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 18,   17,   19,

       0        // eod
};

Q_CONSTINIT const QMetaObject ArtistController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ZN16ArtistControllerE.offsetsAndSizes,
    qt_meta_data_ZN16ArtistControllerE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN16ArtistControllerE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<ArtistController, std::true_type>,
        // method 'sigInfoLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        // method 'sigLoadingImages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QSet<ImageType>, std::false_type>,
        // method 'sigLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        // method 'sigImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'sigLoadImagesStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        // method 'sigDownloadProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'sigSaved'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        // method 'onAllDownloadsFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>,
        // method 'onFanartLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>
    >,
    nullptr
} };

void ArtistController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ArtistController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigInfoLoadDone((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1]))); break;
        case 1: _t->sigLoadingImages((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QSet<ImageType>>>(_a[2]))); break;
        case 2: _t->sigLoadDone((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1]))); break;
        case 3: _t->sigImage((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 4: _t->sigLoadImagesStarted((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1]))); break;
        case 5: _t->sigDownloadProgress((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->sigSaved((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1]))); break;
        case 7: _t->onAllDownloadsFinished(); break;
        case 8: _t->onDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        case 9: _t->onFanartLoadDone((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<ImageType> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (ArtistController::*)(Artist * );
            if (_q_method_type _q_method = &ArtistController::sigInfoLoadDone; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (ArtistController::*)(Artist * , QSet<ImageType> );
            if (_q_method_type _q_method = &ArtistController::sigLoadingImages; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (ArtistController::*)(Artist * );
            if (_q_method_type _q_method = &ArtistController::sigLoadDone; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (ArtistController::*)(Artist * , ImageType , QByteArray );
            if (_q_method_type _q_method = &ArtistController::sigImage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (ArtistController::*)(Artist * );
            if (_q_method_type _q_method = &ArtistController::sigLoadImagesStarted; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
        {
            using _q_method_type = void (ArtistController::*)(Artist * , int , int );
            if (_q_method_type _q_method = &ArtistController::sigDownloadProgress; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 5;
                return;
            }
        }
        {
            using _q_method_type = void (ArtistController::*)(Artist * );
            if (_q_method_type _q_method = &ArtistController::sigSaved; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 6;
                return;
            }
        }
    }
}

const QMetaObject *ArtistController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ArtistController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN16ArtistControllerE.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ArtistController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void ArtistController::sigInfoLoadDone(Artist * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ArtistController::sigLoadingImages(Artist * _t1, QSet<ImageType> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ArtistController::sigLoadDone(Artist * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ArtistController::sigImage(Artist * _t1, ImageType _t2, QByteArray _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ArtistController::sigLoadImagesStarted(Artist * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ArtistController::sigDownloadProgress(Artist * _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void ArtistController::sigSaved(Artist * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
