/****************************************************************************
** Meta object code from reading C++ file 'ConcertWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/concerts/ConcertWidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ConcertWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ConcertWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN13ConcertWidgetE = QtMocHelpers::stringData(
    "ConcertWidget",
    "setActionSearchEnabled",
    "",
    "MainWidgets",
    "setActionSaveEnabled",
    "clear",
    "setConcert",
    "Concert*",
    "concert",
    "onStartScraperSearch",
    "onSaveInformation",
    "onSaveAll",
    "onPlayConcert",
    "setEnabledTrue",
    "setDisabledTrue",
    "setBigWindow",
    "bigWindow",
    "updateConcertInfo",
    "onInfoChanged",
    "onInfoLoadDone",
    "onLoadDone",
    "onLoadImagesStarted",
    "onLoadingImages",
    "QSet<ImageType>",
    "imageTypes",
    "onDownloadProgress",
    "current",
    "maximum",
    "onSetImage",
    "ImageType",
    "type",
    "imageData",
    "onChooseImage",
    "onDeleteImage",
    "onImageDropped",
    "imageType",
    "imageUrl",
    "concertNameChanged",
    "text",
    "addGenre",
    "genre",
    "removeGenre",
    "addTag",
    "tag",
    "removeTag",
    "onRevertChanges",
    "onArtPageOne",
    "onArtPageTwo",
    "onRemoveExtraFanart",
    "file",
    "image",
    "onAddExtraFanart",
    "onExtraFanartDropped",
    "QList<QUrl>",
    "updateImage",
    "ClosableImage*"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN13ConcertWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,  230,    2, 0x06,    1 /* Public */,
       4,    2,  235,    2, 0x06,    4 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       5,    0,  240,    2, 0x0a,    7 /* Public */,
       6,    1,  241,    2, 0x0a,    8 /* Public */,
       9,    0,  244,    2, 0x0a,   10 /* Public */,
      10,    0,  245,    2, 0x0a,   11 /* Public */,
      11,    0,  246,    2, 0x0a,   12 /* Public */,
      12,    0,  247,    2, 0x0a,   13 /* Public */,
      13,    1,  248,    2, 0x0a,   14 /* Public */,
      13,    0,  251,    2, 0x2a,   16 /* Public | MethodCloned */,
      14,    0,  252,    2, 0x0a,   17 /* Public */,
      15,    1,  253,    2, 0x0a,   18 /* Public */,
      17,    0,  256,    2, 0x0a,   20 /* Public */,
      18,    0,  257,    2, 0x08,   21 /* Private */,
      19,    1,  258,    2, 0x08,   22 /* Private */,
      20,    1,  261,    2, 0x08,   24 /* Private */,
      21,    1,  264,    2, 0x08,   26 /* Private */,
      22,    2,  267,    2, 0x08,   28 /* Private */,
      25,    3,  272,    2, 0x08,   31 /* Private */,
      28,    3,  279,    2, 0x08,   35 /* Private */,
      32,    0,  286,    2, 0x08,   39 /* Private */,
      33,    0,  287,    2, 0x08,   40 /* Private */,
      34,    2,  288,    2, 0x08,   41 /* Private */,
      37,    1,  293,    2, 0x08,   44 /* Private */,
      39,    1,  296,    2, 0x08,   46 /* Private */,
      41,    1,  299,    2, 0x08,   48 /* Private */,
      42,    1,  302,    2, 0x08,   50 /* Private */,
      44,    1,  305,    2, 0x08,   52 /* Private */,
      45,    0,  308,    2, 0x08,   54 /* Private */,
      46,    0,  309,    2, 0x08,   55 /* Private */,
      47,    0,  310,    2, 0x08,   56 /* Private */,
      48,    1,  311,    2, 0x08,   57 /* Private */,
      48,    1,  314,    2, 0x08,   59 /* Private */,
      51,    0,  317,    2, 0x08,   61 /* Private */,
      52,    1,  318,    2, 0x08,   62 /* Private */,
      54,    2,  321,    2, 0x08,   64 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 23,    8,   24,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int, QMetaType::Int,    8,   26,   27,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 29, QMetaType::QByteArray,    8,   30,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 29, QMetaType::QUrl,   35,   36,
    QMetaType::Void, QMetaType::QString,   38,
    QMetaType::Void, QMetaType::QString,   40,
    QMetaType::Void, QMetaType::QString,   40,
    QMetaType::Void, QMetaType::QString,   43,
    QMetaType::Void, QMetaType::QString,   43,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Void, QMetaType::QByteArray,   50,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 53,   36,
    QMetaType::Void, 0x80000000 | 29, 0x80000000 | 55,   35,   50,

       0        // eod
};

Q_CONSTINIT const QMetaObject ConcertWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN13ConcertWidgetE.offsetsAndSizes,
    qt_meta_data_ZN13ConcertWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN13ConcertWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<ConcertWidget, std::true_type>,
        // method 'setActionSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'setActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'clear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setConcert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        // method 'onStartScraperSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveAll'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPlayConcert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setEnabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        // method 'setEnabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setDisabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setBigWindow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'updateConcertInfo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onInfoChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onInfoLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        // method 'onLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        // method 'onLoadImagesStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        // method 'onLoadingImages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QSet<ImageType>, std::false_type>,
        // method 'onDownloadProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onSetImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'onChooseImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDeleteImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImageDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QUrl, std::false_type>,
        // method 'concertNameChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'addGenre'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'removeGenre'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'addTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'removeTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRevertChanges'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtPageOne'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtPageTwo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRemoveExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'onAddExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onExtraFanartDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<QUrl>, std::false_type>,
        // method 'updateImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<ClosableImage *, std::false_type>
    >,
    nullptr
} };

void ConcertWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConcertWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setActionSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->setActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 2: _t->clear(); break;
        case 3: _t->setConcert((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1]))); break;
        case 4: _t->onStartScraperSearch(); break;
        case 5: _t->onSaveInformation(); break;
        case 6: _t->onSaveAll(); break;
        case 7: _t->onPlayConcert(); break;
        case 8: _t->setEnabledTrue((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1]))); break;
        case 9: _t->setEnabledTrue(); break;
        case 10: _t->setDisabledTrue(); break;
        case 11: _t->setBigWindow((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->updateConcertInfo(); break;
        case 13: _t->onInfoChanged(); break;
        case 14: _t->onInfoLoadDone((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1]))); break;
        case 15: _t->onLoadDone((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1]))); break;
        case 16: _t->onLoadImagesStarted((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1]))); break;
        case 17: _t->onLoadingImages((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QSet<ImageType>>>(_a[2]))); break;
        case 18: _t->onDownloadProgress((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 19: _t->onSetImage((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 20: _t->onChooseImage(); break;
        case 21: _t->onDeleteImage(); break;
        case 22: _t->onImageDropped((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 23: _t->concertNameChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->addGenre((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->removeGenre((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->addTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->removeTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->onRevertChanges(); break;
        case 29: _t->onArtPageOne(); break;
        case 30: _t->onArtPageTwo(); break;
        case 31: _t->onRemoveExtraFanart((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->onRemoveExtraFanart((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 33: _t->onAddExtraFanart(); break;
        case 34: _t->onExtraFanartDropped((*reinterpret_cast< std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 35: _t->updateImage((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<ClosableImage*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<ImageType> >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QUrl> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (ConcertWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &ConcertWidget::setActionSearchEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (ConcertWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &ConcertWidget::setActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *ConcertWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConcertWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN13ConcertWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ConcertWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void ConcertWidget::setActionSearchEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ConcertWidget::setActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
