/********************************************************************************
** Form generated from reading UI file 'MovieDuplicates.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIEDUPLICATES_H
#define UI_MOVIEDUPLICATES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyTableView.h"

QT_BEGIN_NAMESPACE

class Ui_MovieDuplicates
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *movieDuplicatesWidget;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnDetect;
    QSpacerItem *horizontalSpacer;
    QFrame *line;
    QSplitter *splitter;
    MyTableView *movies;
    QTableWidget *duplicates;

    void setupUi(QWidget *MovieDuplicates)
    {
        if (MovieDuplicates->objectName().isEmpty())
            MovieDuplicates->setObjectName("MovieDuplicates");
        MovieDuplicates->resize(937, 648);
        MovieDuplicates->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(MovieDuplicates);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        movieDuplicatesWidget = new QWidget(MovieDuplicates);
        movieDuplicatesWidget->setObjectName("movieDuplicatesWidget");
        verticalLayout_2 = new QVBoxLayout(movieDuplicatesWidget);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(12, 12, 12, 12);
        btnDetect = new QPushButton(movieDuplicatesWidget);
        btnDetect->setObjectName("btnDetect");

        horizontalLayout->addWidget(btnDetect);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        line = new QFrame(movieDuplicatesWidget);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Plain);
        line->setFrameShape(QFrame::Shape::HLine);

        verticalLayout_2->addWidget(line);

        splitter = new QSplitter(movieDuplicatesWidget);
        splitter->setObjectName("splitter");
        splitter->setFrameShape(QFrame::NoFrame);
        splitter->setLineWidth(1);
        splitter->setOrientation(Qt::Horizontal);
        splitter->setHandleWidth(1);
        movies = new MyTableView(splitter);
        movies->setObjectName("movies");
        movies->setMinimumSize(QSize(200, 0));
        movies->setContextMenuPolicy(Qt::CustomContextMenu);
        movies->setFrameShape(QFrame::NoFrame);
        movies->setEditTriggers(QAbstractItemView::NoEditTriggers);
        movies->setAlternatingRowColors(true);
        movies->setSelectionMode(QAbstractItemView::SingleSelection);
        movies->setSelectionBehavior(QAbstractItemView::SelectRows);
        movies->setShowGrid(false);
        movies->setSortingEnabled(true);
        splitter->addWidget(movies);
        movies->horizontalHeader()->setVisible(false);
        movies->verticalHeader()->setVisible(false);
        movies->verticalHeader()->setMinimumSectionSize(22);
        movies->verticalHeader()->setDefaultSectionSize(22);
        duplicates = new QTableWidget(splitter);
        if (duplicates->columnCount() < 1)
            duplicates->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        __qtablewidgetitem->setText(QString::fromUtf8("col"));
        duplicates->setHorizontalHeaderItem(0, __qtablewidgetitem);
        duplicates->setObjectName("duplicates");
        duplicates->setFrameShape(QFrame::NoFrame);
        duplicates->setEditTriggers(QAbstractItemView::NoEditTriggers);
        duplicates->setSelectionMode(QAbstractItemView::NoSelection);
        duplicates->setShowGrid(false);
        splitter->addWidget(duplicates);
        duplicates->horizontalHeader()->setVisible(false);
        duplicates->horizontalHeader()->setStretchLastSection(true);
        duplicates->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(splitter);

        verticalLayout_2->setStretch(2, 1);

        verticalLayout->addWidget(movieDuplicatesWidget);


        retranslateUi(MovieDuplicates);

        QMetaObject::connectSlotsByName(MovieDuplicates);
    } // setupUi

    void retranslateUi(QWidget *MovieDuplicates)
    {
        btnDetect->setText(QCoreApplication::translate("MovieDuplicates", "Detect duplicate movies", nullptr));
        (void)MovieDuplicates;
    } // retranslateUi

};

namespace Ui {
    class MovieDuplicates: public Ui_MovieDuplicates {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIEDUPLICATES_H
