/****************************************************************************
** Meta object code from reading C++ file 'MakeMkvDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/import/MakeMkvDialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MakeMkvDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MakeMkvDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN13MakeMkvDialogE = QtMocHelpers::stringData(
    "MakeMkvDialog",
    "exec",
    "",
    "reject",
    "accept",
    "onGotDrives",
    "QMap<int,QString>",
    "drives",
    "onScanDrive",
    "onScanFinished",
    "title",
    "QMap<int,MakeMkvCon::Track>",
    "tracks",
    "onImportTracks",
    "onImportComplete",
    "onMovieChosen",
    "onLoadDone",
    "Movie*",
    "movie",
    "onImport",
    "onDiscBackedUp",
    "onTrackImported",
    "trackId",
    "onImportProgress",
    "value",
    "max"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN13MakeMkvDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   98,    2, 0x0a,    1 /* Public */,
       3,    0,   99,    2, 0x0a,    2 /* Public */,
       4,    0,  100,    2, 0x0a,    3 /* Public */,
       5,    1,  101,    2, 0x08,    4 /* Private */,
       8,    0,  104,    2, 0x08,    6 /* Private */,
       9,    2,  105,    2, 0x08,    7 /* Private */,
      13,    0,  110,    2, 0x08,   10 /* Private */,
      14,    0,  111,    2, 0x08,   11 /* Private */,
      15,    0,  112,    2, 0x08,   12 /* Private */,
      16,    1,  113,    2, 0x08,   13 /* Private */,
      19,    0,  116,    2, 0x08,   15 /* Private */,
      20,    0,  117,    2, 0x08,   16 /* Private */,
      21,    1,  118,    2, 0x08,   17 /* Private */,
      23,    2,  121,    2, 0x08,   19 /* Private */,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 11,   10,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   24,   25,

       0        // eod
};

Q_CONSTINIT const QMetaObject MakeMkvDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN13MakeMkvDialogE.offsetsAndSizes,
    qt_meta_data_ZN13MakeMkvDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN13MakeMkvDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MakeMkvDialog, std::true_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'accept'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onGotDrives'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<int,QString>, std::false_type>,
        // method 'onScanDrive'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onScanFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<int,MakeMkvCon::Track>, std::false_type>,
        // method 'onImportTracks'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImportComplete'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onMovieChosen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onImport'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDiscBackedUp'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTrackImported'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onImportProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>
    >,
    nullptr
} };

void MakeMkvDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MakeMkvDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->reject(); break;
        case 2: _t->accept(); break;
        case 3: _t->onGotDrives((*reinterpret_cast< std::add_pointer_t<QMap<int,QString>>>(_a[1]))); break;
        case 4: _t->onScanDrive(); break;
        case 5: _t->onScanFinished((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<int,MakeMkvCon::Track>>>(_a[2]))); break;
        case 6: _t->onImportTracks(); break;
        case 7: _t->onImportComplete(); break;
        case 8: _t->onMovieChosen(); break;
        case 9: _t->onLoadDone((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 10: _t->onImport(); break;
        case 11: _t->onDiscBackedUp(); break;
        case 12: _t->onTrackImported((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->onImportProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MakeMkvDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MakeMkvDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN13MakeMkvDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int MakeMkvDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
