/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/main/MainWindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN10MainWindowE = QtMocHelpers::stringData(
    "MainWindow",
    "setNewMarks",
    "",
    "progressProgress",
    "current",
    "max",
    "id",
    "progressFinished",
    "progressStarted",
    "msg",
    "onMenu",
    "QToolButton*",
    "button",
    "onActionSearch",
    "onActionSave",
    "showAboutDialog",
    "onActionSaveAll",
    "onActionReload",
    "onActionXbmc",
    "onActionRename",
    "onFilterChanged",
    "QList<Filter*>",
    "filters",
    "text",
    "onSetSaveEnabled",
    "enabled",
    "MainWidgets",
    "widget",
    "onSetSearchEnabled",
    "moveSplitter",
    "pos",
    "index",
    "onTriggerReloadAll",
    "onKodiSyncFinished",
    "onFilesRenamed",
    "RenameType",
    "type",
    "onRenewModels",
    "onJumpToMovie",
    "Movie*",
    "movie",
    "updateTvShows",
    "onCommandBarOpen"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN10MainWindowE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  164,    2, 0x0a,    1 /* Public */,
       3,    3,  165,    2, 0x08,    2 /* Private */,
       7,    1,  172,    2, 0x08,    6 /* Private */,
       8,    2,  175,    2, 0x08,    8 /* Private */,
      10,    1,  180,    2, 0x08,   11 /* Private */,
      10,    0,  183,    2, 0x28,   13 /* Private | MethodCloned */,
      13,    0,  184,    2, 0x08,   14 /* Private */,
      14,    0,  185,    2, 0x08,   15 /* Private */,
      15,    0,  186,    2, 0x08,   16 /* Private */,
      16,    0,  187,    2, 0x08,   17 /* Private */,
      17,    0,  188,    2, 0x08,   18 /* Private */,
      18,    0,  189,    2, 0x08,   19 /* Private */,
      19,    0,  190,    2, 0x08,   20 /* Private */,
      20,    2,  191,    2, 0x08,   21 /* Private */,
      24,    2,  196,    2, 0x08,   24 /* Private */,
      28,    2,  201,    2, 0x08,   27 /* Private */,
      29,    2,  206,    2, 0x08,   30 /* Private */,
      32,    0,  211,    2, 0x08,   33 /* Private */,
      33,    0,  212,    2, 0x08,   34 /* Private */,
      34,    1,  213,    2, 0x08,   35 /* Private */,
      34,    0,  216,    2, 0x28,   37 /* Private | MethodCloned */,
      37,    0,  217,    2, 0x08,   38 /* Private */,
      38,    1,  218,    2, 0x08,   39 /* Private */,
      41,    0,  221,    2, 0x08,   41 /* Private */,
      42,    0,  222,    2, 0x08,   42 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    4,    5,    6,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    9,    6,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21, QMetaType::QString,   22,   23,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 26,   25,   27,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 26,   25,   27,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   30,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 35,   36,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 39,   40,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_ZN10MainWindowE.offsetsAndSizes,
    qt_meta_data_ZN10MainWindowE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN10MainWindowE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MainWindow, std::true_type>,
        // method 'setNewMarks'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'progressProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'progressFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'progressStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QToolButton *, std::false_type>,
        // method 'onMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActionSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActionSave'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'showAboutDialog'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActionSaveAll'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActionReload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActionXbmc'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActionRename'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onFilterChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<Filter*>, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onSetSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'onSetSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'moveSplitter'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onTriggerReloadAll'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onKodiSyncFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onFilesRenamed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<RenameType, std::false_type>,
        // method 'onFilesRenamed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRenewModels'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onJumpToMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'updateTvShows'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onCommandBarOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setNewMarks(); break;
        case 1: _t->progressProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->progressFinished((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->progressStarted((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->onMenu((*reinterpret_cast< std::add_pointer_t<QToolButton*>>(_a[1]))); break;
        case 5: _t->onMenu(); break;
        case 6: _t->onActionSearch(); break;
        case 7: _t->onActionSave(); break;
        case 8: _t->showAboutDialog(); break;
        case 9: _t->onActionSaveAll(); break;
        case 10: _t->onActionReload(); break;
        case 11: _t->onActionXbmc(); break;
        case 12: _t->onActionRename(); break;
        case 13: _t->onFilterChanged((*reinterpret_cast< std::add_pointer_t<QList<Filter*>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->onSetSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 15: _t->onSetSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 16: _t->moveSplitter((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->onTriggerReloadAll(); break;
        case 18: _t->onKodiSyncFinished(); break;
        case 19: _t->onFilesRenamed((*reinterpret_cast< std::add_pointer_t<RenameType>>(_a[1]))); break;
        case 20: _t->onFilesRenamed(); break;
        case 21: _t->onRenewModels(); break;
        case 22: _t->onJumpToMovie((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 23: _t->updateTvShows(); break;
        case 24: _t->onCommandBarOpen(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QToolButton* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Filter*> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN10MainWindowE.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
