/********************************************************************************
** Form generated from reading UI file 'ConcertInfoWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONCERTINFOWIDGET_H
#define UI_CONCERTINFOWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/Badge.h"
#include "ui/small_widgets/RatingsWidget.h"

QT_BEGIN_NAMESPACE

class Ui_ConcertInfoWidget
{
public:
    QFormLayout *formLayout;
    QLabel *lbl_files;
    QLineEdit *files;
    QLabel *lblIDs;
    QHBoxLayout *horizontalLayout_2;
    QFormLayout *formLayoutTmdb;
    QLabel *lblTmdbId;
    QLineEdit *tmdbId;
    QFormLayout *formLayoutImdb;
    QLineEdit *imdbId;
    QLabel *lblImdbId;
    QLabel *lbl_title;
    QLineEdit *title;
    QLabel *lblOriginalTitle;
    QLineEdit *originalTitle;
    QLabel *lbl_artist;
    QLineEdit *artist;
    QLabel *lbl_album;
    QLineEdit *album;
    QLabel *lbl_tagline;
    QLineEdit *tagline;
    QLabel *lbl_rating;
    QVBoxLayout *ratingLayout;
    QHBoxLayout *horizontalLayout_3;
    QLabel *lblUserRating;
    QDoubleSpinBox *userRating;
    QSpacerItem *horizontalSpacer;
    RatingsWidget *ratings;
    QLabel *lbl_released;
    QDateEdit *released;
    QLabel *lbl_runtime;
    QSpinBox *runtime;
    QLabel *lbl_certification;
    QComboBox *certification;
    QLabel *lbl_trailer;
    QLineEdit *trailer;
    QLabel *lbl_playCount;
    QHBoxLayout *idsLayout;
    QSpinBox *playcount;
    QSpacerItem *horizontalSpacer_4;
    QLabel *lbl_lastPlayed;
    QDateTimeEdit *lastPlayed;
    QSpacerItem *horizontalSpacer_5;
    Badge *badgeWatched;
    QLabel *lbl_overview;
    QPlainTextEdit *overview;

    void setupUi(QWidget *ConcertInfoWidget)
    {
        if (ConcertInfoWidget->objectName().isEmpty())
            ConcertInfoWidget->setObjectName("ConcertInfoWidget");
        ConcertInfoWidget->resize(715, 601);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ConcertInfoWidget->sizePolicy().hasHeightForWidth());
        ConcertInfoWidget->setSizePolicy(sizePolicy);
        formLayout = new QFormLayout(ConcertInfoWidget);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setContentsMargins(0, 0, 0, 0);
        lbl_files = new QLabel(ConcertInfoWidget);
        lbl_files->setObjectName("lbl_files");

        formLayout->setWidget(0, QFormLayout::LabelRole, lbl_files);

        files = new QLineEdit(ConcertInfoWidget);
        files->setObjectName("files");
        files->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, files);

        lblIDs = new QLabel(ConcertInfoWidget);
        lblIDs->setObjectName("lblIDs");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblIDs);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        formLayoutTmdb = new QFormLayout();
        formLayoutTmdb->setObjectName("formLayoutTmdb");
        lblTmdbId = new QLabel(ConcertInfoWidget);
        lblTmdbId->setObjectName("lblTmdbId");

        formLayoutTmdb->setWidget(0, QFormLayout::LabelRole, lblTmdbId);

        tmdbId = new QLineEdit(ConcertInfoWidget);
        tmdbId->setObjectName("tmdbId");
        tmdbId->setMaximumSize(QSize(150, 16777215));

        formLayoutTmdb->setWidget(0, QFormLayout::FieldRole, tmdbId);


        horizontalLayout_2->addLayout(formLayoutTmdb);

        formLayoutImdb = new QFormLayout();
        formLayoutImdb->setObjectName("formLayoutImdb");
        imdbId = new QLineEdit(ConcertInfoWidget);
        imdbId->setObjectName("imdbId");
        imdbId->setMaximumSize(QSize(150, 16777215));

        formLayoutImdb->setWidget(0, QFormLayout::FieldRole, imdbId);

        lblImdbId = new QLabel(ConcertInfoWidget);
        lblImdbId->setObjectName("lblImdbId");

        formLayoutImdb->setWidget(0, QFormLayout::LabelRole, lblImdbId);


        horizontalLayout_2->addLayout(formLayoutImdb);


        formLayout->setLayout(1, QFormLayout::FieldRole, horizontalLayout_2);

        lbl_title = new QLabel(ConcertInfoWidget);
        lbl_title->setObjectName("lbl_title");

        formLayout->setWidget(2, QFormLayout::LabelRole, lbl_title);

        title = new QLineEdit(ConcertInfoWidget);
        title->setObjectName("title");

        formLayout->setWidget(2, QFormLayout::FieldRole, title);

        lblOriginalTitle = new QLabel(ConcertInfoWidget);
        lblOriginalTitle->setObjectName("lblOriginalTitle");

        formLayout->setWidget(3, QFormLayout::LabelRole, lblOriginalTitle);

        originalTitle = new QLineEdit(ConcertInfoWidget);
        originalTitle->setObjectName("originalTitle");

        formLayout->setWidget(3, QFormLayout::FieldRole, originalTitle);

        lbl_artist = new QLabel(ConcertInfoWidget);
        lbl_artist->setObjectName("lbl_artist");

        formLayout->setWidget(4, QFormLayout::LabelRole, lbl_artist);

        artist = new QLineEdit(ConcertInfoWidget);
        artist->setObjectName("artist");

        formLayout->setWidget(4, QFormLayout::FieldRole, artist);

        lbl_album = new QLabel(ConcertInfoWidget);
        lbl_album->setObjectName("lbl_album");

        formLayout->setWidget(5, QFormLayout::LabelRole, lbl_album);

        album = new QLineEdit(ConcertInfoWidget);
        album->setObjectName("album");

        formLayout->setWidget(5, QFormLayout::FieldRole, album);

        lbl_tagline = new QLabel(ConcertInfoWidget);
        lbl_tagline->setObjectName("lbl_tagline");

        formLayout->setWidget(6, QFormLayout::LabelRole, lbl_tagline);

        tagline = new QLineEdit(ConcertInfoWidget);
        tagline->setObjectName("tagline");

        formLayout->setWidget(6, QFormLayout::FieldRole, tagline);

        lbl_rating = new QLabel(ConcertInfoWidget);
        lbl_rating->setObjectName("lbl_rating");

        formLayout->setWidget(7, QFormLayout::LabelRole, lbl_rating);

        ratingLayout = new QVBoxLayout();
        ratingLayout->setObjectName("ratingLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        lblUserRating = new QLabel(ConcertInfoWidget);
        lblUserRating->setObjectName("lblUserRating");

        horizontalLayout_3->addWidget(lblUserRating);

        userRating = new QDoubleSpinBox(ConcertInfoWidget);
        userRating->setObjectName("userRating");
        userRating->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        userRating->setDecimals(2);
        userRating->setMaximum(10.000000000000000);

        horizontalLayout_3->addWidget(userRating);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        ratingLayout->addLayout(horizontalLayout_3);

        ratings = new RatingsWidget(ConcertInfoWidget);
        ratings->setObjectName("ratings");

        ratingLayout->addWidget(ratings);


        formLayout->setLayout(7, QFormLayout::FieldRole, ratingLayout);

        lbl_released = new QLabel(ConcertInfoWidget);
        lbl_released->setObjectName("lbl_released");

        formLayout->setWidget(8, QFormLayout::LabelRole, lbl_released);

        released = new QDateEdit(ConcertInfoWidget);
        released->setObjectName("released");
        released->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        released->setDisplayFormat(QString::fromUtf8("yyyy"));

        formLayout->setWidget(8, QFormLayout::FieldRole, released);

        lbl_runtime = new QLabel(ConcertInfoWidget);
        lbl_runtime->setObjectName("lbl_runtime");

        formLayout->setWidget(9, QFormLayout::LabelRole, lbl_runtime);

        runtime = new QSpinBox(ConcertInfoWidget);
        runtime->setObjectName("runtime");
        runtime->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        runtime->setMaximum(999);

        formLayout->setWidget(9, QFormLayout::FieldRole, runtime);

        lbl_certification = new QLabel(ConcertInfoWidget);
        lbl_certification->setObjectName("lbl_certification");

        formLayout->setWidget(10, QFormLayout::LabelRole, lbl_certification);

        certification = new QComboBox(ConcertInfoWidget);
        certification->setObjectName("certification");
        certification->setEditable(true);
        certification->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);

        formLayout->setWidget(10, QFormLayout::FieldRole, certification);

        lbl_trailer = new QLabel(ConcertInfoWidget);
        lbl_trailer->setObjectName("lbl_trailer");

        formLayout->setWidget(11, QFormLayout::LabelRole, lbl_trailer);

        trailer = new QLineEdit(ConcertInfoWidget);
        trailer->setObjectName("trailer");

        formLayout->setWidget(11, QFormLayout::FieldRole, trailer);

        lbl_playCount = new QLabel(ConcertInfoWidget);
        lbl_playCount->setObjectName("lbl_playCount");

        formLayout->setWidget(12, QFormLayout::LabelRole, lbl_playCount);

        idsLayout = new QHBoxLayout();
        idsLayout->setObjectName("idsLayout");
        playcount = new QSpinBox(ConcertInfoWidget);
        playcount->setObjectName("playcount");
        playcount->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        playcount->setMaximum(99);

        idsLayout->addWidget(playcount);

        horizontalSpacer_4 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        idsLayout->addItem(horizontalSpacer_4);

        lbl_lastPlayed = new QLabel(ConcertInfoWidget);
        lbl_lastPlayed->setObjectName("lbl_lastPlayed");

        idsLayout->addWidget(lbl_lastPlayed);

        lastPlayed = new QDateTimeEdit(ConcertInfoWidget);
        lastPlayed->setObjectName("lastPlayed");
        lastPlayed->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        idsLayout->addWidget(lastPlayed);

        horizontalSpacer_5 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        idsLayout->addItem(horizontalSpacer_5);

        badgeWatched = new Badge(ConcertInfoWidget);
        badgeWatched->setObjectName("badgeWatched");

        idsLayout->addWidget(badgeWatched);


        formLayout->setLayout(12, QFormLayout::FieldRole, idsLayout);

        lbl_overview = new QLabel(ConcertInfoWidget);
        lbl_overview->setObjectName("lbl_overview");

        formLayout->setWidget(13, QFormLayout::LabelRole, lbl_overview);

        overview = new QPlainTextEdit(ConcertInfoWidget);
        overview->setObjectName("overview");

        formLayout->setWidget(13, QFormLayout::FieldRole, overview);


        retranslateUi(ConcertInfoWidget);

        QMetaObject::connectSlotsByName(ConcertInfoWidget);
    } // setupUi

    void retranslateUi(QWidget *ConcertInfoWidget)
    {
        lbl_files->setText(QCoreApplication::translate("ConcertInfoWidget", "Files", nullptr));
        lblIDs->setText(QCoreApplication::translate("ConcertInfoWidget", "IDs", nullptr));
        lblTmdbId->setText(QCoreApplication::translate("ConcertInfoWidget", "TMDb ID", nullptr));
        lblImdbId->setText(QCoreApplication::translate("ConcertInfoWidget", "IMDb ID", nullptr));
        lbl_title->setText(QCoreApplication::translate("ConcertInfoWidget", "Title", nullptr));
        title->setText(QString());
        lblOriginalTitle->setText(QCoreApplication::translate("ConcertInfoWidget", "Original Title", nullptr));
        lbl_artist->setText(QCoreApplication::translate("ConcertInfoWidget", "Artist", nullptr));
        lbl_album->setText(QCoreApplication::translate("ConcertInfoWidget", "Album", nullptr));
        lbl_tagline->setText(QCoreApplication::translate("ConcertInfoWidget", "Tagline", nullptr));
        lbl_rating->setText(QCoreApplication::translate("ConcertInfoWidget", "Rating", nullptr));
        lblUserRating->setText(QCoreApplication::translate("ConcertInfoWidget", "User Rating", nullptr));
        lbl_released->setText(QCoreApplication::translate("ConcertInfoWidget", "Released", nullptr));
        lbl_runtime->setText(QCoreApplication::translate("ConcertInfoWidget", "Runtime", nullptr));
        runtime->setSuffix(QCoreApplication::translate("ConcertInfoWidget", " Minutes", nullptr));
        lbl_certification->setText(QCoreApplication::translate("ConcertInfoWidget", "Certification", nullptr));
        lbl_trailer->setText(QCoreApplication::translate("ConcertInfoWidget", "Trailer", nullptr));
        lbl_playCount->setText(QCoreApplication::translate("ConcertInfoWidget", "Play Count", nullptr));
        lbl_lastPlayed->setText(QCoreApplication::translate("ConcertInfoWidget", "Last Played", nullptr));
        badgeWatched->setText(QCoreApplication::translate("ConcertInfoWidget", "Watched", nullptr));
        badgeWatched->setProperty("activeText", QVariant(QCoreApplication::translate("ConcertInfoWidget", "Watched", nullptr)));
        badgeWatched->setProperty("inactiveText", QVariant(QCoreApplication::translate("ConcertInfoWidget", "Not watched", nullptr)));
        lbl_overview->setText(QCoreApplication::translate("ConcertInfoWidget", "Overview", nullptr));
        (void)ConcertInfoWidget;
    } // retranslateUi

};

namespace Ui {
    class ConcertInfoWidget: public Ui_ConcertInfoWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONCERTINFOWIDGET_H
