/****************************************************************************
** Meta object code from reading C++ file 'TvShowFilesWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowFilesWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowFilesWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17TvShowFilesWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN17TvShowFilesWidgetE = QtMocHelpers::stringData(
    "TvShowFilesWidget",
    "sigEpisodeSelected",
    "",
    "TvShowEpisode*",
    "episode",
    "sigTvShowSelected",
    "TvShow*",
    "show",
    "sigSeasonSelected",
    "SeasonNumber",
    "season",
    "sigNothingSelected",
    "sigStartSearch",
    "renewModel",
    "force",
    "emitLastSelection",
    "multiScrape",
    "onItemSelected",
    "QModelIndex",
    "current",
    "previous",
    "showContextMenu",
    "point",
    "scanForEpisodes",
    "markAsWatched",
    "markAsUnwatched",
    "loadStreamDetails",
    "markForSyncBool",
    "markForSync",
    "unmarkForSync",
    "openFolder",
    "openNfo",
    "showMissingEpisodes",
    "hideSpecialsInMissingEpisodes",
    "updateStatusLabel",
    "playEpisode",
    "idx"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN17TvShowFilesWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,  158,    2, 0x06,    1 /* Public */,
       5,    1,  161,    2, 0x06,    3 /* Public */,
       8,    2,  164,    2, 0x06,    5 /* Public */,
      11,    0,  169,    2, 0x06,    8 /* Public */,
      12,    0,  170,    2, 0x06,    9 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      13,    1,  171,    2, 0x0a,   10 /* Public */,
      13,    0,  174,    2, 0x2a,   12 /* Public | MethodCloned */,
      15,    0,  175,    2, 0x0a,   13 /* Public */,
      16,    0,  176,    2, 0x0a,   14 /* Public */,
      17,    2,  177,    2, 0x08,   15 /* Private */,
      21,    1,  182,    2, 0x08,   18 /* Private */,
      23,    0,  185,    2, 0x08,   20 /* Private */,
      24,    0,  186,    2, 0x08,   21 /* Private */,
      25,    0,  187,    2, 0x08,   22 /* Private */,
      26,    0,  188,    2, 0x08,   23 /* Private */,
      27,    1,  189,    2, 0x08,   24 /* Private */,
      28,    0,  192,    2, 0x08,   26 /* Private */,
      29,    0,  193,    2, 0x08,   27 /* Private */,
      30,    0,  194,    2, 0x08,   28 /* Private */,
      31,    0,  195,    2, 0x08,   29 /* Private */,
      32,    0,  196,    2, 0x08,   30 /* Private */,
      33,    0,  197,    2, 0x08,   31 /* Private */,
      34,    0,  198,    2, 0x08,   32 /* Private */,
      35,    1,  199,    2, 0x08,   33 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 9,    7,   10,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18, 0x80000000 | 18,   19,   20,
    QMetaType::Void, QMetaType::QPoint,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   28,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,   36,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowFilesWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN17TvShowFilesWidgetE.offsetsAndSizes,
    qt_meta_data_ZN17TvShowFilesWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN17TvShowFilesWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowFilesWidget, std::true_type>,
        // method 'sigEpisodeSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShowEpisode *, std::false_type>,
        // method 'sigTvShowSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        // method 'sigSeasonSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<SeasonNumber, std::false_type>,
        // method 'sigNothingSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'sigStartSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'renewModel'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'renewModel'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'emitLastSelection'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'multiScrape'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onItemSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QModelIndex &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QModelIndex &, std::false_type>,
        // method 'showContextMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'scanForEpisodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'markAsWatched'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'markAsUnwatched'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'loadStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'markForSyncBool'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'markForSync'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'unmarkForSync'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'openFolder'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'openNfo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'showMissingEpisodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'hideSpecialsInMissingEpisodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateStatusLabel'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'playEpisode'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QModelIndex, std::false_type>
    >,
    nullptr
} };

void TvShowFilesWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowFilesWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigEpisodeSelected((*reinterpret_cast< std::add_pointer_t<TvShowEpisode*>>(_a[1]))); break;
        case 1: _t->sigTvShowSelected((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1]))); break;
        case 2: _t->sigSeasonSelected((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<SeasonNumber>>(_a[2]))); break;
        case 3: _t->sigNothingSelected(); break;
        case 4: _t->sigStartSearch(); break;
        case 5: _t->renewModel((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->renewModel(); break;
        case 7: _t->emitLastSelection(); break;
        case 8: _t->multiScrape(); break;
        case 9: _t->onItemSelected((*reinterpret_cast< std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 10: _t->showContextMenu((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 11: _t->scanForEpisodes(); break;
        case 12: _t->markAsWatched(); break;
        case 13: _t->markAsUnwatched(); break;
        case 14: _t->loadStreamDetails(); break;
        case 15: _t->markForSyncBool((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->markForSync(); break;
        case 17: _t->unmarkForSync(); break;
        case 18: _t->openFolder(); break;
        case 19: _t->openNfo(); break;
        case 20: _t->showMissingEpisodes(); break;
        case 21: _t->hideSpecialsInMissingEpisodes(); break;
        case 22: _t->updateStatusLabel(); break;
        case 23: _t->playEpisode((*reinterpret_cast< std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShowEpisode* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (TvShowFilesWidget::*)(TvShowEpisode * );
            if (_q_method_type _q_method = &TvShowFilesWidget::sigEpisodeSelected; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowFilesWidget::*)(TvShow * );
            if (_q_method_type _q_method = &TvShowFilesWidget::sigTvShowSelected; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowFilesWidget::*)(TvShow * , SeasonNumber );
            if (_q_method_type _q_method = &TvShowFilesWidget::sigSeasonSelected; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowFilesWidget::*)();
            if (_q_method_type _q_method = &TvShowFilesWidget::sigNothingSelected; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowFilesWidget::*)();
            if (_q_method_type _q_method = &TvShowFilesWidget::sigStartSearch; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
    }
}

const QMetaObject *TvShowFilesWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowFilesWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN17TvShowFilesWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TvShowFilesWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void TvShowFilesWidget::sigEpisodeSelected(TvShowEpisode * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TvShowFilesWidget::sigTvShowSelected(TvShow * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TvShowFilesWidget::sigSeasonSelected(TvShow * _t1, SeasonNumber _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TvShowFilesWidget::sigNothingSelected()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TvShowFilesWidget::sigStartSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
