/********************************************************************************
** Form generated from reading UI file 'ImageDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEDIALOG_H
#define UI_IMAGEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/ImageGallery.h"
#include "ui/small_widgets/LanguageCombo.h"
#include "ui/small_widgets/MyLineEdit.h"
#include "ui/small_widgets/MyTableWidget.h"

QT_BEGIN_NAMESPACE

class Ui_ImageDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QComboBox *imageProvider;
    MyLineEdit *searchTerm;
    LanguageCombo *comboLanguage;
    QLabel *imageSource;
    QLabel *lblErrorMessage;
    QLabel *lblSuccessMessage;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *results;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_2;
    MyTableWidget *table;
    QWidget *page_3;
    QVBoxLayout *verticalLayout_4;
    QLabel *noResultsLabel;
    ImageGallery *gallery;
    QHBoxLayout *horizontalLayout;
    QToolButton *buttonZoomOut;
    QSlider *previewSizeSlider;
    QToolButton *buttonZoomIn;
    QLabel *labelSpinner;
    QLabel *labelLoading;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonChoose;
    QPushButton *buttonClose;
    QPushButton *btnAcceptImages;

    void setupUi(QDialog *ImageDialog)
    {
        if (ImageDialog->objectName().isEmpty())
            ImageDialog->setObjectName("ImageDialog");
        ImageDialog->setWindowModality(Qt::WindowModal);
        ImageDialog->resize(702, 600);
        ImageDialog->setModal(true);
        verticalLayout = new QVBoxLayout(ImageDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        imageProvider = new QComboBox(ImageDialog);
        imageProvider->setObjectName("imageProvider");

        horizontalLayout_2->addWidget(imageProvider);

        searchTerm = new MyLineEdit(ImageDialog);
        searchTerm->setObjectName("searchTerm");

        horizontalLayout_2->addWidget(searchTerm);

        comboLanguage = new LanguageCombo(ImageDialog);
        comboLanguage->setObjectName("comboLanguage");
        comboLanguage->setMinimumSize(QSize(120, 0));

        horizontalLayout_2->addWidget(comboLanguage);


        verticalLayout->addLayout(horizontalLayout_2);

        imageSource = new QLabel(ImageDialog);
        imageSource->setObjectName("imageSource");
        imageSource->setText(QString::fromUtf8("Image source"));
        imageSource->setOpenExternalLinks(true);

        verticalLayout->addWidget(imageSource);

        lblErrorMessage = new QLabel(ImageDialog);
        lblErrorMessage->setObjectName("lblErrorMessage");
        lblErrorMessage->setText(QString::fromUtf8("error message here"));

        verticalLayout->addWidget(lblErrorMessage);

        lblSuccessMessage = new QLabel(ImageDialog);
        lblSuccessMessage->setObjectName("lblSuccessMessage");
        lblSuccessMessage->setText(QString::fromUtf8("success message here"));

        verticalLayout->addWidget(lblSuccessMessage);

        stackedWidget = new QStackedWidget(ImageDialog);
        stackedWidget->setObjectName("stackedWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(stackedWidget->sizePolicy().hasHeightForWidth());
        stackedWidget->setSizePolicy(sizePolicy);
        stackedWidget->setAutoFillBackground(true);
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_3 = new QVBoxLayout(page);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        results = new QTableWidget(page);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(results);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_2 = new QVBoxLayout(page_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        table = new MyTableWidget(page_2);
        if (table->columnCount() < 5)
            table->setColumnCount(5);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(0, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(1, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(2, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(3, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(4, __qtablewidgetitem5);
        table->setObjectName("table");
        table->setMinimumSize(QSize(400, 0));
        table->setAcceptDrops(true);
        table->setEditTriggers(QAbstractItemView::NoEditTriggers);
        table->setSelectionMode(QAbstractItemView::SingleSelection);
        table->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        table->setShowGrid(false);
        table->horizontalHeader()->setVisible(false);
        table->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(table);

        stackedWidget->addWidget(page_2);
        page_3 = new QWidget();
        page_3->setObjectName("page_3");
        verticalLayout_4 = new QVBoxLayout(page_3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        noResultsLabel = new QLabel(page_3);
        noResultsLabel->setObjectName("noResultsLabel");
        noResultsLabel->setText(QString::fromUtf8("No images found"));
        noResultsLabel->setTextFormat(Qt::RichText);
        noResultsLabel->setAlignment(Qt::AlignCenter);
        noResultsLabel->setOpenExternalLinks(true);

        verticalLayout_4->addWidget(noResultsLabel);

        stackedWidget->addWidget(page_3);

        verticalLayout->addWidget(stackedWidget);

        gallery = new ImageGallery(ImageDialog);
        gallery->setObjectName("gallery");
        gallery->setMinimumSize(QSize(0, 100));
        gallery->setMaximumSize(QSize(16777215, 100));

        verticalLayout->addWidget(gallery);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        buttonZoomOut = new QToolButton(ImageDialog);
        buttonZoomOut->setObjectName("buttonZoomOut");
        buttonZoomOut->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/zoom_out.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonZoomOut->setIcon(icon);

        horizontalLayout->addWidget(buttonZoomOut);

        previewSizeSlider = new QSlider(ImageDialog);
        previewSizeSlider->setObjectName("previewSizeSlider");
        previewSizeSlider->setMinimumSize(QSize(100, 0));
        previewSizeSlider->setMaximumSize(QSize(100, 16777215));
        previewSizeSlider->setMinimum(4);
        previewSizeSlider->setMaximum(20);
        previewSizeSlider->setSingleStep(1);
        previewSizeSlider->setPageStep(1);
        previewSizeSlider->setValue(8);
        previewSizeSlider->setOrientation(Qt::Horizontal);

        horizontalLayout->addWidget(previewSizeSlider);

        buttonZoomIn = new QToolButton(ImageDialog);
        buttonZoomIn->setObjectName("buttonZoomIn");
        buttonZoomIn->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/zoom_in.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonZoomIn->setIcon(icon1);

        horizontalLayout->addWidget(buttonZoomIn);

        labelSpinner = new QLabel(ImageDialog);
        labelSpinner->setObjectName("labelSpinner");

        horizontalLayout->addWidget(labelSpinner);

        labelLoading = new QLabel(ImageDialog);
        labelLoading->setObjectName("labelLoading");

        horizontalLayout->addWidget(labelLoading);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonChoose = new QPushButton(ImageDialog);
        buttonChoose->setObjectName("buttonChoose");
        buttonChoose->setFocusPolicy(Qt::NoFocus);
        buttonChoose->setAutoDefault(false);

        horizontalLayout->addWidget(buttonChoose);

        buttonClose = new QPushButton(ImageDialog);
        buttonClose->setObjectName("buttonClose");
        buttonClose->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(buttonClose);

        btnAcceptImages = new QPushButton(ImageDialog);
        btnAcceptImages->setObjectName("btnAcceptImages");
        btnAcceptImages->setAutoDefault(false);

        horizontalLayout->addWidget(btnAcceptImages);

        horizontalLayout->setStretch(4, 1);

        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(4, 1);

        retranslateUi(ImageDialog);

        stackedWidget->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(ImageDialog);
    } // setupUi

    void retranslateUi(QDialog *ImageDialog)
    {
        ImageDialog->setWindowTitle(QCoreApplication::translate("ImageDialog", "Choose an Image", nullptr));
#if QT_CONFIG(tooltip)
        searchTerm->setToolTip(QCoreApplication::translate("ImageDialog", "Enter a search term or URL", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        comboLanguage->setToolTip(QCoreApplication::translate("ImageDialog", "Language to use when searching for a TV show by title.", nullptr));
#endif // QT_CONFIG(tooltip)
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("ImageDialog", "Movie", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = table->horizontalHeaderItem(0);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("ImageDialog", "Neue Spalte", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = table->horizontalHeaderItem(1);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("ImageDialog", "Neue Spalte", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = table->horizontalHeaderItem(2);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("ImageDialog", "Neue Spalte", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = table->horizontalHeaderItem(3);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("ImageDialog", "Neue Spalte", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = table->horizontalHeaderItem(4);
        ___qtablewidgetitem5->setText(QCoreApplication::translate("ImageDialog", "Neue Spalte", nullptr));
#if QT_CONFIG(tooltip)
        buttonZoomOut->setToolTip(QCoreApplication::translate("ImageDialog", "Zoom out", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonZoomOut->setText(QString());
#if QT_CONFIG(tooltip)
        previewSizeSlider->setToolTip(QCoreApplication::translate("ImageDialog", "Preview size", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonZoomIn->setToolTip(QCoreApplication::translate("ImageDialog", "Zoom in", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonZoomIn->setText(QString());
        labelSpinner->setText(QString());
        labelLoading->setText(QCoreApplication::translate("ImageDialog", "Loading...", nullptr));
        buttonChoose->setText(QCoreApplication::translate("ImageDialog", "Choose Local Image", nullptr));
        buttonClose->setText(QCoreApplication::translate("ImageDialog", "Close", nullptr));
        btnAcceptImages->setText(QCoreApplication::translate("ImageDialog", "Accept Images", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImageDialog: public Ui_ImageDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGEDIALOG_H
