/****************************************************************************
** Meta object code from reading C++ file 'CertificationWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/movies/CertificationWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CertificationWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19CertificationWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN19CertificationWidgetE = QtMocHelpers::stringData(
    "CertificationWidget",
    "setActionSaveEnabled",
    "",
    "MainWidgets",
    "sigJumpToMovie",
    "Movie*",
    "onSaveInformation",
    "loadCertifications",
    "splitter",
    "QSplitter*",
    "addCertification",
    "deleteCertification",
    "onCertificationNameChanged",
    "QTableWidgetItem*",
    "item",
    "onCertificationSelected",
    "addMovie",
    "removeMovie",
    "showCertificationsContextMenu",
    "point",
    "onJumpToMovie"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN19CertificationWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,   92,    2, 0x06,    1 /* Public */,
       4,    1,   97,    2, 0x06,    4 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       6,    0,  100,    2, 0x0a,    6 /* Public */,
       7,    0,  101,    2, 0x0a,    7 /* Public */,
       8,    0,  102,    2, 0x0a,    8 /* Public */,
      10,    0,  103,    2, 0x08,    9 /* Private */,
      11,    0,  104,    2, 0x08,   10 /* Private */,
      12,    1,  105,    2, 0x08,   11 /* Private */,
      15,    0,  108,    2, 0x08,   13 /* Private */,
      16,    0,  109,    2, 0x08,   14 /* Private */,
      17,    0,  110,    2, 0x08,   15 /* Private */,
      18,    1,  111,    2, 0x08,   16 /* Private */,
      20,    1,  114,    2, 0x08,   18 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, 0x80000000 | 5,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   19,
    QMetaType::Void, 0x80000000 | 13,   14,

       0        // eod
};

Q_CONSTINIT const QMetaObject CertificationWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN19CertificationWidgetE.offsetsAndSizes,
    qt_meta_data_ZN19CertificationWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN19CertificationWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<CertificationWidget, std::true_type>,
        // method 'setActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigJumpToMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onSaveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'loadCertifications'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'splitter'
        QtPrivate::TypeAndForceComplete<QSplitter *, std::false_type>,
        // method 'addCertification'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'deleteCertification'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onCertificationNameChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onCertificationSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'addMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'removeMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'showCertificationsContextMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'onJumpToMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>
    >,
    nullptr
} };

void CertificationWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CertificationWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->sigJumpToMovie((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 2: _t->onSaveInformation(); break;
        case 3: _t->loadCertifications(); break;
        case 4: { QSplitter* _r = _t->splitter();
            if (_a[0]) *reinterpret_cast< QSplitter**>(_a[0]) = std::move(_r); }  break;
        case 5: _t->addCertification(); break;
        case 6: _t->deleteCertification(); break;
        case 7: _t->onCertificationNameChanged((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 8: _t->onCertificationSelected(); break;
        case 9: _t->addMovie(); break;
        case 10: _t->removeMovie(); break;
        case 11: _t->showCertificationsContextMenu((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 12: _t->onJumpToMovie((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (CertificationWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &CertificationWidget::setActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (CertificationWidget::*)(Movie * );
            if (_q_method_type _q_method = &CertificationWidget::sigJumpToMovie; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *CertificationWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CertificationWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN19CertificationWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int CertificationWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void CertificationWidget::setActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CertificationWidget::sigJumpToMovie(Movie * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
