/****************************************************************************
** Meta object code from reading C++ file 'FileScannerDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/main/FileScannerDialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FileScannerDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17FileScannerDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN17FileScannerDialogE = QtMocHelpers::stringData(
    "FileScannerDialog",
    "exec",
    "",
    "reject",
    "onProgress",
    "current",
    "max",
    "onProgressPercent",
    "percent",
    "onCurrentDir",
    "dir",
    "onStartMovieScanner",
    "onStartMovieScannerForce",
    "onStartMovieScannerCache",
    "onStartTvShowScanner",
    "onStartTvShowScannerForce",
    "onStartTvShowScannerCache",
    "onStartConcertScanner",
    "onStartConcertScannerForce",
    "onStartConcertScannerCache",
    "onStartEpisodeScanner",
    "onStartMusicScanner",
    "onStartMusicScannerForce",
    "onStartMusicScannerCache"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN17FileScannerDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  122,    2, 0x0a,    1 /* Public */,
       3,    0,  123,    2, 0x0a,    2 /* Public */,
       4,    2,  124,    2, 0x08,    3 /* Private */,
       7,    1,  129,    2, 0x08,    6 /* Private */,
       9,    1,  132,    2, 0x08,    8 /* Private */,
      11,    0,  135,    2, 0x08,   10 /* Private */,
      12,    0,  136,    2, 0x08,   11 /* Private */,
      13,    0,  137,    2, 0x08,   12 /* Private */,
      14,    0,  138,    2, 0x08,   13 /* Private */,
      15,    0,  139,    2, 0x08,   14 /* Private */,
      16,    0,  140,    2, 0x08,   15 /* Private */,
      17,    0,  141,    2, 0x08,   16 /* Private */,
      18,    0,  142,    2, 0x08,   17 /* Private */,
      19,    0,  143,    2, 0x08,   18 /* Private */,
      20,    0,  144,    2, 0x08,   19 /* Private */,
      21,    0,  145,    2, 0x08,   20 /* Private */,
      22,    0,  146,    2, 0x08,   21 /* Private */,
      23,    0,  147,    2, 0x08,   22 /* Private */,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    5,    6,
    QMetaType::Void, QMetaType::Float,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject FileScannerDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN17FileScannerDialogE.offsetsAndSizes,
    qt_meta_data_ZN17FileScannerDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN17FileScannerDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FileScannerDialog, std::true_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onProgressPercent'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<float, std::false_type>,
        // method 'onCurrentDir'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onStartMovieScanner'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartMovieScannerForce'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartMovieScannerCache'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartTvShowScanner'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartTvShowScannerForce'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartTvShowScannerCache'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartConcertScanner'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartConcertScannerForce'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartConcertScannerCache'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartEpisodeScanner'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartMusicScanner'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartMusicScannerForce'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartMusicScannerCache'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void FileScannerDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileScannerDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->reject(); break;
        case 2: _t->onProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->onProgressPercent((*reinterpret_cast< std::add_pointer_t<float>>(_a[1]))); break;
        case 4: _t->onCurrentDir((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->onStartMovieScanner(); break;
        case 6: _t->onStartMovieScannerForce(); break;
        case 7: _t->onStartMovieScannerCache(); break;
        case 8: _t->onStartTvShowScanner(); break;
        case 9: _t->onStartTvShowScannerForce(); break;
        case 10: _t->onStartTvShowScannerCache(); break;
        case 11: _t->onStartConcertScanner(); break;
        case 12: _t->onStartConcertScannerForce(); break;
        case 13: _t->onStartConcertScannerCache(); break;
        case 14: _t->onStartEpisodeScanner(); break;
        case 15: _t->onStartMusicScanner(); break;
        case 16: _t->onStartMusicScannerForce(); break;
        case 17: _t->onStartMusicScannerCache(); break;
        default: ;
        }
    }
}

const QMetaObject *FileScannerDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileScannerDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN17FileScannerDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int FileScannerDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
