/********************************************************************************
** Form generated from reading UI file 'NetworkSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NETWORKSETTINGSWIDGET_H
#define UI_NETWORKSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_NetworkSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QFormLayout *formLayout_2;
    QLabel *lblEnableProxy;
    QCheckBox *chkUseProxy;
    QLabel *lblUseProxyForKodi;
    QCheckBox *chkUseProxyForKodi;
    QComboBox *proxyType;
    QLabel *lblType;
    QLabel *lblHost;
    QLabel *lblPort;
    QLineEdit *proxyHost;
    QSpinBox *proxyPort;
    QLabel *lblUsername;
    QLineEdit *proxyUsername;
    QLabel *lblPassword;
    QLineEdit *proxyPassword;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *NetworkSettingsWidget)
    {
        if (NetworkSettingsWidget->objectName().isEmpty())
            NetworkSettingsWidget->setObjectName("NetworkSettingsWidget");
        NetworkSettingsWidget->resize(970, 625);
        verticalLayout_3 = new QVBoxLayout(NetworkSettingsWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        lblEnableProxy = new QLabel(NetworkSettingsWidget);
        lblEnableProxy->setObjectName("lblEnableProxy");

        formLayout_2->setWidget(0, QFormLayout::LabelRole, lblEnableProxy);

        chkUseProxy = new QCheckBox(NetworkSettingsWidget);
        chkUseProxy->setObjectName("chkUseProxy");
        chkUseProxy->setChecked(false);

        formLayout_2->setWidget(0, QFormLayout::FieldRole, chkUseProxy);

        lblUseProxyForKodi = new QLabel(NetworkSettingsWidget);
        lblUseProxyForKodi->setObjectName("lblUseProxyForKodi");

        formLayout_2->setWidget(1, QFormLayout::LabelRole, lblUseProxyForKodi);

        chkUseProxyForKodi = new QCheckBox(NetworkSettingsWidget);
        chkUseProxyForKodi->setObjectName("chkUseProxyForKodi");

        formLayout_2->setWidget(1, QFormLayout::FieldRole, chkUseProxyForKodi);

        proxyType = new QComboBox(NetworkSettingsWidget);
        proxyType->addItem(QString());
        proxyType->addItem(QString::fromUtf8("SOCKS5"));
        proxyType->setObjectName("proxyType");

        formLayout_2->setWidget(2, QFormLayout::FieldRole, proxyType);

        lblType = new QLabel(NetworkSettingsWidget);
        lblType->setObjectName("lblType");

        formLayout_2->setWidget(2, QFormLayout::LabelRole, lblType);

        lblHost = new QLabel(NetworkSettingsWidget);
        lblHost->setObjectName("lblHost");

        formLayout_2->setWidget(3, QFormLayout::LabelRole, lblHost);

        lblPort = new QLabel(NetworkSettingsWidget);
        lblPort->setObjectName("lblPort");

        formLayout_2->setWidget(4, QFormLayout::LabelRole, lblPort);

        proxyHost = new QLineEdit(NetworkSettingsWidget);
        proxyHost->setObjectName("proxyHost");

        formLayout_2->setWidget(3, QFormLayout::FieldRole, proxyHost);

        proxyPort = new QSpinBox(NetworkSettingsWidget);
        proxyPort->setObjectName("proxyPort");
        proxyPort->setButtonSymbols(QAbstractSpinBox::NoButtons);
        proxyPort->setMaximum(99999);

        formLayout_2->setWidget(4, QFormLayout::FieldRole, proxyPort);

        lblUsername = new QLabel(NetworkSettingsWidget);
        lblUsername->setObjectName("lblUsername");

        formLayout_2->setWidget(5, QFormLayout::LabelRole, lblUsername);

        proxyUsername = new QLineEdit(NetworkSettingsWidget);
        proxyUsername->setObjectName("proxyUsername");

        formLayout_2->setWidget(5, QFormLayout::FieldRole, proxyUsername);

        lblPassword = new QLabel(NetworkSettingsWidget);
        lblPassword->setObjectName("lblPassword");

        formLayout_2->setWidget(6, QFormLayout::LabelRole, lblPassword);

        proxyPassword = new QLineEdit(NetworkSettingsWidget);
        proxyPassword->setObjectName("proxyPassword");
        proxyPassword->setEchoMode(QLineEdit::Password);

        formLayout_2->setWidget(6, QFormLayout::FieldRole, proxyPassword);


        verticalLayout_3->addLayout(formLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        retranslateUi(NetworkSettingsWidget);

        QMetaObject::connectSlotsByName(NetworkSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *NetworkSettingsWidget)
    {
        lblEnableProxy->setText(QCoreApplication::translate("NetworkSettingsWidget", "Enable Proxy", nullptr));
        chkUseProxy->setText(QString());
        lblUseProxyForKodi->setText(QCoreApplication::translate("NetworkSettingsWidget", "Use Proxy for Kodi", nullptr));
        chkUseProxyForKodi->setText(QString());
        proxyType->setItemText(0, QCoreApplication::translate("NetworkSettingsWidget", "HTTP", nullptr));

        lblType->setText(QCoreApplication::translate("NetworkSettingsWidget", "Type", "Proxy Type"));
        lblHost->setText(QCoreApplication::translate("NetworkSettingsWidget", "Host", nullptr));
        lblPort->setText(QCoreApplication::translate("NetworkSettingsWidget", "Port", nullptr));
        lblUsername->setText(QCoreApplication::translate("NetworkSettingsWidget", "Username", nullptr));
        lblPassword->setText(QCoreApplication::translate("NetworkSettingsWidget", "Password", nullptr));
        (void)NetworkSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class NetworkSettingsWidget: public Ui_NetworkSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NETWORKSETTINGSWIDGET_H
