/****************************************************************************
** Meta object code from reading C++ file 'TvShowWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12TvShowWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN12TvShowWidgetE = QtMocHelpers::stringData(
    "TvShowWidget",
    "sigSetActionSearchEnabled",
    "",
    "MainWidgets",
    "sigSetActionSaveEnabled",
    "sigDownloadsStarted",
    "sigDownloadsProgress",
    "sigDownloadsFinished",
    "onTvShowSelected",
    "TvShow*",
    "show",
    "onSeasonSelected",
    "SeasonNumber",
    "season",
    "onEpisodeSelected",
    "TvShowEpisode*",
    "episode",
    "onTvShowSetEnabledTrue",
    "onEpisodeSetEnabledTrue",
    "onSetDisabledTrue",
    "onClear",
    "onSaveInformation",
    "onSaveAll",
    "onStartScraperSearch",
    "setBigWindow",
    "bigWindow"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN12TvShowWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,  116,    2, 0x06,    1 /* Public */,
       4,    2,  121,    2, 0x06,    4 /* Public */,
       5,    2,  126,    2, 0x06,    7 /* Public */,
       6,    3,  131,    2, 0x06,   10 /* Public */,
       7,    1,  138,    2, 0x06,   14 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       8,    1,  141,    2, 0x0a,   16 /* Public */,
      11,    2,  144,    2, 0x0a,   18 /* Public */,
      14,    1,  149,    2, 0x0a,   21 /* Public */,
      17,    2,  152,    2, 0x0a,   23 /* Public */,
      17,    1,  157,    2, 0x2a,   26 /* Public | MethodCloned */,
      18,    1,  160,    2, 0x0a,   28 /* Public */,
      19,    0,  163,    2, 0x0a,   30 /* Public */,
      20,    0,  164,    2, 0x0a,   31 /* Public */,
      21,    0,  165,    2, 0x0a,   32 /* Public */,
      22,    0,  166,    2, 0x0a,   33 /* Public */,
      23,    0,  167,    2, 0x0a,   34 /* Public */,
      24,    1,  168,    2, 0x0a,   35 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 12,   10,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 12,   10,   13,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   25,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN12TvShowWidgetE.offsetsAndSizes,
    qt_meta_data_ZN12TvShowWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN12TvShowWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowWidget, std::true_type>,
        // method 'sigSetActionSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigSetActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigDownloadsStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'sigDownloadsProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'sigDownloadsFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onTvShowSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        // method 'onSeasonSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<SeasonNumber, std::false_type>,
        // method 'onEpisodeSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShowEpisode *, std::false_type>,
        // method 'onTvShowSetEnabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<SeasonNumber, std::false_type>,
        // method 'onTvShowSetEnabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        // method 'onEpisodeSetEnabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShowEpisode *, std::false_type>,
        // method 'onSetDisabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onClear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveAll'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartScraperSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setBigWindow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>
    >,
    nullptr
} };

void TvShowWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigSetActionSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->sigSetActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 2: _t->sigDownloadsStarted((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->sigDownloadsProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->sigDownloadsFinished((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->onTvShowSelected((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1]))); break;
        case 6: _t->onSeasonSelected((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<SeasonNumber>>(_a[2]))); break;
        case 7: _t->onEpisodeSelected((*reinterpret_cast< std::add_pointer_t<TvShowEpisode*>>(_a[1]))); break;
        case 8: _t->onTvShowSetEnabledTrue((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<SeasonNumber>>(_a[2]))); break;
        case 9: _t->onTvShowSetEnabledTrue((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1]))); break;
        case 10: _t->onEpisodeSetEnabledTrue((*reinterpret_cast< std::add_pointer_t<TvShowEpisode*>>(_a[1]))); break;
        case 11: _t->onSetDisabledTrue(); break;
        case 12: _t->onClear(); break;
        case 13: _t->onSaveInformation(); break;
        case 14: _t->onSaveAll(); break;
        case 15: _t->onStartScraperSearch(); break;
        case 16: _t->setBigWindow((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (TvShowWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidget::sigSetActionSearchEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidget::sigSetActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidget::*)(QString , int );
            if (_q_method_type _q_method = &TvShowWidget::sigDownloadsStarted; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidget::*)(int , int , int );
            if (_q_method_type _q_method = &TvShowWidget::sigDownloadsProgress; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidget::*)(int );
            if (_q_method_type _q_method = &TvShowWidget::sigDownloadsFinished; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
    }
}

const QMetaObject *TvShowWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN12TvShowWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TvShowWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void TvShowWidget::sigSetActionSearchEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TvShowWidget::sigSetActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TvShowWidget::sigDownloadsStarted(QString _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TvShowWidget::sigDownloadsProgress(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TvShowWidget::sigDownloadsFinished(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
