/********************************************************************************
** Form generated from reading UI file 'MusicWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICWIDGET_H
#define UI_MUSICWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/music/MusicWidgetAlbum.h"
#include "ui/music/MusicWidgetArtist.h"

QT_BEGIN_NAMESPACE

class Ui_MusicWidget
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *stackedWidget;
    MusicWidgetArtist *artist;
    MusicWidgetAlbum *album;

    void setupUi(QWidget *MusicWidget)
    {
        if (MusicWidget->objectName().isEmpty())
            MusicWidget->setObjectName("MusicWidget");
        MusicWidget->resize(910, 662);
        MusicWidget->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(MusicWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(MusicWidget);
        stackedWidget->setObjectName("stackedWidget");
        artist = new MusicWidgetArtist();
        artist->setObjectName("artist");
        stackedWidget->addWidget(artist);
        album = new MusicWidgetAlbum();
        album->setObjectName("album");
        stackedWidget->addWidget(album);

        verticalLayout->addWidget(stackedWidget);


        retranslateUi(MusicWidget);

        QMetaObject::connectSlotsByName(MusicWidget);
    } // setupUi

    void retranslateUi(QWidget *MusicWidget)
    {
        (void)MusicWidget;
    } // retranslateUi

};

namespace Ui {
    class MusicWidget: public Ui_MusicWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICWIDGET_H
