/****************************************************************************
** Meta object code from reading C++ file 'TvShowMultiScrapeDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowMultiScrapeDialog.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowMultiScrapeDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23TvShowMultiScrapeDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN23TvShowMultiScrapeDialogE = QtMocHelpers::stringData(
    "TvShowMultiScrapeDialog",
    "exec",
    "",
    "reject",
    "accept",
    "onShowInfoToggled",
    "onEpisodeInfoToggled",
    "onChkAllShowInfosToggled",
    "onChkAllEpisodeInfosToggled",
    "onStartScraping",
    "onScrapingFinished",
    "onSearchFinished",
    "mediaelch::scraper::ShowSearchJob*",
    "searchJob",
    "scrapeNext",
    "onInfoLoadDone",
    "TvShow*",
    "show",
    "QSet<ShowScraperInfo>",
    "details",
    "onEpisodeLoadDone",
    "onLoadDone",
    "QMap<ImageType,QList<Poster>>",
    "posters",
    "onDownloadFinished",
    "DownloadManagerElement",
    "elem",
    "onScraperChanged",
    "index",
    "onLanguageChanged",
    "onSeasonOrderChanged"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN23TvShowMultiScrapeDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  122,    2, 0x0a,    1 /* Public */,
       3,    0,  123,    2, 0x0a,    2 /* Public */,
       4,    0,  124,    2, 0x0a,    3 /* Public */,
       5,    0,  125,    2, 0x08,    4 /* Private */,
       6,    0,  126,    2, 0x08,    5 /* Private */,
       7,    0,  127,    2, 0x08,    6 /* Private */,
       8,    0,  128,    2, 0x08,    7 /* Private */,
       9,    0,  129,    2, 0x08,    8 /* Private */,
      10,    0,  130,    2, 0x08,    9 /* Private */,
      11,    1,  131,    2, 0x08,   10 /* Private */,
      14,    0,  134,    2, 0x08,   12 /* Private */,
      15,    2,  135,    2, 0x08,   13 /* Private */,
      20,    0,  140,    2, 0x08,   16 /* Private */,
      21,    2,  141,    2, 0x08,   17 /* Private */,
      24,    1,  146,    2, 0x08,   20 /* Private */,
      27,    1,  149,    2, 0x08,   22 /* Private */,
      29,    0,  152,    2, 0x08,   24 /* Private */,
      30,    1,  153,    2, 0x08,   25 /* Private */,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 18,   17,   19,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 22,   17,   23,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, QMetaType::Int,   28,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   28,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowMultiScrapeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN23TvShowMultiScrapeDialogE.offsetsAndSizes,
    qt_meta_data_ZN23TvShowMultiScrapeDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN23TvShowMultiScrapeDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowMultiScrapeDialog, std::true_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'accept'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onShowInfoToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onEpisodeInfoToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChkAllShowInfosToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChkAllEpisodeInfosToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartScraping'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onScrapingFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSearchFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::ShowSearchJob *, std::false_type>,
        // method 'scrapeNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onInfoLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QSet<ShowScraperInfo>, std::false_type>,
        // method 'onEpisodeLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'onDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>,
        // method 'onScraperChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onLanguageChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSeasonOrderChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>
    >,
    nullptr
} };

void TvShowMultiScrapeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowMultiScrapeDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->reject(); break;
        case 2: _t->accept(); break;
        case 3: _t->onShowInfoToggled(); break;
        case 4: _t->onEpisodeInfoToggled(); break;
        case 5: _t->onChkAllShowInfosToggled(); break;
        case 6: _t->onChkAllEpisodeInfosToggled(); break;
        case 7: _t->onStartScraping(); break;
        case 8: _t->onScrapingFinished(); break;
        case 9: _t->onSearchFinished((*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::ShowSearchJob*>>(_a[1]))); break;
        case 10: _t->scrapeNext(); break;
        case 11: _t->onInfoLoadDone((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QSet<ShowScraperInfo>>>(_a[2]))); break;
        case 12: _t->onEpisodeLoadDone(); break;
        case 13: _t->onLoadDone((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 14: _t->onDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        case 15: _t->onScraperChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->onLanguageChanged(); break;
        case 17: _t->onSeasonOrderChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::ShowSearchJob* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        }
    }
}

const QMetaObject *TvShowMultiScrapeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowMultiScrapeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN23TvShowMultiScrapeDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int TvShowMultiScrapeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
