/********************************************************************************
** Form generated from reading UI file 'ConcertFilesWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONCERTFILESWIDGET_H
#define UI_CONCERTFILESWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyTableView.h"

QT_BEGIN_NAMESPACE

class Ui_ConcertFilesWidget
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *statusLabelWidget;
    QVBoxLayout *verticalLayout_2;
    QLabel *statusLabel;
    MyTableView *files;

    void setupUi(QWidget *ConcertFilesWidget)
    {
        if (ConcertFilesWidget->objectName().isEmpty())
            ConcertFilesWidget->setObjectName("ConcertFilesWidget");
        ConcertFilesWidget->resize(565, 587);
        verticalLayout = new QVBoxLayout(ConcertFilesWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        statusLabelWidget = new QWidget(ConcertFilesWidget);
        statusLabelWidget->setObjectName("statusLabelWidget");
        verticalLayout_2 = new QVBoxLayout(statusLabelWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(6, 4, 6, 4);
        statusLabel = new QLabel(statusLabelWidget);
        statusLabel->setObjectName("statusLabel");
        QFont font;
        font.setBold(true);
        statusLabel->setFont(font);
        statusLabel->setText(QString::fromUtf8("TextLabel"));

        verticalLayout_2->addWidget(statusLabel);


        verticalLayout->addWidget(statusLabelWidget);

        files = new MyTableView(ConcertFilesWidget);
        files->setObjectName("files");
        files->setMinimumSize(QSize(200, 0));
        files->setMouseTracking(true);
        files->setContextMenuPolicy(Qt::CustomContextMenu);
        files->setEditTriggers(QAbstractItemView::NoEditTriggers);
        files->setAlternatingRowColors(true);
        files->setSelectionMode(QAbstractItemView::ExtendedSelection);
        files->setSelectionBehavior(QAbstractItemView::SelectRows);
        files->setShowGrid(false);
        files->setSortingEnabled(true);
        files->setProperty("useSearchOverlay", QVariant(true));
        files->horizontalHeader()->setVisible(false);
        files->horizontalHeader()->setStretchLastSection(true);
        files->verticalHeader()->setVisible(false);
        files->verticalHeader()->setMinimumSectionSize(22);
        files->verticalHeader()->setDefaultSectionSize(22);

        verticalLayout->addWidget(files);


        retranslateUi(ConcertFilesWidget);

        QMetaObject::connectSlotsByName(ConcertFilesWidget);
    } // setupUi

    void retranslateUi(QWidget *ConcertFilesWidget)
    {
        (void)ConcertFilesWidget;
    } // retranslateUi

};

namespace Ui {
    class ConcertFilesWidget: public Ui_ConcertFilesWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONCERTFILESWIDGET_H
