/****************************************************************************
** Meta object code from reading C++ file 'UniversalMusicScraper.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/scrapers/music/UniversalMusicScraper.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UniversalMusicScraper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper24UniversalArtistSearchJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper24UniversalArtistSearchJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::UniversalArtistSearchJob"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper24UniversalArtistSearchJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::UniversalArtistSearchJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ArtistSearchJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper24UniversalArtistSearchJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper24UniversalArtistSearchJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper24UniversalArtistSearchJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<UniversalArtistSearchJob, std::true_type>
    >,
    nullptr
} };

void mediaelch::scraper::UniversalArtistSearchJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UniversalArtistSearchJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *mediaelch::scraper::UniversalArtistSearchJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::UniversalArtistSearchJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper24UniversalArtistSearchJobE.stringdata0))
        return static_cast<void*>(this);
    return ArtistSearchJob::qt_metacast(_clname);
}

int mediaelch::scraper::UniversalArtistSearchJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ArtistSearchJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper23UniversalAlbumSearchJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper23UniversalAlbumSearchJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::UniversalAlbumSearchJob"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper23UniversalAlbumSearchJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::UniversalAlbumSearchJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AlbumSearchJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper23UniversalAlbumSearchJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper23UniversalAlbumSearchJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper23UniversalAlbumSearchJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<UniversalAlbumSearchJob, std::true_type>
    >,
    nullptr
} };

void mediaelch::scraper::UniversalAlbumSearchJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UniversalAlbumSearchJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *mediaelch::scraper::UniversalAlbumSearchJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::UniversalAlbumSearchJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper23UniversalAlbumSearchJobE.stringdata0))
        return static_cast<void*>(this);
    return AlbumSearchJob::qt_metacast(_clname);
}

int mediaelch::scraper::UniversalAlbumSearchJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AlbumSearchJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper24UniversalArtistScrapeJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper24UniversalArtistScrapeJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::UniversalArtistScrapeJob",
    "onArtistLoadFinished",
    ""
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper24UniversalArtistScrapeJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   20,    2, 0x08,    1 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::UniversalArtistScrapeJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ArtistScrapeJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper24UniversalArtistScrapeJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper24UniversalArtistScrapeJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper24UniversalArtistScrapeJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<UniversalArtistScrapeJob, std::true_type>,
        // method 'onArtistLoadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::UniversalArtistScrapeJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UniversalArtistScrapeJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onArtistLoadFinished(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *mediaelch::scraper::UniversalArtistScrapeJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::UniversalArtistScrapeJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper24UniversalArtistScrapeJobE.stringdata0))
        return static_cast<void*>(this);
    return ArtistScrapeJob::qt_metacast(_clname);
}

int mediaelch::scraper::UniversalArtistScrapeJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ArtistScrapeJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE = QtMocHelpers::stringData(
    "mediaelch::scraper::UniversalAlbumScrapeJob",
    "onAlbumLoadFinished",
    ""
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   20,    2, 0x08,    1 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::UniversalAlbumScrapeJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AlbumScrapeJob::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<UniversalAlbumScrapeJob, std::true_type>,
        // method 'onAlbumLoadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::UniversalAlbumScrapeJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UniversalAlbumScrapeJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onAlbumLoadFinished(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *mediaelch::scraper::UniversalAlbumScrapeJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::UniversalAlbumScrapeJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper23UniversalAlbumScrapeJobE.stringdata0))
        return static_cast<void*>(this);
    return AlbumScrapeJob::qt_metacast(_clname);
}

int mediaelch::scraper::UniversalAlbumScrapeJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AlbumScrapeJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper21UniversalMusicScraperE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper21UniversalMusicScraperE = QtMocHelpers::stringData(
    "mediaelch::scraper::UniversalMusicScraper"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper21UniversalMusicScraperE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::UniversalMusicScraper::staticMetaObject = { {
    QMetaObject::SuperData::link<MusicScraper::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper21UniversalMusicScraperE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper21UniversalMusicScraperE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper21UniversalMusicScraperE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<UniversalMusicScraper, std::true_type>
    >,
    nullptr
} };

void mediaelch::scraper::UniversalMusicScraper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UniversalMusicScraper *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *mediaelch::scraper::UniversalMusicScraper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::UniversalMusicScraper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper21UniversalMusicScraperE.stringdata0))
        return static_cast<void*>(this);
    return MusicScraper::qt_metacast(_clname);
}

int mediaelch::scraper::UniversalMusicScraper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MusicScraper::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
