/********************************************************************************
** Form generated from reading UI file 'TvShowSearch.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWSEARCH_H
#define UI_TVSHOWSEARCH_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "ui/tv_show/TvShowSearchWidget.h"

QT_BEGIN_NAMESPACE

class Ui_TvShowSearch
{
public:
    QVBoxLayout *verticalLayout;
    TvShowSearchWidget *tvShowSearchWidget;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClose;
    QPushButton *buttonScrape;

    void setupUi(QDialog *TvShowSearch)
    {
        if (TvShowSearch->objectName().isEmpty())
            TvShowSearch->setObjectName("TvShowSearch");
        TvShowSearch->setWindowModality(Qt::WindowModal);
        TvShowSearch->resize(700, 600);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        TvShowSearch->setWindowIcon(icon);
        TvShowSearch->setModal(true);
        verticalLayout = new QVBoxLayout(TvShowSearch);
        verticalLayout->setObjectName("verticalLayout");
        tvShowSearchWidget = new TvShowSearchWidget(TvShowSearch);
        tvShowSearchWidget->setObjectName("tvShowSearchWidget");

        verticalLayout->addWidget(tvShowSearchWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonClose = new QPushButton(TvShowSearch);
        buttonClose->setObjectName("buttonClose");
        buttonClose->setAutoDefault(false);

        horizontalLayout->addWidget(buttonClose);

        buttonScrape = new QPushButton(TvShowSearch);
        buttonScrape->setObjectName("buttonScrape");
        buttonScrape->setAutoDefault(false);

        horizontalLayout->addWidget(buttonScrape);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(0, 1);

        retranslateUi(TvShowSearch);

        QMetaObject::connectSlotsByName(TvShowSearch);
    } // setupUi

    void retranslateUi(QDialog *TvShowSearch)
    {
        TvShowSearch->setWindowTitle(QCoreApplication::translate("TvShowSearch", "Search Result", nullptr));
        buttonClose->setText(QCoreApplication::translate("TvShowSearch", "Close", nullptr));
        buttonScrape->setText(QCoreApplication::translate("TvShowSearch", "Scrape", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TvShowSearch: public Ui_TvShowSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWSEARCH_H
