/****************************************************************************
** Meta object code from reading C++ file 'MovieFilesWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/movies/MovieFilesWidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MovieFilesWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16MovieFilesWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN16MovieFilesWidgetE = QtMocHelpers::stringData(
    "MovieFilesWidget",
    "noMovieSelected",
    "",
    "movieSelected",
    "Movie*",
    "sigStartSearch",
    "restoreLastSelection",
    "setFilter",
    "QList<Filter*>",
    "filters",
    "text",
    "movieSelectedEmitter",
    "multiScrape",
    "setAlphaListData",
    "itemActivated",
    "QModelIndex",
    "index",
    "previous",
    "onSortByName",
    "onSortByAdded",
    "onSortByYear",
    "onSortBySeen",
    "onSortByNew",
    "showContextMenu",
    "point",
    "markAsWatched",
    "markAsUnwatched",
    "loadStreamDetails",
    "markForSync",
    "unmarkForSync",
    "openFolder",
    "scrollToAlpha",
    "alpha",
    "onLeftEdge",
    "isEdge",
    "onActionMediaStatusColumn",
    "onLabel",
    "updateStatusLabel",
    "playMovie",
    "idx",
    "openNfoFile"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN16MovieFilesWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      28,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  182,    2, 0x06,    1 /* Public */,
       3,    1,  183,    2, 0x06,    2 /* Public */,
       5,    0,  186,    2, 0x06,    4 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       6,    0,  187,    2, 0x0a,    5 /* Public */,
       7,    2,  188,    2, 0x0a,    6 /* Public */,
      11,    0,  193,    2, 0x0a,    9 /* Public */,
      12,    0,  194,    2, 0x0a,   10 /* Public */,
      13,    0,  195,    2, 0x0a,   11 /* Public */,
      14,    2,  196,    2, 0x08,   12 /* Private */,
      18,    0,  201,    2, 0x08,   15 /* Private */,
      19,    0,  202,    2, 0x08,   16 /* Private */,
      20,    0,  203,    2, 0x08,   17 /* Private */,
      21,    0,  204,    2, 0x08,   18 /* Private */,
      22,    0,  205,    2, 0x08,   19 /* Private */,
      23,    1,  206,    2, 0x08,   20 /* Private */,
      25,    0,  209,    2, 0x08,   22 /* Private */,
      26,    0,  210,    2, 0x08,   23 /* Private */,
      27,    0,  211,    2, 0x08,   24 /* Private */,
      28,    0,  212,    2, 0x08,   25 /* Private */,
      29,    0,  213,    2, 0x08,   26 /* Private */,
      30,    0,  214,    2, 0x08,   27 /* Private */,
      31,    1,  215,    2, 0x08,   28 /* Private */,
      33,    1,  218,    2, 0x08,   30 /* Private */,
      35,    0,  221,    2, 0x08,   32 /* Private */,
      36,    0,  222,    2, 0x08,   33 /* Private */,
      37,    0,  223,    2, 0x08,   34 /* Private */,
      38,    1,  224,    2, 0x08,   35 /* Private */,
      40,    0,  227,    2, 0x08,   37 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8, QMetaType::QString,    9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 15,   16,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   32,
    QMetaType::Void, QMetaType::Bool,   34,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,   39,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject MovieFilesWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN16MovieFilesWidgetE.offsetsAndSizes,
    qt_meta_data_ZN16MovieFilesWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN16MovieFilesWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MovieFilesWidget, std::true_type>,
        // method 'noMovieSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'movieSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'sigStartSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'restoreLastSelection'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setFilter'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<Filter*>, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'movieSelectedEmitter'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'multiScrape'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setAlphaListData'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'itemActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QModelIndex, std::false_type>,
        QtPrivate::TypeAndForceComplete<QModelIndex, std::false_type>,
        // method 'onSortByName'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSortByAdded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSortByYear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSortBySeen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSortByNew'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'showContextMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'markAsWatched'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'markAsUnwatched'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'loadStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'markForSync'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'unmarkForSync'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'openFolder'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'scrollToAlpha'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onLeftEdge'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onActionMediaStatusColumn'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onLabel'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateStatusLabel'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'playMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QModelIndex, std::false_type>,
        // method 'openNfoFile'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void MovieFilesWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MovieFilesWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->noMovieSelected(); break;
        case 1: _t->movieSelected((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 2: _t->sigStartSearch(); break;
        case 3: _t->restoreLastSelection(); break;
        case 4: _t->setFilter((*reinterpret_cast< std::add_pointer_t<QList<Filter*>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->movieSelectedEmitter(); break;
        case 6: _t->multiScrape(); break;
        case 7: _t->setAlphaListData(); break;
        case 8: _t->itemActivated((*reinterpret_cast< std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 9: _t->onSortByName(); break;
        case 10: _t->onSortByAdded(); break;
        case 11: _t->onSortByYear(); break;
        case 12: _t->onSortBySeen(); break;
        case 13: _t->onSortByNew(); break;
        case 14: _t->showContextMenu((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 15: _t->markAsWatched(); break;
        case 16: _t->markAsUnwatched(); break;
        case 17: _t->loadStreamDetails(); break;
        case 18: _t->markForSync(); break;
        case 19: _t->unmarkForSync(); break;
        case 20: _t->openFolder(); break;
        case 21: _t->scrollToAlpha((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->onLeftEdge((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->onActionMediaStatusColumn(); break;
        case 24: _t->onLabel(); break;
        case 25: _t->updateStatusLabel(); break;
        case 26: _t->playMovie((*reinterpret_cast< std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 27: _t->openNfoFile(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Filter*> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (MovieFilesWidget::*)();
            if (_q_method_type _q_method = &MovieFilesWidget::noMovieSelected; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (MovieFilesWidget::*)(Movie * );
            if (_q_method_type _q_method = &MovieFilesWidget::movieSelected; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (MovieFilesWidget::*)();
            if (_q_method_type _q_method = &MovieFilesWidget::sigStartSearch; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
    }
}

const QMetaObject *MovieFilesWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MovieFilesWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN16MovieFilesWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MovieFilesWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void MovieFilesWidget::noMovieSelected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MovieFilesWidget::movieSelected(Movie * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void MovieFilesWidget::sigStartSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
