/********************************************************************************
** Form generated from reading UI file 'TvShowMultiScrapeDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWMULTISCRAPEDIALOG_H
#define UI_TVSHOWMULTISCRAPEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/LanguageCombo.h>
#include <ui/small_widgets/MyCheckBox.h>

QT_BEGIN_NAMESPACE

class Ui_TvShowMultiScrapeDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *lblHelp;
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tabShowDetails;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *showInfosGroupBox;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_2;
    QGridLayout *gridLayout_2;
    MyCheckBox *chkTitle;
    MyCheckBox *chkSeasonThumb;
    MyCheckBox *chkSeasonPoster;
    MyCheckBox *chkPoster;
    MyCheckBox *chkNetwork;
    MyCheckBox *chkCertification;
    MyCheckBox *chkRuntime;
    MyCheckBox *chkFirstAired;
    MyCheckBox *chkRating;
    MyCheckBox *chkStatus;
    MyCheckBox *chkOverview;
    MyCheckBox *chkActors;
    MyCheckBox *chkBanner;
    MyCheckBox *chkGenres;
    MyCheckBox *chkFanart;
    MyCheckBox *chkExtraArts;
    MyCheckBox *chkSeasonBanner;
    MyCheckBox *chkTags;
    MyCheckBox *chkSeasonFanart;
    MyCheckBox *chkThumb;
    QFrame *line;
    QCheckBox *chkUnCheckAll;
    QWidget *tabEpisodeDetails;
    QVBoxLayout *verticalLayout_22;
    QGroupBox *episodeInfosGroupBox;
    QVBoxLayout *verticalLayout_5;
    QLabel *lblEpisodeInfosToLoad;
    QGridLayout *gridLayout_22;
    MyCheckBox *chkEpisodeNetwork;
    MyCheckBox *chkEpisodeCertification;
    MyCheckBox *chkEpisodeTitle;
    MyCheckBox *chkEpisodeFirstAired;
    MyCheckBox *chkEpisodeDirector;
    MyCheckBox *chkEpisodeWriter;
    MyCheckBox *chkEpisodeOverview;
    MyCheckBox *chkEpisodeThumbnail;
    MyCheckBox *chkEpisodeTags;
    MyCheckBox *chkEpisodeRating;
    MyCheckBox *chkEpisodeActors;
    QFrame *line2;
    QCheckBox *chkEpisodeUnCheckAll;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *h_seasonTypeLayout;
    QFormLayout *formLayout;
    QLabel *lblScraper;
    QComboBox *comboScraper;
    QLabel *lblLanguage;
    LanguageCombo *comboLanguage;
    QLabel *lblSeasonOrder;
    QComboBox *comboSeasonOrder;
    QCheckBox *chkAutoSave;
    QCheckBox *chkOnlyId;
    QLabel *lblOnlyId;
    QLabel *lblAutoSave;
    QSpacerItem *spacerComboRight;
    QHBoxLayout *horizontalLayout_2;
    QLabel *itemCounter;
    QLabel *title;
    QProgressBar *progressItem;
    QProgressBar *progressAll;
    QPlainTextEdit *txtScraperLog;
    QLabel *lblError;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnCancel;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnClose;
    QPushButton *btnStartScraping;

    void setupUi(QDialog *TvShowMultiScrapeDialog)
    {
        if (TvShowMultiScrapeDialog->objectName().isEmpty())
            TvShowMultiScrapeDialog->setObjectName("TvShowMultiScrapeDialog");
        TvShowMultiScrapeDialog->resize(723, 754);
        TvShowMultiScrapeDialog->setMinimumSize(QSize(550, 0));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        TvShowMultiScrapeDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(TvShowMultiScrapeDialog);
        verticalLayout->setObjectName("verticalLayout");
        lblHelp = new QLabel(TvShowMultiScrapeDialog);
        lblHelp->setObjectName("lblHelp");
        lblHelp->setWordWrap(true);

        verticalLayout->addWidget(lblHelp);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        tabWidget = new QTabWidget(TvShowMultiScrapeDialog);
        tabWidget->setObjectName("tabWidget");
        tabShowDetails = new QWidget();
        tabShowDetails->setObjectName("tabShowDetails");
        verticalLayout_2 = new QVBoxLayout(tabShowDetails);
        verticalLayout_2->setObjectName("verticalLayout_2");
        showInfosGroupBox = new QGroupBox(tabShowDetails);
        showInfosGroupBox->setObjectName("showInfosGroupBox");
        verticalLayout_4 = new QVBoxLayout(showInfosGroupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_2 = new QLabel(showInfosGroupBox);
        label_2->setObjectName("label_2");
        QFont font;
        font.setBold(true);
        label_2->setFont(font);

        verticalLayout_4->addWidget(label_2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        chkTitle = new MyCheckBox(showInfosGroupBox);
        chkTitle->setObjectName("chkTitle");
        chkTitle->setChecked(true);
        chkTitle->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkTitle, 0, 0, 1, 1);

        chkSeasonThumb = new MyCheckBox(showInfosGroupBox);
        chkSeasonThumb->setObjectName("chkSeasonThumb");
        chkSeasonThumb->setChecked(true);
        chkSeasonThumb->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkSeasonThumb, 6, 1, 1, 1);

        chkSeasonPoster = new MyCheckBox(showInfosGroupBox);
        chkSeasonPoster->setObjectName("chkSeasonPoster");
        chkSeasonPoster->setChecked(true);
        chkSeasonPoster->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkSeasonPoster, 5, 1, 1, 1);

        chkPoster = new MyCheckBox(showInfosGroupBox);
        chkPoster->setObjectName("chkPoster");
        chkPoster->setChecked(true);
        chkPoster->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkPoster, 4, 1, 1, 1);

        chkNetwork = new MyCheckBox(showInfosGroupBox);
        chkNetwork->setObjectName("chkNetwork");
        chkNetwork->setChecked(true);
        chkNetwork->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkNetwork, 3, 1, 1, 1);

        chkCertification = new MyCheckBox(showInfosGroupBox);
        chkCertification->setObjectName("chkCertification");
        chkCertification->setChecked(true);
        chkCertification->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkCertification, 2, 1, 1, 1);

        chkRuntime = new MyCheckBox(showInfosGroupBox);
        chkRuntime->setObjectName("chkRuntime");
        chkRuntime->setChecked(true);
        chkRuntime->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkRuntime, 1, 1, 1, 1);

        chkFirstAired = new MyCheckBox(showInfosGroupBox);
        chkFirstAired->setObjectName("chkFirstAired");
        chkFirstAired->setChecked(true);
        chkFirstAired->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkFirstAired, 1, 0, 1, 1);

        chkRating = new MyCheckBox(showInfosGroupBox);
        chkRating->setObjectName("chkRating");
        chkRating->setChecked(true);
        chkRating->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkRating, 0, 2, 1, 1);

        chkStatus = new MyCheckBox(showInfosGroupBox);
        chkStatus->setObjectName("chkStatus");
        chkStatus->setChecked(true);
        chkStatus->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkStatus, 0, 1, 1, 1);

        chkOverview = new MyCheckBox(showInfosGroupBox);
        chkOverview->setObjectName("chkOverview");
        chkOverview->setChecked(true);
        chkOverview->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkOverview, 1, 2, 1, 1);

        chkActors = new MyCheckBox(showInfosGroupBox);
        chkActors->setObjectName("chkActors");
        chkActors->setChecked(true);
        chkActors->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkActors, 2, 2, 1, 1);

        chkBanner = new MyCheckBox(showInfosGroupBox);
        chkBanner->setObjectName("chkBanner");
        chkBanner->setChecked(true);
        chkBanner->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkBanner, 3, 2, 1, 1);

        chkGenres = new MyCheckBox(showInfosGroupBox);
        chkGenres->setObjectName("chkGenres");
        chkGenres->setChecked(true);
        chkGenres->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkGenres, 2, 0, 1, 1);

        chkFanart = new MyCheckBox(showInfosGroupBox);
        chkFanart->setObjectName("chkFanart");
        chkFanart->setChecked(true);
        chkFanart->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkFanart, 3, 0, 1, 1);

        chkExtraArts = new MyCheckBox(showInfosGroupBox);
        chkExtraArts->setObjectName("chkExtraArts");
        chkExtraArts->setChecked(true);
        chkExtraArts->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkExtraArts, 4, 0, 1, 1);

        chkSeasonBanner = new MyCheckBox(showInfosGroupBox);
        chkSeasonBanner->setObjectName("chkSeasonBanner");
        chkSeasonBanner->setChecked(true);
        chkSeasonBanner->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkSeasonBanner, 5, 0, 1, 1);

        chkTags = new MyCheckBox(showInfosGroupBox);
        chkTags->setObjectName("chkTags");
        chkTags->setChecked(true);
        chkTags->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkTags, 6, 0, 1, 1);

        chkSeasonFanart = new MyCheckBox(showInfosGroupBox);
        chkSeasonFanart->setObjectName("chkSeasonFanart");
        chkSeasonFanart->setChecked(true);
        chkSeasonFanart->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_2->addWidget(chkSeasonFanart, 5, 2, 1, 1);

        chkThumb = new MyCheckBox(showInfosGroupBox);
        chkThumb->setObjectName("chkThumb");
        chkThumb->setChecked(true);

        gridLayout_2->addWidget(chkThumb, 4, 2, 1, 1);


        verticalLayout_4->addLayout(gridLayout_2);


        verticalLayout_2->addWidget(showInfosGroupBox);

        line = new QFrame(tabShowDetails);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        chkUnCheckAll = new QCheckBox(tabShowDetails);
        chkUnCheckAll->setObjectName("chkUnCheckAll");
        chkUnCheckAll->setChecked(true);

        verticalLayout_2->addWidget(chkUnCheckAll);

        tabWidget->addTab(tabShowDetails, QString());
        tabEpisodeDetails = new QWidget();
        tabEpisodeDetails->setObjectName("tabEpisodeDetails");
        verticalLayout_22 = new QVBoxLayout(tabEpisodeDetails);
        verticalLayout_22->setObjectName("verticalLayout_22");
        episodeInfosGroupBox = new QGroupBox(tabEpisodeDetails);
        episodeInfosGroupBox->setObjectName("episodeInfosGroupBox");
        verticalLayout_5 = new QVBoxLayout(episodeInfosGroupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        lblEpisodeInfosToLoad = new QLabel(episodeInfosGroupBox);
        lblEpisodeInfosToLoad->setObjectName("lblEpisodeInfosToLoad");
        lblEpisodeInfosToLoad->setFont(font);

        verticalLayout_5->addWidget(lblEpisodeInfosToLoad);

        gridLayout_22 = new QGridLayout();
        gridLayout_22->setObjectName("gridLayout_22");
        chkEpisodeNetwork = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeNetwork->setObjectName("chkEpisodeNetwork");
        chkEpisodeNetwork->setChecked(true);
        chkEpisodeNetwork->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeNetwork, 3, 1, 1, 1);

        chkEpisodeCertification = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeCertification->setObjectName("chkEpisodeCertification");
        chkEpisodeCertification->setChecked(true);
        chkEpisodeCertification->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeCertification, 2, 1, 1, 1);

        chkEpisodeTitle = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeTitle->setObjectName("chkEpisodeTitle");
        chkEpisodeTitle->setChecked(true);
        chkEpisodeTitle->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeTitle, 0, 0, 1, 1);

        chkEpisodeFirstAired = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeFirstAired->setObjectName("chkEpisodeFirstAired");
        chkEpisodeFirstAired->setChecked(true);
        chkEpisodeFirstAired->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeFirstAired, 1, 0, 1, 1);

        chkEpisodeDirector = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeDirector->setObjectName("chkEpisodeDirector");
        chkEpisodeDirector->setChecked(true);

        gridLayout_22->addWidget(chkEpisodeDirector, 2, 0, 1, 1);

        chkEpisodeWriter = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeWriter->setObjectName("chkEpisodeWriter");
        chkEpisodeWriter->setChecked(true);
        chkEpisodeWriter->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_22->addWidget(chkEpisodeWriter, 3, 0, 1, 1);

        chkEpisodeOverview = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeOverview->setObjectName("chkEpisodeOverview");
        chkEpisodeOverview->setChecked(true);
        chkEpisodeOverview->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeOverview, 1, 1, 1, 1);

        chkEpisodeThumbnail = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeThumbnail->setObjectName("chkEpisodeThumbnail");
        chkEpisodeThumbnail->setChecked(true);
        chkEpisodeThumbnail->setProperty("type", QVariant(QString::fromUtf8("show")));

        gridLayout_22->addWidget(chkEpisodeThumbnail, 1, 2, 1, 1);

        chkEpisodeTags = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeTags->setObjectName("chkEpisodeTags");
        chkEpisodeTags->setChecked(true);
        chkEpisodeTags->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeTags, 2, 2, 1, 1);

        chkEpisodeRating = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeRating->setObjectName("chkEpisodeRating");
        chkEpisodeRating->setChecked(true);
        chkEpisodeRating->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeRating, 0, 1, 1, 1);

        chkEpisodeActors = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeActors->setObjectName("chkEpisodeActors");
        chkEpisodeActors->setChecked(true);
        chkEpisodeActors->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_22->addWidget(chkEpisodeActors, 0, 2, 1, 1);


        verticalLayout_5->addLayout(gridLayout_22);


        verticalLayout_22->addWidget(episodeInfosGroupBox);

        line2 = new QFrame(tabEpisodeDetails);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::Shape::HLine);
        line2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_22->addWidget(line2);

        chkEpisodeUnCheckAll = new QCheckBox(tabEpisodeDetails);
        chkEpisodeUnCheckAll->setObjectName("chkEpisodeUnCheckAll");
        chkEpisodeUnCheckAll->setChecked(true);

        verticalLayout_22->addWidget(chkEpisodeUnCheckAll);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_22->addItem(verticalSpacer);

        tabWidget->addTab(tabEpisodeDetails, QString());

        verticalLayout_3->addWidget(tabWidget);


        verticalLayout->addLayout(verticalLayout_3);

        h_seasonTypeLayout = new QHBoxLayout();
        h_seasonTypeLayout->setObjectName("h_seasonTypeLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        lblScraper = new QLabel(TvShowMultiScrapeDialog);
        lblScraper->setObjectName("lblScraper");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblScraper);

        comboScraper = new QComboBox(TvShowMultiScrapeDialog);
        comboScraper->setObjectName("comboScraper");

        formLayout->setWidget(0, QFormLayout::FieldRole, comboScraper);

        lblLanguage = new QLabel(TvShowMultiScrapeDialog);
        lblLanguage->setObjectName("lblLanguage");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblLanguage);

        comboLanguage = new LanguageCombo(TvShowMultiScrapeDialog);
        comboLanguage->setObjectName("comboLanguage");

        formLayout->setWidget(1, QFormLayout::FieldRole, comboLanguage);

        lblSeasonOrder = new QLabel(TvShowMultiScrapeDialog);
        lblSeasonOrder->setObjectName("lblSeasonOrder");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblSeasonOrder);

        comboSeasonOrder = new QComboBox(TvShowMultiScrapeDialog);
        comboSeasonOrder->setObjectName("comboSeasonOrder");
        comboSeasonOrder->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        formLayout->setWidget(2, QFormLayout::FieldRole, comboSeasonOrder);

        chkAutoSave = new QCheckBox(TvShowMultiScrapeDialog);
        chkAutoSave->setObjectName("chkAutoSave");

        formLayout->setWidget(3, QFormLayout::FieldRole, chkAutoSave);

        chkOnlyId = new QCheckBox(TvShowMultiScrapeDialog);
        chkOnlyId->setObjectName("chkOnlyId");

        formLayout->setWidget(4, QFormLayout::FieldRole, chkOnlyId);

        lblOnlyId = new QLabel(TvShowMultiScrapeDialog);
        lblOnlyId->setObjectName("lblOnlyId");
        lblOnlyId->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lblOnlyId->setWordWrap(true);

        formLayout->setWidget(4, QFormLayout::LabelRole, lblOnlyId);

        lblAutoSave = new QLabel(TvShowMultiScrapeDialog);
        lblAutoSave->setObjectName("lblAutoSave");
        lblAutoSave->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lblAutoSave->setWordWrap(true);

        formLayout->setWidget(3, QFormLayout::LabelRole, lblAutoSave);


        h_seasonTypeLayout->addLayout(formLayout);

        spacerComboRight = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        h_seasonTypeLayout->addItem(spacerComboRight);


        verticalLayout->addLayout(h_seasonTypeLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        itemCounter = new QLabel(TvShowMultiScrapeDialog);
        itemCounter->setObjectName("itemCounter");
        itemCounter->setMinimumSize(QSize(30, 0));

        horizontalLayout_2->addWidget(itemCounter);

        title = new QLabel(TvShowMultiScrapeDialog);
        title->setObjectName("title");
        title->setText(QString::fromUtf8("Item"));

        horizontalLayout_2->addWidget(title);

        horizontalLayout_2->setStretch(1, 1);

        verticalLayout->addLayout(horizontalLayout_2);

        progressItem = new QProgressBar(TvShowMultiScrapeDialog);
        progressItem->setObjectName("progressItem");
        progressItem->setValue(24);

        verticalLayout->addWidget(progressItem);

        progressAll = new QProgressBar(TvShowMultiScrapeDialog);
        progressAll->setObjectName("progressAll");
        progressAll->setValue(24);

        verticalLayout->addWidget(progressAll);

        txtScraperLog = new QPlainTextEdit(TvShowMultiScrapeDialog);
        txtScraperLog->setObjectName("txtScraperLog");
        txtScraperLog->setReadOnly(true);

        verticalLayout->addWidget(txtScraperLog);

        lblError = new QLabel(TvShowMultiScrapeDialog);
        lblError->setObjectName("lblError");

        verticalLayout->addWidget(lblError);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnCancel = new QPushButton(TvShowMultiScrapeDialog);
        btnCancel->setObjectName("btnCancel");
        btnCancel->setAutoDefault(false);

        horizontalLayout->addWidget(btnCancel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnClose = new QPushButton(TvShowMultiScrapeDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(true);

        horizontalLayout->addWidget(btnClose);

        btnStartScraping = new QPushButton(TvShowMultiScrapeDialog);
        btnStartScraping->setObjectName("btnStartScraping");

        horizontalLayout->addWidget(btnStartScraping);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(TvShowMultiScrapeDialog);
        QObject::connect(btnCancel, &QPushButton::clicked, TvShowMultiScrapeDialog, qOverload<>(&QDialog::reject));
        QObject::connect(btnClose, &QPushButton::clicked, TvShowMultiScrapeDialog, qOverload<>(&QDialog::accept));

        tabWidget->setCurrentIndex(0);
        btnClose->setDefault(true);
        btnStartScraping->setDefault(true);


        QMetaObject::connectSlotsByName(TvShowMultiScrapeDialog);
    } // setupUi

    void retranslateUi(QDialog *TvShowMultiScrapeDialog)
    {
        TvShowMultiScrapeDialog->setWindowTitle(QCoreApplication::translate("TvShowMultiScrapeDialog", "TV Show Multi Scrape", nullptr));
        lblHelp->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Please select the infos you want to be loaded. MediaElch will use the best result for each TV show and episode you selected.", nullptr));
        label_2->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Infos to load", nullptr));
        chkTitle->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Title", nullptr));
        chkSeasonThumb->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Season Thumb", nullptr));
        chkSeasonPoster->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Season Poster", nullptr));
        chkPoster->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Poster", nullptr));
        chkNetwork->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Network", nullptr));
        chkCertification->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Certification", nullptr));
        chkRuntime->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Runtime", nullptr));
        chkFirstAired->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "First aired", nullptr));
        chkRating->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Rating", nullptr));
        chkStatus->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Status", nullptr));
        chkOverview->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Overview", nullptr));
        chkActors->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Actors", nullptr));
        chkBanner->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Banner", nullptr));
        chkGenres->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Genres", nullptr));
        chkFanart->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Fanart", nullptr));
        chkExtraArts->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Artwork", nullptr));
        chkSeasonBanner->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Season Banner", nullptr));
        chkTags->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Tags", nullptr));
        chkSeasonFanart->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Season Fanart", nullptr));
        chkThumb->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Thumb", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "(Un)Check all", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabShowDetails), QCoreApplication::translate("TvShowMultiScrapeDialog", "Show Details", nullptr));
        lblEpisodeInfosToLoad->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Infos to load", nullptr));
        chkEpisodeNetwork->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Network", nullptr));
        chkEpisodeCertification->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Certification", nullptr));
        chkEpisodeTitle->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Title", nullptr));
        chkEpisodeFirstAired->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "First aired", nullptr));
        chkEpisodeDirector->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Director", nullptr));
        chkEpisodeWriter->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Writer", nullptr));
        chkEpisodeOverview->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Overview", nullptr));
        chkEpisodeThumbnail->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Thumbnail", nullptr));
        chkEpisodeTags->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Tags", nullptr));
        chkEpisodeRating->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Rating", nullptr));
        chkEpisodeActors->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Actors", nullptr));
        chkEpisodeUnCheckAll->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "(Un)Check all", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabEpisodeDetails), QCoreApplication::translate("TvShowMultiScrapeDialog", "Episode Details", nullptr));
        lblScraper->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Scraper", nullptr));
        lblLanguage->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Language", nullptr));
        lblSeasonOrder->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Order for seasons", nullptr));
        chkAutoSave->setText(QString());
        chkOnlyId->setText(QString());
        lblOnlyId->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Update only TV shows/episodes\n"
"which have an ID", nullptr));
        lblAutoSave->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Automatically save each TV show/\n"
"episode after scraping", nullptr));
        itemCounter->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "1/20", nullptr));
        lblError->setText(QString());
        btnCancel->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Cancel", nullptr));
        btnClose->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Close", nullptr));
        btnStartScraping->setText(QCoreApplication::translate("TvShowMultiScrapeDialog", "Start Scraping", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TvShowMultiScrapeDialog: public Ui_TvShowMultiScrapeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWMULTISCRAPEDIALOG_H
