/********************************************************************************
** Form generated from reading UI file 'TvTunesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVTUNESDIALOG_H
#define UI_TVTUNESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include "ui/small_widgets/MyLineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_TvTunesDialog
{
public:
    QVBoxLayout *verticalLayout;
    MyLineEdit *searchString;
    QTableWidget *results;
    QHBoxLayout *horizontalLayout_7;
    QToolButton *btnPlayPause;
    QSlider *seekSlider;
    QLabel *time;
    QHBoxLayout *horizontalLayout_4;
    QProgressBar *progressBar;
    QLabel *progress;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *buttonDownload;
    QPushButton *buttonCancelDownload;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnClose;

    void setupUi(QDialog *TvTunesDialog)
    {
        if (TvTunesDialog->objectName().isEmpty())
            TvTunesDialog->setObjectName("TvTunesDialog");
        TvTunesDialog->setWindowModality(Qt::WindowModal);
        TvTunesDialog->resize(580, 487);
        TvTunesDialog->setModal(true);
        verticalLayout = new QVBoxLayout(TvTunesDialog);
        verticalLayout->setObjectName("verticalLayout");
        searchString = new MyLineEdit(TvTunesDialog);
        searchString->setObjectName("searchString");

        verticalLayout->addWidget(searchString);

        results = new QTableWidget(TvTunesDialog);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(results);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        btnPlayPause = new QToolButton(TvTunesDialog);
        btnPlayPause->setObjectName("btnPlayPause");
        btnPlayPause->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/video_play_64.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnPlayPause->setIcon(icon);

        horizontalLayout_7->addWidget(btnPlayPause);

        seekSlider = new QSlider(TvTunesDialog);
        seekSlider->setObjectName("seekSlider");
        seekSlider->setOrientation(Qt::Horizontal);

        horizontalLayout_7->addWidget(seekSlider);

        time = new QLabel(TvTunesDialog);
        time->setObjectName("time");
        time->setMinimumSize(QSize(80, 0));
        time->setText(QString::fromUtf8("0:00 / 0:00"));
        time->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_7->addWidget(time);


        verticalLayout->addLayout(horizontalLayout_7);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        progressBar = new QProgressBar(TvTunesDialog);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        horizontalLayout_4->addWidget(progressBar);

        progress = new QLabel(TvTunesDialog);
        progress->setObjectName("progress");
        progress->setMinimumSize(QSize(100, 0));
        progress->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_4->addWidget(progress);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);

        buttonDownload = new QPushButton(TvTunesDialog);
        buttonDownload->setObjectName("buttonDownload");

        horizontalLayout_4->addWidget(buttonDownload);

        buttonCancelDownload = new QPushButton(TvTunesDialog);
        buttonCancelDownload->setObjectName("buttonCancelDownload");

        horizontalLayout_4->addWidget(buttonCancelDownload);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnClose = new QPushButton(TvTunesDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(false);

        horizontalLayout->addWidget(btnClose);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(TvTunesDialog);

        QMetaObject::connectSlotsByName(TvTunesDialog);
    } // setupUi

    void retranslateUi(QDialog *TvTunesDialog)
    {
        TvTunesDialog->setWindowTitle(QCoreApplication::translate("TvTunesDialog", "TV Tunes", nullptr));
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("TvTunesDialog", "Result", nullptr));
        btnPlayPause->setText(QString());
        progress->setText(QCoreApplication::translate("TvTunesDialog", "Progress", nullptr));
        buttonDownload->setText(QCoreApplication::translate("TvTunesDialog", "Download", nullptr));
        buttonCancelDownload->setText(QCoreApplication::translate("TvTunesDialog", "Cancel Download", nullptr));
        btnClose->setText(QCoreApplication::translate("TvTunesDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TvTunesDialog: public Ui_TvTunesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVTUNESDIALOG_H
