/********************************************************************************
** Form generated from reading UI file 'ScraperSettingsTable.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRAPERSETTINGSTABLE_H
#define UI_SCRAPERSETTINGSTABLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ScraperSettingsTable
{
public:
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QListWidget *scraperList;
    QCheckBox *chkEnableAdultScrapers;
    QStackedWidget *scraperDetailPages;

    void setupUi(QWidget *ScraperSettingsTable)
    {
        if (ScraperSettingsTable->objectName().isEmpty())
            ScraperSettingsTable->setObjectName("ScraperSettingsTable");
        ScraperSettingsTable->resize(612, 439);
        horizontalLayout = new QHBoxLayout(ScraperSettingsTable);
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        scraperList = new QListWidget(ScraperSettingsTable);
        scraperList->setObjectName("scraperList");
        scraperList->setMinimumSize(QSize(120, 0));
        scraperList->setMaximumSize(QSize(220, 16777215));
        scraperList->setSpacing(2);
        scraperList->setWordWrap(true);

        verticalLayout_2->addWidget(scraperList);

        chkEnableAdultScrapers = new QCheckBox(ScraperSettingsTable);
        chkEnableAdultScrapers->setObjectName("chkEnableAdultScrapers");

        verticalLayout_2->addWidget(chkEnableAdultScrapers);


        horizontalLayout->addLayout(verticalLayout_2);

        scraperDetailPages = new QStackedWidget(ScraperSettingsTable);
        scraperDetailPages->setObjectName("scraperDetailPages");

        horizontalLayout->addWidget(scraperDetailPages);


        retranslateUi(ScraperSettingsTable);

        QMetaObject::connectSlotsByName(ScraperSettingsTable);
    } // setupUi

    void retranslateUi(QWidget *ScraperSettingsTable)
    {
        chkEnableAdultScrapers->setText(QCoreApplication::translate("ScraperSettingsTable", "Enable adult scrapers", nullptr));
        (void)ScraperSettingsTable;
    } // retranslateUi

};

namespace Ui {
    class ScraperSettingsTable: public Ui_ScraperSettingsTable {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCRAPERSETTINGSTABLE_H
