/********************************************************************************
** Form generated from reading UI file 'MusicFilesWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICFILESWIDGET_H
#define UI_MUSICFILESWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MusicTreeView.h"

QT_BEGIN_NAMESPACE

class Ui_MusicFilesWidget
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *statusLabelWidget;
    QVBoxLayout *verticalLayout_2;
    QLabel *statusLabel;
    MusicTreeView *music;

    void setupUi(QWidget *MusicFilesWidget)
    {
        if (MusicFilesWidget->objectName().isEmpty())
            MusicFilesWidget->setObjectName("MusicFilesWidget");
        MusicFilesWidget->resize(369, 676);
        MusicFilesWidget->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(MusicFilesWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        statusLabelWidget = new QWidget(MusicFilesWidget);
        statusLabelWidget->setObjectName("statusLabelWidget");
        verticalLayout_2 = new QVBoxLayout(statusLabelWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(6, 4, 6, 4);
        statusLabel = new QLabel(statusLabelWidget);
        statusLabel->setObjectName("statusLabel");
        QFont font;
        font.setBold(true);
        statusLabel->setFont(font);
        statusLabel->setText(QString::fromUtf8("12 artists, 34 albums"));
        statusLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        verticalLayout_2->addWidget(statusLabel);


        verticalLayout->addWidget(statusLabelWidget);

        music = new MusicTreeView(MusicFilesWidget);
        music->setObjectName("music");
        music->setMinimumSize(QSize(200, 0));
        music->setContextMenuPolicy(Qt::CustomContextMenu);
        music->setFrameShape(QFrame::NoFrame);
        music->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        music->setEditTriggers(QAbstractItemView::NoEditTriggers);
        music->setAlternatingRowColors(true);
        music->setSelectionMode(QAbstractItemView::ExtendedSelection);
        music->setAnimated(true);
        music->setHeaderHidden(true);
        music->setExpandsOnDoubleClick(true);

        verticalLayout->addWidget(music);

        verticalLayout->setStretch(1, 1);

        retranslateUi(MusicFilesWidget);

        QMetaObject::connectSlotsByName(MusicFilesWidget);
    } // setupUi

    void retranslateUi(QWidget *MusicFilesWidget)
    {
        (void)MusicFilesWidget;
    } // retranslateUi

};

namespace Ui {
    class MusicFilesWidget: public Ui_MusicFilesWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICFILESWIDGET_H
