/********************************************************************************
** Form generated from reading UI file 'ConcertWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONCERTWIDGET_H
#define UI_CONCERTWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/concerts/ConcertInfoWidget.h"
#include "ui/concerts/ConcertStreamDetailsWidget.h"
#include "ui/small_widgets/ClosableImage.h"
#include "ui/small_widgets/ImageGallery.h"
#include "ui/small_widgets/MediaFlags.h"
#include "ui/small_widgets/SlidingStackedWidget.h"
#include "ui/small_widgets/TagCloud.h"

QT_BEGIN_NAMESPACE

class Ui_ConcertWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer;
    QToolButton *buttonRevert;
    QLabel *concertName;
    QSpacerItem *horizontalSpacer_2;
    MediaFlags *mediaFlags;
    QToolButton *buttonPlay;
    QSpacerItem *hSpacerRight;
    QGroupBox *concertGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tabInformation;
    QVBoxLayout *verticalLayout_6;
    ConcertInfoWidget *concertInfo;
    QWidget *tabExtended;
    QVBoxLayout *verticalLayout_3;
    TagCloud *genreCloud;
    TagCloud *tagCloud;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabExtraFanart;
    QVBoxLayout *verticalLayout_2;
    ImageGallery *fanarts;
    QHBoxLayout *horizontalLayout_5;
    QLabel *lblExtraFanartHint;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *btnAddExtraFanart;
    QWidget *tabStreamdetails;
    QVBoxLayout *verticalLayout_7;
    ConcertStreamDetailsWidget *concertStreamdetails;
    QFrame *line;
    QVBoxLayout *verticalLayout_4;
    QWidget *artStackedWidgetButtons;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *buttonArtPageOne;
    QPushButton *buttonArtPageTwo;
    QSpacerItem *horizontalSpacer_8;
    SlidingStackedWidget *artStackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_10;
    QLabel *labelPoster;
    ClosableImage *poster;
    QLabel *labelFanart;
    ClosableImage *backdrop;
    QSpacerItem *verticalSpacer;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_11;
    QLabel *labelLogo;
    ClosableImage *logo;
    QLabel *labelClearArt;
    ClosableImage *clearArt;
    QLabel *labelDiscArt;
    ClosableImage *cdArt;
    QSpacerItem *verticalSpacer_7;

    void setupUi(QWidget *ConcertWidget)
    {
        if (ConcertWidget->objectName().isEmpty())
            ConcertWidget->setObjectName("ConcertWidget");
        ConcertWidget->resize(952, 804);
        verticalLayout = new QVBoxLayout(ConcertWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 20);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer);

        buttonRevert = new QToolButton(ConcertWidget);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);
        buttonRevert->setIconSize(QSize(16, 16));

        horizontalLayout_10->addWidget(buttonRevert);

        concertName = new QLabel(ConcertWidget);
        concertName->setObjectName("concertName");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        concertName->setFont(font);
        concertName->setIndent(4);
        concertName->setProperty("isHeader", QVariant(true));

        horizontalLayout_10->addWidget(concertName);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_2);

        mediaFlags = new MediaFlags(ConcertWidget);
        mediaFlags->setObjectName("mediaFlags");

        horizontalLayout_10->addWidget(mediaFlags);

        buttonPlay = new QToolButton(ConcertWidget);
        buttonPlay->setObjectName("buttonPlay");
        buttonPlay->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/actions/32/media-playback-start.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonPlay->setIcon(icon1);

        horizontalLayout_10->addWidget(buttonPlay);

        hSpacerRight = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(hSpacerRight);


        verticalLayout->addLayout(horizontalLayout_10);

        concertGroupBox = new QGroupBox(ConcertWidget);
        concertGroupBox->setObjectName("concertGroupBox");
        horizontalLayout_2 = new QHBoxLayout(concertGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(concertGroupBox);
        tabWidget->setObjectName("tabWidget");
        tabInformation = new QWidget();
        tabInformation->setObjectName("tabInformation");
        verticalLayout_6 = new QVBoxLayout(tabInformation);
        verticalLayout_6->setObjectName("verticalLayout_6");
        concertInfo = new ConcertInfoWidget(tabInformation);
        concertInfo->setObjectName("concertInfo");

        verticalLayout_6->addWidget(concertInfo);

        tabWidget->addTab(tabInformation, QString());
        tabExtended = new QWidget();
        tabExtended->setObjectName("tabExtended");
        verticalLayout_3 = new QVBoxLayout(tabExtended);
        verticalLayout_3->setObjectName("verticalLayout_3");
        genreCloud = new TagCloud(tabExtended);
        genreCloud->setObjectName("genreCloud");

        verticalLayout_3->addWidget(genreCloud);

        tagCloud = new TagCloud(tabExtended);
        tagCloud->setObjectName("tagCloud");

        verticalLayout_3->addWidget(tagCloud);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        tabWidget->addTab(tabExtended, QString());
        tabExtraFanart = new QWidget();
        tabExtraFanart->setObjectName("tabExtraFanart");
        verticalLayout_2 = new QVBoxLayout(tabExtraFanart);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(12, -1, 12, -1);
        fanarts = new ImageGallery(tabExtraFanart);
        fanarts->setObjectName("fanarts");

        verticalLayout_2->addWidget(fanarts);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        lblExtraFanartHint = new QLabel(tabExtraFanart);
        lblExtraFanartHint->setObjectName("lblExtraFanartHint");
        QFont font1;
        font1.setItalic(true);
        lblExtraFanartHint->setFont(font1);
        lblExtraFanartHint->setWordWrap(true);

        horizontalLayout_5->addWidget(lblExtraFanartHint);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        btnAddExtraFanart = new QPushButton(tabExtraFanart);
        btnAddExtraFanart->setObjectName("btnAddExtraFanart");

        horizontalLayout_5->addWidget(btnAddExtraFanart);

        horizontalLayout_5->setStretch(0, 1);

        verticalLayout_2->addLayout(horizontalLayout_5);

        verticalLayout_2->setStretch(0, 1);
        tabWidget->addTab(tabExtraFanart, QString());
        tabStreamdetails = new QWidget();
        tabStreamdetails->setObjectName("tabStreamdetails");
        verticalLayout_7 = new QVBoxLayout(tabStreamdetails);
        verticalLayout_7->setObjectName("verticalLayout_7");
        concertStreamdetails = new ConcertStreamDetailsWidget(tabStreamdetails);
        concertStreamdetails->setObjectName("concertStreamdetails");

        verticalLayout_7->addWidget(concertStreamdetails);

        tabWidget->addTab(tabStreamdetails, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(concertGroupBox);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Shadow::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(0);
        line->setFrameShape(QFrame::Shape::VLine);

        horizontalLayout_2->addWidget(line);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(10, 0, -1, -1);
        artStackedWidgetButtons = new QWidget(concertGroupBox);
        artStackedWidgetButtons->setObjectName("artStackedWidgetButtons");
        horizontalLayout_3 = new QHBoxLayout(artStackedWidgetButtons);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_7 = new QSpacerItem(69, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_7);

        buttonArtPageOne = new QPushButton(artStackedWidgetButtons);
        buttonArtPageOne->setObjectName("buttonArtPageOne");
        buttonArtPageOne->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonArtPageOne->setCheckable(true);
        buttonArtPageOne->setChecked(true);

        horizontalLayout_3->addWidget(buttonArtPageOne);

        buttonArtPageTwo = new QPushButton(artStackedWidgetButtons);
        buttonArtPageTwo->setObjectName("buttonArtPageTwo");
        buttonArtPageTwo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonArtPageTwo->setCheckable(true);

        horizontalLayout_3->addWidget(buttonArtPageTwo);

        horizontalSpacer_8 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_8);

        buttonArtPageTwo->raise();
        buttonArtPageOne->raise();

        verticalLayout_4->addWidget(artStackedWidgetButtons);

        artStackedWidget = new SlidingStackedWidget(concertGroupBox);
        artStackedWidget->setObjectName("artStackedWidget");
        artStackedWidget->setMinimumSize(QSize(180, 0));
        artStackedWidget->setMaximumSize(QSize(180, 16777215));
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_10 = new QVBoxLayout(page);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        labelPoster = new QLabel(page);
        labelPoster->setObjectName("labelPoster");

        verticalLayout_10->addWidget(labelPoster);

        poster = new ClosableImage(page);
        poster->setObjectName("poster");
        poster->setMinimumSize(QSize(180, 290));
        poster->setMaximumSize(QSize(180, 290));
        poster->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        poster->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/poster.png")));
        poster->setAlignment(Qt::AlignmentFlag::AlignCenter);
        poster->setProperty("clickable", QVariant(true));
        poster->setProperty("myFixedHeight", QVariant(290));

        verticalLayout_10->addWidget(poster);

        labelFanart = new QLabel(page);
        labelFanart->setObjectName("labelFanart");

        verticalLayout_10->addWidget(labelFanart);

        backdrop = new ClosableImage(page);
        backdrop->setObjectName("backdrop");
        backdrop->setMinimumSize(QSize(180, 120));
        backdrop->setMaximumSize(QSize(180, 120));
        backdrop->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        backdrop->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/fanart.png")));
        backdrop->setAlignment(Qt::AlignmentFlag::AlignCenter);
        backdrop->setProperty("clickable", QVariant(true));
        backdrop->setProperty("myFixedHeight", QVariant(120));

        verticalLayout_10->addWidget(backdrop);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_10->addItem(verticalSpacer);

        artStackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_11 = new QVBoxLayout(page_2);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_11->setContentsMargins(0, 0, 0, 0);
        labelLogo = new QLabel(page_2);
        labelLogo->setObjectName("labelLogo");

        verticalLayout_11->addWidget(labelLogo);

        logo = new ClosableImage(page_2);
        logo->setObjectName("logo");
        logo->setMinimumSize(QSize(180, 110));
        logo->setMaximumSize(QSize(180, 110));
        logo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        logo->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/logo.png")));
        logo->setAlignment(Qt::AlignmentFlag::AlignCenter);
        logo->setProperty("clickable", QVariant(true));
        logo->setProperty("myFixedHeight", QVariant(110));

        verticalLayout_11->addWidget(logo);

        labelClearArt = new QLabel(page_2);
        labelClearArt->setObjectName("labelClearArt");

        verticalLayout_11->addWidget(labelClearArt);

        clearArt = new ClosableImage(page_2);
        clearArt->setObjectName("clearArt");
        clearArt->setMinimumSize(QSize(180, 155));
        clearArt->setMaximumSize(QSize(180, 155));
        clearArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        clearArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/clear_art.png")));
        clearArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        clearArt->setProperty("clickable", QVariant(true));
        clearArt->setProperty("myFixedHeight", QVariant(155));

        verticalLayout_11->addWidget(clearArt);

        labelDiscArt = new QLabel(page_2);
        labelDiscArt->setObjectName("labelDiscArt");

        verticalLayout_11->addWidget(labelDiscArt);

        cdArt = new ClosableImage(page_2);
        cdArt->setObjectName("cdArt");
        cdArt->setMinimumSize(QSize(180, 200));
        cdArt->setMaximumSize(QSize(180, 200));
        cdArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        cdArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/cd_art.png")));
        cdArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        cdArt->setProperty("clickable", QVariant(true));
        cdArt->setProperty("myFixedHeight", QVariant(200));

        verticalLayout_11->addWidget(cdArt);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_11->addItem(verticalSpacer_7);

        artStackedWidget->addWidget(page_2);

        verticalLayout_4->addWidget(artStackedWidget);

        verticalLayout_4->setStretch(1, 1);

        horizontalLayout_2->addLayout(verticalLayout_4);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout->addWidget(concertGroupBox);

        verticalLayout->setStretch(1, 1);

        retranslateUi(ConcertWidget);
        QObject::connect(buttonArtPageOne, SIGNAL(clicked()), ConcertWidget, SLOT(onArtPageOne()));
        QObject::connect(buttonArtPageTwo, SIGNAL(clicked()), ConcertWidget, SLOT(onArtPageTwo()));

        tabWidget->setCurrentIndex(0);
        artStackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ConcertWidget);
    } // setupUi

    void retranslateUi(QWidget *ConcertWidget)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("ConcertWidget", "Concert has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        concertName->setText(QCoreApplication::translate("ConcertWidget", "Concert Name", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlay->setToolTip(QCoreApplication::translate("ConcertWidget", "Play movie", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlay->setText(QCoreApplication::translate("ConcertWidget", "...", nullptr));
        concertGroupBox->setTitle(QString());
        tabWidget->setTabText(tabWidget->indexOf(tabInformation), QCoreApplication::translate("ConcertWidget", "Information", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtended), QCoreApplication::translate("ConcertWidget", "Extended", nullptr));
        lblExtraFanartHint->setText(QCoreApplication::translate("ConcertWidget", "Hint: Closed images will be deleted on save.", nullptr));
        btnAddExtraFanart->setText(QCoreApplication::translate("ConcertWidget", "Add Images", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtraFanart), QCoreApplication::translate("ConcertWidget", "Extra Fanarts", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabStreamdetails), QCoreApplication::translate("ConcertWidget", "Streamdetails", nullptr));
        buttonArtPageOne->setText(QString());
        buttonArtPageTwo->setText(QString());
        labelPoster->setText(QCoreApplication::translate("ConcertWidget", "Poster", nullptr));
#if QT_CONFIG(tooltip)
        poster->setToolTip(QCoreApplication::translate("ConcertWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        poster->setText(QString());
        labelFanart->setText(QCoreApplication::translate("ConcertWidget", "Fanart", nullptr));
#if QT_CONFIG(tooltip)
        backdrop->setToolTip(QCoreApplication::translate("ConcertWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        backdrop->setText(QString());
        labelLogo->setText(QCoreApplication::translate("ConcertWidget", "Logo", nullptr));
#if QT_CONFIG(tooltip)
        logo->setToolTip(QCoreApplication::translate("ConcertWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        logo->setText(QString());
        labelClearArt->setText(QCoreApplication::translate("ConcertWidget", "Clear Art", nullptr));
#if QT_CONFIG(tooltip)
        clearArt->setToolTip(QCoreApplication::translate("ConcertWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        clearArt->setText(QString());
        labelDiscArt->setText(QCoreApplication::translate("ConcertWidget", "CD Art", nullptr));
#if QT_CONFIG(tooltip)
        cdArt->setToolTip(QCoreApplication::translate("ConcertWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        cdArt->setText(QString());
        (void)ConcertWidget;
    } // retranslateUi

};

namespace Ui {
    class ConcertWidget: public Ui_ConcertWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONCERTWIDGET_H
