/****************************************************************************
** Meta object code from reading C++ file 'TvShowWidgetSeason.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowWidgetSeason.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowWidgetSeason.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TvShowWidgetSeasonE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN18TvShowWidgetSeasonE = QtMocHelpers::stringData(
    "TvShowWidgetSeason",
    "sigSetActionSearchEnabled",
    "",
    "MainWidgets",
    "sigSetActionSaveEnabled",
    "onClear",
    "onSaveInformation",
    "onSetEnabled",
    "enabled",
    "onNameChange",
    "name",
    "onChooseImage",
    "onDeleteImage",
    "onImageDropped",
    "ImageType",
    "imageType",
    "imageUrl",
    "onRevertChanges",
    "onDownloadFinished",
    "DownloadManagerElement",
    "elem"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN18TvShowWidgetSeasonE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,   80,    2, 0x06,    1 /* Public */,
       4,    2,   85,    2, 0x06,    4 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       5,    0,   90,    2, 0x0a,    7 /* Public */,
       6,    0,   91,    2, 0x0a,    8 /* Public */,
       7,    1,   92,    2, 0x0a,    9 /* Public */,
       9,    1,   95,    2, 0x0a,   11 /* Public */,
      11,    0,   98,    2, 0x08,   13 /* Private */,
      12,    0,   99,    2, 0x08,   14 /* Private */,
      13,    2,  100,    2, 0x08,   15 /* Private */,
      17,    0,  105,    2, 0x08,   18 /* Private */,
      18,    1,  106,    2, 0x08,   19 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14, QMetaType::QUrl,   15,   16,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,   20,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowWidgetSeason::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN18TvShowWidgetSeasonE.offsetsAndSizes,
    qt_meta_data_ZN18TvShowWidgetSeasonE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN18TvShowWidgetSeasonE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowWidgetSeason, std::true_type>,
        // method 'sigSetActionSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigSetActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'onClear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSetEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onNameChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onChooseImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDeleteImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImageDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QUrl, std::false_type>,
        // method 'onRevertChanges'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>
    >,
    nullptr
} };

void TvShowWidgetSeason::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowWidgetSeason *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigSetActionSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->sigSetActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 2: _t->onClear(); break;
        case 3: _t->onSaveInformation(); break;
        case 4: _t->onSetEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->onNameChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->onChooseImage(); break;
        case 7: _t->onDeleteImage(); break;
        case 8: _t->onImageDropped((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 9: _t->onRevertChanges(); break;
        case 10: _t->onDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (TvShowWidgetSeason::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidgetSeason::sigSetActionSearchEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidgetSeason::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidgetSeason::sigSetActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *TvShowWidgetSeason::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowWidgetSeason::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN18TvShowWidgetSeasonE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TvShowWidgetSeason::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TvShowWidgetSeason::sigSetActionSearchEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TvShowWidgetSeason::sigSetActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
