/********************************************************************************
** Form generated from reading UI file 'MovieListDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIELISTDIALOG_H
#define UI_MOVIELISTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include "ui/small_widgets/MyLineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_MovieListDialog
{
public:
    QVBoxLayout *verticalLayout;
    MyLineEdit *filter;
    QTableWidget *movies;
    QHBoxLayout *horizontalLayout;
    QPushButton *buttonAddMovies;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClose;

    void setupUi(QDialog *MovieListDialog)
    {
        if (MovieListDialog->objectName().isEmpty())
            MovieListDialog->setObjectName("MovieListDialog");
        MovieListDialog->setWindowModality(Qt::WindowModal);
        MovieListDialog->resize(654, 454);
        MovieListDialog->setModal(true);
        verticalLayout = new QVBoxLayout(MovieListDialog);
        verticalLayout->setObjectName("verticalLayout");
        filter = new MyLineEdit(MovieListDialog);
        filter->setObjectName("filter");

        verticalLayout->addWidget(filter);

        movies = new QTableWidget(MovieListDialog);
        if (movies->columnCount() < 1)
            movies->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        movies->setHorizontalHeaderItem(0, __qtablewidgetitem);
        movies->setObjectName("movies");
        movies->setEditTriggers(QAbstractItemView::NoEditTriggers);
        movies->setAlternatingRowColors(true);
        movies->setSelectionMode(QAbstractItemView::MultiSelection);
        movies->setSelectionBehavior(QAbstractItemView::SelectRows);
        movies->setShowGrid(false);
        movies->horizontalHeader()->setVisible(false);
        movies->horizontalHeader()->setStretchLastSection(true);
        movies->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(movies);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        buttonAddMovies = new QPushButton(MovieListDialog);
        buttonAddMovies->setObjectName("buttonAddMovies");

        horizontalLayout->addWidget(buttonAddMovies);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonClose = new QPushButton(MovieListDialog);
        buttonClose->setObjectName("buttonClose");
        buttonClose->setAutoDefault(false);

        horizontalLayout->addWidget(buttonClose);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(MovieListDialog);

        QMetaObject::connectSlotsByName(MovieListDialog);
    } // setupUi

    void retranslateUi(QDialog *MovieListDialog)
    {
        MovieListDialog->setWindowTitle(QCoreApplication::translate("MovieListDialog", "Choose a movie", nullptr));
        filter->setPlaceholderText(QCoreApplication::translate("MovieListDialog", "Filter", nullptr));
        QTableWidgetItem *___qtablewidgetitem = movies->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("MovieListDialog", "Result", nullptr));
        buttonAddMovies->setText(QCoreApplication::translate("MovieListDialog", "Add selected movies", nullptr));
        buttonClose->setText(QCoreApplication::translate("MovieListDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MovieListDialog: public Ui_MovieListDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIELISTDIALOG_H
