/********************************************************************************
** Form generated from reading UI file 'UnpackButtons.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UNPACKBUTTONS_H
#define UI_UNPACKBUTTONS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_UnpackButtons
{
public:
    QHBoxLayout *horizontalLayout;
    QProgressBar *progressBar;
    QPushButton *btnStop;
    QPushButton *btnUnpack;
    QPushButton *btnUnpackWithPassword;
    QPushButton *btnDelete;

    void setupUi(QWidget *UnpackButtons)
    {
        if (UnpackButtons->objectName().isEmpty())
            UnpackButtons->setObjectName("UnpackButtons");
        UnpackButtons->resize(470, 300);
        horizontalLayout = new QHBoxLayout(UnpackButtons);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        progressBar = new QProgressBar(UnpackButtons);
        progressBar->setObjectName("progressBar");
        progressBar->setMinimum(0);
        progressBar->setMaximum(100);
        progressBar->setValue(24);
        progressBar->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        progressBar->setOrientation(Qt::Horizontal);
        progressBar->setFormat(QString::fromUtf8("%p%"));

        horizontalLayout->addWidget(progressBar);

        btnStop = new QPushButton(UnpackButtons);
        btnStop->setObjectName("btnStop");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/stop_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnStop->setIcon(icon);
        btnStop->setIconSize(QSize(12, 12));

        horizontalLayout->addWidget(btnStop);

        btnUnpack = new QPushButton(UnpackButtons);
        btnUnpack->setObjectName("btnUnpack");

        horizontalLayout->addWidget(btnUnpack);

        btnUnpackWithPassword = new QPushButton(UnpackButtons);
        btnUnpackWithPassword->setObjectName("btnUnpackWithPassword");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/lock_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnUnpackWithPassword->setIcon(icon1);
        btnUnpackWithPassword->setIconSize(QSize(12, 12));

        horizontalLayout->addWidget(btnUnpackWithPassword);

        btnDelete = new QPushButton(UnpackButtons);
        btnDelete->setObjectName("btnDelete");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/img/trash_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnDelete->setIcon(icon2);
        btnDelete->setIconSize(QSize(12, 12));

        horizontalLayout->addWidget(btnDelete);


        retranslateUi(UnpackButtons);

        QMetaObject::connectSlotsByName(UnpackButtons);
    } // setupUi

    void retranslateUi(QWidget *UnpackButtons)
    {
#if QT_CONFIG(tooltip)
        btnStop->setToolTip(QCoreApplication::translate("UnpackButtons", "Cancel extraction", nullptr));
#endif // QT_CONFIG(tooltip)
        btnStop->setText(QString());
#if QT_CONFIG(tooltip)
        btnUnpack->setToolTip(QCoreApplication::translate("UnpackButtons", "Extract without password", nullptr));
#endif // QT_CONFIG(tooltip)
        btnUnpack->setText(QCoreApplication::translate("UnpackButtons", "Extract", nullptr));
#if QT_CONFIG(tooltip)
        btnUnpackWithPassword->setToolTip(QCoreApplication::translate("UnpackButtons", "Extract with password", nullptr));
#endif // QT_CONFIG(tooltip)
        btnUnpackWithPassword->setText(QString());
#if QT_CONFIG(tooltip)
        btnDelete->setToolTip(QCoreApplication::translate("UnpackButtons", "Delete this archive", nullptr));
#endif // QT_CONFIG(tooltip)
        btnDelete->setText(QString());
        (void)UnpackButtons;
    } // retranslateUi

};

namespace Ui {
    class UnpackButtons: public Ui_UnpackButtons {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UNPACKBUTTONS_H
