/********************************************************************************
** Form generated from reading UI file 'MovieFilesWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIEFILESWIDGET_H
#define UI_MOVIEFILESWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyLabel.h"
#include "ui/small_widgets/MyTableView.h"

QT_BEGIN_NAMESPACE

class Ui_MovieFilesWidget
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *statusLabelWidget;
    QVBoxLayout *verticalLayout_2;
    QLabel *statusLabel;
    MyTableView *files;
    QWidget *sortLabelWidget;
    QHBoxLayout *hlayoutSortLabelWidget;
    MyLabel *sortByNew;
    MyLabel *sortByName;
    MyLabel *sortByLastAdded;
    MyLabel *sortBySeen;
    MyLabel *sortByYear;

    void setupUi(QWidget *MovieFilesWidget)
    {
        if (MovieFilesWidget->objectName().isEmpty())
            MovieFilesWidget->setObjectName("MovieFilesWidget");
        MovieFilesWidget->resize(565, 587);
        verticalLayout = new QVBoxLayout(MovieFilesWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        statusLabelWidget = new QWidget(MovieFilesWidget);
        statusLabelWidget->setObjectName("statusLabelWidget");
        verticalLayout_2 = new QVBoxLayout(statusLabelWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(6, 4, 6, 4);
        statusLabel = new QLabel(statusLabelWidget);
        statusLabel->setObjectName("statusLabel");
        QFont font;
        font.setBold(true);
        statusLabel->setFont(font);
        statusLabel->setText(QString::fromUtf8("0 movies"));
        statusLabel->setTextFormat(Qt::PlainText);
        statusLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        verticalLayout_2->addWidget(statusLabel);


        verticalLayout->addWidget(statusLabelWidget);

        files = new MyTableView(MovieFilesWidget);
        files->setObjectName("files");
        files->setMinimumSize(QSize(200, 0));
        files->setMouseTracking(true);
        files->setContextMenuPolicy(Qt::CustomContextMenu);
        files->setFrameShape(QFrame::NoFrame);
        files->setEditTriggers(QAbstractItemView::NoEditTriggers);
        files->setAlternatingRowColors(true);
        files->setSelectionMode(QAbstractItemView::ExtendedSelection);
        files->setSelectionBehavior(QAbstractItemView::SelectRows);
        files->setShowGrid(false);
        files->setSortingEnabled(true);
        files->setProperty("useSearchOverlay", QVariant(true));
        files->horizontalHeader()->setVisible(false);
        files->horizontalHeader()->setMinimumSectionSize(22);
        files->verticalHeader()->setVisible(false);
        files->verticalHeader()->setMinimumSectionSize(22);
        files->verticalHeader()->setDefaultSectionSize(22);

        verticalLayout->addWidget(files);

        sortLabelWidget = new QWidget(MovieFilesWidget);
        sortLabelWidget->setObjectName("sortLabelWidget");
        hlayoutSortLabelWidget = new QHBoxLayout(sortLabelWidget);
        hlayoutSortLabelWidget->setSpacing(0);
        hlayoutSortLabelWidget->setObjectName("hlayoutSortLabelWidget");
        hlayoutSortLabelWidget->setContentsMargins(0, 0, 0, 0);
        sortByNew = new MyLabel(sortLabelWidget);
        sortByNew->setObjectName("sortByNew");
        sortByNew->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        sortByNew->setAlignment(Qt::AlignCenter);
        sortByNew->setProperty("active", QVariant(true));

        hlayoutSortLabelWidget->addWidget(sortByNew);

        sortByName = new MyLabel(sortLabelWidget);
        sortByName->setObjectName("sortByName");
        sortByName->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        sortByName->setAlignment(Qt::AlignCenter);
        sortByName->setProperty("active", QVariant(false));

        hlayoutSortLabelWidget->addWidget(sortByName);

        sortByLastAdded = new MyLabel(sortLabelWidget);
        sortByLastAdded->setObjectName("sortByLastAdded");
        sortByLastAdded->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        sortByLastAdded->setAlignment(Qt::AlignCenter);
        sortByLastAdded->setProperty("active", QVariant(false));

        hlayoutSortLabelWidget->addWidget(sortByLastAdded);

        sortBySeen = new MyLabel(sortLabelWidget);
        sortBySeen->setObjectName("sortBySeen");
        sortBySeen->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        sortBySeen->setAlignment(Qt::AlignCenter);
        sortBySeen->setProperty("active", QVariant(false));

        hlayoutSortLabelWidget->addWidget(sortBySeen);

        sortByYear = new MyLabel(sortLabelWidget);
        sortByYear->setObjectName("sortByYear");
        sortByYear->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        sortByYear->setAlignment(Qt::AlignCenter);
        sortByYear->setProperty("active", QVariant(false));

        hlayoutSortLabelWidget->addWidget(sortByYear);


        verticalLayout->addWidget(sortLabelWidget);


        retranslateUi(MovieFilesWidget);

        QMetaObject::connectSlotsByName(MovieFilesWidget);
    } // setupUi

    void retranslateUi(QWidget *MovieFilesWidget)
    {
        sortByNew->setText(QCoreApplication::translate("MovieFilesWidget", "New", nullptr));
        sortByName->setText(QCoreApplication::translate("MovieFilesWidget", "Name", nullptr));
        sortByLastAdded->setText(QCoreApplication::translate("MovieFilesWidget", "Date Added", nullptr));
        sortBySeen->setText(QCoreApplication::translate("MovieFilesWidget", "Seen", nullptr));
        sortByYear->setText(QCoreApplication::translate("MovieFilesWidget", "Year", nullptr));
        (void)MovieFilesWidget;
    } // retranslateUi

};

namespace Ui {
    class MovieFilesWidget: public Ui_MovieFilesWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIEFILESWIDGET_H
