/********************************************************************************
** Form generated from reading UI file 'MusicSearch.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICSEARCH_H
#define UI_MUSICSEARCH_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "ui/music/MusicSearchWidget.h"

QT_BEGIN_NAMESPACE

class Ui_MusicSearch
{
public:
    QVBoxLayout *verticalLayout;
    MusicSearchWidget *musicSearchWidget;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClose;

    void setupUi(QDialog *MusicSearch)
    {
        if (MusicSearch->objectName().isEmpty())
            MusicSearch->setObjectName("MusicSearch");
        MusicSearch->setWindowModality(Qt::WindowModal);
        MusicSearch->resize(868, 644);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MusicSearch->setWindowIcon(icon);
        MusicSearch->setModal(true);
        verticalLayout = new QVBoxLayout(MusicSearch);
        verticalLayout->setObjectName("verticalLayout");
        musicSearchWidget = new MusicSearchWidget(MusicSearch);
        musicSearchWidget->setObjectName("musicSearchWidget");

        verticalLayout->addWidget(musicSearchWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonClose = new QPushButton(MusicSearch);
        buttonClose->setObjectName("buttonClose");
        buttonClose->setAutoDefault(false);

        horizontalLayout->addWidget(buttonClose);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(0, 1);

        retranslateUi(MusicSearch);

        QMetaObject::connectSlotsByName(MusicSearch);
    } // setupUi

    void retranslateUi(QDialog *MusicSearch)
    {
        MusicSearch->setWindowTitle(QCoreApplication::translate("MusicSearch", "Search Result", nullptr));
        buttonClose->setText(QCoreApplication::translate("MusicSearch", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MusicSearch: public Ui_MusicSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICSEARCH_H
