/********************************************************************************
** Form generated from reading UI file 'SupportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SUPPORTDIALOG_H
#define UI_SUPPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SupportDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLabel *label_2;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_3;
    QLabel *label_4;
    QLabel *label_5;
    QSpacerItem *verticalSpacer_3;
    QLabel *label_6;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnAlreadyDonated;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClose;

    void setupUi(QDialog *SupportDialog)
    {
        if (SupportDialog->objectName().isEmpty())
            SupportDialog->setObjectName("SupportDialog");
        SupportDialog->setWindowModality(Qt::WindowModal);
        SupportDialog->resize(597, 361);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        SupportDialog->setWindowIcon(icon);
        SupportDialog->setModal(true);
        verticalLayout = new QVBoxLayout(SupportDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(SupportDialog);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        label_2 = new QLabel(SupportDialog);
        label_2->setObjectName("label_2");
        label_2->setText(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Helvetica'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&amp;hosted_button_id=6EU9GPSQ8PSUA\"><img src=\":/img/btn_donate_LG.gif\" /></a></p>\n"
"</body></html>"));
        label_2->setTextFormat(Qt::RichText);
        label_2->setOpenExternalLinks(true);

        verticalLayout->addWidget(label_2);

        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_2);

        label_3 = new QLabel(SupportDialog);
        label_3->setObjectName("label_3");
        label_3->setWordWrap(true);

        verticalLayout->addWidget(label_3);

        label_4 = new QLabel(SupportDialog);
        label_4->setObjectName("label_4");
        label_4->setText(QString::fromUtf8("<a href=\"https://www.themoviedb.org\">www.themoviedb.org</a>\n"
""));
        label_4->setOpenExternalLinks(true);

        verticalLayout->addWidget(label_4);

        label_5 = new QLabel(SupportDialog);
        label_5->setObjectName("label_5");
        label_5->setText(QString::fromUtf8("<a href=\"https://www.thetvdb.com\">TheTVDB.com</a>"));
        label_5->setOpenExternalLinks(true);

        verticalLayout->addWidget(label_5);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_3);

        label_6 = new QLabel(SupportDialog);
        label_6->setObjectName("label_6");
        QFont font;
        font.setBold(true);
        label_6->setFont(font);

        verticalLayout->addWidget(label_6);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnAlreadyDonated = new QPushButton(SupportDialog);
        btnAlreadyDonated->setObjectName("btnAlreadyDonated");
        btnAlreadyDonated->setAutoDefault(false);

        horizontalLayout->addWidget(btnAlreadyDonated);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonClose = new QPushButton(SupportDialog);
        buttonClose->setObjectName("buttonClose");

        horizontalLayout->addWidget(buttonClose);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(SupportDialog);
        QObject::connect(buttonClose, &QPushButton::clicked, SupportDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(SupportDialog);
    } // setupUi

    void retranslateUi(QDialog *SupportDialog)
    {
        SupportDialog->setWindowTitle(QCoreApplication::translate("SupportDialog", "Support MediaElch", nullptr));
        label->setText(QCoreApplication::translate("SupportDialog", "If you like MediaElch and wish to support it you have the chance to do so! You can donate as much as you like via PayPal.", nullptr));
        label_3->setText(QCoreApplication::translate("SupportDialog", "MediaElch makes use of various movie and TV show databases. These databases also need your help to keep their services up and running for free. If you don't want to donate you can also contribute information and missing artwork if possible.", nullptr));
        label_6->setText(QCoreApplication::translate("SupportDialog", "Thanks for your help and support!", nullptr));
        btnAlreadyDonated->setText(QCoreApplication::translate("SupportDialog", "Hide donate button", nullptr));
        buttonClose->setText(QCoreApplication::translate("SupportDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SupportDialog: public Ui_SupportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SUPPORTDIALOG_H
