/********************************************************************************
** Form generated from reading UI file 'TvShowSearchWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWSEARCHWIDGET_H
#define UI_TVSHOWSEARCHWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/LanguageCombo.h"
#include "ui/small_widgets/MyCheckBox.h"
#include "ui/small_widgets/MyLineEdit.h"
#include "ui/small_widgets/ScrapePreview.h"

QT_BEGIN_NAMESPACE

class Ui_TvShowSearchWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hLayoutSearchConfig;
    QLabel *lblScraper;
    QComboBox *comboScraper;
    QLabel *lblLanguage;
    LanguageCombo *comboLanguage;
    QSpacerItem *hSpacerDropdowns;
    MyLineEdit *searchString;
    QHBoxLayout *horizontalLayout_4;
    QLabel *lblSuccessMessage;
    QLabel *lblErrorMessage;
    QHBoxLayout *horizontalLayout;
    QSplitter *splitter;
    QTableWidget *results;
    ScrapePreview *tvShowPreview;
    QVBoxLayout *vboxDetails;
    QComboBox *comboUpdate;
    QTabWidget *tabsInfos;
    QWidget *tabShowDetails;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *showInfosGroupBox;
    QVBoxLayout *vboxShowDetails;
    QHBoxLayout *verticalLayout_41;
    QVBoxLayout *verticalLayout_421;
    MyCheckBox *chkTitle;
    MyCheckBox *chkStatus;
    MyCheckBox *chkRating;
    MyCheckBox *chkFirstAired;
    MyCheckBox *chkRuntime;
    MyCheckBox *chkCertification;
    MyCheckBox *chkOverview;
    MyCheckBox *chkGenres;
    MyCheckBox *chkTags;
    MyCheckBox *chkNetwork;
    QSpacerItem *verticalSpacer_31;
    QVBoxLayout *verticalLayout_43;
    MyCheckBox *chkActors;
    MyCheckBox *chkFanart;
    MyCheckBox *chkPoster;
    MyCheckBox *chkSeasonPoster;
    MyCheckBox *chkSeasonFanart;
    MyCheckBox *chkSeasonBanner;
    MyCheckBox *chkSeasonThumb;
    MyCheckBox *chkBanner;
    MyCheckBox *chkExtraArts;
    MyCheckBox *chkThumb;
    QSpacerItem *verticalSpacer_32;
    QFrame *line2;
    QCheckBox *chkUnCheckAll;
    QLabel *lblEpisodeOnlyType;
    QSpacerItem *verticalSpacer_313;
    QWidget *tabEpisodeDetails;
    QVBoxLayout *verticalLayout_72;
    QGroupBox *episodeInfosGroupBox;
    QVBoxLayout *vboxEpisodeDetails;
    QHBoxLayout *verticalLayout_412;
    QVBoxLayout *verticalLayout_422;
    MyCheckBox *chkEpisodeTitle;
    MyCheckBox *chkEpisodeOverview;
    MyCheckBox *chkEpisodeRating;
    MyCheckBox *chkEpisodeFirstAired;
    MyCheckBox *chkEpisodeCertification;
    QSpacerItem *verticalSpacer_312;
    QVBoxLayout *verticalLayout_431;
    MyCheckBox *chkEpisodeDirector;
    MyCheckBox *chkEpisodeWriter;
    MyCheckBox *chkEpisodeNetwork;
    MyCheckBox *chkEpisodeActors;
    MyCheckBox *chkEpisodeTags;
    MyCheckBox *chkEpisodeThumbnail;
    QSpacerItem *verticalSpacer_322;
    QFrame *line3;
    QCheckBox *chkEpisodeUnCheckAll;
    QFormLayout *formLayout;
    QLabel *lblSeasonOrder;
    QComboBox *comboSeasonOrder;
    QLabel *lblShowOnlyType;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *TvShowSearchWidget)
    {
        if (TvShowSearchWidget->objectName().isEmpty())
            TvShowSearchWidget->setObjectName("TvShowSearchWidget");
        TvShowSearchWidget->resize(740, 600);
        verticalLayout = new QVBoxLayout(TvShowSearchWidget);
        verticalLayout->setObjectName("verticalLayout");
        hLayoutSearchConfig = new QHBoxLayout();
        hLayoutSearchConfig->setObjectName("hLayoutSearchConfig");
        lblScraper = new QLabel(TvShowSearchWidget);
        lblScraper->setObjectName("lblScraper");

        hLayoutSearchConfig->addWidget(lblScraper);

        comboScraper = new QComboBox(TvShowSearchWidget);
        comboScraper->setObjectName("comboScraper");

        hLayoutSearchConfig->addWidget(comboScraper);

        lblLanguage = new QLabel(TvShowSearchWidget);
        lblLanguage->setObjectName("lblLanguage");

        hLayoutSearchConfig->addWidget(lblLanguage);

        comboLanguage = new LanguageCombo(TvShowSearchWidget);
        comboLanguage->setObjectName("comboLanguage");
        comboLanguage->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        hLayoutSearchConfig->addWidget(comboLanguage);

        hSpacerDropdowns = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hLayoutSearchConfig->addItem(hSpacerDropdowns);


        verticalLayout->addLayout(hLayoutSearchConfig);

        searchString = new MyLineEdit(TvShowSearchWidget);
        searchString->setObjectName("searchString");

        verticalLayout->addWidget(searchString);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        lblSuccessMessage = new QLabel(TvShowSearchWidget);
        lblSuccessMessage->setObjectName("lblSuccessMessage");
        lblSuccessMessage->setText(QString::fromUtf8(""));
        lblSuccessMessage->setTextFormat(Qt::PlainText);

        horizontalLayout_4->addWidget(lblSuccessMessage);

        lblErrorMessage = new QLabel(TvShowSearchWidget);
        lblErrorMessage->setObjectName("lblErrorMessage");
        lblErrorMessage->setText(QString::fromUtf8(""));
        lblErrorMessage->setTextFormat(Qt::PlainText);

        horizontalLayout_4->addWidget(lblErrorMessage);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        splitter = new QSplitter(TvShowSearchWidget);
        splitter->setObjectName("splitter");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(splitter->sizePolicy().hasHeightForWidth());
        splitter->setSizePolicy(sizePolicy);
        splitter->setOrientation(Qt::Vertical);
        splitter->setOpaqueResize(false);
        splitter->setChildrenCollapsible(false);
        results = new QTableWidget(splitter);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(results->sizePolicy().hasHeightForWidth());
        results->setSizePolicy(sizePolicy1);
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        splitter->addWidget(results);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);
        tvShowPreview = new ScrapePreview(splitter);
        tvShowPreview->setObjectName("tvShowPreview");
        splitter->addWidget(tvShowPreview);

        horizontalLayout->addWidget(splitter);

        vboxDetails = new QVBoxLayout();
        vboxDetails->setObjectName("vboxDetails");
        comboUpdate = new QComboBox(TvShowSearchWidget);
        comboUpdate->addItem(QString());
        comboUpdate->addItem(QString());
        comboUpdate->addItem(QString());
        comboUpdate->addItem(QString());
        comboUpdate->addItem(QString());
        comboUpdate->setObjectName("comboUpdate");

        vboxDetails->addWidget(comboUpdate);

        tabsInfos = new QTabWidget(TvShowSearchWidget);
        tabsInfos->setObjectName("tabsInfos");
        tabShowDetails = new QWidget();
        tabShowDetails->setObjectName("tabShowDetails");
        verticalLayout_6 = new QVBoxLayout(tabShowDetails);
        verticalLayout_6->setObjectName("verticalLayout_6");
        showInfosGroupBox = new QGroupBox(tabShowDetails);
        showInfosGroupBox->setObjectName("showInfosGroupBox");
        showInfosGroupBox->setFlat(true);
        vboxShowDetails = new QVBoxLayout(showInfosGroupBox);
        vboxShowDetails->setObjectName("vboxShowDetails");
        vboxShowDetails->setContentsMargins(0, 0, 0, 0);
        verticalLayout_41 = new QHBoxLayout();
        verticalLayout_41->setObjectName("verticalLayout_41");
        verticalLayout_421 = new QVBoxLayout();
        verticalLayout_421->setObjectName("verticalLayout_421");
        chkTitle = new MyCheckBox(showInfosGroupBox);
        chkTitle->setObjectName("chkTitle");
        chkTitle->setChecked(true);

        verticalLayout_421->addWidget(chkTitle);

        chkStatus = new MyCheckBox(showInfosGroupBox);
        chkStatus->setObjectName("chkStatus");
        chkStatus->setChecked(true);

        verticalLayout_421->addWidget(chkStatus);

        chkRating = new MyCheckBox(showInfosGroupBox);
        chkRating->setObjectName("chkRating");
        chkRating->setChecked(true);

        verticalLayout_421->addWidget(chkRating);

        chkFirstAired = new MyCheckBox(showInfosGroupBox);
        chkFirstAired->setObjectName("chkFirstAired");
        chkFirstAired->setChecked(true);

        verticalLayout_421->addWidget(chkFirstAired);

        chkRuntime = new MyCheckBox(showInfosGroupBox);
        chkRuntime->setObjectName("chkRuntime");
        chkRuntime->setChecked(true);

        verticalLayout_421->addWidget(chkRuntime);

        chkCertification = new MyCheckBox(showInfosGroupBox);
        chkCertification->setObjectName("chkCertification");
        chkCertification->setChecked(true);

        verticalLayout_421->addWidget(chkCertification);

        chkOverview = new MyCheckBox(showInfosGroupBox);
        chkOverview->setObjectName("chkOverview");
        chkOverview->setChecked(true);

        verticalLayout_421->addWidget(chkOverview);

        chkGenres = new MyCheckBox(showInfosGroupBox);
        chkGenres->setObjectName("chkGenres");
        chkGenres->setChecked(true);

        verticalLayout_421->addWidget(chkGenres);

        chkTags = new MyCheckBox(showInfosGroupBox);
        chkTags->setObjectName("chkTags");
        chkTags->setChecked(true);

        verticalLayout_421->addWidget(chkTags);

        chkNetwork = new MyCheckBox(showInfosGroupBox);
        chkNetwork->setObjectName("chkNetwork");
        chkNetwork->setChecked(true);

        verticalLayout_421->addWidget(chkNetwork);

        verticalSpacer_31 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_421->addItem(verticalSpacer_31);


        verticalLayout_41->addLayout(verticalLayout_421);

        verticalLayout_43 = new QVBoxLayout();
        verticalLayout_43->setObjectName("verticalLayout_43");
        chkActors = new MyCheckBox(showInfosGroupBox);
        chkActors->setObjectName("chkActors");
        chkActors->setChecked(true);

        verticalLayout_43->addWidget(chkActors);

        chkFanart = new MyCheckBox(showInfosGroupBox);
        chkFanart->setObjectName("chkFanart");
        chkFanart->setChecked(true);

        verticalLayout_43->addWidget(chkFanart);

        chkPoster = new MyCheckBox(showInfosGroupBox);
        chkPoster->setObjectName("chkPoster");
        chkPoster->setChecked(true);

        verticalLayout_43->addWidget(chkPoster);

        chkSeasonPoster = new MyCheckBox(showInfosGroupBox);
        chkSeasonPoster->setObjectName("chkSeasonPoster");
        chkSeasonPoster->setChecked(true);

        verticalLayout_43->addWidget(chkSeasonPoster);

        chkSeasonFanart = new MyCheckBox(showInfosGroupBox);
        chkSeasonFanart->setObjectName("chkSeasonFanart");
        chkSeasonFanart->setChecked(true);

        verticalLayout_43->addWidget(chkSeasonFanart);

        chkSeasonBanner = new MyCheckBox(showInfosGroupBox);
        chkSeasonBanner->setObjectName("chkSeasonBanner");
        chkSeasonBanner->setChecked(true);

        verticalLayout_43->addWidget(chkSeasonBanner);

        chkSeasonThumb = new MyCheckBox(showInfosGroupBox);
        chkSeasonThumb->setObjectName("chkSeasonThumb");
        chkSeasonThumb->setChecked(true);

        verticalLayout_43->addWidget(chkSeasonThumb);

        chkBanner = new MyCheckBox(showInfosGroupBox);
        chkBanner->setObjectName("chkBanner");
        chkBanner->setChecked(true);

        verticalLayout_43->addWidget(chkBanner);

        chkExtraArts = new MyCheckBox(showInfosGroupBox);
        chkExtraArts->setObjectName("chkExtraArts");
        chkExtraArts->setChecked(true);

        verticalLayout_43->addWidget(chkExtraArts);

        chkThumb = new MyCheckBox(showInfosGroupBox);
        chkThumb->setObjectName("chkThumb");
        chkThumb->setChecked(true);

        verticalLayout_43->addWidget(chkThumb);

        verticalSpacer_32 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_43->addItem(verticalSpacer_32);


        verticalLayout_41->addLayout(verticalLayout_43);


        vboxShowDetails->addLayout(verticalLayout_41);

        line2 = new QFrame(showInfosGroupBox);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::Shape::HLine);
        line2->setFrameShadow(QFrame::Shadow::Sunken);

        vboxShowDetails->addWidget(line2);

        chkUnCheckAll = new QCheckBox(showInfosGroupBox);
        chkUnCheckAll->setObjectName("chkUnCheckAll");
        chkUnCheckAll->setChecked(true);

        vboxShowDetails->addWidget(chkUnCheckAll);


        verticalLayout_6->addWidget(showInfosGroupBox);

        lblEpisodeOnlyType = new QLabel(tabShowDetails);
        lblEpisodeOnlyType->setObjectName("lblEpisodeOnlyType");
        lblEpisodeOnlyType->setWordWrap(true);

        verticalLayout_6->addWidget(lblEpisodeOnlyType);

        verticalSpacer_313 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_313);

        tabsInfos->addTab(tabShowDetails, QString());
        tabEpisodeDetails = new QWidget();
        tabEpisodeDetails->setObjectName("tabEpisodeDetails");
        verticalLayout_72 = new QVBoxLayout(tabEpisodeDetails);
        verticalLayout_72->setObjectName("verticalLayout_72");
        episodeInfosGroupBox = new QGroupBox(tabEpisodeDetails);
        episodeInfosGroupBox->setObjectName("episodeInfosGroupBox");
        episodeInfosGroupBox->setFlat(true);
        vboxEpisodeDetails = new QVBoxLayout(episodeInfosGroupBox);
        vboxEpisodeDetails->setObjectName("vboxEpisodeDetails");
        vboxEpisodeDetails->setContentsMargins(0, 0, 0, 0);
        verticalLayout_412 = new QHBoxLayout();
        verticalLayout_412->setObjectName("verticalLayout_412");
        verticalLayout_422 = new QVBoxLayout();
        verticalLayout_422->setObjectName("verticalLayout_422");
        chkEpisodeTitle = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeTitle->setObjectName("chkEpisodeTitle");
        chkEpisodeTitle->setChecked(true);

        verticalLayout_422->addWidget(chkEpisodeTitle);

        chkEpisodeOverview = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeOverview->setObjectName("chkEpisodeOverview");
        chkEpisodeOverview->setChecked(true);

        verticalLayout_422->addWidget(chkEpisodeOverview);

        chkEpisodeRating = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeRating->setObjectName("chkEpisodeRating");
        chkEpisodeRating->setChecked(true);

        verticalLayout_422->addWidget(chkEpisodeRating);

        chkEpisodeFirstAired = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeFirstAired->setObjectName("chkEpisodeFirstAired");
        chkEpisodeFirstAired->setChecked(true);

        verticalLayout_422->addWidget(chkEpisodeFirstAired);

        chkEpisodeCertification = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeCertification->setObjectName("chkEpisodeCertification");
        chkEpisodeCertification->setChecked(true);

        verticalLayout_422->addWidget(chkEpisodeCertification);

        verticalSpacer_312 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_422->addItem(verticalSpacer_312);


        verticalLayout_412->addLayout(verticalLayout_422);

        verticalLayout_431 = new QVBoxLayout();
        verticalLayout_431->setObjectName("verticalLayout_431");
        chkEpisodeDirector = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeDirector->setObjectName("chkEpisodeDirector");
        chkEpisodeDirector->setChecked(true);

        verticalLayout_431->addWidget(chkEpisodeDirector);

        chkEpisodeWriter = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeWriter->setObjectName("chkEpisodeWriter");
        chkEpisodeWriter->setChecked(true);

        verticalLayout_431->addWidget(chkEpisodeWriter);

        chkEpisodeNetwork = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeNetwork->setObjectName("chkEpisodeNetwork");
        chkEpisodeNetwork->setChecked(true);

        verticalLayout_431->addWidget(chkEpisodeNetwork);

        chkEpisodeActors = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeActors->setObjectName("chkEpisodeActors");
        chkEpisodeActors->setChecked(true);

        verticalLayout_431->addWidget(chkEpisodeActors);

        chkEpisodeTags = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeTags->setObjectName("chkEpisodeTags");
        chkEpisodeTags->setChecked(true);

        verticalLayout_431->addWidget(chkEpisodeTags);

        chkEpisodeThumbnail = new MyCheckBox(episodeInfosGroupBox);
        chkEpisodeThumbnail->setObjectName("chkEpisodeThumbnail");
        chkEpisodeThumbnail->setChecked(true);

        verticalLayout_431->addWidget(chkEpisodeThumbnail);

        verticalSpacer_322 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_431->addItem(verticalSpacer_322);


        verticalLayout_412->addLayout(verticalLayout_431);


        vboxEpisodeDetails->addLayout(verticalLayout_412);

        line3 = new QFrame(episodeInfosGroupBox);
        line3->setObjectName("line3");
        line3->setFrameShape(QFrame::Shape::HLine);
        line3->setFrameShadow(QFrame::Shadow::Sunken);

        vboxEpisodeDetails->addWidget(line3);

        chkEpisodeUnCheckAll = new QCheckBox(episodeInfosGroupBox);
        chkEpisodeUnCheckAll->setObjectName("chkEpisodeUnCheckAll");
        chkEpisodeUnCheckAll->setChecked(true);

        vboxEpisodeDetails->addWidget(chkEpisodeUnCheckAll);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        lblSeasonOrder = new QLabel(episodeInfosGroupBox);
        lblSeasonOrder->setObjectName("lblSeasonOrder");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblSeasonOrder);

        comboSeasonOrder = new QComboBox(episodeInfosGroupBox);
        comboSeasonOrder->setObjectName("comboSeasonOrder");

        formLayout->setWidget(0, QFormLayout::FieldRole, comboSeasonOrder);


        vboxEpisodeDetails->addLayout(formLayout);


        verticalLayout_72->addWidget(episodeInfosGroupBox);

        lblShowOnlyType = new QLabel(tabEpisodeDetails);
        lblShowOnlyType->setObjectName("lblShowOnlyType");
        lblShowOnlyType->setWordWrap(true);

        verticalLayout_72->addWidget(lblShowOnlyType);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_72->addItem(verticalSpacer_3);

        tabsInfos->addTab(tabEpisodeDetails, QString());

        vboxDetails->addWidget(tabsInfos);


        horizontalLayout->addLayout(vboxDetails);

        horizontalLayout->setStretch(0, 1);

        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(TvShowSearchWidget);

        tabsInfos->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(TvShowSearchWidget);
    } // setupUi

    void retranslateUi(QWidget *TvShowSearchWidget)
    {
        lblScraper->setText(QCoreApplication::translate("TvShowSearchWidget", "Scraper", nullptr));
        lblLanguage->setText(QCoreApplication::translate("TvShowSearchWidget", "Language", nullptr));
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("TvShowSearchWidget", "Result", nullptr));
        comboUpdate->setItemText(0, QCoreApplication::translate("TvShowSearchWidget", "Update TV Show only", nullptr));
        comboUpdate->setItemText(1, QCoreApplication::translate("TvShowSearchWidget", "Update TV Show and new Episodes", nullptr));
        comboUpdate->setItemText(2, QCoreApplication::translate("TvShowSearchWidget", "Update TV Show and all Episodes", nullptr));
        comboUpdate->setItemText(3, QCoreApplication::translate("TvShowSearchWidget", "Update new Episodes", nullptr));
        comboUpdate->setItemText(4, QCoreApplication::translate("TvShowSearchWidget", "Update all episodes", nullptr));

        showInfosGroupBox->setTitle(QString());
        chkTitle->setText(QCoreApplication::translate("TvShowSearchWidget", "Title", nullptr));
        chkStatus->setText(QCoreApplication::translate("TvShowSearchWidget", "Status", nullptr));
        chkRating->setText(QCoreApplication::translate("TvShowSearchWidget", "Rating", nullptr));
        chkFirstAired->setText(QCoreApplication::translate("TvShowSearchWidget", "First aired", nullptr));
        chkRuntime->setText(QCoreApplication::translate("TvShowSearchWidget", "Runtime", nullptr));
        chkCertification->setText(QCoreApplication::translate("TvShowSearchWidget", "Certification", nullptr));
        chkOverview->setText(QCoreApplication::translate("TvShowSearchWidget", "Overview", nullptr));
        chkGenres->setText(QCoreApplication::translate("TvShowSearchWidget", "Genres", nullptr));
        chkTags->setText(QCoreApplication::translate("TvShowSearchWidget", "Tags", nullptr));
        chkNetwork->setText(QCoreApplication::translate("TvShowSearchWidget", "Network", nullptr));
        chkActors->setText(QCoreApplication::translate("TvShowSearchWidget", "Actors", nullptr));
        chkFanart->setText(QCoreApplication::translate("TvShowSearchWidget", "Fanart", nullptr));
        chkPoster->setText(QCoreApplication::translate("TvShowSearchWidget", "Poster", nullptr));
        chkSeasonPoster->setText(QCoreApplication::translate("TvShowSearchWidget", "Season Poster", nullptr));
        chkSeasonFanart->setText(QCoreApplication::translate("TvShowSearchWidget", "Season Fanart", nullptr));
        chkSeasonBanner->setText(QCoreApplication::translate("TvShowSearchWidget", "Season Banner", nullptr));
        chkSeasonThumb->setText(QCoreApplication::translate("TvShowSearchWidget", "Season Thumb", nullptr));
        chkBanner->setText(QCoreApplication::translate("TvShowSearchWidget", "Banner", nullptr));
        chkExtraArts->setText(QCoreApplication::translate("TvShowSearchWidget", "Artwork", nullptr));
        chkThumb->setText(QCoreApplication::translate("TvShowSearchWidget", "Thumb", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("TvShowSearchWidget", "(Un)Check all", nullptr));
        lblEpisodeOnlyType->setText(QCoreApplication::translate("TvShowSearchWidget", "No show details will be loaded because only episodes were selected.", nullptr));
        tabsInfos->setTabText(tabsInfos->indexOf(tabShowDetails), QCoreApplication::translate("TvShowSearchWidget", "Show Details", nullptr));
        episodeInfosGroupBox->setTitle(QString());
        chkEpisodeTitle->setText(QCoreApplication::translate("TvShowSearchWidget", "Title", nullptr));
        chkEpisodeOverview->setText(QCoreApplication::translate("TvShowSearchWidget", "Overview", nullptr));
        chkEpisodeRating->setText(QCoreApplication::translate("TvShowSearchWidget", "Rating", nullptr));
        chkEpisodeFirstAired->setText(QCoreApplication::translate("TvShowSearchWidget", "First aired", nullptr));
        chkEpisodeCertification->setText(QCoreApplication::translate("TvShowSearchWidget", "Certification", nullptr));
        chkEpisodeDirector->setText(QCoreApplication::translate("TvShowSearchWidget", "Director", nullptr));
        chkEpisodeWriter->setText(QCoreApplication::translate("TvShowSearchWidget", "Writer", nullptr));
        chkEpisodeNetwork->setText(QCoreApplication::translate("TvShowSearchWidget", "Network", nullptr));
        chkEpisodeActors->setText(QCoreApplication::translate("TvShowSearchWidget", "Actors", nullptr));
        chkEpisodeTags->setText(QCoreApplication::translate("TvShowSearchWidget", "Tags", nullptr));
        chkEpisodeThumbnail->setText(QCoreApplication::translate("TvShowSearchWidget", "Thumbnail", nullptr));
        chkEpisodeUnCheckAll->setText(QCoreApplication::translate("TvShowSearchWidget", "(Un)Check all", nullptr));
        lblSeasonOrder->setText(QCoreApplication::translate("TvShowSearchWidget", "Season order", nullptr));
        lblShowOnlyType->setText(QCoreApplication::translate("TvShowSearchWidget", "No episodes will be loaded. If you want to load episodes, change the update mode below.", nullptr));
        tabsInfos->setTabText(tabsInfos->indexOf(tabEpisodeDetails), QCoreApplication::translate("TvShowSearchWidget", "Episode Details", nullptr));
        (void)TvShowSearchWidget;
    } // retranslateUi

};

namespace Ui {
    class TvShowSearchWidget: public Ui_TvShowSearchWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWSEARCHWIDGET_H
