/********************************************************************************
** Form generated from reading UI file 'ConcertSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONCERTSETTINGSWIDGET_H
#define UI_CONCERTSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/PlaceholderLineEdit.h>

QT_BEGIN_NAMESPACE

class Ui_ConcertSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_8;
    QLabel *lblConcertIntro;
    QLabel *lblConcertPlaceholderIntro;
    QFormLayout *formLayout_7;
    QLabel *lblNfo;
    PlaceholderLineEdit *concertNfo;
    QLabel *lblPoster;
    PlaceholderLineEdit *concertPoster;
    QLabel *lblBackdrop;
    PlaceholderLineEdit *concertBackdrop;
    QLabel *lblLogo;
    PlaceholderLineEdit *concertLogo;
    QLabel *lblClearArt;
    PlaceholderLineEdit *concertClearArt;
    QLabel *lblDiscArt;
    PlaceholderLineEdit *concertDiscArt;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ConcertSettingsWidget)
    {
        if (ConcertSettingsWidget->objectName().isEmpty())
            ConcertSettingsWidget->setObjectName("ConcertSettingsWidget");
        ConcertSettingsWidget->resize(970, 625);
        verticalLayout_8 = new QVBoxLayout(ConcertSettingsWidget);
        verticalLayout_8->setObjectName("verticalLayout_8");
        lblConcertIntro = new QLabel(ConcertSettingsWidget);
        lblConcertIntro->setObjectName("lblConcertIntro");
        lblConcertIntro->setWordWrap(true);

        verticalLayout_8->addWidget(lblConcertIntro);

        lblConcertPlaceholderIntro = new QLabel(ConcertSettingsWidget);
        lblConcertPlaceholderIntro->setObjectName("lblConcertPlaceholderIntro");
        lblConcertPlaceholderIntro->setWordWrap(true);

        verticalLayout_8->addWidget(lblConcertPlaceholderIntro);

        formLayout_7 = new QFormLayout();
        formLayout_7->setObjectName("formLayout_7");
        formLayout_7->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        lblNfo = new QLabel(ConcertSettingsWidget);
        lblNfo->setObjectName("lblNfo");

        formLayout_7->setWidget(0, QFormLayout::LabelRole, lblNfo);

        concertNfo = new PlaceholderLineEdit(ConcertSettingsWidget);
        concertNfo->setObjectName("concertNfo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(concertNfo->sizePolicy().hasHeightForWidth());
        concertNfo->setSizePolicy(sizePolicy);
        concertNfo->setMinimumSize(QSize(200, 0));
        concertNfo->setMaximumSize(QSize(400, 16777215));

        formLayout_7->setWidget(0, QFormLayout::FieldRole, concertNfo);

        lblPoster = new QLabel(ConcertSettingsWidget);
        lblPoster->setObjectName("lblPoster");

        formLayout_7->setWidget(1, QFormLayout::LabelRole, lblPoster);

        concertPoster = new PlaceholderLineEdit(ConcertSettingsWidget);
        concertPoster->setObjectName("concertPoster");
        sizePolicy.setHeightForWidth(concertPoster->sizePolicy().hasHeightForWidth());
        concertPoster->setSizePolicy(sizePolicy);
        concertPoster->setMinimumSize(QSize(200, 0));
        concertPoster->setMaximumSize(QSize(400, 16777215));

        formLayout_7->setWidget(1, QFormLayout::FieldRole, concertPoster);

        lblBackdrop = new QLabel(ConcertSettingsWidget);
        lblBackdrop->setObjectName("lblBackdrop");

        formLayout_7->setWidget(2, QFormLayout::LabelRole, lblBackdrop);

        concertBackdrop = new PlaceholderLineEdit(ConcertSettingsWidget);
        concertBackdrop->setObjectName("concertBackdrop");
        sizePolicy.setHeightForWidth(concertBackdrop->sizePolicy().hasHeightForWidth());
        concertBackdrop->setSizePolicy(sizePolicy);
        concertBackdrop->setMinimumSize(QSize(200, 0));
        concertBackdrop->setMaximumSize(QSize(400, 16777215));

        formLayout_7->setWidget(2, QFormLayout::FieldRole, concertBackdrop);

        lblLogo = new QLabel(ConcertSettingsWidget);
        lblLogo->setObjectName("lblLogo");

        formLayout_7->setWidget(3, QFormLayout::LabelRole, lblLogo);

        concertLogo = new PlaceholderLineEdit(ConcertSettingsWidget);
        concertLogo->setObjectName("concertLogo");
        sizePolicy.setHeightForWidth(concertLogo->sizePolicy().hasHeightForWidth());
        concertLogo->setSizePolicy(sizePolicy);
        concertLogo->setMinimumSize(QSize(200, 0));
        concertLogo->setMaximumSize(QSize(400, 16777215));

        formLayout_7->setWidget(3, QFormLayout::FieldRole, concertLogo);

        lblClearArt = new QLabel(ConcertSettingsWidget);
        lblClearArt->setObjectName("lblClearArt");

        formLayout_7->setWidget(4, QFormLayout::LabelRole, lblClearArt);

        concertClearArt = new PlaceholderLineEdit(ConcertSettingsWidget);
        concertClearArt->setObjectName("concertClearArt");
        sizePolicy.setHeightForWidth(concertClearArt->sizePolicy().hasHeightForWidth());
        concertClearArt->setSizePolicy(sizePolicy);
        concertClearArt->setMinimumSize(QSize(200, 0));
        concertClearArt->setMaximumSize(QSize(400, 16777215));

        formLayout_7->setWidget(4, QFormLayout::FieldRole, concertClearArt);

        lblDiscArt = new QLabel(ConcertSettingsWidget);
        lblDiscArt->setObjectName("lblDiscArt");

        formLayout_7->setWidget(5, QFormLayout::LabelRole, lblDiscArt);

        concertDiscArt = new PlaceholderLineEdit(ConcertSettingsWidget);
        concertDiscArt->setObjectName("concertDiscArt");
        sizePolicy.setHeightForWidth(concertDiscArt->sizePolicy().hasHeightForWidth());
        concertDiscArt->setSizePolicy(sizePolicy);
        concertDiscArt->setMinimumSize(QSize(200, 0));
        concertDiscArt->setMaximumSize(QSize(400, 16777215));

        formLayout_7->setWidget(5, QFormLayout::FieldRole, concertDiscArt);


        verticalLayout_8->addLayout(formLayout_7);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer);

        verticalLayout_8->setStretch(2, 1);

        retranslateUi(ConcertSettingsWidget);

        QMetaObject::connectSlotsByName(ConcertSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *ConcertSettingsWidget)
    {
        lblConcertIntro->setText(QCoreApplication::translate("ConcertSettingsWidget", "Below you see the filenames which are used for loading and saving your concerts. You can edit them as you like, if you want to use multiple files separate them by comma.", nullptr));
        lblConcertPlaceholderIntro->setText(QCoreApplication::translate("ConcertSettingsWidget", "You can use the placeholder <baseFileName> which is the filename without extension.", nullptr));
        lblNfo->setText(QCoreApplication::translate("ConcertSettingsWidget", "Nfo", nullptr));
        lblPoster->setText(QCoreApplication::translate("ConcertSettingsWidget", "Poster", nullptr));
        lblBackdrop->setText(QCoreApplication::translate("ConcertSettingsWidget", "Backdrop", nullptr));
        lblLogo->setText(QCoreApplication::translate("ConcertSettingsWidget", "Logo", nullptr));
        lblClearArt->setText(QCoreApplication::translate("ConcertSettingsWidget", "Clear Art", nullptr));
        lblDiscArt->setText(QCoreApplication::translate("ConcertSettingsWidget", "Disc Art", nullptr));
        (void)ConcertSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class ConcertSettingsWidget: public Ui_ConcertSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONCERTSETTINGSWIDGET_H
