/********************************************************************************
** Form generated from reading UI file 'ImagePreviewDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEPREVIEWDIALOG_H
#define UI_IMAGEPREVIEWDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ImagePreviewDialog
{
public:
    QVBoxLayout *verticalLayout;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QLabel *image;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClose;

    void setupUi(QDialog *ImagePreviewDialog)
    {
        if (ImagePreviewDialog->objectName().isEmpty())
            ImagePreviewDialog->setObjectName("ImagePreviewDialog");
        ImagePreviewDialog->setWindowModality(Qt::WindowModal);
        ImagePreviewDialog->resize(837, 636);
        ImagePreviewDialog->setModal(true);
        verticalLayout = new QVBoxLayout(ImagePreviewDialog);
        verticalLayout->setObjectName("verticalLayout");
        scrollArea = new QScrollArea(ImagePreviewDialog);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 811, 568));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        image = new QLabel(scrollAreaWidgetContents);
        image->setObjectName("image");
        image->setText(QString::fromUtf8(""));
        image->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(image);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonClose = new QPushButton(ImagePreviewDialog);
        buttonClose->setObjectName("buttonClose");

        horizontalLayout->addWidget(buttonClose);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(ImagePreviewDialog);

        QMetaObject::connectSlotsByName(ImagePreviewDialog);
    } // setupUi

    void retranslateUi(QDialog *ImagePreviewDialog)
    {
        ImagePreviewDialog->setWindowTitle(QCoreApplication::translate("ImagePreviewDialog", "Preview", nullptr));
        buttonClose->setText(QCoreApplication::translate("ImagePreviewDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImagePreviewDialog: public Ui_ImagePreviewDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGEPREVIEWDIALOG_H
