/********************************************************************************
** Form generated from reading UI file 'TvShowWidgetSeason.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWWIDGETSEASON_H
#define UI_TVSHOWWIDGETSEASON_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/ClosableImage.h"

QT_BEGIN_NAMESPACE

class Ui_TvShowWidgetSeason
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *buttonRevert;
    QLabel *title;
    QSpacerItem *horizontalSpacer_3;
    QLabel *missingLabel;
    QGroupBox *seasonGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tabInformation;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *seasonFormLayout;
    QLineEdit *seasonName;
    QLabel *lblSeasonName;
    QFrame *line;
    QVBoxLayout *verticalLayout_3;
    QLabel *labelThumb;
    ClosableImage *thumb;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *labelPoster;
    ClosableImage *poster;
    QLabel *labelFanart;
    ClosableImage *backdrop;
    QLabel *labelBanner;
    ClosableImage *banner;
    QSpacerItem *verticalSpacer_4;

    void setupUi(QWidget *TvShowWidgetSeason)
    {
        if (TvShowWidgetSeason->objectName().isEmpty())
            TvShowWidgetSeason->setObjectName("TvShowWidgetSeason");
        TvShowWidgetSeason->resize(962, 732);
        verticalLayout_2 = new QVBoxLayout(TvShowWidgetSeason);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 20);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(-1, 20, -1, -1);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        buttonRevert = new QToolButton(TvShowWidgetSeason);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);

        horizontalLayout_3->addWidget(buttonRevert);

        title = new QLabel(TvShowWidgetSeason);
        title->setObjectName("title");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        title->setFont(font);
        title->setIndent(4);
        title->setProperty("isHeader", QVariant(true));

        horizontalLayout_3->addWidget(title);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_4->addLayout(horizontalLayout_3);


        horizontalLayout->addLayout(verticalLayout_4);

        missingLabel = new QLabel(TvShowWidgetSeason);
        missingLabel->setObjectName("missingLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(missingLabel->sizePolicy().hasHeightForWidth());
        missingLabel->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(missingLabel);

        horizontalLayout->setStretch(0, 1);

        verticalLayout_2->addLayout(horizontalLayout);

        seasonGroupBox = new QGroupBox(TvShowWidgetSeason);
        seasonGroupBox->setObjectName("seasonGroupBox");
        horizontalLayout_2 = new QHBoxLayout(seasonGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(seasonGroupBox);
        tabWidget->setObjectName("tabWidget");
        tabInformation = new QWidget();
        tabInformation->setObjectName("tabInformation");
        verticalLayout_5 = new QVBoxLayout(tabInformation);
        verticalLayout_5->setObjectName("verticalLayout_5");
        seasonFormLayout = new QFormLayout();
        seasonFormLayout->setObjectName("seasonFormLayout");
        seasonName = new QLineEdit(tabInformation);
        seasonName->setObjectName("seasonName");

        seasonFormLayout->setWidget(0, QFormLayout::FieldRole, seasonName);

        lblSeasonName = new QLabel(tabInformation);
        lblSeasonName->setObjectName("lblSeasonName");

        seasonFormLayout->setWidget(0, QFormLayout::LabelRole, lblSeasonName);


        verticalLayout_5->addLayout(seasonFormLayout);

        tabWidget->addTab(tabInformation, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(seasonGroupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_2->addWidget(line);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        labelThumb = new QLabel(seasonGroupBox);
        labelThumb->setObjectName("labelThumb");

        verticalLayout_3->addWidget(labelThumb);

        thumb = new ClosableImage(seasonGroupBox);
        thumb->setObjectName("thumb");
        thumb->setMinimumSize(QSize(180, 102));
        thumb->setMaximumSize(QSize(180, 102));
        thumb->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        thumb->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/thumb.png")));
        thumb->setAlignment(Qt::AlignCenter);
        thumb->setProperty("clickable", QVariant(true));
        thumb->setProperty("myFixedHeight", QVariant(102));

        verticalLayout_3->addWidget(thumb);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout_3);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        labelPoster = new QLabel(seasonGroupBox);
        labelPoster->setObjectName("labelPoster");

        verticalLayout->addWidget(labelPoster);

        poster = new ClosableImage(seasonGroupBox);
        poster->setObjectName("poster");
        poster->setMinimumSize(QSize(180, 290));
        poster->setMaximumSize(QSize(180, 290));
        poster->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        poster->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/poster.png")));
        poster->setAlignment(Qt::AlignCenter);
        poster->setProperty("clickable", QVariant(true));
        poster->setProperty("myFixedHeight", QVariant(290));

        verticalLayout->addWidget(poster);

        labelFanart = new QLabel(seasonGroupBox);
        labelFanart->setObjectName("labelFanart");

        verticalLayout->addWidget(labelFanart);

        backdrop = new ClosableImage(seasonGroupBox);
        backdrop->setObjectName("backdrop");
        backdrop->setMinimumSize(QSize(180, 120));
        backdrop->setMaximumSize(QSize(180, 120));
        backdrop->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        backdrop->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/fanart.png")));
        backdrop->setAlignment(Qt::AlignCenter);
        backdrop->setProperty("clickable", QVariant(true));
        backdrop->setProperty("myFixedHeight", QVariant(120));

        verticalLayout->addWidget(backdrop);

        labelBanner = new QLabel(seasonGroupBox);
        labelBanner->setObjectName("labelBanner");

        verticalLayout->addWidget(labelBanner);

        banner = new ClosableImage(seasonGroupBox);
        banner->setObjectName("banner");
        banner->setMinimumSize(QSize(180, 53));
        banner->setMaximumSize(QSize(180, 53));
        banner->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        banner->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/banner.png")));
        banner->setAlignment(Qt::AlignCenter);
        banner->setProperty("clickable", QVariant(true));
        banner->setProperty("myFixedHeight", QVariant(53));

        verticalLayout->addWidget(banner);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_4);


        horizontalLayout_2->addLayout(verticalLayout);


        verticalLayout_2->addWidget(seasonGroupBox);

        verticalLayout_2->setStretch(1, 1);

        retranslateUi(TvShowWidgetSeason);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(TvShowWidgetSeason);
    } // setupUi

    void retranslateUi(QWidget *TvShowWidgetSeason)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("TvShowWidgetSeason", "Episode has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        title->setText(QCoreApplication::translate("TvShowWidgetSeason", "Title", nullptr));
        missingLabel->setText(QCoreApplication::translate("TvShowWidgetSeason", "Season missing", nullptr));
        seasonGroupBox->setTitle(QString());
        lblSeasonName->setText(QCoreApplication::translate("TvShowWidgetSeason", "Season Name", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabInformation), QCoreApplication::translate("TvShowWidgetSeason", "Information", nullptr));
        labelThumb->setText(QCoreApplication::translate("TvShowWidgetSeason", "Thumb", nullptr));
#if QT_CONFIG(tooltip)
        thumb->setToolTip(QCoreApplication::translate("TvShowWidgetSeason", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        thumb->setText(QString());
        labelPoster->setText(QCoreApplication::translate("TvShowWidgetSeason", "Poster", nullptr));
#if QT_CONFIG(tooltip)
        poster->setToolTip(QCoreApplication::translate("TvShowWidgetSeason", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        poster->setText(QString());
        labelFanart->setText(QCoreApplication::translate("TvShowWidgetSeason", "Fanart", nullptr));
#if QT_CONFIG(tooltip)
        backdrop->setToolTip(QCoreApplication::translate("TvShowWidgetSeason", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        backdrop->setText(QString());
        labelBanner->setText(QCoreApplication::translate("TvShowWidgetSeason", "Banner", nullptr));
#if QT_CONFIG(tooltip)
        banner->setToolTip(QCoreApplication::translate("TvShowWidgetSeason", "Click to change", nullptr));
#endif // QT_CONFIG(tooltip)
        banner->setText(QString());
        (void)TvShowWidgetSeason;
    } // retranslateUi

};

namespace Ui {
    class TvShowWidgetSeason: public Ui_TvShowWidgetSeason {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWWIDGETSEASON_H
