/********************************************************************************
** Form generated from reading UI file 'ConcertSearchWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONCERTSEARCHWIDGET_H
#define UI_CONCERTSEARCHWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/LanguageCombo.h>
#include <ui/small_widgets/MyCheckBox.h>
#include "ui/small_widgets/MyLineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_ConcertSearchWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_5;
    QLabel *lblScraper;
    QComboBox *comboScraper;
    QLabel *lblLanguage;
    LanguageCombo *comboLanguage;
    QSpacerItem *horizontalSpacer_3;
    MyLineEdit *searchString;
    QLabel *lblSuccessMessage;
    QLabel *lblErrorMessage;
    QHBoxLayout *horizontalLayout_4;
    QTableWidget *results;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *concertInfosGroupBox;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_3;
    QSpacerItem *verticalSpacer_3;
    MyCheckBox *chkTitle;
    MyCheckBox *chkTagline;
    MyCheckBox *chkRating;
    MyCheckBox *chkReleased;
    MyCheckBox *chkRuntime;
    MyCheckBox *chkCertification;
    MyCheckBox *chkTrailer;
    MyCheckBox *chkOverview;
    MyCheckBox *chkPoster;
    MyCheckBox *chkBackdrop;
    MyCheckBox *chkGenres;
    MyCheckBox *chkExtraArts;
    QFrame *line_2;
    QCheckBox *chkUnCheckAll;
    QSpacerItem *verticalSpacer_4;

    void setupUi(QWidget *ConcertSearchWidget)
    {
        if (ConcertSearchWidget->objectName().isEmpty())
            ConcertSearchWidget->setObjectName("ConcertSearchWidget");
        ConcertSearchWidget->resize(746, 590);
        verticalLayout = new QVBoxLayout(ConcertSearchWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        lblScraper = new QLabel(ConcertSearchWidget);
        lblScraper->setObjectName("lblScraper");

        horizontalLayout_5->addWidget(lblScraper);

        comboScraper = new QComboBox(ConcertSearchWidget);
        comboScraper->setObjectName("comboScraper");

        horizontalLayout_5->addWidget(comboScraper);

        lblLanguage = new QLabel(ConcertSearchWidget);
        lblLanguage->setObjectName("lblLanguage");

        horizontalLayout_5->addWidget(lblLanguage);

        comboLanguage = new LanguageCombo(ConcertSearchWidget);
        comboLanguage->setObjectName("comboLanguage");

        horizontalLayout_5->addWidget(comboLanguage);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);


        verticalLayout->addLayout(horizontalLayout_5);

        searchString = new MyLineEdit(ConcertSearchWidget);
        searchString->setObjectName("searchString");

        verticalLayout->addWidget(searchString);

        lblSuccessMessage = new QLabel(ConcertSearchWidget);
        lblSuccessMessage->setObjectName("lblSuccessMessage");
        lblSuccessMessage->setText(QString::fromUtf8("success"));

        verticalLayout->addWidget(lblSuccessMessage);

        lblErrorMessage = new QLabel(ConcertSearchWidget);
        lblErrorMessage->setObjectName("lblErrorMessage");
        lblErrorMessage->setText(QString::fromUtf8("failure"));

        verticalLayout->addWidget(lblErrorMessage);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        results = new QTableWidget(ConcertSearchWidget);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);

        horizontalLayout_4->addWidget(results);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        concertInfosGroupBox = new QGroupBox(ConcertSearchWidget);
        concertInfosGroupBox->setObjectName("concertInfosGroupBox");
        verticalLayout_5 = new QVBoxLayout(concertInfosGroupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        label_3 = new QLabel(concertInfosGroupBox);
        label_3->setObjectName("label_3");
        QFont font;
        font.setBold(true);
        label_3->setFont(font);

        verticalLayout_5->addWidget(label_3);

        verticalSpacer_3 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_5->addItem(verticalSpacer_3);

        chkTitle = new MyCheckBox(concertInfosGroupBox);
        chkTitle->setObjectName("chkTitle");

        verticalLayout_5->addWidget(chkTitle);

        chkTagline = new MyCheckBox(concertInfosGroupBox);
        chkTagline->setObjectName("chkTagline");

        verticalLayout_5->addWidget(chkTagline);

        chkRating = new MyCheckBox(concertInfosGroupBox);
        chkRating->setObjectName("chkRating");

        verticalLayout_5->addWidget(chkRating);

        chkReleased = new MyCheckBox(concertInfosGroupBox);
        chkReleased->setObjectName("chkReleased");

        verticalLayout_5->addWidget(chkReleased);

        chkRuntime = new MyCheckBox(concertInfosGroupBox);
        chkRuntime->setObjectName("chkRuntime");

        verticalLayout_5->addWidget(chkRuntime);

        chkCertification = new MyCheckBox(concertInfosGroupBox);
        chkCertification->setObjectName("chkCertification");

        verticalLayout_5->addWidget(chkCertification);

        chkTrailer = new MyCheckBox(concertInfosGroupBox);
        chkTrailer->setObjectName("chkTrailer");

        verticalLayout_5->addWidget(chkTrailer);

        chkOverview = new MyCheckBox(concertInfosGroupBox);
        chkOverview->setObjectName("chkOverview");

        verticalLayout_5->addWidget(chkOverview);

        chkPoster = new MyCheckBox(concertInfosGroupBox);
        chkPoster->setObjectName("chkPoster");

        verticalLayout_5->addWidget(chkPoster);

        chkBackdrop = new MyCheckBox(concertInfosGroupBox);
        chkBackdrop->setObjectName("chkBackdrop");

        verticalLayout_5->addWidget(chkBackdrop);

        chkGenres = new MyCheckBox(concertInfosGroupBox);
        chkGenres->setObjectName("chkGenres");

        verticalLayout_5->addWidget(chkGenres);

        chkExtraArts = new MyCheckBox(concertInfosGroupBox);
        chkExtraArts->setObjectName("chkExtraArts");

        verticalLayout_5->addWidget(chkExtraArts);

        line_2 = new QFrame(concertInfosGroupBox);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_5->addWidget(line_2);

        chkUnCheckAll = new QCheckBox(concertInfosGroupBox);
        chkUnCheckAll->setObjectName("chkUnCheckAll");

        verticalLayout_5->addWidget(chkUnCheckAll);


        verticalLayout_2->addWidget(concertInfosGroupBox);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_4);


        horizontalLayout_4->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout_4);


        retranslateUi(ConcertSearchWidget);

        QMetaObject::connectSlotsByName(ConcertSearchWidget);
    } // setupUi

    void retranslateUi(QWidget *ConcertSearchWidget)
    {
        lblScraper->setText(QCoreApplication::translate("ConcertSearchWidget", "Scraper", nullptr));
        lblLanguage->setText(QCoreApplication::translate("ConcertSearchWidget", "Language", nullptr));
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("ConcertSearchWidget", "Result", nullptr));
        concertInfosGroupBox->setTitle(QString());
        label_3->setText(QCoreApplication::translate("ConcertSearchWidget", "Infos to load", nullptr));
        chkTitle->setText(QCoreApplication::translate("ConcertSearchWidget", "Title", nullptr));
        chkTagline->setText(QCoreApplication::translate("ConcertSearchWidget", "Tagline", nullptr));
        chkRating->setText(QCoreApplication::translate("ConcertSearchWidget", "Rating", nullptr));
        chkReleased->setText(QCoreApplication::translate("ConcertSearchWidget", "Released", nullptr));
        chkRuntime->setText(QCoreApplication::translate("ConcertSearchWidget", "Runtime", nullptr));
        chkCertification->setText(QCoreApplication::translate("ConcertSearchWidget", "Certification", nullptr));
        chkTrailer->setText(QCoreApplication::translate("ConcertSearchWidget", "Trailer", nullptr));
        chkOverview->setText(QCoreApplication::translate("ConcertSearchWidget", "Overview", nullptr));
        chkPoster->setText(QCoreApplication::translate("ConcertSearchWidget", "Poster", nullptr));
        chkBackdrop->setText(QCoreApplication::translate("ConcertSearchWidget", "Backdrop", nullptr));
        chkGenres->setText(QCoreApplication::translate("ConcertSearchWidget", "Genres", nullptr));
#if QT_CONFIG(tooltip)
        chkExtraArts->setToolTip(QCoreApplication::translate("ConcertSearchWidget", "Logo, Clear Art, CD Art", nullptr));
#endif // QT_CONFIG(tooltip)
        chkExtraArts->setText(QCoreApplication::translate("ConcertSearchWidget", "Artwork", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("ConcertSearchWidget", "(Un)Check all", nullptr));
        (void)ConcertSearchWidget;
    } // retranslateUi

};

namespace Ui {
    class ConcertSearchWidget: public Ui_ConcertSearchWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONCERTSEARCHWIDGET_H
