/********************************************************************************
** Form generated from reading UI file 'MusicWidgetArtist.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICWIDGETARTIST_H
#define UI_MUSICWIDGETARTIST_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/ClosableImage.h"
#include "ui/small_widgets/ImageGallery.h"
#include "ui/small_widgets/TagCloud.h"

QT_BEGIN_NAMESPACE

class Ui_MusicWidgetArtist
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer_4;
    QToolButton *buttonRevert;
    QLabel *artistName;
    QSpacerItem *horizontalSpacer_11;
    QGroupBox *artistGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_6;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *path;
    QLineEdit *name;
    QLabel *label_2;
    QLineEdit *born;
    QLineEdit *formed;
    QLineEdit *yearsActive;
    QLabel *label_3;
    QLabel *label_21;
    QLabel *label_6;
    QLabel *label_22;
    QLabel *label_23;
    QLineEdit *disbanded;
    QLineEdit *died;
    QLabel *label_15;
    QLabel *label_4;
    QLineEdit *musicBrainzId;
    QPlainTextEdit *biography;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    TagCloud *genreCloud;
    TagCloud *styleCloud;
    TagCloud *moodCloud;
    QSpacerItem *verticalSpacer_3;
    QWidget *tab_4;
    QHBoxLayout *horizontalLayout_3;
    QTableWidget *discography;
    QVBoxLayout *verticalLayout_5;
    QPushButton *btnAddAlbum;
    QPushButton *btnRemoveAlbum;
    QSpacerItem *verticalSpacer_5;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_2;
    ImageGallery *fanarts;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_16;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *btnAddExtraFanart;
    QFrame *line;
    QWidget *artWidget;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_4;
    QLabel *labelThumb;
    ClosableImage *thumb;
    QSpacerItem *verticalSpacer_2;
    QLabel *labelFanart;
    ClosableImage *fanart;
    QSpacerItem *verticalSpacer_4;
    QLabel *labelLogo;
    ClosableImage *logo;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *MusicWidgetArtist)
    {
        if (MusicWidgetArtist->objectName().isEmpty())
            MusicWidgetArtist->setObjectName("MusicWidgetArtist");
        MusicWidgetArtist->resize(954, 771);
        MusicWidgetArtist->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(MusicWidgetArtist);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 0);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_4);

        buttonRevert = new QToolButton(MusicWidgetArtist);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);
        buttonRevert->setIconSize(QSize(16, 16));

        horizontalLayout_10->addWidget(buttonRevert);

        artistName = new QLabel(MusicWidgetArtist);
        artistName->setObjectName("artistName");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        artistName->setFont(font);
        artistName->setIndent(4);
        artistName->setProperty("isHeader", QVariant(true));

        horizontalLayout_10->addWidget(artistName);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_11);


        verticalLayout->addLayout(horizontalLayout_10);

        artistGroupBox = new QGroupBox(MusicWidgetArtist);
        artistGroupBox->setObjectName("artistGroupBox");
        horizontalLayout_2 = new QHBoxLayout(artistGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(artistGroupBox);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_6 = new QVBoxLayout(tab);
        verticalLayout_6->setObjectName("verticalLayout_6");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setRowWrapPolicy(QFormLayout::RowWrapPolicy::DontWrapRows);
        formLayout->setLabelAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        formLayout->setHorizontalSpacing(20);
        label = new QLabel(tab);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        path = new QLineEdit(tab);
        path->setObjectName("path");
        path->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, path);

        name = new QLineEdit(tab);
        name->setObjectName("name");
        name->setProperty("item", QVariant(QString::fromUtf8("name")));

        formLayout->setWidget(2, QFormLayout::FieldRole, name);

        label_2 = new QLabel(tab);
        label_2->setObjectName("label_2");

        formLayout->setWidget(2, QFormLayout::LabelRole, label_2);

        born = new QLineEdit(tab);
        born->setObjectName("born");
        born->setProperty("item", QVariant(QString::fromUtf8("born")));

        formLayout->setWidget(3, QFormLayout::FieldRole, born);

        formed = new QLineEdit(tab);
        formed->setObjectName("formed");
        formed->setProperty("item", QVariant(QString::fromUtf8("formed")));

        formLayout->setWidget(4, QFormLayout::FieldRole, formed);

        yearsActive = new QLineEdit(tab);
        yearsActive->setObjectName("yearsActive");
        yearsActive->setProperty("item", QVariant(QString::fromUtf8("yearsActive")));

        formLayout->setWidget(5, QFormLayout::FieldRole, yearsActive);

        label_3 = new QLabel(tab);
        label_3->setObjectName("label_3");

        formLayout->setWidget(3, QFormLayout::LabelRole, label_3);

        label_21 = new QLabel(tab);
        label_21->setObjectName("label_21");

        formLayout->setWidget(4, QFormLayout::LabelRole, label_21);

        label_6 = new QLabel(tab);
        label_6->setObjectName("label_6");

        formLayout->setWidget(5, QFormLayout::LabelRole, label_6);

        label_22 = new QLabel(tab);
        label_22->setObjectName("label_22");

        formLayout->setWidget(6, QFormLayout::LabelRole, label_22);

        label_23 = new QLabel(tab);
        label_23->setObjectName("label_23");

        formLayout->setWidget(7, QFormLayout::LabelRole, label_23);

        disbanded = new QLineEdit(tab);
        disbanded->setObjectName("disbanded");
        disbanded->setProperty("item", QVariant(QString::fromUtf8("disbanded")));

        formLayout->setWidget(6, QFormLayout::FieldRole, disbanded);

        died = new QLineEdit(tab);
        died->setObjectName("died");
        died->setProperty("item", QVariant(QString::fromUtf8("died")));

        formLayout->setWidget(7, QFormLayout::FieldRole, died);

        label_15 = new QLabel(tab);
        label_15->setObjectName("label_15");

        formLayout->setWidget(8, QFormLayout::LabelRole, label_15);

        label_4 = new QLabel(tab);
        label_4->setObjectName("label_4");

        formLayout->setWidget(1, QFormLayout::LabelRole, label_4);

        musicBrainzId = new QLineEdit(tab);
        musicBrainzId->setObjectName("musicBrainzId");
        musicBrainzId->setProperty("item", QVariant(QString::fromUtf8("mbid")));

        formLayout->setWidget(1, QFormLayout::FieldRole, musicBrainzId);

        biography = new QPlainTextEdit(tab);
        biography->setObjectName("biography");

        formLayout->setWidget(8, QFormLayout::FieldRole, biography);


        verticalLayout_6->addLayout(formLayout);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        genreCloud = new TagCloud(tab_2);
        genreCloud->setObjectName("genreCloud");
        genreCloud->setProperty("item", QVariant(QString::fromUtf8("genre")));

        verticalLayout_3->addWidget(genreCloud);

        styleCloud = new TagCloud(tab_2);
        styleCloud->setObjectName("styleCloud");
        styleCloud->setProperty("item", QVariant(QString::fromUtf8("style")));

        verticalLayout_3->addWidget(styleCloud);

        moodCloud = new TagCloud(tab_2);
        moodCloud->setObjectName("moodCloud");
        moodCloud->setProperty("item", QVariant(QString::fromUtf8("mood")));

        verticalLayout_3->addWidget(moodCloud);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        verticalLayout_3->setStretch(0, 2);
        verticalLayout_3->setStretch(1, 2);
        verticalLayout_3->setStretch(2, 2);
        tabWidget->addTab(tab_2, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        horizontalLayout_3 = new QHBoxLayout(tab_4);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        discography = new QTableWidget(tab_4);
        if (discography->columnCount() < 2)
            discography->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        discography->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        discography->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        discography->setObjectName("discography");
        discography->setEditTriggers(QAbstractItemView::EditTrigger::DoubleClicked);
        discography->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        discography->setShowGrid(false);
        discography->horizontalHeader()->setDefaultSectionSize(200);
        discography->verticalHeader()->setVisible(false);

        horizontalLayout_3->addWidget(discography);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        btnAddAlbum = new QPushButton(tab_4);
        btnAddAlbum->setObjectName("btnAddAlbum");

        verticalLayout_5->addWidget(btnAddAlbum);

        btnRemoveAlbum = new QPushButton(tab_4);
        btnRemoveAlbum->setObjectName("btnRemoveAlbum");

        verticalLayout_5->addWidget(btnRemoveAlbum);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_5);


        horizontalLayout_3->addLayout(verticalLayout_5);

        horizontalLayout_3->setStretch(0, 1);
        tabWidget->addTab(tab_4, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_2 = new QVBoxLayout(tab_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(12, -1, 12, -1);
        fanarts = new ImageGallery(tab_3);
        fanarts->setObjectName("fanarts");

        verticalLayout_2->addWidget(fanarts);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_16 = new QLabel(tab_3);
        label_16->setObjectName("label_16");
        QFont font1;
        font1.setFamilies({QString::fromUtf8("Helvetica Neue")});
        label_16->setFont(font1);
        label_16->setWordWrap(true);

        horizontalLayout_5->addWidget(label_16);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        btnAddExtraFanart = new QPushButton(tab_3);
        btnAddExtraFanart->setObjectName("btnAddExtraFanart");

        horizontalLayout_5->addWidget(btnAddExtraFanart);

        horizontalLayout_5->setStretch(0, 1);

        verticalLayout_2->addLayout(horizontalLayout_5);

        verticalLayout_2->setStretch(0, 1);
        tabWidget->addTab(tab_3, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(artistGroupBox);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Shadow::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(0);
        line->setFrameShape(QFrame::Shape::VLine);

        horizontalLayout_2->addWidget(line);

        artWidget = new QWidget(artistGroupBox);
        artWidget->setObjectName("artWidget");
        horizontalLayout = new QHBoxLayout(artWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(10, 0, -1, -1);
        labelThumb = new QLabel(artWidget);
        labelThumb->setObjectName("labelThumb");

        verticalLayout_4->addWidget(labelThumb);

        thumb = new ClosableImage(artWidget);
        thumb->setObjectName("thumb");
        thumb->setMinimumSize(QSize(180, 180));
        thumb->setMaximumSize(QSize(180, 180));
        thumb->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        thumb->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/thumb.png")));
        thumb->setAlignment(Qt::AlignmentFlag::AlignCenter);
        thumb->setProperty("clickable", QVariant(true));
        thumb->setProperty("myFixedHeight", QVariant(180));

        verticalLayout_4->addWidget(thumb);

        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_2);

        labelFanart = new QLabel(artWidget);
        labelFanart->setObjectName("labelFanart");

        verticalLayout_4->addWidget(labelFanart);

        fanart = new ClosableImage(artWidget);
        fanart->setObjectName("fanart");
        fanart->setMinimumSize(QSize(180, 120));
        fanart->setMaximumSize(QSize(180, 120));
        fanart->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        fanart->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/fanart.png")));
        fanart->setAlignment(Qt::AlignmentFlag::AlignCenter);
        fanart->setProperty("clickable", QVariant(true));
        fanart->setProperty("myFixedHeight", QVariant(120));

        verticalLayout_4->addWidget(fanart);

        verticalSpacer_4 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_4);

        labelLogo = new QLabel(artWidget);
        labelLogo->setObjectName("labelLogo");

        verticalLayout_4->addWidget(labelLogo);

        logo = new ClosableImage(artWidget);
        logo->setObjectName("logo");
        logo->setMinimumSize(QSize(180, 102));
        logo->setMaximumSize(QSize(180, 102));
        logo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        logo->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/logo.png")));
        logo->setAlignment(Qt::AlignmentFlag::AlignCenter);
        logo->setProperty("clickable", QVariant(true));
        logo->setProperty("myFixedHeight", QVariant(102));

        verticalLayout_4->addWidget(logo);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_4);


        horizontalLayout_2->addWidget(artWidget);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout->addWidget(artistGroupBox);

        verticalLayout->setStretch(1, 1);

        retranslateUi(MusicWidgetArtist);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MusicWidgetArtist);
    } // setupUi

    void retranslateUi(QWidget *MusicWidgetArtist)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("MusicWidgetArtist", "Artist has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        artistName->setText(QCoreApplication::translate("MusicWidgetArtist", "Artist Name", nullptr));
        artistGroupBox->setTitle(QString());
        label->setText(QCoreApplication::translate("MusicWidgetArtist", "Path", nullptr));
        label_2->setText(QCoreApplication::translate("MusicWidgetArtist", "Name", nullptr));
        label_3->setText(QCoreApplication::translate("MusicWidgetArtist", "Born", nullptr));
        label_21->setText(QCoreApplication::translate("MusicWidgetArtist", "Formed", nullptr));
        label_6->setText(QCoreApplication::translate("MusicWidgetArtist", "Years active", nullptr));
        label_22->setText(QCoreApplication::translate("MusicWidgetArtist", "Disbanded", nullptr));
        label_23->setText(QCoreApplication::translate("MusicWidgetArtist", "Died", nullptr));
        label_15->setText(QCoreApplication::translate("MusicWidgetArtist", "Biography", nullptr));
        label_4->setText(QCoreApplication::translate("MusicWidgetArtist", "MusicBrainz ID", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("MusicWidgetArtist", "Information", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("MusicWidgetArtist", "Extended", nullptr));
        QTableWidgetItem *___qtablewidgetitem = discography->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("MusicWidgetArtist", "Title", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = discography->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("MusicWidgetArtist", "Year", nullptr));
        btnAddAlbum->setText(QCoreApplication::translate("MusicWidgetArtist", "Add Album", nullptr));
        btnRemoveAlbum->setText(QCoreApplication::translate("MusicWidgetArtist", "Remove Album", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("MusicWidgetArtist", "Discography", nullptr));
        label_16->setText(QCoreApplication::translate("MusicWidgetArtist", "Hint: Closed images will be deleted on save.", nullptr));
        btnAddExtraFanart->setText(QCoreApplication::translate("MusicWidgetArtist", "Add Images", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("MusicWidgetArtist", "Extra Fanarts", nullptr));
        labelThumb->setText(QCoreApplication::translate("MusicWidgetArtist", "Thumb", nullptr));
#if QT_CONFIG(tooltip)
        thumb->setToolTip(QCoreApplication::translate("MusicWidgetArtist", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        thumb->setText(QString());
        thumb->setProperty("title", QVariant(QCoreApplication::translate("MusicWidgetArtist", "Thumb", nullptr)));
        labelFanart->setText(QCoreApplication::translate("MusicWidgetArtist", "Fanart", nullptr));
#if QT_CONFIG(tooltip)
        fanart->setToolTip(QCoreApplication::translate("MusicWidgetArtist", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        fanart->setText(QString());
        fanart->setProperty("title", QVariant(QCoreApplication::translate("MusicWidgetArtist", "Fanart", nullptr)));
        labelLogo->setText(QCoreApplication::translate("MusicWidgetArtist", "Logo", nullptr));
#if QT_CONFIG(tooltip)
        logo->setToolTip(QCoreApplication::translate("MusicWidgetArtist", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        logo->setText(QString());
        logo->setProperty("title", QVariant(QCoreApplication::translate("MusicWidgetArtist", "Thumb", nullptr)));
        (void)MusicWidgetArtist;
    } // retranslateUi

};

namespace Ui {
    class MusicWidgetArtist: public Ui_MusicWidgetArtist {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICWIDGETARTIST_H
