/********************************************************************************
** Form generated from reading UI file 'TvShowSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWSETTINGSWIDGET_H
#define UI_TVSHOWSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/PlaceholderLineEdit.h>

QT_BEGIN_NAMESPACE

class Ui_TvShowSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_7;
    QLabel *lblTvShowIntro;
    QLabel *lblTvShowPlaceholders;
    QFormLayout *formLayout_6;
    QLabel *label_26;
    QLabel *label_27;
    QLabel *label_28;
    QLabel *label_29;
    QLabel *label_32;
    QLabel *label_34;
    QLabel *label_30;
    QLabel *label_31;
    QLabel *label_35;
    QLabel *label_33;
    QLabel *label_36;
    QLabel *label_37;
    PlaceholderLineEdit *showNfo;
    PlaceholderLineEdit *showEpisodeNfo;
    PlaceholderLineEdit *showPoster;
    PlaceholderLineEdit *showBackdrop;
    PlaceholderLineEdit *showBanner;
    PlaceholderLineEdit *showLogo;
    PlaceholderLineEdit *showClearArt;
    PlaceholderLineEdit *showCharacterArt;
    PlaceholderLineEdit *showEpisodeThumbnail;
    PlaceholderLineEdit *showSeasonPoster;
    PlaceholderLineEdit *showSeasonBackdrop;
    PlaceholderLineEdit *showSeasonBanner;
    QLabel *label_51;
    QLabel *label_52;
    PlaceholderLineEdit *showThumb;
    PlaceholderLineEdit *showSeasonThumb;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *TvShowSettingsWidget)
    {
        if (TvShowSettingsWidget->objectName().isEmpty())
            TvShowSettingsWidget->setObjectName("TvShowSettingsWidget");
        TvShowSettingsWidget->resize(970, 625);
        verticalLayout_7 = new QVBoxLayout(TvShowSettingsWidget);
        verticalLayout_7->setObjectName("verticalLayout_7");
        lblTvShowIntro = new QLabel(TvShowSettingsWidget);
        lblTvShowIntro->setObjectName("lblTvShowIntro");
        lblTvShowIntro->setWordWrap(true);

        verticalLayout_7->addWidget(lblTvShowIntro);

        lblTvShowPlaceholders = new QLabel(TvShowSettingsWidget);
        lblTvShowPlaceholders->setObjectName("lblTvShowPlaceholders");
        lblTvShowPlaceholders->setWordWrap(true);

        verticalLayout_7->addWidget(lblTvShowPlaceholders);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName("formLayout_6");
        formLayout_6->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        label_26 = new QLabel(TvShowSettingsWidget);
        label_26->setObjectName("label_26");

        formLayout_6->setWidget(0, QFormLayout::LabelRole, label_26);

        label_27 = new QLabel(TvShowSettingsWidget);
        label_27->setObjectName("label_27");

        formLayout_6->setWidget(2, QFormLayout::LabelRole, label_27);

        label_28 = new QLabel(TvShowSettingsWidget);
        label_28->setObjectName("label_28");

        formLayout_6->setWidget(3, QFormLayout::LabelRole, label_28);

        label_29 = new QLabel(TvShowSettingsWidget);
        label_29->setObjectName("label_29");

        formLayout_6->setWidget(4, QFormLayout::LabelRole, label_29);

        label_32 = new QLabel(TvShowSettingsWidget);
        label_32->setObjectName("label_32");

        formLayout_6->setWidget(5, QFormLayout::LabelRole, label_32);

        label_34 = new QLabel(TvShowSettingsWidget);
        label_34->setObjectName("label_34");

        formLayout_6->setWidget(7, QFormLayout::LabelRole, label_34);

        label_30 = new QLabel(TvShowSettingsWidget);
        label_30->setObjectName("label_30");

        formLayout_6->setWidget(8, QFormLayout::LabelRole, label_30);

        label_31 = new QLabel(TvShowSettingsWidget);
        label_31->setObjectName("label_31");

        formLayout_6->setWidget(10, QFormLayout::LabelRole, label_31);

        label_35 = new QLabel(TvShowSettingsWidget);
        label_35->setObjectName("label_35");

        formLayout_6->setWidget(11, QFormLayout::LabelRole, label_35);

        label_33 = new QLabel(TvShowSettingsWidget);
        label_33->setObjectName("label_33");

        formLayout_6->setWidget(1, QFormLayout::LabelRole, label_33);

        label_36 = new QLabel(TvShowSettingsWidget);
        label_36->setObjectName("label_36");

        formLayout_6->setWidget(9, QFormLayout::LabelRole, label_36);

        label_37 = new QLabel(TvShowSettingsWidget);
        label_37->setObjectName("label_37");

        formLayout_6->setWidget(12, QFormLayout::LabelRole, label_37);

        showNfo = new PlaceholderLineEdit(TvShowSettingsWidget);
        showNfo->setObjectName("showNfo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(showNfo->sizePolicy().hasHeightForWidth());
        showNfo->setSizePolicy(sizePolicy);
        showNfo->setMinimumSize(QSize(200, 0));
        showNfo->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(0, QFormLayout::FieldRole, showNfo);

        showEpisodeNfo = new PlaceholderLineEdit(TvShowSettingsWidget);
        showEpisodeNfo->setObjectName("showEpisodeNfo");
        sizePolicy.setHeightForWidth(showEpisodeNfo->sizePolicy().hasHeightForWidth());
        showEpisodeNfo->setSizePolicy(sizePolicy);
        showEpisodeNfo->setMinimumSize(QSize(200, 0));
        showEpisodeNfo->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(1, QFormLayout::FieldRole, showEpisodeNfo);

        showPoster = new PlaceholderLineEdit(TvShowSettingsWidget);
        showPoster->setObjectName("showPoster");
        sizePolicy.setHeightForWidth(showPoster->sizePolicy().hasHeightForWidth());
        showPoster->setSizePolicy(sizePolicy);
        showPoster->setMinimumSize(QSize(200, 0));
        showPoster->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(2, QFormLayout::FieldRole, showPoster);

        showBackdrop = new PlaceholderLineEdit(TvShowSettingsWidget);
        showBackdrop->setObjectName("showBackdrop");
        sizePolicy.setHeightForWidth(showBackdrop->sizePolicy().hasHeightForWidth());
        showBackdrop->setSizePolicy(sizePolicy);
        showBackdrop->setMinimumSize(QSize(200, 0));
        showBackdrop->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(3, QFormLayout::FieldRole, showBackdrop);

        showBanner = new PlaceholderLineEdit(TvShowSettingsWidget);
        showBanner->setObjectName("showBanner");
        sizePolicy.setHeightForWidth(showBanner->sizePolicy().hasHeightForWidth());
        showBanner->setSizePolicy(sizePolicy);
        showBanner->setMinimumSize(QSize(200, 0));
        showBanner->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(4, QFormLayout::FieldRole, showBanner);

        showLogo = new PlaceholderLineEdit(TvShowSettingsWidget);
        showLogo->setObjectName("showLogo");
        sizePolicy.setHeightForWidth(showLogo->sizePolicy().hasHeightForWidth());
        showLogo->setSizePolicy(sizePolicy);
        showLogo->setMinimumSize(QSize(200, 0));
        showLogo->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(5, QFormLayout::FieldRole, showLogo);

        showClearArt = new PlaceholderLineEdit(TvShowSettingsWidget);
        showClearArt->setObjectName("showClearArt");
        sizePolicy.setHeightForWidth(showClearArt->sizePolicy().hasHeightForWidth());
        showClearArt->setSizePolicy(sizePolicy);
        showClearArt->setMinimumSize(QSize(200, 0));
        showClearArt->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(7, QFormLayout::FieldRole, showClearArt);

        showCharacterArt = new PlaceholderLineEdit(TvShowSettingsWidget);
        showCharacterArt->setObjectName("showCharacterArt");
        sizePolicy.setHeightForWidth(showCharacterArt->sizePolicy().hasHeightForWidth());
        showCharacterArt->setSizePolicy(sizePolicy);
        showCharacterArt->setMinimumSize(QSize(200, 0));
        showCharacterArt->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(8, QFormLayout::FieldRole, showCharacterArt);

        showEpisodeThumbnail = new PlaceholderLineEdit(TvShowSettingsWidget);
        showEpisodeThumbnail->setObjectName("showEpisodeThumbnail");
        sizePolicy.setHeightForWidth(showEpisodeThumbnail->sizePolicy().hasHeightForWidth());
        showEpisodeThumbnail->setSizePolicy(sizePolicy);
        showEpisodeThumbnail->setMinimumSize(QSize(200, 0));
        showEpisodeThumbnail->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(9, QFormLayout::FieldRole, showEpisodeThumbnail);

        showSeasonPoster = new PlaceholderLineEdit(TvShowSettingsWidget);
        showSeasonPoster->setObjectName("showSeasonPoster");
        sizePolicy.setHeightForWidth(showSeasonPoster->sizePolicy().hasHeightForWidth());
        showSeasonPoster->setSizePolicy(sizePolicy);
        showSeasonPoster->setMinimumSize(QSize(200, 0));
        showSeasonPoster->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(10, QFormLayout::FieldRole, showSeasonPoster);

        showSeasonBackdrop = new PlaceholderLineEdit(TvShowSettingsWidget);
        showSeasonBackdrop->setObjectName("showSeasonBackdrop");
        sizePolicy.setHeightForWidth(showSeasonBackdrop->sizePolicy().hasHeightForWidth());
        showSeasonBackdrop->setSizePolicy(sizePolicy);
        showSeasonBackdrop->setMinimumSize(QSize(200, 0));
        showSeasonBackdrop->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(11, QFormLayout::FieldRole, showSeasonBackdrop);

        showSeasonBanner = new PlaceholderLineEdit(TvShowSettingsWidget);
        showSeasonBanner->setObjectName("showSeasonBanner");
        sizePolicy.setHeightForWidth(showSeasonBanner->sizePolicy().hasHeightForWidth());
        showSeasonBanner->setSizePolicy(sizePolicy);
        showSeasonBanner->setMinimumSize(QSize(200, 0));
        showSeasonBanner->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(12, QFormLayout::FieldRole, showSeasonBanner);

        label_51 = new QLabel(TvShowSettingsWidget);
        label_51->setObjectName("label_51");

        formLayout_6->setWidget(13, QFormLayout::LabelRole, label_51);

        label_52 = new QLabel(TvShowSettingsWidget);
        label_52->setObjectName("label_52");

        formLayout_6->setWidget(6, QFormLayout::LabelRole, label_52);

        showThumb = new PlaceholderLineEdit(TvShowSettingsWidget);
        showThumb->setObjectName("showThumb");
        sizePolicy.setHeightForWidth(showThumb->sizePolicy().hasHeightForWidth());
        showThumb->setSizePolicy(sizePolicy);
        showThumb->setMinimumSize(QSize(200, 0));
        showThumb->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(6, QFormLayout::FieldRole, showThumb);

        showSeasonThumb = new PlaceholderLineEdit(TvShowSettingsWidget);
        showSeasonThumb->setObjectName("showSeasonThumb");
        sizePolicy.setHeightForWidth(showSeasonThumb->sizePolicy().hasHeightForWidth());
        showSeasonThumb->setSizePolicy(sizePolicy);
        showSeasonThumb->setMinimumSize(QSize(200, 0));
        showSeasonThumb->setMaximumSize(QSize(400, 16777215));

        formLayout_6->setWidget(13, QFormLayout::FieldRole, showSeasonThumb);


        verticalLayout_7->addLayout(formLayout_6);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer);

        verticalLayout_7->setStretch(2, 1);

        retranslateUi(TvShowSettingsWidget);

        QMetaObject::connectSlotsByName(TvShowSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *TvShowSettingsWidget)
    {
        lblTvShowIntro->setText(QCoreApplication::translate("TvShowSettingsWidget", "Below you see the filenames which are used for loading and saving your TV shows. You can edit them as you like, if you want to use multiple files separate them by comma.", nullptr));
        lblTvShowPlaceholders->setText(QCoreApplication::translate("TvShowSettingsWidget", "You can use the placeholder <baseFileName> which is the filename without extension and for season posters <seasonNumber> which is the season number.", nullptr));
        label_26->setText(QCoreApplication::translate("TvShowSettingsWidget", "Show nfo", nullptr));
        label_27->setText(QCoreApplication::translate("TvShowSettingsWidget", "Poster", nullptr));
        label_28->setText(QCoreApplication::translate("TvShowSettingsWidget", "Backdrop", nullptr));
        label_29->setText(QCoreApplication::translate("TvShowSettingsWidget", "Banner", nullptr));
        label_32->setText(QCoreApplication::translate("TvShowSettingsWidget", "Logo", nullptr));
        label_34->setText(QCoreApplication::translate("TvShowSettingsWidget", "Clear Art", nullptr));
        label_30->setText(QCoreApplication::translate("TvShowSettingsWidget", "Character Art", nullptr));
        label_31->setText(QCoreApplication::translate("TvShowSettingsWidget", "Season Poster", nullptr));
        label_35->setText(QCoreApplication::translate("TvShowSettingsWidget", "Season Backdrop", nullptr));
        label_33->setText(QCoreApplication::translate("TvShowSettingsWidget", "Episode nfo", nullptr));
        label_36->setText(QCoreApplication::translate("TvShowSettingsWidget", "Episode thumbnail", nullptr));
        label_37->setText(QCoreApplication::translate("TvShowSettingsWidget", "Season Banner", nullptr));
        label_51->setText(QCoreApplication::translate("TvShowSettingsWidget", "Season Thumb", nullptr));
        label_52->setText(QCoreApplication::translate("TvShowSettingsWidget", "Thumb", nullptr));
        (void)TvShowSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class TvShowSettingsWidget: public Ui_TvShowSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWSETTINGSWIDGET_H
