/********************************************************************************
** Form generated from reading UI file 'DownloadsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOWNLOADSWIDGET_H
#define UI_DOWNLOADSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DownloadsWidget
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *widget;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelArchives;
    QTableWidget *tablePackages;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelImportable;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnImportMakeMkv;
    QTableWidget *tableImports;

    void setupUi(QWidget *DownloadsWidget)
    {
        if (DownloadsWidget->objectName().isEmpty())
            DownloadsWidget->setObjectName("DownloadsWidget");
        DownloadsWidget->resize(997, 596);
        verticalLayout = new QVBoxLayout(DownloadsWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(DownloadsWidget);
        widget->setObjectName("widget");
        verticalLayout_2 = new QVBoxLayout(widget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 12, 0, 12);
        labelArchives = new QLabel(widget);
        labelArchives->setObjectName("labelArchives");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        labelArchives->setFont(font);
        labelArchives->setIndent(12);
        labelArchives->setProperty("isHeader", QVariant(true));

        verticalLayout_2->addWidget(labelArchives);

        tablePackages = new QTableWidget(widget);
        if (tablePackages->columnCount() < 5)
            tablePackages->setColumnCount(5);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        tablePackages->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        tablePackages->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        tablePackages->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        tablePackages->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        tablePackages->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        tablePackages->setObjectName("tablePackages");
        tablePackages->setFrameShape(QFrame::NoFrame);
        tablePackages->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tablePackages->setAlternatingRowColors(true);
        tablePackages->setSelectionMode(QAbstractItemView::SingleSelection);
        tablePackages->setSelectionBehavior(QAbstractItemView::SelectRows);
        tablePackages->setShowGrid(false);
        tablePackages->setSortingEnabled(true);
        tablePackages->horizontalHeader()->setVisible(true);
        tablePackages->horizontalHeader()->setDefaultSectionSize(200);
        tablePackages->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(tablePackages);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_2->addItem(verticalSpacer);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelImportable = new QLabel(widget);
        labelImportable->setObjectName("labelImportable");
        labelImportable->setFont(font);
        labelImportable->setIndent(12);
        labelImportable->setProperty("isHeader", QVariant(true));

        horizontalLayout_2->addWidget(labelImportable);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        btnImportMakeMkv = new QPushButton(widget);
        btnImportMakeMkv->setObjectName("btnImportMakeMkv");
        btnImportMakeMkv->setProperty("buttonstyle", QVariant(QString::fromUtf8("info")));

        horizontalLayout_2->addWidget(btnImportMakeMkv);


        verticalLayout_2->addLayout(horizontalLayout_2);

        tableImports = new QTableWidget(widget);
        if (tableImports->columnCount() < 6)
            tableImports->setColumnCount(6);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        tableImports->setHorizontalHeaderItem(0, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        tableImports->setHorizontalHeaderItem(1, __qtablewidgetitem6);
        QTableWidgetItem *__qtablewidgetitem7 = new QTableWidgetItem();
        tableImports->setHorizontalHeaderItem(2, __qtablewidgetitem7);
        QTableWidgetItem *__qtablewidgetitem8 = new QTableWidgetItem();
        tableImports->setHorizontalHeaderItem(3, __qtablewidgetitem8);
        QTableWidgetItem *__qtablewidgetitem9 = new QTableWidgetItem();
        tableImports->setHorizontalHeaderItem(4, __qtablewidgetitem9);
        QTableWidgetItem *__qtablewidgetitem10 = new QTableWidgetItem();
        tableImports->setHorizontalHeaderItem(5, __qtablewidgetitem10);
        tableImports->setObjectName("tableImports");
        tableImports->setFrameShape(QFrame::NoFrame);
        tableImports->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableImports->setAlternatingRowColors(true);
        tableImports->setSelectionMode(QAbstractItemView::SingleSelection);
        tableImports->setSelectionBehavior(QAbstractItemView::SelectRows);
        tableImports->setShowGrid(false);
        tableImports->setSortingEnabled(true);
        tableImports->horizontalHeader()->setVisible(true);
        tableImports->horizontalHeader()->setDefaultSectionSize(170);
        tableImports->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(tableImports);


        verticalLayout->addWidget(widget);


        retranslateUi(DownloadsWidget);

        QMetaObject::connectSlotsByName(DownloadsWidget);
    } // setupUi

    void retranslateUi(QWidget *DownloadsWidget)
    {
        labelArchives->setText(QCoreApplication::translate("DownloadsWidget", "Archives", nullptr));
        QTableWidgetItem *___qtablewidgetitem = tablePackages->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("DownloadsWidget", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = tablePackages->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("DownloadsWidget", "Files", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = tablePackages->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("DownloadsWidget", "Size", nullptr));
        labelImportable->setText(QCoreApplication::translate("DownloadsWidget", "Importable items", nullptr));
        btnImportMakeMkv->setText(QCoreApplication::translate("DownloadsWidget", "Import movie with MakeMKV", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = tableImports->horizontalHeaderItem(0);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("DownloadsWidget", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = tableImports->horizontalHeaderItem(1);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("DownloadsWidget", "Files", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = tableImports->horizontalHeaderItem(2);
        ___qtablewidgetitem5->setText(QCoreApplication::translate("DownloadsWidget", "Size", nullptr));
        QTableWidgetItem *___qtablewidgetitem6 = tableImports->horizontalHeaderItem(3);
        ___qtablewidgetitem6->setText(QCoreApplication::translate("DownloadsWidget", "Import Type", nullptr));
#if QT_CONFIG(tooltip)
        ___qtablewidgetitem6->setToolTip(QCoreApplication::translate("DownloadsWidget", "Type of this file.", nullptr));
#endif // QT_CONFIG(tooltip)
        QTableWidgetItem *___qtablewidgetitem7 = tableImports->horizontalHeaderItem(4);
        ___qtablewidgetitem7->setText(QCoreApplication::translate("DownloadsWidget", "Import Directory", nullptr));
#if QT_CONFIG(tooltip)
        ___qtablewidgetitem7->setToolTip(QCoreApplication::translate("DownloadsWidget", "Where to store the file when importing it.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)DownloadsWidget;
    } // retranslateUi

};

namespace Ui {
    class DownloadsWidget: public Ui_DownloadsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DOWNLOADSWIDGET_H
