/********************************************************************************
** Form generated from reading UI file 'MovieScraperInfoWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIESCRAPERINFOWIDGET_H
#define UI_MOVIESCRAPERINFOWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MovieScraperInfoWidget
{
public:
    QFormLayout *formLayout;
    QLabel *lblName;
    QLabel *txtName;
    QLabel *lblId;
    QLabel *txtId;
    QLabel *lblDescription;
    QPlainTextEdit *txtDescription;
    QLabel *lblWebsite;
    QLabel *txtWebsite;
    QLabel *lblTOS;
    QLabel *txtTOS;
    QLabel *lblPrivacyPolicy;
    QLabel *txtPrivacyPolicy;
    QLabel *lblHelpUrl;
    QLabel *txtHelp;

    void setupUi(QWidget *MovieScraperInfoWidget)
    {
        if (MovieScraperInfoWidget->objectName().isEmpty())
            MovieScraperInfoWidget->setObjectName("MovieScraperInfoWidget");
        MovieScraperInfoWidget->resize(612, 439);
        formLayout = new QFormLayout(MovieScraperInfoWidget);
        formLayout->setObjectName("formLayout");
        lblName = new QLabel(MovieScraperInfoWidget);
        lblName->setObjectName("lblName");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblName);

        txtName = new QLabel(MovieScraperInfoWidget);
        txtName->setObjectName("txtName");
        QFont font;
        font.setBold(true);
        txtName->setFont(font);
        txtName->setText(QString::fromUtf8("Scraper Name"));

        formLayout->setWidget(0, QFormLayout::FieldRole, txtName);

        lblId = new QLabel(MovieScraperInfoWidget);
        lblId->setObjectName("lblId");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblId);

        txtId = new QLabel(MovieScraperInfoWidget);
        txtId->setObjectName("txtId");
        txtId->setText(QString::fromUtf8("example-identifier"));

        formLayout->setWidget(1, QFormLayout::FieldRole, txtId);

        lblDescription = new QLabel(MovieScraperInfoWidget);
        lblDescription->setObjectName("lblDescription");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblDescription);

        txtDescription = new QPlainTextEdit(MovieScraperInfoWidget);
        txtDescription->setObjectName("txtDescription");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(txtDescription->sizePolicy().hasHeightForWidth());
        txtDescription->setSizePolicy(sizePolicy);
        txtDescription->setMinimumSize(QSize(180, 120));
        txtDescription->setMaximumSize(QSize(600, 360));
        txtDescription->setTabChangesFocus(true);
        txtDescription->setUndoRedoEnabled(false);
        txtDescription->setReadOnly(true);
        txtDescription->setPlainText(QString::fromUtf8("This is the scraper\n"
"description which may span\n"
"multiple lines.\n"
""));

        formLayout->setWidget(2, QFormLayout::FieldRole, txtDescription);

        lblWebsite = new QLabel(MovieScraperInfoWidget);
        lblWebsite->setObjectName("lblWebsite");
        lblWebsite->setTextFormat(Qt::PlainText);

        formLayout->setWidget(3, QFormLayout::LabelRole, lblWebsite);

        txtWebsite = new QLabel(MovieScraperInfoWidget);
        txtWebsite->setObjectName("txtWebsite");
        txtWebsite->setText(QString::fromUtf8("<a href=\"https://example.com\">https://example.com</a>"));
        txtWebsite->setOpenExternalLinks(true);

        formLayout->setWidget(3, QFormLayout::FieldRole, txtWebsite);

        lblTOS = new QLabel(MovieScraperInfoWidget);
        lblTOS->setObjectName("lblTOS");

        formLayout->setWidget(4, QFormLayout::LabelRole, lblTOS);

        txtTOS = new QLabel(MovieScraperInfoWidget);
        txtTOS->setObjectName("txtTOS");
        txtTOS->setText(QString::fromUtf8("<a href=\"https://example.com\">https://example.com</a>"));
        txtTOS->setOpenExternalLinks(true);

        formLayout->setWidget(4, QFormLayout::FieldRole, txtTOS);

        lblPrivacyPolicy = new QLabel(MovieScraperInfoWidget);
        lblPrivacyPolicy->setObjectName("lblPrivacyPolicy");

        formLayout->setWidget(5, QFormLayout::LabelRole, lblPrivacyPolicy);

        txtPrivacyPolicy = new QLabel(MovieScraperInfoWidget);
        txtPrivacyPolicy->setObjectName("txtPrivacyPolicy");
        txtPrivacyPolicy->setText(QString::fromUtf8("<a href=\"https://example.com\">https://example.com</a>"));
        txtPrivacyPolicy->setOpenExternalLinks(true);

        formLayout->setWidget(5, QFormLayout::FieldRole, txtPrivacyPolicy);

        lblHelpUrl = new QLabel(MovieScraperInfoWidget);
        lblHelpUrl->setObjectName("lblHelpUrl");

        formLayout->setWidget(6, QFormLayout::LabelRole, lblHelpUrl);

        txtHelp = new QLabel(MovieScraperInfoWidget);
        txtHelp->setObjectName("txtHelp");
        txtHelp->setText(QString::fromUtf8("<a href=\"https://example.com\">https://example.com</a>"));
        txtHelp->setOpenExternalLinks(true);

        formLayout->setWidget(6, QFormLayout::FieldRole, txtHelp);


        retranslateUi(MovieScraperInfoWidget);

        QMetaObject::connectSlotsByName(MovieScraperInfoWidget);
    } // setupUi

    void retranslateUi(QWidget *MovieScraperInfoWidget)
    {
        lblName->setText(QCoreApplication::translate("MovieScraperInfoWidget", "Name", nullptr));
        lblId->setText(QCoreApplication::translate("MovieScraperInfoWidget", "ID", nullptr));
#if QT_CONFIG(tooltip)
        txtId->setToolTip(QCoreApplication::translate("MovieScraperInfoWidget", "Internal TV scraper ID. Used as key in MediaElch's settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblDescription->setText(QCoreApplication::translate("MovieScraperInfoWidget", "Description", nullptr));
        lblWebsite->setText(QCoreApplication::translate("MovieScraperInfoWidget", "Website", nullptr));
#if QT_CONFIG(tooltip)
        txtWebsite->setToolTip(QCoreApplication::translate("MovieScraperInfoWidget", "The scraper's main website.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblTOS->setText(QCoreApplication::translate("MovieScraperInfoWidget", "Terms of Service", nullptr));
#if QT_CONFIG(tooltip)
        txtTOS->setToolTip(QCoreApplication::translate("MovieScraperInfoWidget", "Terms of service of the TV scraper.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblPrivacyPolicy->setText(QCoreApplication::translate("MovieScraperInfoWidget", "Privacy Policy", nullptr));
#if QT_CONFIG(tooltip)
        txtPrivacyPolicy->setToolTip(QCoreApplication::translate("MovieScraperInfoWidget", "Privacy Policy of the scraper.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblHelpUrl->setText(QCoreApplication::translate("MovieScraperInfoWidget", "Help", nullptr));
#if QT_CONFIG(tooltip)
        txtHelp->setToolTip(QCoreApplication::translate("MovieScraperInfoWidget", "Where to get help for the TV scraper.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)MovieScraperInfoWidget;
    } // retranslateUi

};

namespace Ui {
    class MovieScraperInfoWidget: public Ui_MovieScraperInfoWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIESCRAPERINFOWIDGET_H
