/****************************************************************************
** Meta object code from reading C++ file 'SetsWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/ui/movie_sets/SetsWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SetsWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10SetsWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN10SetsWidgetE = QtMocHelpers::stringData(
    "SetsWidget",
    "setActionSaveEnabled",
    "",
    "MainWidgets",
    "sigJumpToMovie",
    "Movie*",
    "movie",
    "loadSets",
    "saveSet",
    "splitter",
    "QSplitter*",
    "onSetSelected",
    "clear",
    "onSortTitleChanged",
    "QTableWidgetItem*",
    "item",
    "onAddMovieSet",
    "onRemoveMovieSet",
    "onAddMovie",
    "onRemoveMovie",
    "chooseSetPoster",
    "chooseSetBackdrop",
    "onPreviewPoster",
    "onPreviewBackdrop",
    "showSetsContextMenu",
    "point",
    "onSetNameChanged",
    "onDownloadFinished",
    "DownloadManagerElement",
    "elem",
    "onJumpToMovie"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN10SetsWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,  134,    2, 0x06,    1 /* Public */,
       4,    1,  139,    2, 0x06,    4 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       7,    0,  142,    2, 0x0a,    6 /* Public */,
       8,    0,  143,    2, 0x0a,    7 /* Public */,
       9,    0,  144,    2, 0x0a,    8 /* Public */,
      11,    0,  145,    2, 0x08,    9 /* Private */,
      12,    0,  146,    2, 0x08,   10 /* Private */,
      13,    1,  147,    2, 0x08,   11 /* Private */,
      16,    0,  150,    2, 0x08,   13 /* Private */,
      17,    0,  151,    2, 0x08,   14 /* Private */,
      18,    0,  152,    2, 0x08,   15 /* Private */,
      19,    0,  153,    2, 0x08,   16 /* Private */,
      20,    0,  154,    2, 0x08,   17 /* Private */,
      21,    0,  155,    2, 0x08,   18 /* Private */,
      22,    0,  156,    2, 0x08,   19 /* Private */,
      23,    0,  157,    2, 0x08,   20 /* Private */,
      24,    1,  158,    2, 0x08,   21 /* Private */,
      26,    1,  161,    2, 0x08,   23 /* Private */,
      27,    1,  164,    2, 0x08,   25 /* Private */,
      30,    1,  167,    2, 0x08,   27 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, 0x80000000 | 5,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   25,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, 0x80000000 | 14,   15,

       0        // eod
};

Q_CONSTINIT const QMetaObject SetsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN10SetsWidgetE.offsetsAndSizes,
    qt_meta_data_ZN10SetsWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN10SetsWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<SetsWidget, std::true_type>,
        // method 'setActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigJumpToMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'loadSets'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveSet'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'splitter'
        QtPrivate::TypeAndForceComplete<QSplitter *, std::false_type>,
        // method 'onSetSelected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'clear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSortTitleChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onAddMovieSet'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveMovieSet'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAddMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'chooseSetPoster'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'chooseSetBackdrop'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPreviewPoster'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPreviewBackdrop'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'showSetsContextMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'onSetNameChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>,
        // method 'onJumpToMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>
    >,
    nullptr
} };

void SetsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SetsWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->sigJumpToMovie((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 2: _t->loadSets(); break;
        case 3: _t->saveSet(); break;
        case 4: { QSplitter* _r = _t->splitter();
            if (_a[0]) *reinterpret_cast< QSplitter**>(_a[0]) = std::move(_r); }  break;
        case 5: _t->onSetSelected(); break;
        case 6: _t->clear(); break;
        case 7: _t->onSortTitleChanged((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 8: _t->onAddMovieSet(); break;
        case 9: _t->onRemoveMovieSet(); break;
        case 10: _t->onAddMovie(); break;
        case 11: _t->onRemoveMovie(); break;
        case 12: _t->chooseSetPoster(); break;
        case 13: _t->chooseSetBackdrop(); break;
        case 14: _t->onPreviewPoster(); break;
        case 15: _t->onPreviewBackdrop(); break;
        case 16: _t->showSetsContextMenu((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 17: _t->onSetNameChanged((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 18: _t->onDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        case 19: _t->onJumpToMovie((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (SetsWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &SetsWidget::setActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (SetsWidget::*)(Movie * );
            if (_q_method_type _q_method = &SetsWidget::sigJumpToMovie; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *SetsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SetsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN10SetsWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SetsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void SetsWidget::setActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SetsWidget::sigJumpToMovie(Movie * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
