/********************************************************************************
** Form generated from reading UI file 'MusicSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICSETTINGSWIDGET_H
#define UI_MUSICSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MusicSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_17;
    QLabel *lblMusicIntro;
    QFormLayout *formLayout_9;
    QLabel *lblArtistThumbnail;
    QLineEdit *artistThumb;
    QLabel *lblArtistFanart;
    QLineEdit *artistFanart;
    QLabel *lblArtistLogo;
    QLineEdit *artistLogo;
    QLabel *lblArtistThumbnail_2;
    QLineEdit *albumThumb;
    QLabel *lblAlbumDiscArt;
    QLineEdit *albumDiscArt;
    QLabel *lblDownlaodExtraFanart;
    QSpinBox *artistExtraFanarts;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *MusicSettingsWidget)
    {
        if (MusicSettingsWidget->objectName().isEmpty())
            MusicSettingsWidget->setObjectName("MusicSettingsWidget");
        MusicSettingsWidget->resize(970, 625);
        verticalLayout_17 = new QVBoxLayout(MusicSettingsWidget);
        verticalLayout_17->setObjectName("verticalLayout_17");
        lblMusicIntro = new QLabel(MusicSettingsWidget);
        lblMusicIntro->setObjectName("lblMusicIntro");
        lblMusicIntro->setWordWrap(true);

        verticalLayout_17->addWidget(lblMusicIntro);

        formLayout_9 = new QFormLayout();
        formLayout_9->setObjectName("formLayout_9");
        formLayout_9->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        lblArtistThumbnail = new QLabel(MusicSettingsWidget);
        lblArtistThumbnail->setObjectName("lblArtistThumbnail");

        formLayout_9->setWidget(0, QFormLayout::LabelRole, lblArtistThumbnail);

        artistThumb = new QLineEdit(MusicSettingsWidget);
        artistThumb->setObjectName("artistThumb");
        artistThumb->setMaximumSize(QSize(400, 16777215));

        formLayout_9->setWidget(0, QFormLayout::FieldRole, artistThumb);

        lblArtistFanart = new QLabel(MusicSettingsWidget);
        lblArtistFanart->setObjectName("lblArtistFanart");

        formLayout_9->setWidget(1, QFormLayout::LabelRole, lblArtistFanart);

        artistFanart = new QLineEdit(MusicSettingsWidget);
        artistFanart->setObjectName("artistFanart");
        artistFanart->setMaximumSize(QSize(400, 16777215));

        formLayout_9->setWidget(1, QFormLayout::FieldRole, artistFanart);

        lblArtistLogo = new QLabel(MusicSettingsWidget);
        lblArtistLogo->setObjectName("lblArtistLogo");

        formLayout_9->setWidget(2, QFormLayout::LabelRole, lblArtistLogo);

        artistLogo = new QLineEdit(MusicSettingsWidget);
        artistLogo->setObjectName("artistLogo");
        artistLogo->setMaximumSize(QSize(400, 16777215));

        formLayout_9->setWidget(2, QFormLayout::FieldRole, artistLogo);

        lblArtistThumbnail_2 = new QLabel(MusicSettingsWidget);
        lblArtistThumbnail_2->setObjectName("lblArtistThumbnail_2");

        formLayout_9->setWidget(3, QFormLayout::LabelRole, lblArtistThumbnail_2);

        albumThumb = new QLineEdit(MusicSettingsWidget);
        albumThumb->setObjectName("albumThumb");
        albumThumb->setMaximumSize(QSize(400, 16777215));

        formLayout_9->setWidget(3, QFormLayout::FieldRole, albumThumb);

        lblAlbumDiscArt = new QLabel(MusicSettingsWidget);
        lblAlbumDiscArt->setObjectName("lblAlbumDiscArt");

        formLayout_9->setWidget(4, QFormLayout::LabelRole, lblAlbumDiscArt);

        albumDiscArt = new QLineEdit(MusicSettingsWidget);
        albumDiscArt->setObjectName("albumDiscArt");
        albumDiscArt->setMaximumSize(QSize(400, 16777215));

        formLayout_9->setWidget(4, QFormLayout::FieldRole, albumDiscArt);

        lblDownlaodExtraFanart = new QLabel(MusicSettingsWidget);
        lblDownlaodExtraFanart->setObjectName("lblDownlaodExtraFanart");

        formLayout_9->setWidget(5, QFormLayout::LabelRole, lblDownlaodExtraFanart);

        artistExtraFanarts = new QSpinBox(MusicSettingsWidget);
        artistExtraFanarts->setObjectName("artistExtraFanarts");
        artistExtraFanarts->setMaximum(20);

        formLayout_9->setWidget(5, QFormLayout::FieldRole, artistExtraFanarts);


        verticalLayout_17->addLayout(formLayout_9);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_17->addItem(verticalSpacer);

        verticalLayout_17->setStretch(1, 1);

        retranslateUi(MusicSettingsWidget);

        QMetaObject::connectSlotsByName(MusicSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *MusicSettingsWidget)
    {
        lblMusicIntro->setText(QCoreApplication::translate("MusicSettingsWidget", "Below you see the filenames which are used for loading and saving your artists and albums. You can edit them as you like, if you want to use multiple files separate them by comma.", nullptr));
        lblArtistThumbnail->setText(QCoreApplication::translate("MusicSettingsWidget", "Artist Thumbnail", nullptr));
        lblArtistFanart->setText(QCoreApplication::translate("MusicSettingsWidget", "Artist Fanart", nullptr));
        lblArtistLogo->setText(QCoreApplication::translate("MusicSettingsWidget", "Artist Logo", nullptr));
        lblArtistThumbnail_2->setText(QCoreApplication::translate("MusicSettingsWidget", "Album Thumbnail", nullptr));
        lblAlbumDiscArt->setText(QCoreApplication::translate("MusicSettingsWidget", "Album Disc Art", nullptr));
        lblDownlaodExtraFanart->setText(QCoreApplication::translate("MusicSettingsWidget", "Download Extra Fanarts for Artists", nullptr));
        (void)MusicSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class MusicSettingsWidget: public Ui_MusicSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICSETTINGSWIDGET_H
