/********************************************************************************
** Form generated from reading UI file 'TvShowWidgetTvShow.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWWIDGETTVSHOW_H
#define UI_TVSHOWWIDGETTVSHOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/Badge.h"
#include "ui/small_widgets/ClosableImage.h"
#include "ui/small_widgets/ImageGallery.h"
#include "ui/small_widgets/MyLabel.h"
#include "ui/small_widgets/MySpinBox.h"
#include "ui/small_widgets/RatingsWidget.h"
#include "ui/small_widgets/SlidingStackedWidget.h"
#include "ui/small_widgets/TagCloud.h"

QT_BEGIN_NAMESPACE

class Ui_TvShowWidgetTvShow
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_9;
    QSpacerItem *horizontalSpacer;
    QToolButton *buttonRevert;
    QLabel *showTitle;
    QGroupBox *tvShowGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tabInformation;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *lblDirectory;
    QLineEdit *dir;
    QLabel *lblIds;
    QGridLayout *showIdLayout;
    QPushButton *btnTmdb;
    QLineEdit *tvmazeId;
    QPushButton *btnTvmaze;
    QLabel *lblTvdbId;
    QLabel *lblTvMaze;
    QSpacerItem *horizontalSpacer_9;
    QLineEdit *tvdbId;
    QLineEdit *tmdbId;
    QLabel *lblImdbId;
    QLineEdit *imdbId;
    QLabel *lblTmdbId;
    QPushButton *btnImdb;
    QSpacerItem *horizontalSpacer_10;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *horizontalSpacer_11;
    QLabel *lblTitle;
    QLineEdit *title;
    QLabel *lblOriginalTitle;
    QLineEdit *originalTitle;
    QLabel *lblSortTitle;
    QLineEdit *sortTitle;
    QLabel *lblStatus;
    QComboBox *comboStatus;
    QLabel *lblRating;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_5;
    QLabel *lblUserRating;
    QSpacerItem *horizontalSpacer_2;
    QDoubleSpinBox *userRating;
    QSpacerItem *horizontalSpacer_userRating;
    QLabel *lblTop250;
    MySpinBox *top250;
    QSpacerItem *horizontalSpacer_5;
    RatingsWidget *ratings;
    QLabel *lblCertification;
    QComboBox *certification;
    QLabel *lblFirstAired;
    QLabel *lblRuntime;
    QSpinBox *runtime;
    QLabel *lblStudio;
    QLineEdit *studio;
    QLabel *lblTvTune;
    QHBoxLayout *horizontalLayout;
    Badge *badgeTuneExisting;
    Badge *badgeTuneMissing;
    QToolButton *btnDownloadTune;
    QLabel *lblOverview;
    QHBoxLayout *firstAiredLayout;
    QDateEdit *firstAired;
    QLabel *lblMissingFirstAired;
    QSpacerItem *horizontalSpacer_12;
    QPlainTextEdit *overview;
    QWidget *tabExtended;
    QVBoxLayout *verticalLayout_8;
    QFormLayout *formLayout_2;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_3;
    QTableWidget *actors;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout_8;
    QPushButton *buttonRemoveActor;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *buttonAddActor;
    QSpacerItem *verticalSpacer;
    MyLabel *actor;
    MyLabel *actorResolution;
    QSpacerItem *verticalSpacer_5;
    TagCloud *genreCloud;
    TagCloud *tagCloud;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabExtraFanarts;
    QVBoxLayout *verticalLayout_3;
    ImageGallery *fanarts;
    QHBoxLayout *horizontalLayout_12;
    QLabel *lblHintClosedImage;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *btnAddExtraFanart;
    QFrame *line;
    QVBoxLayout *verticalLayout_6;
    QWidget *artStackedWidgetButtons;
    QHBoxLayout *horizontalLayout_11;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *buttonArtPageOne;
    QPushButton *buttonArtPageTwo;
    QSpacerItem *horizontalSpacer_8;
    SlidingStackedWidget *artStackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_5;
    QLabel *labelPoster;
    ClosableImage *poster;
    QLabel *labelFanart;
    ClosableImage *backdrop;
    QLabel *labelThumb;
    ClosableImage *thumb;
    QSpacerItem *verticalSpacer_4;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_7;
    QLabel *labelLogo;
    ClosableImage *logo;
    QLabel *labelClearArt;
    ClosableImage *clearArt;
    QLabel *labelCharacterArt;
    ClosableImage *characterArt;
    QLabel *labelBanner;
    ClosableImage *banner;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *TvShowWidgetTvShow)
    {
        if (TvShowWidgetTvShow->objectName().isEmpty())
            TvShowWidgetTvShow->setObjectName("TvShowWidgetTvShow");
        TvShowWidgetTvShow->resize(954, 835);
        verticalLayout = new QVBoxLayout(TvShowWidgetTvShow);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 21);
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer);

        buttonRevert = new QToolButton(TvShowWidgetTvShow);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);

        horizontalLayout_9->addWidget(buttonRevert);

        showTitle = new QLabel(TvShowWidgetTvShow);
        showTitle->setObjectName("showTitle");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        showTitle->setFont(font);
        showTitle->setIndent(4);
        showTitle->setProperty("isHeader", QVariant(true));

        horizontalLayout_9->addWidget(showTitle);


        verticalLayout->addLayout(horizontalLayout_9);

        tvShowGroupBox = new QGroupBox(TvShowWidgetTvShow);
        tvShowGroupBox->setObjectName("tvShowGroupBox");
        horizontalLayout_2 = new QHBoxLayout(tvShowGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(tvShowGroupBox);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        tabInformation = new QWidget();
        tabInformation->setObjectName("tabInformation");
        verticalLayout_2 = new QVBoxLayout(tabInformation);
        verticalLayout_2->setObjectName("verticalLayout_2");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setRowWrapPolicy(QFormLayout::RowWrapPolicy::WrapLongRows);
        lblDirectory = new QLabel(tabInformation);
        lblDirectory->setObjectName("lblDirectory");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblDirectory);

        dir = new QLineEdit(tabInformation);
        dir->setObjectName("dir");
        dir->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, dir);

        lblIds = new QLabel(tabInformation);
        lblIds->setObjectName("lblIds");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblIds);

        showIdLayout = new QGridLayout();
        showIdLayout->setObjectName("showIdLayout");
        btnTmdb = new QPushButton(tabInformation);
        btnTmdb->setObjectName("btnTmdb");

        showIdLayout->addWidget(btnTmdb, 1, 2, 1, 1);

        tvmazeId = new QLineEdit(tabInformation);
        tvmazeId->setObjectName("tvmazeId");
        tvmazeId->setMaximumSize(QSize(150, 16777215));

        showIdLayout->addWidget(tvmazeId, 1, 6, 1, 1);

        btnTvmaze = new QPushButton(tabInformation);
        btnTvmaze->setObjectName("btnTvmaze");

        showIdLayout->addWidget(btnTvmaze, 1, 7, 1, 1);

        lblTvdbId = new QLabel(tabInformation);
        lblTvdbId->setObjectName("lblTvdbId");

        showIdLayout->addWidget(lblTvdbId, 0, 0, 1, 1);

        lblTvMaze = new QLabel(tabInformation);
        lblTvMaze->setObjectName("lblTvMaze");

        showIdLayout->addWidget(lblTvMaze, 1, 4, 1, 1);

        horizontalSpacer_9 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        showIdLayout->addItem(horizontalSpacer_9, 1, 3, 1, 1);

        tvdbId = new QLineEdit(tabInformation);
        tvdbId->setObjectName("tvdbId");
        tvdbId->setMaximumSize(QSize(150, 16777215));

        showIdLayout->addWidget(tvdbId, 0, 1, 1, 1);

        tmdbId = new QLineEdit(tabInformation);
        tmdbId->setObjectName("tmdbId");
        tmdbId->setMaximumSize(QSize(150, 16777215));

        showIdLayout->addWidget(tmdbId, 1, 1, 1, 1);

        lblImdbId = new QLabel(tabInformation);
        lblImdbId->setObjectName("lblImdbId");

        showIdLayout->addWidget(lblImdbId, 0, 4, 1, 1);

        imdbId = new QLineEdit(tabInformation);
        imdbId->setObjectName("imdbId");
        imdbId->setMaximumSize(QSize(150, 16777215));

        showIdLayout->addWidget(imdbId, 0, 6, 1, 1);

        lblTmdbId = new QLabel(tabInformation);
        lblTmdbId->setObjectName("lblTmdbId");

        showIdLayout->addWidget(lblTmdbId, 1, 0, 1, 1);

        btnImdb = new QPushButton(tabInformation);
        btnImdb->setObjectName("btnImdb");

        showIdLayout->addWidget(btnImdb, 0, 7, 1, 1);

        horizontalSpacer_10 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        showIdLayout->addItem(horizontalSpacer_10, 0, 3, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        showIdLayout->addItem(horizontalSpacer_6, 0, 8, 1, 1);

        horizontalSpacer_11 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        showIdLayout->addItem(horizontalSpacer_11, 1, 8, 1, 1);


        formLayout->setLayout(1, QFormLayout::FieldRole, showIdLayout);

        lblTitle = new QLabel(tabInformation);
        lblTitle->setObjectName("lblTitle");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblTitle);

        title = new QLineEdit(tabInformation);
        title->setObjectName("title");

        formLayout->setWidget(2, QFormLayout::FieldRole, title);

        lblOriginalTitle = new QLabel(tabInformation);
        lblOriginalTitle->setObjectName("lblOriginalTitle");

        formLayout->setWidget(3, QFormLayout::LabelRole, lblOriginalTitle);

        originalTitle = new QLineEdit(tabInformation);
        originalTitle->setObjectName("originalTitle");

        formLayout->setWidget(3, QFormLayout::FieldRole, originalTitle);

        lblSortTitle = new QLabel(tabInformation);
        lblSortTitle->setObjectName("lblSortTitle");

        formLayout->setWidget(4, QFormLayout::LabelRole, lblSortTitle);

        sortTitle = new QLineEdit(tabInformation);
        sortTitle->setObjectName("sortTitle");

        formLayout->setWidget(4, QFormLayout::FieldRole, sortTitle);

        lblStatus = new QLabel(tabInformation);
        lblStatus->setObjectName("lblStatus");

        formLayout->setWidget(5, QFormLayout::LabelRole, lblStatus);

        comboStatus = new QComboBox(tabInformation);
        comboStatus->addItem(QString::fromUtf8(""));
        comboStatus->addItem(QString());
        comboStatus->addItem(QString());
        comboStatus->setObjectName("comboStatus");
        comboStatus->setMinimumContentsLength(10);

        formLayout->setWidget(5, QFormLayout::FieldRole, comboStatus);

        lblRating = new QLabel(tabInformation);
        lblRating->setObjectName("lblRating");

        formLayout->setWidget(6, QFormLayout::LabelRole, lblRating);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        lblUserRating = new QLabel(tabInformation);
        lblUserRating->setObjectName("lblUserRating");

        horizontalLayout_5->addWidget(lblUserRating);

        horizontalSpacer_2 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);

        userRating = new QDoubleSpinBox(tabInformation);
        userRating->setObjectName("userRating");
        userRating->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        userRating->setDecimals(2);
        userRating->setMaximum(10.000000000000000);

        horizontalLayout_5->addWidget(userRating);

        horizontalSpacer_userRating = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_userRating);

        lblTop250 = new QLabel(tabInformation);
        lblTop250->setObjectName("lblTop250");

        horizontalLayout_5->addWidget(lblTop250);

        top250 = new MySpinBox(tabInformation);
        top250->setObjectName("top250");
        top250->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        top250->setMaximum(250);

        horizontalLayout_5->addWidget(top250);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);


        verticalLayout_4->addLayout(horizontalLayout_5);

        ratings = new RatingsWidget(tabInformation);
        ratings->setObjectName("ratings");

        verticalLayout_4->addWidget(ratings);


        formLayout->setLayout(6, QFormLayout::FieldRole, verticalLayout_4);

        lblCertification = new QLabel(tabInformation);
        lblCertification->setObjectName("lblCertification");

        formLayout->setWidget(7, QFormLayout::LabelRole, lblCertification);

        certification = new QComboBox(tabInformation);
        certification->setObjectName("certification");
        certification->setMinimumSize(QSize(140, 0));
        certification->setEditable(true);
        certification->setMinimumContentsLength(10);

        formLayout->setWidget(7, QFormLayout::FieldRole, certification);

        lblFirstAired = new QLabel(tabInformation);
        lblFirstAired->setObjectName("lblFirstAired");

        formLayout->setWidget(8, QFormLayout::LabelRole, lblFirstAired);

        lblRuntime = new QLabel(tabInformation);
        lblRuntime->setObjectName("lblRuntime");

        formLayout->setWidget(9, QFormLayout::LabelRole, lblRuntime);

        runtime = new QSpinBox(tabInformation);
        runtime->setObjectName("runtime");
        runtime->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        runtime->setMaximum(999);

        formLayout->setWidget(9, QFormLayout::FieldRole, runtime);

        lblStudio = new QLabel(tabInformation);
        lblStudio->setObjectName("lblStudio");

        formLayout->setWidget(10, QFormLayout::LabelRole, lblStudio);

        studio = new QLineEdit(tabInformation);
        studio->setObjectName("studio");

        formLayout->setWidget(10, QFormLayout::FieldRole, studio);

        lblTvTune = new QLabel(tabInformation);
        lblTvTune->setObjectName("lblTvTune");

        formLayout->setWidget(11, QFormLayout::LabelRole, lblTvTune);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        badgeTuneExisting = new Badge(tabInformation);
        badgeTuneExisting->setObjectName("badgeTuneExisting");

        horizontalLayout->addWidget(badgeTuneExisting);

        badgeTuneMissing = new Badge(tabInformation);
        badgeTuneMissing->setObjectName("badgeTuneMissing");

        horizontalLayout->addWidget(badgeTuneMissing);

        btnDownloadTune = new QToolButton(tabInformation);
        btnDownloadTune->setObjectName("btnDownloadTune");
        btnDownloadTune->setMinimumSize(QSize(22, 22));
        btnDownloadTune->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout->addWidget(btnDownloadTune);


        formLayout->setLayout(11, QFormLayout::FieldRole, horizontalLayout);

        lblOverview = new QLabel(tabInformation);
        lblOverview->setObjectName("lblOverview");

        formLayout->setWidget(12, QFormLayout::LabelRole, lblOverview);

        firstAiredLayout = new QHBoxLayout();
        firstAiredLayout->setObjectName("firstAiredLayout");
        firstAired = new QDateEdit(tabInformation);
        firstAired->setObjectName("firstAired");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(firstAired->sizePolicy().hasHeightForWidth());
        firstAired->setSizePolicy(sizePolicy);
        firstAired->setMinimumSize(QSize(140, 0));
        firstAired->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        firstAiredLayout->addWidget(firstAired);

        lblMissingFirstAired = new QLabel(tabInformation);
        lblMissingFirstAired->setObjectName("lblMissingFirstAired");
        lblMissingFirstAired->setTextFormat(Qt::TextFormat::RichText);

        firstAiredLayout->addWidget(lblMissingFirstAired);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        firstAiredLayout->addItem(horizontalSpacer_12);

        firstAiredLayout->setStretch(2, 1);

        formLayout->setLayout(8, QFormLayout::FieldRole, firstAiredLayout);

        overview = new QPlainTextEdit(tabInformation);
        overview->setObjectName("overview");

        formLayout->setWidget(12, QFormLayout::FieldRole, overview);


        verticalLayout_2->addLayout(formLayout);

        tabWidget->addTab(tabInformation, QString());
        tabExtended = new QWidget();
        tabExtended->setObjectName("tabExtended");
        verticalLayout_8 = new QVBoxLayout(tabExtended);
        verticalLayout_8->setObjectName("verticalLayout_8");
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        label_3 = new QLabel(tabExtended);
        label_3->setObjectName("label_3");

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_3);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        actors = new QTableWidget(tabExtended);
        if (actors->columnCount() < 2)
            actors->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        actors->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        actors->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        actors->setObjectName("actors");
        actors->setEditTriggers(QAbstractItemView::EditTrigger::DoubleClicked);
        actors->setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
        actors->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        actors->setShowGrid(false);
        actors->verticalHeader()->setVisible(false);

        horizontalLayout_3->addWidget(actors);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setSpacing(0);
        verticalLayout_9->setObjectName("verticalLayout_9");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        buttonRemoveActor = new QPushButton(tabExtended);
        buttonRemoveActor->setObjectName("buttonRemoveActor");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/minus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRemoveActor->setIcon(icon1);

        horizontalLayout_8->addWidget(buttonRemoveActor);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_3);

        buttonAddActor = new QPushButton(tabExtended);
        buttonAddActor->setObjectName("buttonAddActor");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/img/plus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonAddActor->setIcon(icon2);

        horizontalLayout_8->addWidget(buttonAddActor);


        verticalLayout_9->addLayout(horizontalLayout_8);

        verticalSpacer = new QSpacerItem(20, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_9->addItem(verticalSpacer);

        actor = new MyLabel(tabExtended);
        actor->setObjectName("actor");
        actor->setMinimumSize(QSize(120, 180));
        actor->setMaximumSize(QSize(120, 180));
        actor->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        actor->setPixmap(QPixmap(QString::fromUtf8(":/img/man.png")));
        actor->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout_9->addWidget(actor);

        actorResolution = new MyLabel(tabExtended);
        actorResolution->setObjectName("actorResolution");
        actorResolution->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        verticalLayout_9->addWidget(actorResolution);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_5);


        horizontalLayout_3->addLayout(verticalLayout_9);

        horizontalLayout_3->setStretch(0, 1);

        formLayout_2->setLayout(0, QFormLayout::FieldRole, horizontalLayout_3);


        verticalLayout_8->addLayout(formLayout_2);

        genreCloud = new TagCloud(tabExtended);
        genreCloud->setObjectName("genreCloud");

        verticalLayout_8->addWidget(genreCloud);

        tagCloud = new TagCloud(tabExtended);
        tagCloud->setObjectName("tagCloud");

        verticalLayout_8->addWidget(tagCloud);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_2);

        verticalLayout_8->setStretch(1, 1);
        verticalLayout_8->setStretch(2, 1);
        tabWidget->addTab(tabExtended, QString());
        tabExtraFanarts = new QWidget();
        tabExtraFanarts->setObjectName("tabExtraFanarts");
        verticalLayout_3 = new QVBoxLayout(tabExtraFanarts);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(12, -1, 12, -1);
        fanarts = new ImageGallery(tabExtraFanarts);
        fanarts->setObjectName("fanarts");

        verticalLayout_3->addWidget(fanarts);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        lblHintClosedImage = new QLabel(tabExtraFanarts);
        lblHintClosedImage->setObjectName("lblHintClosedImage");
        QFont font1;
        font1.setItalic(true);
        lblHintClosedImage->setFont(font1);
        lblHintClosedImage->setWordWrap(true);

        horizontalLayout_12->addWidget(lblHintClosedImage);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_4);

        btnAddExtraFanart = new QPushButton(tabExtraFanarts);
        btnAddExtraFanart->setObjectName("btnAddExtraFanart");

        horizontalLayout_12->addWidget(btnAddExtraFanart);

        horizontalLayout_12->setStretch(0, 1);

        verticalLayout_3->addLayout(horizontalLayout_12);

        verticalLayout_3->setStretch(0, 1);
        tabWidget->addTab(tabExtraFanarts, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(tvShowGroupBox);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Shadow::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(0);
        line->setFrameShape(QFrame::Shape::VLine);

        horizontalLayout_2->addWidget(line);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setSpacing(0);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(10, 0, -1, -1);
        artStackedWidgetButtons = new QWidget(tvShowGroupBox);
        artStackedWidgetButtons->setObjectName("artStackedWidgetButtons");
        horizontalLayout_11 = new QHBoxLayout(artStackedWidgetButtons);
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        horizontalLayout_11->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_7 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_7);

        buttonArtPageOne = new QPushButton(artStackedWidgetButtons);
        buttonArtPageOne->setObjectName("buttonArtPageOne");
        buttonArtPageOne->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonArtPageOne->setCheckable(true);
        buttonArtPageOne->setChecked(true);

        horizontalLayout_11->addWidget(buttonArtPageOne);

        buttonArtPageTwo = new QPushButton(artStackedWidgetButtons);
        buttonArtPageTwo->setObjectName("buttonArtPageTwo");
        buttonArtPageTwo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonArtPageTwo->setCheckable(true);

        horizontalLayout_11->addWidget(buttonArtPageTwo);

        horizontalSpacer_8 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_8);


        verticalLayout_6->addWidget(artStackedWidgetButtons);

        artStackedWidget = new SlidingStackedWidget(tvShowGroupBox);
        artStackedWidget->setObjectName("artStackedWidget");
        artStackedWidget->setMinimumSize(QSize(180, 0));
        artStackedWidget->setMaximumSize(QSize(180, 16777215));
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_5 = new QVBoxLayout(page);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        labelPoster = new QLabel(page);
        labelPoster->setObjectName("labelPoster");

        verticalLayout_5->addWidget(labelPoster);

        poster = new ClosableImage(page);
        poster->setObjectName("poster");
        poster->setMinimumSize(QSize(180, 290));
        poster->setMaximumSize(QSize(180, 290));
        poster->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        poster->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/poster.png")));
        poster->setAlignment(Qt::AlignmentFlag::AlignCenter);
        poster->setProperty("clickable", QVariant(true));
        poster->setProperty("myFixedHeight", QVariant(290));

        verticalLayout_5->addWidget(poster);

        labelFanart = new QLabel(page);
        labelFanart->setObjectName("labelFanart");

        verticalLayout_5->addWidget(labelFanart);

        backdrop = new ClosableImage(page);
        backdrop->setObjectName("backdrop");
        backdrop->setMinimumSize(QSize(180, 120));
        backdrop->setMaximumSize(QSize(180, 120));
        backdrop->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        backdrop->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/fanart.png")));
        backdrop->setAlignment(Qt::AlignmentFlag::AlignCenter);
        backdrop->setProperty("clickable", QVariant(true));
        backdrop->setProperty("myFixedHeight", QVariant(120));

        verticalLayout_5->addWidget(backdrop);

        labelThumb = new QLabel(page);
        labelThumb->setObjectName("labelThumb");

        verticalLayout_5->addWidget(labelThumb);

        thumb = new ClosableImage(page);
        thumb->setObjectName("thumb");
        thumb->setMinimumSize(QSize(180, 102));
        thumb->setMaximumSize(QSize(180, 102));
        thumb->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        thumb->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/thumb.png")));
        thumb->setAlignment(Qt::AlignmentFlag::AlignCenter);
        thumb->setProperty("clickable", QVariant(true));
        thumb->setProperty("myFixedHeight", QVariant(102));

        verticalLayout_5->addWidget(thumb);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_4);

        artStackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_7 = new QVBoxLayout(page_2);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        labelLogo = new QLabel(page_2);
        labelLogo->setObjectName("labelLogo");

        verticalLayout_7->addWidget(labelLogo);

        logo = new ClosableImage(page_2);
        logo->setObjectName("logo");
        logo->setMinimumSize(QSize(180, 110));
        logo->setMaximumSize(QSize(180, 110));
        logo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        logo->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/logo.png")));
        logo->setAlignment(Qt::AlignmentFlag::AlignCenter);
        logo->setProperty("clickable", QVariant(true));
        logo->setProperty("myFixedHeight", QVariant(110));

        verticalLayout_7->addWidget(logo);

        labelClearArt = new QLabel(page_2);
        labelClearArt->setObjectName("labelClearArt");

        verticalLayout_7->addWidget(labelClearArt);

        clearArt = new ClosableImage(page_2);
        clearArt->setObjectName("clearArt");
        clearArt->setMinimumSize(QSize(180, 155));
        clearArt->setMaximumSize(QSize(180, 155));
        clearArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        clearArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/clear_art.png")));
        clearArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        clearArt->setProperty("clickable", QVariant(true));
        clearArt->setProperty("myFixedHeight", QVariant(155));

        verticalLayout_7->addWidget(clearArt);

        labelCharacterArt = new QLabel(page_2);
        labelCharacterArt->setObjectName("labelCharacterArt");

        verticalLayout_7->addWidget(labelCharacterArt);

        characterArt = new ClosableImage(page_2);
        characterArt->setObjectName("characterArt");
        characterArt->setMinimumSize(QSize(180, 200));
        characterArt->setMaximumSize(QSize(180, 200));
        characterArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        characterArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/character_art.png")));
        characterArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        characterArt->setProperty("clickable", QVariant(true));
        characterArt->setProperty("myFixedHeight", QVariant(200));

        verticalLayout_7->addWidget(characterArt);

        labelBanner = new QLabel(page_2);
        labelBanner->setObjectName("labelBanner");

        verticalLayout_7->addWidget(labelBanner);

        banner = new ClosableImage(page_2);
        banner->setObjectName("banner");
        banner->setMinimumSize(QSize(180, 53));
        banner->setMaximumSize(QSize(200, 53));
        banner->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        banner->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/banner.png")));
        banner->setAlignment(Qt::AlignmentFlag::AlignCenter);
        banner->setProperty("clickable", QVariant(true));
        banner->setProperty("myFixedHeight", QVariant(53));

        verticalLayout_7->addWidget(banner);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_3);

        artStackedWidget->addWidget(page_2);

        verticalLayout_6->addWidget(artStackedWidget);

        verticalLayout_6->setStretch(1, 1);

        horizontalLayout_2->addLayout(verticalLayout_6);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout->addWidget(tvShowGroupBox);

        verticalLayout->setStretch(1, 1);
#if QT_CONFIG(shortcut)
        lblTvdbId->setBuddy(tvdbId);
        lblTvMaze->setBuddy(tvmazeId);
        lblImdbId->setBuddy(imdbId);
        lblTmdbId->setBuddy(tmdbId);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TvShowWidgetTvShow);
        QObject::connect(buttonArtPageOne, SIGNAL(clicked()), TvShowWidgetTvShow, SLOT(onArtPageOne()));
        QObject::connect(buttonArtPageTwo, SIGNAL(clicked()), TvShowWidgetTvShow, SLOT(onArtPageTwo()));

        tabWidget->setCurrentIndex(0);
        artStackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(TvShowWidgetTvShow);
    } // setupUi

    void retranslateUi(QWidget *TvShowWidgetTvShow)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "TV Show has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        showTitle->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Show Title", nullptr));
        tvShowGroupBox->setTitle(QString());
        lblDirectory->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Dir", nullptr));
        lblIds->setText(QCoreApplication::translate("TvShowWidgetTvShow", "IDs", nullptr));
#if QT_CONFIG(tooltip)
        btnTmdb->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Open TV show on TMDB.com", nullptr));
#endif // QT_CONFIG(tooltip)
        btnTmdb->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Go", nullptr));
#if QT_CONFIG(tooltip)
        btnTvmaze->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Open TV show on TVmaze.com", nullptr));
#endif // QT_CONFIG(tooltip)
        btnTvmaze->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Go", nullptr));
        lblTvdbId->setText(QCoreApplication::translate("TvShowWidgetTvShow", "TheTVDB", nullptr));
        lblTvMaze->setText(QCoreApplication::translate("TvShowWidgetTvShow", "TVmaze", nullptr));
        lblImdbId->setText(QCoreApplication::translate("TvShowWidgetTvShow", "IMDb", nullptr));
        lblTmdbId->setText(QCoreApplication::translate("TvShowWidgetTvShow", "TMDB", nullptr));
#if QT_CONFIG(tooltip)
        btnImdb->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Open TV show on IMDb.com", nullptr));
#endif // QT_CONFIG(tooltip)
        btnImdb->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Go", nullptr));
        lblTitle->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Name", nullptr));
        lblOriginalTitle->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Original Title", nullptr));
        lblSortTitle->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Sort Title", nullptr));
        lblStatus->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Status", nullptr));
        comboStatus->setItemText(1, QCoreApplication::translate("TvShowWidgetTvShow", "Continuing", nullptr));
        comboStatus->setItemText(2, QCoreApplication::translate("TvShowWidgetTvShow", "Ended", nullptr));

        lblRating->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Rating", nullptr));
        lblUserRating->setText(QCoreApplication::translate("TvShowWidgetTvShow", "User Rating", nullptr));
        lblTop250->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Top 250", nullptr));
        lblCertification->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Certification", nullptr));
        lblFirstAired->setText(QCoreApplication::translate("TvShowWidgetTvShow", "First Aired", nullptr));
        lblRuntime->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Runtime", nullptr));
        runtime->setSuffix(QCoreApplication::translate("TvShowWidgetTvShow", " Minutes", nullptr));
        lblStudio->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Studio", nullptr));
        lblTvTune->setText(QCoreApplication::translate("TvShowWidgetTvShow", "TV Tune", nullptr));
        badgeTuneExisting->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Existing", nullptr));
        badgeTuneMissing->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Missing", nullptr));
#if QT_CONFIG(tooltip)
        btnDownloadTune->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Download Theme", nullptr));
#endif // QT_CONFIG(tooltip)
        btnDownloadTune->setText(QString());
        lblOverview->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Overview", nullptr));
        firstAired->setDisplayFormat(QCoreApplication::translate("TvShowWidgetTvShow", "yyyy-MM-dd", nullptr));
        lblMissingFirstAired->setText(QCoreApplication::translate("TvShowWidgetTvShow", "<i>missing</i>", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabInformation), QCoreApplication::translate("TvShowWidgetTvShow", "Information", nullptr));
        label_3->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Actors", nullptr));
        QTableWidgetItem *___qtablewidgetitem = actors->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Actor", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = actors->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Role", nullptr));
#if QT_CONFIG(tooltip)
        buttonRemoveActor->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Remove Actor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveActor->setText(QString());
#if QT_CONFIG(tooltip)
        buttonAddActor->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Add Actor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddActor->setText(QString());
#if QT_CONFIG(tooltip)
        actor->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to change", nullptr));
#endif // QT_CONFIG(tooltip)
        actor->setText(QString());
        actorResolution->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Resolution", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtended), QCoreApplication::translate("TvShowWidgetTvShow", "Extended", nullptr));
        lblHintClosedImage->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Hint: Closed images will be deleted on save.", nullptr));
        btnAddExtraFanart->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Add Images", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtraFanarts), QCoreApplication::translate("TvShowWidgetTvShow", "Extra Fanarts", nullptr));
        buttonArtPageOne->setText(QString());
        buttonArtPageTwo->setText(QString());
        labelPoster->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Poster", nullptr));
#if QT_CONFIG(tooltip)
        poster->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        poster->setText(QString());
        labelFanart->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Fanart", nullptr));
#if QT_CONFIG(tooltip)
        backdrop->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        backdrop->setText(QString());
        labelThumb->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Thumb", nullptr));
#if QT_CONFIG(tooltip)
        thumb->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        thumb->setText(QString());
        labelLogo->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Logo", nullptr));
#if QT_CONFIG(tooltip)
        logo->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        logo->setText(QString());
        labelClearArt->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Clear Art", nullptr));
#if QT_CONFIG(tooltip)
        clearArt->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        clearArt->setText(QString());
        labelCharacterArt->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Character Art", nullptr));
#if QT_CONFIG(tooltip)
        characterArt->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        characterArt->setText(QString());
        labelBanner->setText(QCoreApplication::translate("TvShowWidgetTvShow", "Banner", nullptr));
#if QT_CONFIG(tooltip)
        banner->setToolTip(QCoreApplication::translate("TvShowWidgetTvShow", "Click to change", nullptr));
#endif // QT_CONFIG(tooltip)
        banner->setText(QString());
        (void)TvShowWidgetTvShow;
    } // retranslateUi

};

namespace Ui {
    class TvShowWidgetTvShow: public Ui_TvShowWidgetTvShow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWWIDGETTVSHOW_H
