/****************************************************************************
** Meta object code from reading C++ file 'ImageProvider.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/scrapers/image/ImageProvider.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ImageProvider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper13ImageProviderE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper13ImageProviderE = QtMocHelpers::stringData(
    "mediaelch::scraper::ImageProvider",
    "sigSearchDone",
    "",
    "QList<ScraperSearchResult>",
    "mediaelch::ScraperError",
    "error",
    "sigImagesLoaded",
    "QList<Poster>",
    "sigMovieImagesLoaded",
    "Movie*",
    "QMap<ImageType,QList<Poster>>",
    "sigConcertImagesLoaded",
    "Concert*",
    "sigTvShowImagesLoaded",
    "TvShow*",
    "sigArtistImagesLoaded",
    "Artist*",
    "sigAlbumImagesLoaded",
    "Album*",
    "searchMovie",
    "searchStr",
    "limit",
    "searchConcert",
    "searchTvShow",
    "mediaelch::Locale",
    "locale",
    "searchArtist",
    "searchAlbum",
    "artistName"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper13ImageProviderE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,   86,    2, 0x06,    1 /* Public */,
       6,    2,   91,    2, 0x06,    4 /* Public */,
       8,    2,   96,    2, 0x06,    7 /* Public */,
      11,    2,  101,    2, 0x06,   10 /* Public */,
      13,    2,  106,    2, 0x06,   13 /* Public */,
      15,    2,  111,    2, 0x06,   16 /* Public */,
      17,    2,  116,    2, 0x06,   19 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      19,    2,  121,    2, 0x0a,   22 /* Public */,
      22,    2,  126,    2, 0x0a,   25 /* Public */,
      23,    3,  131,    2, 0x0a,   28 /* Public */,
      26,    2,  138,    2, 0x0a,   32 /* Public */,
      27,    3,  143,    2, 0x0a,   35 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 4,    2,    5,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 4,    2,    5,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 10,    2,    2,
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 10,    2,    2,
    QMetaType::Void, 0x80000000 | 14, 0x80000000 | 10,    2,    2,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 10,    2,    2,
    QMetaType::Void, 0x80000000 | 18, 0x80000000 | 10,    2,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   20,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   20,   21,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 24, QMetaType::Int,   20,   25,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   20,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   28,   20,   21,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::ImageProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper13ImageProviderE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper13ImageProviderE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper13ImageProviderE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<ImageProvider, std::true_type>,
        // method 'sigSearchDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<ScraperSearchResult>, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::ScraperError, std::false_type>,
        // method 'sigImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<Poster>, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::ScraperError, std::false_type>,
        // method 'sigMovieImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'sigConcertImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Concert *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'sigTvShowImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<TvShow *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'sigArtistImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'sigAlbumImagesLoaded'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Album *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMap<ImageType,QVector<Poster>>, std::false_type>,
        // method 'searchMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchConcert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchTvShow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::Locale, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchArtist'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchAlbum'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::ImageProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImageProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigSearchDone((*reinterpret_cast< std::add_pointer_t<QList<ScraperSearchResult>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::ScraperError>>(_a[2]))); break;
        case 1: _t->sigImagesLoaded((*reinterpret_cast< std::add_pointer_t<QList<Poster>>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::ScraperError>>(_a[2]))); break;
        case 2: _t->sigMovieImagesLoaded((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 3: _t->sigConcertImagesLoaded((*reinterpret_cast< std::add_pointer_t<Concert*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 4: _t->sigTvShowImagesLoaded((*reinterpret_cast< std::add_pointer_t<TvShow*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 5: _t->sigArtistImagesLoaded((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 6: _t->sigAlbumImagesLoaded((*reinterpret_cast< std::add_pointer_t<Album*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QMap<ImageType,QList<Poster>>>>(_a[2]))); break;
        case 7: _t->searchMovie((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->searchConcert((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->searchTvShow((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::Locale>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->searchArtist((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->searchAlbum((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Concert* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TvShow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (ImageProvider::*)(QVector<ScraperSearchResult> , mediaelch::ScraperError );
            if (_q_method_type _q_method = &ImageProvider::sigSearchDone; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (ImageProvider::*)(QVector<Poster> , mediaelch::ScraperError );
            if (_q_method_type _q_method = &ImageProvider::sigImagesLoaded; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (ImageProvider::*)(Movie * , QMap<ImageType,QVector<Poster>> );
            if (_q_method_type _q_method = &ImageProvider::sigMovieImagesLoaded; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (ImageProvider::*)(Concert * , QMap<ImageType,QVector<Poster>> );
            if (_q_method_type _q_method = &ImageProvider::sigConcertImagesLoaded; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (ImageProvider::*)(TvShow * , QMap<ImageType,QVector<Poster>> );
            if (_q_method_type _q_method = &ImageProvider::sigTvShowImagesLoaded; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
        {
            using _q_method_type = void (ImageProvider::*)(Artist * , QMap<ImageType,QVector<Poster>> );
            if (_q_method_type _q_method = &ImageProvider::sigArtistImagesLoaded; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 5;
                return;
            }
        }
        {
            using _q_method_type = void (ImageProvider::*)(Album * , QMap<ImageType,QVector<Poster>> );
            if (_q_method_type _q_method = &ImageProvider::sigAlbumImagesLoaded; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 6;
                return;
            }
        }
    }
}

const QMetaObject *mediaelch::scraper::ImageProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::ImageProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper13ImageProviderE.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ScraperInterface"))
        return static_cast< ScraperInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int mediaelch::scraper::ImageProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void mediaelch::scraper::ImageProvider::sigSearchDone(QVector<ScraperSearchResult> _t1, mediaelch::ScraperError _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void mediaelch::scraper::ImageProvider::sigImagesLoaded(QVector<Poster> _t1, mediaelch::ScraperError _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void mediaelch::scraper::ImageProvider::sigMovieImagesLoaded(Movie * _t1, QMap<ImageType,QVector<Poster>> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void mediaelch::scraper::ImageProvider::sigConcertImagesLoaded(Concert * _t1, QMap<ImageType,QVector<Poster>> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void mediaelch::scraper::ImageProvider::sigTvShowImagesLoaded(TvShow * _t1, QMap<ImageType,QVector<Poster>> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void mediaelch::scraper::ImageProvider::sigArtistImagesLoaded(Artist * _t1, QMap<ImageType,QVector<Poster>> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void mediaelch::scraper::ImageProvider::sigAlbumImagesLoaded(Album * _t1, QMap<ImageType,QVector<Poster>> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
