/********************************************************************************
** Form generated from reading UI file 'GlobalSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GLOBALSETTINGSWIDGET_H
#define UI_GLOBALSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_GlobalSettingsWidget
{
public:
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *globalSettingsTab;
    QWidget *globalSettingsDirectoriesTab;
    QVBoxLayout *verticalLayout;
    QScrollArea *globalSettingsScrollArea;
    QWidget *globalSettingsScrollAreaContent;
    QVBoxLayout *verticalLayout_3;
    QLabel *labelGlobal;
    QTableWidget *dirs;
    QHBoxLayout *horizontalLayout;
    QPushButton *buttonAddDir;
    QPushButton *buttonRemoveDir;
    QPushButton *buttonMovieFilesToDirs;
    QSpacerItem *horizontalSpacer;
    QFormLayout *formLayout_3;
    QLabel *lblYouTubePlugin;
    QCheckBox *useYoutubePluginUrls;
    QLabel *lblAutomaticStreamDetails;
    QCheckBox *chkAutoLoadStreamDetails;
    QLabel *lblIgnoreArticles;
    QCheckBox *chkIgnoreArticlesWhenSorting;
    QLabel *lblDownloadActorImages;
    QCheckBox *chkDownloadActorImages;
    QLabel *lblCheckForUpdates;
    QCheckBox *chkCheckForUpdates;
    QLabel *lblExcludeWords;
    QTextEdit *excludeWordsText;
    QLabel *lblStartupSection;
    QComboBox *comboStartupSection;
    QSpacerItem *verticalSpacer;
    QWidget *globalSettingsAppearanceTab;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *lblTheme;
    QComboBox *comboTheme;

    void setupUi(QWidget *GlobalSettingsWidget)
    {
        if (GlobalSettingsWidget->objectName().isEmpty())
            GlobalSettingsWidget->setObjectName("GlobalSettingsWidget");
        GlobalSettingsWidget->resize(970, 625);
        horizontalLayout_2 = new QHBoxLayout(GlobalSettingsWidget);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        globalSettingsTab = new QTabWidget(GlobalSettingsWidget);
        globalSettingsTab->setObjectName("globalSettingsTab");
        globalSettingsDirectoriesTab = new QWidget();
        globalSettingsDirectoriesTab->setObjectName("globalSettingsDirectoriesTab");
        verticalLayout = new QVBoxLayout(globalSettingsDirectoriesTab);
        verticalLayout->setObjectName("verticalLayout");
        globalSettingsScrollArea = new QScrollArea(globalSettingsDirectoriesTab);
        globalSettingsScrollArea->setObjectName("globalSettingsScrollArea");
        globalSettingsScrollArea->setFrameShape(QFrame::NoFrame);
        globalSettingsScrollArea->setWidgetResizable(true);
        globalSettingsScrollAreaContent = new QWidget();
        globalSettingsScrollAreaContent->setObjectName("globalSettingsScrollAreaContent");
        globalSettingsScrollAreaContent->setGeometry(QRect(0, 0, 905, 687));
        verticalLayout_3 = new QVBoxLayout(globalSettingsScrollAreaContent);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        labelGlobal = new QLabel(globalSettingsScrollAreaContent);
        labelGlobal->setObjectName("labelGlobal");
        labelGlobal->setWordWrap(true);

        verticalLayout_3->addWidget(labelGlobal);

        dirs = new QTableWidget(globalSettingsScrollAreaContent);
        if (dirs->columnCount() < 5)
            dirs->setColumnCount(5);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        dirs->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        dirs->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        dirs->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        dirs->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        dirs->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        dirs->setObjectName("dirs");
        dirs->setMinimumSize(QSize(0, 100));
        dirs->setMaximumSize(QSize(16777215, 200));
        dirs->setSelectionMode(QAbstractItemView::SingleSelection);
        dirs->horizontalHeader()->setDefaultSectionSize(120);
        dirs->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(dirs);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        buttonAddDir = new QPushButton(globalSettingsScrollAreaContent);
        buttonAddDir->setObjectName("buttonAddDir");

        horizontalLayout->addWidget(buttonAddDir);

        buttonRemoveDir = new QPushButton(globalSettingsScrollAreaContent);
        buttonRemoveDir->setObjectName("buttonRemoveDir");

        horizontalLayout->addWidget(buttonRemoveDir);

        buttonMovieFilesToDirs = new QPushButton(globalSettingsScrollAreaContent);
        buttonMovieFilesToDirs->setObjectName("buttonMovieFilesToDirs");

        horizontalLayout->addWidget(buttonMovieFilesToDirs);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        lblYouTubePlugin = new QLabel(globalSettingsScrollAreaContent);
        lblYouTubePlugin->setObjectName("lblYouTubePlugin");

        formLayout_3->setWidget(0, QFormLayout::LabelRole, lblYouTubePlugin);

        useYoutubePluginUrls = new QCheckBox(globalSettingsScrollAreaContent);
        useYoutubePluginUrls->setObjectName("useYoutubePluginUrls");

        formLayout_3->setWidget(0, QFormLayout::FieldRole, useYoutubePluginUrls);

        lblAutomaticStreamDetails = new QLabel(globalSettingsScrollAreaContent);
        lblAutomaticStreamDetails->setObjectName("lblAutomaticStreamDetails");

        formLayout_3->setWidget(1, QFormLayout::LabelRole, lblAutomaticStreamDetails);

        chkAutoLoadStreamDetails = new QCheckBox(globalSettingsScrollAreaContent);
        chkAutoLoadStreamDetails->setObjectName("chkAutoLoadStreamDetails");

        formLayout_3->setWidget(1, QFormLayout::FieldRole, chkAutoLoadStreamDetails);

        lblIgnoreArticles = new QLabel(globalSettingsScrollAreaContent);
        lblIgnoreArticles->setObjectName("lblIgnoreArticles");

        formLayout_3->setWidget(2, QFormLayout::LabelRole, lblIgnoreArticles);

        chkIgnoreArticlesWhenSorting = new QCheckBox(globalSettingsScrollAreaContent);
        chkIgnoreArticlesWhenSorting->setObjectName("chkIgnoreArticlesWhenSorting");

        formLayout_3->setWidget(2, QFormLayout::FieldRole, chkIgnoreArticlesWhenSorting);

        lblDownloadActorImages = new QLabel(globalSettingsScrollAreaContent);
        lblDownloadActorImages->setObjectName("lblDownloadActorImages");

        formLayout_3->setWidget(3, QFormLayout::LabelRole, lblDownloadActorImages);

        chkDownloadActorImages = new QCheckBox(globalSettingsScrollAreaContent);
        chkDownloadActorImages->setObjectName("chkDownloadActorImages");

        formLayout_3->setWidget(3, QFormLayout::FieldRole, chkDownloadActorImages);

        lblCheckForUpdates = new QLabel(globalSettingsScrollAreaContent);
        lblCheckForUpdates->setObjectName("lblCheckForUpdates");

        formLayout_3->setWidget(4, QFormLayout::LabelRole, lblCheckForUpdates);

        chkCheckForUpdates = new QCheckBox(globalSettingsScrollAreaContent);
        chkCheckForUpdates->setObjectName("chkCheckForUpdates");

        formLayout_3->setWidget(4, QFormLayout::FieldRole, chkCheckForUpdates);

        lblExcludeWords = new QLabel(globalSettingsScrollAreaContent);
        lblExcludeWords->setObjectName("lblExcludeWords");
        lblExcludeWords->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        lblExcludeWords->setWordWrap(true);

        formLayout_3->setWidget(6, QFormLayout::LabelRole, lblExcludeWords);

        excludeWordsText = new QTextEdit(globalSettingsScrollAreaContent);
        excludeWordsText->setObjectName("excludeWordsText");
        excludeWordsText->setMaximumSize(QSize(16777215, 100));

        formLayout_3->setWidget(6, QFormLayout::FieldRole, excludeWordsText);

        lblStartupSection = new QLabel(globalSettingsScrollAreaContent);
        lblStartupSection->setObjectName("lblStartupSection");

        formLayout_3->setWidget(5, QFormLayout::LabelRole, lblStartupSection);

        comboStartupSection = new QComboBox(globalSettingsScrollAreaContent);
        comboStartupSection->setObjectName("comboStartupSection");

        formLayout_3->setWidget(5, QFormLayout::FieldRole, comboStartupSection);


        verticalLayout_3->addLayout(formLayout_3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        verticalLayout_3->setStretch(1, 1);
        globalSettingsScrollArea->setWidget(globalSettingsScrollAreaContent);

        verticalLayout->addWidget(globalSettingsScrollArea);

        globalSettingsTab->addTab(globalSettingsDirectoriesTab, QString());
        globalSettingsAppearanceTab = new QWidget();
        globalSettingsAppearanceTab->setObjectName("globalSettingsAppearanceTab");
        verticalLayout_2 = new QVBoxLayout(globalSettingsAppearanceTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        lblTheme = new QLabel(globalSettingsAppearanceTab);
        lblTheme->setObjectName("lblTheme");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblTheme);

        comboTheme = new QComboBox(globalSettingsAppearanceTab);
        comboTheme->setObjectName("comboTheme");

        formLayout->setWidget(0, QFormLayout::FieldRole, comboTheme);


        verticalLayout_2->addLayout(formLayout);

        globalSettingsTab->addTab(globalSettingsAppearanceTab, QString());

        horizontalLayout_2->addWidget(globalSettingsTab);


        retranslateUi(GlobalSettingsWidget);

        globalSettingsTab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GlobalSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *GlobalSettingsWidget)
    {
        labelGlobal->setText(QCoreApplication::translate("GlobalSettingsWidget", "Add one or more directories containing your movies, TV Shows, concerts, music or files to import.\n"
"TV Show Episodes have to be in subfolders with the name of the show.\n"
"The directories containing your music must contain subdirectories for each artist which contain directories of albums.", nullptr));
        QTableWidgetItem *___qtablewidgetitem = dirs->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("GlobalSettingsWidget", "Type", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = dirs->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("GlobalSettingsWidget", "Path", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = dirs->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("GlobalSettingsWidget", "Sep. folders", nullptr));
#if QT_CONFIG(tooltip)
        ___qtablewidgetitem2->setToolTip(QCoreApplication::translate("GlobalSettingsWidget", "Items are in separate folders", nullptr));
#endif // QT_CONFIG(tooltip)
        QTableWidgetItem *___qtablewidgetitem3 = dirs->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("GlobalSettingsWidget", "Reload On Start", nullptr));
#if QT_CONFIG(tooltip)
        ___qtablewidgetitem3->setToolTip(QCoreApplication::translate("GlobalSettingsWidget", "Automatically reload contents on start", nullptr));
#endif // QT_CONFIG(tooltip)
        QTableWidgetItem *___qtablewidgetitem4 = dirs->horizontalHeaderItem(4);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("GlobalSettingsWidget", "Disabled", nullptr));
#if QT_CONFIG(tooltip)
        ___qtablewidgetitem4->setToolTip(QCoreApplication::translate("GlobalSettingsWidget", "Ignore this folder.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddDir->setText(QCoreApplication::translate("GlobalSettingsWidget", "Add", nullptr));
        buttonRemoveDir->setText(QCoreApplication::translate("GlobalSettingsWidget", "Remove", nullptr));
#if QT_CONFIG(tooltip)
        buttonMovieFilesToDirs->setToolTip(QCoreApplication::translate("GlobalSettingsWidget", "<html><head/><body><p>Sort movies into separate directories</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonMovieFilesToDirs->setText(QCoreApplication::translate("GlobalSettingsWidget", "Organize", nullptr));
        lblYouTubePlugin->setText(QCoreApplication::translate("GlobalSettingsWidget", "Store trailer URLs in YouTube Plugin format", nullptr));
        useYoutubePluginUrls->setText(QString());
        lblAutomaticStreamDetails->setText(QCoreApplication::translate("GlobalSettingsWidget", "Automatically load and save stream details from files", nullptr));
        chkAutoLoadStreamDetails->setText(QString());
        lblIgnoreArticles->setText(QCoreApplication::translate("GlobalSettingsWidget", "Ignore articles when sorting (\"The\")", nullptr));
        chkIgnoreArticlesWhenSorting->setText(QString());
        lblDownloadActorImages->setText(QCoreApplication::translate("GlobalSettingsWidget", "Download actor images", nullptr));
        chkDownloadActorImages->setText(QString());
        lblCheckForUpdates->setText(QCoreApplication::translate("GlobalSettingsWidget", "Check for Updates on start", nullptr));
        chkCheckForUpdates->setText(QString());
        lblExcludeWords->setText(QCoreApplication::translate("GlobalSettingsWidget", "Words to exclude from media names (separated by commas and non case-sensitive)", nullptr));
#if QT_CONFIG(tooltip)
        excludeWordsText->setToolTip(QCoreApplication::translate("GlobalSettingsWidget", "Words to exclude from media names (separated by commas and non case-sensitive)", nullptr));
#endif // QT_CONFIG(tooltip)
        lblStartupSection->setText(QCoreApplication::translate("GlobalSettingsWidget", "Startup section", nullptr));
        globalSettingsTab->setTabText(globalSettingsTab->indexOf(globalSettingsDirectoriesTab), QCoreApplication::translate("GlobalSettingsWidget", "Directories", nullptr));
        lblTheme->setText(QCoreApplication::translate("GlobalSettingsWidget", "Main Window Theme", nullptr));
        globalSettingsTab->setTabText(globalSettingsTab->indexOf(globalSettingsAppearanceTab), QCoreApplication::translate("GlobalSettingsWidget", "Appearance", nullptr));
        (void)GlobalSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class GlobalSettingsWidget: public Ui_GlobalSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GLOBALSETTINGSWIDGET_H
