/********************************************************************************
** Form generated from reading UI file 'MusicWidgetAlbum.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICWIDGETALBUM_H
#define UI_MUSICWIDGETALBUM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/ClosableImage.h"
#include "ui/small_widgets/ImageGallery.h"
#include "ui/small_widgets/TagCloud.h"

QT_BEGIN_NAMESPACE

class Ui_MusicWidgetAlbum
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer_4;
    QToolButton *buttonRevert;
    QLabel *albumName;
    QSpacerItem *horizontalSpacer_11;
    QGroupBox *albumGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tabInformation;
    QVBoxLayout *verticalLayout_6;
    QFormLayout *formLayout;
    QLabel *lblPath;
    QLineEdit *path;
    QLineEdit *artist;
    QLabel *lblArtist;
    QLineEdit *title;
    QLineEdit *label;
    QLabel *lblTitle;
    QLabel *lblLabel;
    QLabel *lblRating;
    QLabel *lblYear;
    QLabel *lblReleaseDate;
    QLabel *lblReview;
    QSpinBox *year;
    QLineEdit *releaseDate;
    QDoubleSpinBox *rating;
    QLabel *lblAlbumId;
    QLineEdit *musicBrainzAlbumId;
    QLabel *lblReleaseGroupId;
    QLineEdit *musicBrainzReleaseGroupId;
    QPlainTextEdit *review;
    QWidget *tabExtended;
    QVBoxLayout *verticalLayout_3;
    TagCloud *genreCloud;
    TagCloud *styleCloud;
    TagCloud *moodCloud;
    QSpacerItem *verticalSpacer_3;
    QWidget *tabBooklet;
    QVBoxLayout *bookletVerticalLayout;
    ImageGallery *booklets;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_16;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *btnAddExtraFanart;
    QFrame *line;
    QWidget *artWidget;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_4;
    QLabel *labelCover;
    ClosableImage *cover;
    QSpacerItem *verticalSpacer_2;
    QLabel *labelDiscArt;
    ClosableImage *discArt;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *MusicWidgetAlbum)
    {
        if (MusicWidgetAlbum->objectName().isEmpty())
            MusicWidgetAlbum->setObjectName("MusicWidgetAlbum");
        MusicWidgetAlbum->resize(931, 693);
        MusicWidgetAlbum->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(MusicWidgetAlbum);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 0);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_4);

        buttonRevert = new QToolButton(MusicWidgetAlbum);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);
        buttonRevert->setIconSize(QSize(16, 16));

        horizontalLayout_10->addWidget(buttonRevert);

        albumName = new QLabel(MusicWidgetAlbum);
        albumName->setObjectName("albumName");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica Neue")});
        font.setPointSize(22);
        albumName->setFont(font);
        albumName->setIndent(4);
        albumName->setProperty("isHeader", QVariant(true));

        horizontalLayout_10->addWidget(albumName);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_11);


        verticalLayout->addLayout(horizontalLayout_10);

        albumGroupBox = new QGroupBox(MusicWidgetAlbum);
        albumGroupBox->setObjectName("albumGroupBox");
        horizontalLayout_2 = new QHBoxLayout(albumGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(albumGroupBox);
        tabWidget->setObjectName("tabWidget");
        tabInformation = new QWidget();
        tabInformation->setObjectName("tabInformation");
        verticalLayout_6 = new QVBoxLayout(tabInformation);
        verticalLayout_6->setObjectName("verticalLayout_6");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setRowWrapPolicy(QFormLayout::RowWrapPolicy::DontWrapRows);
        formLayout->setLabelAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        formLayout->setHorizontalSpacing(20);
        lblPath = new QLabel(tabInformation);
        lblPath->setObjectName("lblPath");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblPath);

        path = new QLineEdit(tabInformation);
        path->setObjectName("path");
        path->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, path);

        artist = new QLineEdit(tabInformation);
        artist->setObjectName("artist");
        artist->setProperty("item", QVariant(QString::fromUtf8("artist")));

        formLayout->setWidget(3, QFormLayout::FieldRole, artist);

        lblArtist = new QLabel(tabInformation);
        lblArtist->setObjectName("lblArtist");

        formLayout->setWidget(3, QFormLayout::LabelRole, lblArtist);

        title = new QLineEdit(tabInformation);
        title->setObjectName("title");
        title->setProperty("item", QVariant(QString::fromUtf8("title")));

        formLayout->setWidget(4, QFormLayout::FieldRole, title);

        label = new QLineEdit(tabInformation);
        label->setObjectName("label");
        label->setProperty("item", QVariant(QString::fromUtf8("label")));

        formLayout->setWidget(5, QFormLayout::FieldRole, label);

        lblTitle = new QLabel(tabInformation);
        lblTitle->setObjectName("lblTitle");

        formLayout->setWidget(4, QFormLayout::LabelRole, lblTitle);

        lblLabel = new QLabel(tabInformation);
        lblLabel->setObjectName("lblLabel");

        formLayout->setWidget(5, QFormLayout::LabelRole, lblLabel);

        lblRating = new QLabel(tabInformation);
        lblRating->setObjectName("lblRating");

        formLayout->setWidget(6, QFormLayout::LabelRole, lblRating);

        lblYear = new QLabel(tabInformation);
        lblYear->setObjectName("lblYear");

        formLayout->setWidget(7, QFormLayout::LabelRole, lblYear);

        lblReleaseDate = new QLabel(tabInformation);
        lblReleaseDate->setObjectName("lblReleaseDate");

        formLayout->setWidget(8, QFormLayout::LabelRole, lblReleaseDate);

        lblReview = new QLabel(tabInformation);
        lblReview->setObjectName("lblReview");

        formLayout->setWidget(9, QFormLayout::LabelRole, lblReview);

        year = new QSpinBox(tabInformation);
        year->setObjectName("year");
        year->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        year->setMaximum(2050);

        formLayout->setWidget(7, QFormLayout::FieldRole, year);

        releaseDate = new QLineEdit(tabInformation);
        releaseDate->setObjectName("releaseDate");
        releaseDate->setProperty("item", QVariant(QString::fromUtf8("releaseDate")));

        formLayout->setWidget(8, QFormLayout::FieldRole, releaseDate);

        rating = new QDoubleSpinBox(tabInformation);
        rating->setObjectName("rating");
        rating->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        rating->setDecimals(1);
        rating->setMaximum(10.000000000000000);

        formLayout->setWidget(6, QFormLayout::FieldRole, rating);

        lblAlbumId = new QLabel(tabInformation);
        lblAlbumId->setObjectName("lblAlbumId");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblAlbumId);

        musicBrainzAlbumId = new QLineEdit(tabInformation);
        musicBrainzAlbumId->setObjectName("musicBrainzAlbumId");
        musicBrainzAlbumId->setProperty("item", QVariant(QString::fromUtf8("mbAlbumId")));

        formLayout->setWidget(1, QFormLayout::FieldRole, musicBrainzAlbumId);

        lblReleaseGroupId = new QLabel(tabInformation);
        lblReleaseGroupId->setObjectName("lblReleaseGroupId");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblReleaseGroupId);

        musicBrainzReleaseGroupId = new QLineEdit(tabInformation);
        musicBrainzReleaseGroupId->setObjectName("musicBrainzReleaseGroupId");
        musicBrainzReleaseGroupId->setProperty("item", QVariant(QString::fromUtf8("mbReleaseGroupId")));

        formLayout->setWidget(2, QFormLayout::FieldRole, musicBrainzReleaseGroupId);

        review = new QPlainTextEdit(tabInformation);
        review->setObjectName("review");

        formLayout->setWidget(9, QFormLayout::FieldRole, review);


        verticalLayout_6->addLayout(formLayout);

        tabWidget->addTab(tabInformation, QString());
        tabExtended = new QWidget();
        tabExtended->setObjectName("tabExtended");
        verticalLayout_3 = new QVBoxLayout(tabExtended);
        verticalLayout_3->setObjectName("verticalLayout_3");
        genreCloud = new TagCloud(tabExtended);
        genreCloud->setObjectName("genreCloud");
        genreCloud->setProperty("item", QVariant(QString::fromUtf8("genre")));

        verticalLayout_3->addWidget(genreCloud);

        styleCloud = new TagCloud(tabExtended);
        styleCloud->setObjectName("styleCloud");
        styleCloud->setProperty("item", QVariant(QString::fromUtf8("style")));

        verticalLayout_3->addWidget(styleCloud);

        moodCloud = new TagCloud(tabExtended);
        moodCloud->setObjectName("moodCloud");
        moodCloud->setProperty("item", QVariant(QString::fromUtf8("mood")));

        verticalLayout_3->addWidget(moodCloud);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        verticalLayout_3->setStretch(0, 2);
        verticalLayout_3->setStretch(1, 2);
        verticalLayout_3->setStretch(2, 2);
        tabWidget->addTab(tabExtended, QString());
        tabBooklet = new QWidget();
        tabBooklet->setObjectName("tabBooklet");
        bookletVerticalLayout = new QVBoxLayout(tabBooklet);
        bookletVerticalLayout->setObjectName("bookletVerticalLayout");
        bookletVerticalLayout->setContentsMargins(12, -1, 12, -1);
        booklets = new ImageGallery(tabBooklet);
        booklets->setObjectName("booklets");

        bookletVerticalLayout->addWidget(booklets);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_16 = new QLabel(tabBooklet);
        label_16->setObjectName("label_16");
        QFont font1;
        font1.setFamilies({QString::fromUtf8("Helvetica Neue")});
        label_16->setFont(font1);
        label_16->setWordWrap(true);

        horizontalLayout_5->addWidget(label_16);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        btnAddExtraFanart = new QPushButton(tabBooklet);
        btnAddExtraFanart->setObjectName("btnAddExtraFanart");

        horizontalLayout_5->addWidget(btnAddExtraFanart);

        horizontalLayout_5->setStretch(0, 1);

        bookletVerticalLayout->addLayout(horizontalLayout_5);

        bookletVerticalLayout->setStretch(0, 1);
        tabWidget->addTab(tabBooklet, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(albumGroupBox);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Shadow::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(0);
        line->setFrameShape(QFrame::Shape::VLine);

        horizontalLayout_2->addWidget(line);

        artWidget = new QWidget(albumGroupBox);
        artWidget->setObjectName("artWidget");
        horizontalLayout = new QHBoxLayout(artWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(10, 0, -1, -1);
        labelCover = new QLabel(artWidget);
        labelCover->setObjectName("labelCover");

        verticalLayout_4->addWidget(labelCover);

        cover = new ClosableImage(artWidget);
        cover->setObjectName("cover");
        cover->setMinimumSize(QSize(180, 180));
        cover->setMaximumSize(QSize(180, 180));
        cover->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        cover->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/poster.png")));
        cover->setAlignment(Qt::AlignmentFlag::AlignCenter);
        cover->setProperty("clickable", QVariant(true));
        cover->setProperty("myFixedHeight", QVariant(180));

        verticalLayout_4->addWidget(cover);

        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_2);

        labelDiscArt = new QLabel(artWidget);
        labelDiscArt->setObjectName("labelDiscArt");

        verticalLayout_4->addWidget(labelDiscArt);

        discArt = new ClosableImage(artWidget);
        discArt->setObjectName("discArt");
        discArt->setMinimumSize(QSize(180, 180));
        discArt->setMaximumSize(QSize(180, 180));
        discArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        discArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/cd_art.png")));
        discArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        discArt->setProperty("clickable", QVariant(true));
        discArt->setProperty("myFixedHeight", QVariant(180));

        verticalLayout_4->addWidget(discArt);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_4);


        horizontalLayout_2->addWidget(artWidget);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout->addWidget(albumGroupBox);

        verticalLayout->setStretch(1, 1);

        retranslateUi(MusicWidgetAlbum);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MusicWidgetAlbum);
    } // setupUi

    void retranslateUi(QWidget *MusicWidgetAlbum)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("MusicWidgetAlbum", "Album has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        albumName->setText(QCoreApplication::translate("MusicWidgetAlbum", "Album Name", nullptr));
        albumGroupBox->setTitle(QString());
        lblPath->setText(QCoreApplication::translate("MusicWidgetAlbum", "Path", nullptr));
        lblArtist->setText(QCoreApplication::translate("MusicWidgetAlbum", "Artist", nullptr));
        lblTitle->setText(QCoreApplication::translate("MusicWidgetAlbum", "Title", nullptr));
        lblLabel->setText(QCoreApplication::translate("MusicWidgetAlbum", "Label", nullptr));
        lblRating->setText(QCoreApplication::translate("MusicWidgetAlbum", "Rating", nullptr));
        lblYear->setText(QCoreApplication::translate("MusicWidgetAlbum", "Year", nullptr));
        lblReleaseDate->setText(QCoreApplication::translate("MusicWidgetAlbum", "Release Date", nullptr));
        lblReview->setText(QCoreApplication::translate("MusicWidgetAlbum", "Review", nullptr));
        lblAlbumId->setText(QCoreApplication::translate("MusicWidgetAlbum", "MusicBrainz Album ID", nullptr));
        lblReleaseGroupId->setText(QCoreApplication::translate("MusicWidgetAlbum", "MusicBrainz Release Group ID", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabInformation), QCoreApplication::translate("MusicWidgetAlbum", "Information", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtended), QCoreApplication::translate("MusicWidgetAlbum", "Extended", nullptr));
        label_16->setText(QCoreApplication::translate("MusicWidgetAlbum", "Hint: Closed images will be deleted on save.", nullptr));
        btnAddExtraFanart->setText(QCoreApplication::translate("MusicWidgetAlbum", "Add Images", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabBooklet), QCoreApplication::translate("MusicWidgetAlbum", "Booklet", nullptr));
        labelCover->setText(QCoreApplication::translate("MusicWidgetAlbum", "Cover", nullptr));
#if QT_CONFIG(tooltip)
        cover->setToolTip(QCoreApplication::translate("MusicWidgetAlbum", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        cover->setText(QString());
        cover->setProperty("title", QVariant(QCoreApplication::translate("MusicWidgetAlbum", "Cover", nullptr)));
        labelDiscArt->setText(QCoreApplication::translate("MusicWidgetAlbum", "Disc Art", nullptr));
#if QT_CONFIG(tooltip)
        discArt->setToolTip(QCoreApplication::translate("MusicWidgetAlbum", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        discArt->setText(QString());
        discArt->setProperty("title", QVariant(QCoreApplication::translate("MusicWidgetAlbum", "Disc Art", nullptr)));
        (void)MusicWidgetAlbum;
    } // retranslateUi

};

namespace Ui {
    class MusicWidgetAlbum: public Ui_MusicWidgetAlbum {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICWIDGETALBUM_H
