/********************************************************************************
** Form generated from reading UI file 'ImportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTDIALOG_H
#define UI_IMPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/concerts/ConcertSearchWidget.h"
#include "ui/movies/MovieSearchWidget.h"
#include "ui/renamer/RenamerPlaceholders.h"
#include "ui/small_widgets/Badge.h"
#include "ui/small_widgets/SlidingStackedWidget.h"
#include "ui/tv_show/TvShowSearchWidget.h"

QT_BEGIN_NAMESPACE

class Ui_ImportDialog
{
public:
    QVBoxLayout *verticalLayout;
    SlidingStackedWidget *stackedWidget;
    MovieSearchWidget *movieSearchWidget;
    QWidget *importPage2;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *loading;
    Badge *badgeSuccess;
    QLabel *labelLoading;
    QSpacerItem *horizontalSpacer_2;
    RenamerPlaceholders *placeholders;
    QFormLayout *formLayout;
    QLabel *labelDirectoryNaming;
    QLineEdit *directoryNaming;
    QLabel *label;
    QLineEdit *fileNaming;
    QCheckBox *chkSeasonDirectories;
    QLabel *labelUseSeasonDirectories;
    QLabel *labelSeasonNaming;
    QLineEdit *seasonNaming;
    QCheckBox *chkKeepSourceFiles;
    QLabel *label_5;
    QSpacerItem *verticalSpacer;
    QProgressBar *progressBar;
    ConcertSearchWidget *concertSearchWidget;
    TvShowSearchWidget *tvShowSearchWidget;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnReject;
    QPushButton *btnAccept;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnImport;

    void setupUi(QDialog *ImportDialog)
    {
        if (ImportDialog->objectName().isEmpty())
            ImportDialog->setObjectName("ImportDialog");
        ImportDialog->setWindowModality(Qt::WindowModal);
        ImportDialog->resize(695, 598);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        ImportDialog->setWindowIcon(icon);
        ImportDialog->setModal(true);
        verticalLayout = new QVBoxLayout(ImportDialog);
        verticalLayout->setObjectName("verticalLayout");
        stackedWidget = new SlidingStackedWidget(ImportDialog);
        stackedWidget->setObjectName("stackedWidget");
        movieSearchWidget = new MovieSearchWidget();
        movieSearchWidget->setObjectName("movieSearchWidget");
        stackedWidget->addWidget(movieSearchWidget);
        importPage2 = new QWidget();
        importPage2->setObjectName("importPage2");
        verticalLayout_3 = new QVBoxLayout(importPage2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        loading = new QLabel(importPage2);
        loading->setObjectName("loading");

        horizontalLayout_2->addWidget(loading);

        badgeSuccess = new Badge(importPage2);
        badgeSuccess->setObjectName("badgeSuccess");

        horizontalLayout_2->addWidget(badgeSuccess);

        labelLoading = new QLabel(importPage2);
        labelLoading->setObjectName("labelLoading");

        horizontalLayout_2->addWidget(labelLoading);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        horizontalLayout_2->setStretch(2, 1);

        verticalLayout_3->addLayout(horizontalLayout_2);

        placeholders = new RenamerPlaceholders(importPage2);
        placeholders->setObjectName("placeholders");

        verticalLayout_3->addWidget(placeholders);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        labelDirectoryNaming = new QLabel(importPage2);
        labelDirectoryNaming->setObjectName("labelDirectoryNaming");

        formLayout->setWidget(0, QFormLayout::LabelRole, labelDirectoryNaming);

        directoryNaming = new QLineEdit(importPage2);
        directoryNaming->setObjectName("directoryNaming");

        formLayout->setWidget(0, QFormLayout::FieldRole, directoryNaming);

        label = new QLabel(importPage2);
        label->setObjectName("label");

        formLayout->setWidget(1, QFormLayout::LabelRole, label);

        fileNaming = new QLineEdit(importPage2);
        fileNaming->setObjectName("fileNaming");

        formLayout->setWidget(1, QFormLayout::FieldRole, fileNaming);

        chkSeasonDirectories = new QCheckBox(importPage2);
        chkSeasonDirectories->setObjectName("chkSeasonDirectories");

        formLayout->setWidget(2, QFormLayout::FieldRole, chkSeasonDirectories);

        labelUseSeasonDirectories = new QLabel(importPage2);
        labelUseSeasonDirectories->setObjectName("labelUseSeasonDirectories");

        formLayout->setWidget(2, QFormLayout::LabelRole, labelUseSeasonDirectories);

        labelSeasonNaming = new QLabel(importPage2);
        labelSeasonNaming->setObjectName("labelSeasonNaming");

        formLayout->setWidget(3, QFormLayout::LabelRole, labelSeasonNaming);

        seasonNaming = new QLineEdit(importPage2);
        seasonNaming->setObjectName("seasonNaming");

        formLayout->setWidget(3, QFormLayout::FieldRole, seasonNaming);

        chkKeepSourceFiles = new QCheckBox(importPage2);
        chkKeepSourceFiles->setObjectName("chkKeepSourceFiles");

        formLayout->setWidget(4, QFormLayout::FieldRole, chkKeepSourceFiles);

        label_5 = new QLabel(importPage2);
        label_5->setObjectName("label_5");

        formLayout->setWidget(4, QFormLayout::LabelRole, label_5);


        verticalLayout_3->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        progressBar = new QProgressBar(importPage2);
        progressBar->setObjectName("progressBar");
        progressBar->setMinimum(0);
        progressBar->setMaximum(100);
        progressBar->setValue(0);

        verticalLayout_3->addWidget(progressBar);

        stackedWidget->addWidget(importPage2);
        concertSearchWidget = new ConcertSearchWidget();
        concertSearchWidget->setObjectName("concertSearchWidget");
        stackedWidget->addWidget(concertSearchWidget);
        tvShowSearchWidget = new TvShowSearchWidget();
        tvShowSearchWidget->setObjectName("tvShowSearchWidget");
        stackedWidget->addWidget(tvShowSearchWidget);

        verticalLayout->addWidget(stackedWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnReject = new QPushButton(ImportDialog);
        btnReject->setObjectName("btnReject");
        btnReject->setAutoDefault(false);

        horizontalLayout->addWidget(btnReject);

        btnAccept = new QPushButton(ImportDialog);
        btnAccept->setObjectName("btnAccept");

        horizontalLayout->addWidget(btnAccept);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnImport = new QPushButton(ImportDialog);
        btnImport->setObjectName("btnImport");

        horizontalLayout->addWidget(btnImport);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(0, 1);

        retranslateUi(ImportDialog);
        QObject::connect(btnReject, &QPushButton::clicked, ImportDialog, qOverload<>(&QDialog::reject));
        QObject::connect(btnAccept, &QPushButton::clicked, ImportDialog, qOverload<>(&QDialog::accept));

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ImportDialog);
    } // setupUi

    void retranslateUi(QDialog *ImportDialog)
    {
        ImportDialog->setWindowTitle(QCoreApplication::translate("ImportDialog", "MediaElch", nullptr));
        loading->setText(QCoreApplication::translate("ImportDialog", "Loading", nullptr));
        badgeSuccess->setText(QCoreApplication::translate("ImportDialog", "Success", nullptr));
        labelLoading->setText(QCoreApplication::translate("ImportDialog", "Loading movie...", nullptr));
        labelDirectoryNaming->setText(QCoreApplication::translate("ImportDialog", "Directory Naming", nullptr));
        label->setText(QCoreApplication::translate("ImportDialog", "File Naming", nullptr));
        chkSeasonDirectories->setText(QString());
        labelUseSeasonDirectories->setText(QCoreApplication::translate("ImportDialog", "Use Season Directories", nullptr));
        labelSeasonNaming->setText(QCoreApplication::translate("ImportDialog", "Season Directory Naming", nullptr));
        chkKeepSourceFiles->setText(QString());
        label_5->setText(QCoreApplication::translate("ImportDialog", "Keep source files after import", nullptr));
        btnReject->setText(QCoreApplication::translate("ImportDialog", "Close", nullptr));
        btnAccept->setText(QCoreApplication::translate("ImportDialog", "Close", nullptr));
        btnImport->setText(QCoreApplication::translate("ImportDialog", "Import", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImportDialog: public Ui_ImportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMPORTDIALOG_H
