/********************************************************************************
** Form generated from reading UI file 'Navbar.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NAVBAR_H
#define UI_NAVBAR_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/FilterWidget.h"

QT_BEGIN_NAMESPACE

class Ui_Navbar
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *widget;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *btnSearch;
    QToolButton *btnSave;
    QToolButton *btnSaveAll;
    QToolButton *btnRename;
    QToolButton *btnSync;
    QToolButton *btnExport;
    QToolButton *btnReload;
    QToolButton *btnSettings;
    QToolButton *btnAbout;
    QSpacerItem *horizontalSpacer;
    FilterWidget *filterWidget;
    QPushButton *btnDonate;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *Navbar)
    {
        if (Navbar->objectName().isEmpty())
            Navbar->setObjectName("Navbar");
        Navbar->resize(1091, 84);
        Navbar->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(Navbar);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(Navbar);
        widget->setObjectName("widget");
        horizontalLayout_2 = new QHBoxLayout(widget);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        btnSearch = new QToolButton(widget);
        btnSearch->setObjectName("btnSearch");
        btnSearch->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/navbar/edit-download.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnSearch->setIcon(icon);
        btnSearch->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnSearch);

        btnSave = new QToolButton(widget);
        btnSave->setObjectName("btnSave");
        btnSave->setText(QString::fromUtf8(""));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/navbar/document-save.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnSave->setIcon(icon1);
        btnSave->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnSave);

        btnSaveAll = new QToolButton(widget);
        btnSaveAll->setObjectName("btnSaveAll");
        btnSaveAll->setText(QString::fromUtf8(""));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/navbar/document-save-all.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnSaveAll->setIcon(icon2);
        btnSaveAll->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnSaveAll);

        btnRename = new QToolButton(widget);
        btnRename->setObjectName("btnRename");
        btnRename->setText(QString::fromUtf8(""));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/navbar/document-open.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnRename->setIcon(icon3);
        btnRename->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnRename);

        btnSync = new QToolButton(widget);
        btnSync->setObjectName("btnSync");
        btnSync->setText(QString::fromUtf8(""));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/navbar/vcs-merge.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnSync->setIcon(icon4);
        btnSync->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnSync);

        btnExport = new QToolButton(widget);
        btnExport->setObjectName("btnExport");
        btnExport->setText(QString::fromUtf8(""));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/navbar/document-export.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnExport->setIcon(icon5);
        btnExport->setIconSize(QSize(32, 32));
        btnExport->setPopupMode(QToolButton::InstantPopup);

        horizontalLayout_2->addWidget(btnExport);

        btnReload = new QToolButton(widget);
        btnReload->setObjectName("btnReload");
        btnReload->setText(QString::fromUtf8(""));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/navbar/folder-sync.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnReload->setIcon(icon6);
        btnReload->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnReload);

        btnSettings = new QToolButton(widget);
        btnSettings->setObjectName("btnSettings");
        btnSettings->setText(QString::fromUtf8(""));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/navbar/configure.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnSettings->setIcon(icon7);
        btnSettings->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnSettings);

        btnAbout = new QToolButton(widget);
        btnAbout->setObjectName("btnAbout");
        btnAbout->setText(QString::fromUtf8(""));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/navbar/help-about.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnAbout->setIcon(icon8);
        btnAbout->setIconSize(QSize(32, 32));

        horizontalLayout_2->addWidget(btnAbout);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        filterWidget = new FilterWidget(widget);
        filterWidget->setObjectName("filterWidget");
        filterWidget->setMaximumSize(QSize(500, 16777215));

        horizontalLayout_2->addWidget(filterWidget);

        btnDonate = new QPushButton(widget);
        btnDonate->setObjectName("btnDonate");
        btnDonate->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/img/money_64.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnDonate->setIcon(icon9);
        btnDonate->setIconSize(QSize(12, 12));

        horizontalLayout_2->addWidget(btnDonate);

        horizontalSpacer_2 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        horizontalLayout_2->setStretch(10, 1);

        verticalLayout->addWidget(widget);


        retranslateUi(Navbar);

        QMetaObject::connectSlotsByName(Navbar);
    } // setupUi

    void retranslateUi(QWidget *Navbar)
    {
#if QT_CONFIG(tooltip)
        btnSearch->setToolTip(QCoreApplication::translate("Navbar", "Scrape", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSave->setToolTip(QCoreApplication::translate("Navbar", "Save", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSaveAll->setToolTip(QCoreApplication::translate("Navbar", "Save All", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnRename->setToolTip(QCoreApplication::translate("Navbar", "Rename selected files", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSync->setToolTip(QCoreApplication::translate("Navbar", "Synchronize to Kodi", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnExport->setToolTip(QCoreApplication::translate("Navbar", "Export Database", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnReload->setToolTip(QCoreApplication::translate("Navbar", "Reload", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSettings->setToolTip(QCoreApplication::translate("Navbar", "Settings", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnAbout->setToolTip(QCoreApplication::translate("Navbar", "About", nullptr));
#endif // QT_CONFIG(tooltip)
        btnDonate->setText(QCoreApplication::translate("Navbar", "Donate", nullptr));
        (void)Navbar;
    } // retranslateUi

};

namespace Ui {
    class Navbar: public Ui_Navbar {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NAVBAR_H
