/********************************************************************************
** Form generated from reading UI file 'RatingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RATINGSWIDGET_H
#define UI_RATINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RatingsWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QTableView *ratings;
    QVBoxLayout *verticalLayout_8;
    QPushButton *btnAddRating;
    QPushButton *btnRemoveRating;

    void setupUi(QWidget *RatingsWidget)
    {
        if (RatingsWidget->objectName().isEmpty())
            RatingsWidget->setObjectName("RatingsWidget");
        RatingsWidget->resize(400, 118);
        horizontalLayout = new QHBoxLayout(RatingsWidget);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        ratings = new QTableView(RatingsWidget);
        ratings->setObjectName("ratings");
        ratings->setMinimumSize(QSize(0, 100));
        ratings->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        ratings->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        ratings->setEditTriggers(QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed);
        ratings->setDragEnabled(true);
        ratings->setSelectionMode(QAbstractItemView::SingleSelection);
        ratings->setSelectionBehavior(QAbstractItemView::SelectRows);
        ratings->setShowGrid(false);
        ratings->setWordWrap(false);
        ratings->setCornerButtonEnabled(false);
        ratings->horizontalHeader()->setStretchLastSection(true);
        ratings->verticalHeader()->setVisible(false);

        horizontalLayout->addWidget(ratings);

        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName("verticalLayout_8");
        btnAddRating = new QPushButton(RatingsWidget);
        btnAddRating->setObjectName("btnAddRating");
        btnAddRating->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/plus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnAddRating->setIcon(icon);

        verticalLayout_8->addWidget(btnAddRating);

        btnRemoveRating = new QPushButton(RatingsWidget);
        btnRemoveRating->setObjectName("btnRemoveRating");
        btnRemoveRating->setText(QString::fromUtf8(""));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/minus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnRemoveRating->setIcon(icon1);

        verticalLayout_8->addWidget(btnRemoveRating);


        horizontalLayout->addLayout(verticalLayout_8);


        retranslateUi(RatingsWidget);

        QMetaObject::connectSlotsByName(RatingsWidget);
    } // setupUi

    void retranslateUi(QWidget *RatingsWidget)
    {
        RatingsWidget->setWindowTitle(QCoreApplication::translate("RatingsWidget", "Form", nullptr));
#if QT_CONFIG(tooltip)
        btnAddRating->setToolTip(QCoreApplication::translate("RatingsWidget", "Add rating", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnRemoveRating->setToolTip(QCoreApplication::translate("RatingsWidget", "Remove selected rating", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class RatingsWidget: public Ui_RatingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RATINGSWIDGET_H
