/********************************************************************************
** Form generated from reading UI file 'RenamerDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RENAMERDIALOG_H
#define UI_RENAMERDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/PlaceholderLineEdit.h>
#include "ui/renamer/RenamerPlaceholders.h"

QT_BEGIN_NAMESPACE

class Ui_RenamerDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tabPattern;
    QVBoxLayout *verticalLayout_2;
    QLabel *helpLabel;
    QLabel *infoLabel;
    RenamerPlaceholders *placeholders;
    QGridLayout *gridLayout;
    QCheckBox *chkSeasonDirectories;
    QLabel *label_15;
    PlaceholderLineEdit *directoryNaming;
    QCheckBox *chkDirectoryNaming;
    QCheckBox *chkFileNaming;
    PlaceholderLineEdit *fileNamingMulti;
    QLabel *label;
    PlaceholderLineEdit *fileNaming;
    QLabel *label_2;
    QLabel *labelSeasonDirectory;
    PlaceholderLineEdit *seasonNaming;
    QSpacerItem *verticalSpacer;
    QWidget *tabResults;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *resultsTable;
    QTextEdit *results;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnClose;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnDryRun;
    QPushButton *btnRename;

    void setupUi(QDialog *RenamerDialog)
    {
        if (RenamerDialog->objectName().isEmpty())
            RenamerDialog->setObjectName("RenamerDialog");
        RenamerDialog->setWindowModality(Qt::WindowModal);
        RenamerDialog->resize(700, 600);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        RenamerDialog->setWindowIcon(icon);
        RenamerDialog->setModal(true);
        verticalLayout = new QVBoxLayout(RenamerDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(RenamerDialog);
        tabWidget->setObjectName("tabWidget");
        tabPattern = new QWidget();
        tabPattern->setObjectName("tabPattern");
        verticalLayout_2 = new QVBoxLayout(tabPattern);
        verticalLayout_2->setObjectName("verticalLayout_2");
        helpLabel = new QLabel(tabPattern);
        helpLabel->setObjectName("helpLabel");
        helpLabel->setText(QString::fromUtf8("Help"));
        helpLabel->setWordWrap(true);
        helpLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(helpLabel);

        infoLabel = new QLabel(tabPattern);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setText(QString::fromUtf8("Info"));
        infoLabel->setWordWrap(true);

        verticalLayout_2->addWidget(infoLabel);

        placeholders = new RenamerPlaceholders(tabPattern);
        placeholders->setObjectName("placeholders");
        placeholders->setMinimumSize(QSize(0, 200));

        verticalLayout_2->addWidget(placeholders);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        chkSeasonDirectories = new QCheckBox(tabPattern);
        chkSeasonDirectories->setObjectName("chkSeasonDirectories");
        chkSeasonDirectories->setChecked(true);

        gridLayout->addWidget(chkSeasonDirectories, 5, 0, 1, 1);

        label_15 = new QLabel(tabPattern);
        label_15->setObjectName("label_15");

        gridLayout->addWidget(label_15, 2, 0, 1, 1);

        directoryNaming = new PlaceholderLineEdit(tabPattern);
        directoryNaming->setObjectName("directoryNaming");
        directoryNaming->setCurrentText(QString::fromUtf8("<title> (<year>)"));
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(directoryNaming->sizePolicy().hasHeightForWidth());
        directoryNaming->setSizePolicy(sizePolicy);
        directoryNaming->setMinimumSize(QSize(200, 0));
        directoryNaming->setMaximumSize(QSize(500, 16777215));

        gridLayout->addWidget(directoryNaming, 4, 1, 1, 1);

        chkDirectoryNaming = new QCheckBox(tabPattern);
        chkDirectoryNaming->setObjectName("chkDirectoryNaming");
        chkDirectoryNaming->setChecked(true);

        gridLayout->addWidget(chkDirectoryNaming, 3, 0, 1, 1);

        chkFileNaming = new QCheckBox(tabPattern);
        chkFileNaming->setObjectName("chkFileNaming");
        chkFileNaming->setChecked(true);

        gridLayout->addWidget(chkFileNaming, 0, 0, 1, 1);

        fileNamingMulti = new PlaceholderLineEdit(tabPattern);
        fileNamingMulti->setObjectName("fileNamingMulti");
        fileNamingMulti->setCurrentText(QString::fromUtf8("<title>-part<partNo>.<extension>"));
        sizePolicy.setHeightForWidth(fileNamingMulti->sizePolicy().hasHeightForWidth());
        fileNamingMulti->setSizePolicy(sizePolicy);
        fileNamingMulti->setMinimumSize(QSize(200, 0));
        fileNamingMulti->setMaximumSize(QSize(500, 16777215));

        gridLayout->addWidget(fileNamingMulti, 2, 1, 1, 1);

        label = new QLabel(tabPattern);
        label->setObjectName("label");

        gridLayout->addWidget(label, 4, 0, 1, 1);

        fileNaming = new PlaceholderLineEdit(tabPattern);
        fileNaming->setObjectName("fileNaming");
        fileNaming->setCurrentText(QString::fromUtf8("<title>.<extension>"));
        sizePolicy.setHeightForWidth(fileNaming->sizePolicy().hasHeightForWidth());
        fileNaming->setSizePolicy(sizePolicy);
        fileNaming->setMinimumSize(QSize(200, 0));
        fileNaming->setMaximumSize(QSize(500, 16777215));

        gridLayout->addWidget(fileNaming, 1, 1, 1, 1);

        label_2 = new QLabel(tabPattern);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        labelSeasonDirectory = new QLabel(tabPattern);
        labelSeasonDirectory->setObjectName("labelSeasonDirectory");

        gridLayout->addWidget(labelSeasonDirectory, 6, 0, 1, 1);

        seasonNaming = new PlaceholderLineEdit(tabPattern);
        seasonNaming->setObjectName("seasonNaming");
        sizePolicy.setHeightForWidth(seasonNaming->sizePolicy().hasHeightForWidth());
        seasonNaming->setSizePolicy(sizePolicy);
        seasonNaming->setMinimumSize(QSize(200, 0));
        seasonNaming->setMaximumSize(QSize(500, 16777215));

        gridLayout->addWidget(seasonNaming, 6, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        verticalLayout_2->setStretch(2, 1);
        tabWidget->addTab(tabPattern, QString());
        tabResults = new QWidget();
        tabResults->setObjectName("tabResults");
        verticalLayout_3 = new QVBoxLayout(tabResults);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        resultsTable = new QTableWidget(tabResults);
        if (resultsTable->columnCount() < 3)
            resultsTable->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        __qtablewidgetitem->setText(QString::fromUtf8("Neue Spalte"));
        resultsTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        __qtablewidgetitem1->setText(QString::fromUtf8("Neue Spalte"));
        resultsTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        __qtablewidgetitem2->setText(QString::fromUtf8("Neue Spalte"));
        resultsTable->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        resultsTable->setObjectName("resultsTable");
        resultsTable->setFrameShape(QFrame::NoFrame);
        resultsTable->setFrameShadow(QFrame::Plain);
        resultsTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        resultsTable->horizontalHeader()->setVisible(false);
        resultsTable->horizontalHeader()->setDefaultSectionSize(100);
        resultsTable->verticalHeader()->setVisible(false);
        resultsTable->verticalHeader()->setMinimumSectionSize(22);
        resultsTable->verticalHeader()->setDefaultSectionSize(22);

        verticalLayout_3->addWidget(resultsTable);

        results = new QTextEdit(tabResults);
        results->setObjectName("results");
        results->setMaximumSize(QSize(16777215, 110));
        results->setFrameShape(QFrame::NoFrame);
        results->setFrameShadow(QFrame::Plain);
        results->setReadOnly(true);
        results->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'.SF NS Text'; font-size:13pt;\"><br /></p></body></html>"));

        verticalLayout_3->addWidget(results);

        tabWidget->addTab(tabResults, QString());

        verticalLayout->addWidget(tabWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnClose = new QPushButton(RenamerDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(false);

        horizontalLayout->addWidget(btnClose);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnDryRun = new QPushButton(RenamerDialog);
        btnDryRun->setObjectName("btnDryRun");
        btnDryRun->setAutoDefault(false);

        horizontalLayout->addWidget(btnDryRun);

        btnRename = new QPushButton(RenamerDialog);
        btnRename->setObjectName("btnRename");
        btnRename->setAutoDefault(false);

        horizontalLayout->addWidget(btnRename);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(0, 1);
        QWidget::setTabOrder(chkFileNaming, fileNaming);
        QWidget::setTabOrder(fileNaming, fileNamingMulti);
        QWidget::setTabOrder(fileNamingMulti, chkDirectoryNaming);
        QWidget::setTabOrder(chkDirectoryNaming, directoryNaming);
        QWidget::setTabOrder(directoryNaming, chkSeasonDirectories);
        QWidget::setTabOrder(chkSeasonDirectories, seasonNaming);
        QWidget::setTabOrder(seasonNaming, btnClose);
        QWidget::setTabOrder(btnClose, btnDryRun);
        QWidget::setTabOrder(btnDryRun, btnRename);

        retranslateUi(RenamerDialog);
        QObject::connect(btnClose, &QPushButton::clicked, RenamerDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(RenamerDialog);
    } // setupUi

    void retranslateUi(QDialog *RenamerDialog)
    {
        RenamerDialog->setWindowTitle(QCoreApplication::translate("RenamerDialog", "Renamer", nullptr));
        chkSeasonDirectories->setText(QCoreApplication::translate("RenamerDialog", "Use Season Directories", nullptr));
        label_15->setText(QCoreApplication::translate("RenamerDialog", "Multi-File Naming", nullptr));
        chkDirectoryNaming->setText(QCoreApplication::translate("RenamerDialog", "Rename Directories", nullptr));
        chkFileNaming->setText(QCoreApplication::translate("RenamerDialog", "Rename Files", nullptr));
        label->setText(QCoreApplication::translate("RenamerDialog", "Directory Naming", nullptr));
        label_2->setText(QCoreApplication::translate("RenamerDialog", "File Naming", nullptr));
        labelSeasonDirectory->setText(QCoreApplication::translate("RenamerDialog", "Season Directory Naming", nullptr));
        seasonNaming->setCurrentText(QCoreApplication::translate("RenamerDialog", "Season <season>", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabPattern), QCoreApplication::translate("RenamerDialog", "Pattern", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabResults), QCoreApplication::translate("RenamerDialog", "Results", nullptr));
        btnClose->setText(QCoreApplication::translate("RenamerDialog", "Close", nullptr));
        btnDryRun->setText(QCoreApplication::translate("RenamerDialog", "Dry Run", nullptr));
        btnRename->setText(QCoreApplication::translate("RenamerDialog", "Rename", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RenamerDialog: public Ui_RenamerDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RENAMERDIALOG_H
