/********************************************************************************
** Form generated from reading UI file 'TvShowWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TVSHOWWIDGET_H
#define UI_TVSHOWWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/tv_show/TvShowWidgetEpisode.h"
#include "ui/tv_show/TvShowWidgetSeason.h"
#include "ui/tv_show/TvShowWidgetTvShow.h"

QT_BEGIN_NAMESPACE

class Ui_TvShowWidget
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *stackedWidget;
    TvShowWidgetTvShow *tvShowWidget;
    TvShowWidgetEpisode *episodeWidget;
    TvShowWidgetSeason *seasonWidget;

    void setupUi(QWidget *TvShowWidget)
    {
        if (TvShowWidget->objectName().isEmpty())
            TvShowWidget->setObjectName("TvShowWidget");
        TvShowWidget->resize(924, 793);
        verticalLayout = new QVBoxLayout(TvShowWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(TvShowWidget);
        stackedWidget->setObjectName("stackedWidget");
        tvShowWidget = new TvShowWidgetTvShow();
        tvShowWidget->setObjectName("tvShowWidget");
        stackedWidget->addWidget(tvShowWidget);
        episodeWidget = new TvShowWidgetEpisode();
        episodeWidget->setObjectName("episodeWidget");
        stackedWidget->addWidget(episodeWidget);
        seasonWidget = new TvShowWidgetSeason();
        seasonWidget->setObjectName("seasonWidget");
        stackedWidget->addWidget(seasonWidget);

        verticalLayout->addWidget(stackedWidget);


        retranslateUi(TvShowWidget);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(TvShowWidget);
    } // setupUi

    void retranslateUi(QWidget *TvShowWidget)
    {
        (void)TvShowWidget;
    } // retranslateUi

};

namespace Ui {
    class TvShowWidget: public Ui_TvShowWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TVSHOWWIDGET_H
