/********************************************************************************
** Form generated from reading UI file 'ConcertStreamDetailsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONCERTSTREAMDETAILSWIDGET_H
#define UI_CONCERTSTREAMDETAILSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/StereoModeComboBox.h"

QT_BEGIN_NAMESPACE

class Ui_ConcertStreamDetailsWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QGridLayout *streamDetails;
    QLabel *lblCodec;
    QDoubleSpinBox *videoAspectRatio;
    QTimeEdit *videoDuration;
    QLineEdit *videoScantype;
    QLabel *lblVideo;
    QLabel *labelStreamDetailsAudio;
    QLabel *lblScantype;
    QLabel *lblDuration;
    QHBoxLayout *horizontalLayout_16;
    QSpinBox *videoWidth;
    QLabel *lblResolutionTimes;
    QSpinBox *videoHeight;
    QSpacerItem *horizontalSpacer_10;
    QLabel *lblAspectRatio;
    QLineEdit *videoCodec;
    QLabel *lblResolution;
    QLabel *lblStereoMode;
    StereoModeComboBox *stereoMode;
    QHBoxLayout *horizontalLayout_17;
    QPushButton *buttonReloadStreamDetails;
    QLabel *lblReloadStreamDetailsError;
    QSpacerItem *horizontalSpacer_9;
    QSpacerItem *verticalSpacer_8;

    void setupUi(QWidget *ConcertStreamDetailsWidget)
    {
        if (ConcertStreamDetailsWidget->objectName().isEmpty())
            ConcertStreamDetailsWidget->setObjectName("ConcertStreamDetailsWidget");
        ConcertStreamDetailsWidget->resize(675, 400);
        verticalLayout_2 = new QVBoxLayout(ConcertStreamDetailsWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        streamDetails = new QGridLayout();
        streamDetails->setObjectName("streamDetails");
        lblCodec = new QLabel(ConcertStreamDetailsWidget);
        lblCodec->setObjectName("lblCodec");
        lblCodec->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblCodec, 1, 0, 1, 1);

        videoAspectRatio = new QDoubleSpinBox(ConcertStreamDetailsWidget);
        videoAspectRatio->setObjectName("videoAspectRatio");
        videoAspectRatio->setMaximumSize(QSize(100, 16777215));
        videoAspectRatio->setButtonSymbols(QAbstractSpinBox::NoButtons);
        videoAspectRatio->setDecimals(3);

        streamDetails->addWidget(videoAspectRatio, 3, 1, 1, 1);

        videoDuration = new QTimeEdit(ConcertStreamDetailsWidget);
        videoDuration->setObjectName("videoDuration");
        videoDuration->setMaximumSize(QSize(200, 16777215));
        videoDuration->setButtonSymbols(QAbstractSpinBox::NoButtons);

        streamDetails->addWidget(videoDuration, 6, 1, 1, 1);

        videoScantype = new QLineEdit(ConcertStreamDetailsWidget);
        videoScantype->setObjectName("videoScantype");
        videoScantype->setMaximumSize(QSize(200, 16777215));

        streamDetails->addWidget(videoScantype, 4, 1, 1, 1);

        lblVideo = new QLabel(ConcertStreamDetailsWidget);
        lblVideo->setObjectName("lblVideo");
        QFont font;
        font.setBold(true);
        lblVideo->setFont(font);
        lblVideo->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblVideo, 0, 0, 1, 1);

        labelStreamDetailsAudio = new QLabel(ConcertStreamDetailsWidget);
        labelStreamDetailsAudio->setObjectName("labelStreamDetailsAudio");
        labelStreamDetailsAudio->setFont(font);
        labelStreamDetailsAudio->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(labelStreamDetailsAudio, 7, 0, 1, 1);

        lblScantype = new QLabel(ConcertStreamDetailsWidget);
        lblScantype->setObjectName("lblScantype");
        lblScantype->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblScantype, 4, 0, 1, 1);

        lblDuration = new QLabel(ConcertStreamDetailsWidget);
        lblDuration->setObjectName("lblDuration");
        lblDuration->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblDuration, 6, 0, 1, 1);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        videoWidth = new QSpinBox(ConcertStreamDetailsWidget);
        videoWidth->setObjectName("videoWidth");
        videoWidth->setButtonSymbols(QAbstractSpinBox::NoButtons);
        videoWidth->setMaximum(9999);

        horizontalLayout_16->addWidget(videoWidth);

        lblResolutionTimes = new QLabel(ConcertStreamDetailsWidget);
        lblResolutionTimes->setObjectName("lblResolutionTimes");
        lblResolutionTimes->setText(QString::fromUtf8("x"));

        horizontalLayout_16->addWidget(lblResolutionTimes);

        videoHeight = new QSpinBox(ConcertStreamDetailsWidget);
        videoHeight->setObjectName("videoHeight");
        videoHeight->setButtonSymbols(QAbstractSpinBox::NoButtons);
        videoHeight->setMaximum(99999);

        horizontalLayout_16->addWidget(videoHeight);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_10);


        streamDetails->addLayout(horizontalLayout_16, 2, 1, 1, 1);

        lblAspectRatio = new QLabel(ConcertStreamDetailsWidget);
        lblAspectRatio->setObjectName("lblAspectRatio");
        lblAspectRatio->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblAspectRatio, 3, 0, 1, 1);

        videoCodec = new QLineEdit(ConcertStreamDetailsWidget);
        videoCodec->setObjectName("videoCodec");
        videoCodec->setMaximumSize(QSize(100, 16777215));

        streamDetails->addWidget(videoCodec, 1, 1, 1, 1);

        lblResolution = new QLabel(ConcertStreamDetailsWidget);
        lblResolution->setObjectName("lblResolution");
        lblResolution->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblResolution, 2, 0, 1, 1);

        lblStereoMode = new QLabel(ConcertStreamDetailsWidget);
        lblStereoMode->setObjectName("lblStereoMode");
        lblStereoMode->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        streamDetails->addWidget(lblStereoMode, 5, 0, 1, 1);

        stereoMode = new StereoModeComboBox(ConcertStreamDetailsWidget);
        stereoMode->setObjectName("stereoMode");

        streamDetails->addWidget(stereoMode, 5, 1, 1, 1);


        verticalLayout_2->addLayout(streamDetails);

        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        buttonReloadStreamDetails = new QPushButton(ConcertStreamDetailsWidget);
        buttonReloadStreamDetails->setObjectName("buttonReloadStreamDetails");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_right_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonReloadStreamDetails->setIcon(icon);

        horizontalLayout_17->addWidget(buttonReloadStreamDetails);

        lblReloadStreamDetailsError = new QLabel(ConcertStreamDetailsWidget);
        lblReloadStreamDetailsError->setObjectName("lblReloadStreamDetailsError");
        lblReloadStreamDetailsError->setText(QString::fromUtf8("Error"));

        horizontalLayout_17->addWidget(lblReloadStreamDetailsError);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_9);


        verticalLayout_2->addLayout(horizontalLayout_17);

        verticalSpacer_8 = new QSpacerItem(20, 98, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_8);


        retranslateUi(ConcertStreamDetailsWidget);

        QMetaObject::connectSlotsByName(ConcertStreamDetailsWidget);
    } // setupUi

    void retranslateUi(QWidget *ConcertStreamDetailsWidget)
    {
        lblCodec->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Codec", nullptr));
        videoDuration->setDisplayFormat(QCoreApplication::translate("ConcertStreamDetailsWidget", "HH:mm:ss", nullptr));
        lblVideo->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Video", nullptr));
        labelStreamDetailsAudio->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Audio", nullptr));
        lblScantype->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Scantype", nullptr));
        lblDuration->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Duration", nullptr));
        lblAspectRatio->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Aspect Ratio", nullptr));
        lblResolution->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Resolution", nullptr));
        lblStereoMode->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Stereo Mode", nullptr));
        buttonReloadStreamDetails->setText(QCoreApplication::translate("ConcertStreamDetailsWidget", "Reload from File", nullptr));
        (void)ConcertStreamDetailsWidget;
    } // retranslateUi

};

namespace Ui {
    class ConcertStreamDetailsWidget: public Ui_ConcertStreamDetailsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONCERTSTREAMDETAILSWIDGET_H
