/********************************************************************************
** Form generated from reading UI file 'TrailerDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRAILERDIALOG_H
#define UI_TRAILERDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyLineEdit.h"
#include "ui/small_widgets/SlidingStackedWidget.h"

QT_BEGIN_NAMESPACE

class Ui_TrailerDialog
{
public:
    QVBoxLayout *verticalLayout;
    SlidingStackedWidget *stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QComboBox *comboScraper;
    MyLineEdit *searchString;
    QTableWidget *results;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *buttonClose;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *trailers;
    QHBoxLayout *horizontalLayout;
    QPushButton *buttonBackToResults;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClose2;
    QWidget *page_trailerPreview;
    QVBoxLayout *verticalLayout_4;
    QWidget *video;
    QHBoxLayout *horizontalLayout_7;
    QToolButton *btnPlayPause;
    QSlider *seekSlider;
    QLabel *time;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QLineEdit *url;
    QHBoxLayout *horizontalLayout_4;
    QProgressBar *progressBar;
    QLabel *progress;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *buttonDownload;
    QPushButton *buttonCancelDownload;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *buttonBackToTrailers;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *buttonClose3;

    void setupUi(QDialog *TrailerDialog)
    {
        if (TrailerDialog->objectName().isEmpty())
            TrailerDialog->setObjectName("TrailerDialog");
        TrailerDialog->setWindowModality(Qt::ApplicationModal);
        TrailerDialog->resize(843, 644);
        TrailerDialog->setModal(true);
        verticalLayout = new QVBoxLayout(TrailerDialog);
        verticalLayout->setObjectName("verticalLayout");
        stackedWidget = new SlidingStackedWidget(TrailerDialog);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(page);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        comboScraper = new QComboBox(page);
        comboScraper->setObjectName("comboScraper");

        horizontalLayout_2->addWidget(comboScraper);

        searchString = new MyLineEdit(page);
        searchString->setObjectName("searchString");

        horizontalLayout_2->addWidget(searchString);


        verticalLayout_2->addLayout(horizontalLayout_2);

        results = new QTableWidget(page);
        if (results->columnCount() < 1)
            results->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        results->setHorizontalHeaderItem(0, __qtablewidgetitem);
        results->setObjectName("results");
        results->setEditTriggers(QAbstractItemView::NoEditTriggers);
        results->setAlternatingRowColors(true);
        results->setSelectionMode(QAbstractItemView::SingleSelection);
        results->setSelectionBehavior(QAbstractItemView::SelectRows);
        results->setShowGrid(false);
        results->horizontalHeader()->setVisible(false);
        results->horizontalHeader()->setStretchLastSection(true);
        results->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(results);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        buttonClose = new QPushButton(page);
        buttonClose->setObjectName("buttonClose");
        buttonClose->setAutoDefault(false);

        horizontalLayout_5->addWidget(buttonClose);


        verticalLayout_2->addLayout(horizontalLayout_5);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_3 = new QVBoxLayout(page_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        trailers = new QTableWidget(page_2);
        if (trailers->columnCount() < 3)
            trailers->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        trailers->setHorizontalHeaderItem(0, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        trailers->setHorizontalHeaderItem(1, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        trailers->setHorizontalHeaderItem(2, __qtablewidgetitem3);
        trailers->setObjectName("trailers");
        trailers->setEditTriggers(QAbstractItemView::NoEditTriggers);
        trailers->setAlternatingRowColors(true);
        trailers->setSelectionMode(QAbstractItemView::SingleSelection);
        trailers->setSelectionBehavior(QAbstractItemView::SelectRows);
        trailers->setShowGrid(false);
        trailers->setWordWrap(true);
        trailers->horizontalHeader()->setVisible(false);
        trailers->horizontalHeader()->setStretchLastSection(true);
        trailers->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(trailers);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        buttonBackToResults = new QPushButton(page_2);
        buttonBackToResults->setObjectName("buttonBackToResults");

        horizontalLayout->addWidget(buttonBackToResults);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonClose2 = new QPushButton(page_2);
        buttonClose2->setObjectName("buttonClose2");

        horizontalLayout->addWidget(buttonClose2);


        verticalLayout_3->addLayout(horizontalLayout);

        stackedWidget->addWidget(page_2);
        page_trailerPreview = new QWidget();
        page_trailerPreview->setObjectName("page_trailerPreview");
        verticalLayout_4 = new QVBoxLayout(page_trailerPreview);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        video = new QWidget(page_trailerPreview);
        video->setObjectName("video");

        verticalLayout_4->addWidget(video);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        btnPlayPause = new QToolButton(page_trailerPreview);
        btnPlayPause->setObjectName("btnPlayPause");
        btnPlayPause->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/video_play_64.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        btnPlayPause->setIcon(icon);

        horizontalLayout_7->addWidget(btnPlayPause);

        seekSlider = new QSlider(page_trailerPreview);
        seekSlider->setObjectName("seekSlider");
        seekSlider->setOrientation(Qt::Horizontal);

        horizontalLayout_7->addWidget(seekSlider);

        time = new QLabel(page_trailerPreview);
        time->setObjectName("time");
        time->setMinimumSize(QSize(80, 0));
        time->setText(QString::fromUtf8("0:00 / 0:00"));
        time->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_7->addWidget(time);


        verticalLayout_4->addLayout(horizontalLayout_7);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_2 = new QLabel(page_trailerPreview);
        label_2->setObjectName("label_2");

        horizontalLayout_3->addWidget(label_2);

        url = new QLineEdit(page_trailerPreview);
        url->setObjectName("url");
        url->setReadOnly(true);

        horizontalLayout_3->addWidget(url);


        verticalLayout_4->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        progressBar = new QProgressBar(page_trailerPreview);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        horizontalLayout_4->addWidget(progressBar);

        progress = new QLabel(page_trailerPreview);
        progress->setObjectName("progress");
        progress->setMinimumSize(QSize(100, 0));
        progress->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_4->addWidget(progress);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);

        buttonDownload = new QPushButton(page_trailerPreview);
        buttonDownload->setObjectName("buttonDownload");

        horizontalLayout_4->addWidget(buttonDownload);

        buttonCancelDownload = new QPushButton(page_trailerPreview);
        buttonCancelDownload->setObjectName("buttonCancelDownload");

        horizontalLayout_4->addWidget(buttonCancelDownload);


        verticalLayout_4->addLayout(horizontalLayout_4);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        buttonBackToTrailers = new QPushButton(page_trailerPreview);
        buttonBackToTrailers->setObjectName("buttonBackToTrailers");

        horizontalLayout_6->addWidget(buttonBackToTrailers);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);

        buttonClose3 = new QPushButton(page_trailerPreview);
        buttonClose3->setObjectName("buttonClose3");

        horizontalLayout_6->addWidget(buttonClose3);


        verticalLayout_4->addLayout(horizontalLayout_6);

        verticalLayout_4->setStretch(0, 1);
        stackedWidget->addWidget(page_trailerPreview);

        verticalLayout->addWidget(stackedWidget);


        retranslateUi(TrailerDialog);
        QObject::connect(buttonClose, &QPushButton::clicked, TrailerDialog, qOverload<>(&QDialog::reject));
        QObject::connect(buttonClose2, &QPushButton::clicked, TrailerDialog, qOverload<>(&QDialog::reject));
        QObject::connect(buttonClose3, &QPushButton::clicked, TrailerDialog, qOverload<>(&QDialog::reject));

        stackedWidget->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(TrailerDialog);
    } // setupUi

    void retranslateUi(QDialog *TrailerDialog)
    {
        TrailerDialog->setWindowTitle(QCoreApplication::translate("TrailerDialog", "Trailer", nullptr));
        label->setText(QCoreApplication::translate("TrailerDialog", "Scraper", nullptr));
        QTableWidgetItem *___qtablewidgetitem = results->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("TrailerDialog", "Result", nullptr));
        buttonClose->setText(QCoreApplication::translate("TrailerDialog", "Close", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = trailers->horizontalHeaderItem(0);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("TrailerDialog", "Preview", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = trailers->horizontalHeaderItem(1);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("TrailerDialog", "Language", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = trailers->horizontalHeaderItem(2);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("TrailerDialog", "Title", nullptr));
        buttonBackToResults->setText(QCoreApplication::translate("TrailerDialog", "Back to Search Results", nullptr));
        buttonClose2->setText(QCoreApplication::translate("TrailerDialog", "Close", nullptr));
        btnPlayPause->setText(QString());
        label_2->setText(QCoreApplication::translate("TrailerDialog", "URL", nullptr));
        progress->setText(QCoreApplication::translate("TrailerDialog", "Progress", nullptr));
        buttonDownload->setText(QCoreApplication::translate("TrailerDialog", "Download", nullptr));
        buttonCancelDownload->setText(QCoreApplication::translate("TrailerDialog", "Cancel Download", nullptr));
        buttonBackToTrailers->setText(QCoreApplication::translate("TrailerDialog", "Back to Trailers", nullptr));
        buttonClose3->setText(QCoreApplication::translate("TrailerDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TrailerDialog: public Ui_TrailerDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRAILERDIALOG_H
