/****************************************************************************
** Meta object code from reading C++ file 'TvShowWidgetEpisode.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/tv_show/TvShowWidgetEpisode.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TvShowWidgetEpisode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19TvShowWidgetEpisodeE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN19TvShowWidgetEpisodeE = QtMocHelpers::stringData(
    "TvShowWidgetEpisode",
    "sigSetActionSearchEnabled",
    "",
    "MainWidgets",
    "sigSetActionSaveEnabled",
    "onSetEnabled",
    "enabled",
    "onClear",
    "onSaveInformation",
    "onStartScraperSearch",
    "onPlayEpisode",
    "onAddDirector",
    "onRemoveDirector",
    "onAddWriter",
    "onRemoveWriter",
    "onChooseThumbnail",
    "onDeleteThumbnail",
    "onImageDropped",
    "ImageType",
    "imageType",
    "imageUrl",
    "onPosterDownloadFinished",
    "DownloadManagerElement",
    "elem",
    "onLoadDone",
    "QSet<EpisodeScraperInfo>",
    "details",
    "onRevertChanges",
    "onCaptureImage",
    "type",
    "onImdbIdChanged",
    "imdbid",
    "onTvdbIdChanged",
    "tvdbid",
    "onTmdbIdChanged",
    "tmdbId",
    "onTvmazeIdChanged",
    "tvmazeId",
    "onNameChange",
    "text",
    "onShowTitleChange",
    "onSeasonChange",
    "value",
    "onEpisodeChange",
    "onDisplaySeasonChange",
    "onDisplayEpisodeChange",
    "onTop250Change",
    "onCertificationChange",
    "onFirstAiredChange",
    "date",
    "onPlayCountChange",
    "onLastPlayedChange",
    "dateTime",
    "onStudioChange",
    "onEpBookmarkChange",
    "time",
    "onOverviewChange",
    "onDirectorEdited",
    "QTableWidgetItem*",
    "item",
    "onWriterEdited",
    "onStreamDetailsEdited",
    "onReloadStreamDetails",
    "updateStreamDetails",
    "reloadedFromFile",
    "onAddTag",
    "tag",
    "onRemoveTag",
    "onAddActor",
    "onRemoveActor",
    "onActorChanged",
    "onChangeActorImage",
    "onActorEdited",
    "onImdbIdOpen",
    "onTvMazeIdOpen"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN19TvShowWidgetEpisodeE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      51,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,  320,    2, 0x06,    1 /* Public */,
       4,    2,  325,    2, 0x06,    4 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       5,    1,  330,    2, 0x0a,    7 /* Public */,
       7,    0,  333,    2, 0x0a,    9 /* Public */,
       8,    0,  334,    2, 0x0a,   10 /* Public */,
       9,    0,  335,    2, 0x0a,   11 /* Public */,
      10,    0,  336,    2, 0x0a,   12 /* Public */,
      11,    0,  337,    2, 0x08,   13 /* Private */,
      12,    0,  338,    2, 0x08,   14 /* Private */,
      13,    0,  339,    2, 0x08,   15 /* Private */,
      14,    0,  340,    2, 0x08,   16 /* Private */,
      15,    0,  341,    2, 0x08,   17 /* Private */,
      16,    0,  342,    2, 0x08,   18 /* Private */,
      17,    2,  343,    2, 0x08,   19 /* Private */,
      21,    1,  348,    2, 0x08,   22 /* Private */,
      24,    1,  351,    2, 0x08,   24 /* Private */,
      27,    0,  354,    2, 0x08,   26 /* Private */,
      28,    1,  355,    2, 0x08,   27 /* Private */,
      30,    1,  358,    2, 0x08,   29 /* Private */,
      32,    1,  361,    2, 0x08,   31 /* Private */,
      34,    1,  364,    2, 0x08,   33 /* Private */,
      36,    1,  367,    2, 0x08,   35 /* Private */,
      38,    1,  370,    2, 0x08,   37 /* Private */,
      40,    1,  373,    2, 0x08,   39 /* Private */,
      41,    1,  376,    2, 0x08,   41 /* Private */,
      43,    1,  379,    2, 0x08,   43 /* Private */,
      44,    1,  382,    2, 0x08,   45 /* Private */,
      45,    1,  385,    2, 0x08,   47 /* Private */,
      46,    1,  388,    2, 0x08,   49 /* Private */,
      47,    1,  391,    2, 0x08,   51 /* Private */,
      48,    1,  394,    2, 0x08,   53 /* Private */,
      50,    1,  397,    2, 0x08,   55 /* Private */,
      51,    1,  400,    2, 0x08,   57 /* Private */,
      53,    1,  403,    2, 0x08,   59 /* Private */,
      54,    1,  406,    2, 0x08,   61 /* Private */,
      56,    0,  409,    2, 0x08,   63 /* Private */,
      57,    1,  410,    2, 0x08,   64 /* Private */,
      60,    1,  413,    2, 0x08,   66 /* Private */,
      61,    0,  416,    2, 0x08,   68 /* Private */,
      62,    0,  417,    2, 0x08,   69 /* Private */,
      63,    1,  418,    2, 0x08,   70 /* Private */,
      63,    0,  421,    2, 0x28,   72 /* Private | MethodCloned */,
      65,    1,  422,    2, 0x08,   73 /* Private */,
      67,    1,  425,    2, 0x08,   75 /* Private */,
      68,    0,  428,    2, 0x08,   77 /* Private */,
      69,    0,  429,    2, 0x08,   78 /* Private */,
      70,    0,  430,    2, 0x08,   79 /* Private */,
      71,    0,  431,    2, 0x08,   80 /* Private */,
      72,    1,  432,    2, 0x08,   81 /* Private */,
      73,    0,  435,    2, 0x08,   83 /* Private */,
      74,    0,  436,    2, 0x08,   84 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 3,    2,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18, QMetaType::QUrl,   19,   20,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,   29,
    QMetaType::Void, QMetaType::QString,   31,
    QMetaType::Void, QMetaType::QString,   33,
    QMetaType::Void, QMetaType::QString,   35,
    QMetaType::Void, QMetaType::QString,   37,
    QMetaType::Void, QMetaType::QString,   39,
    QMetaType::Void, QMetaType::QString,   39,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void, QMetaType::QString,   39,
    QMetaType::Void, QMetaType::QDate,   49,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void, QMetaType::QDateTime,   52,
    QMetaType::Void, QMetaType::QString,   39,
    QMetaType::Void, QMetaType::QTime,   55,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 58,   59,
    QMetaType::Void, 0x80000000 | 58,   59,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   64,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   66,
    QMetaType::Void, QMetaType::QString,   66,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 58,   59,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject TvShowWidgetEpisode::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN19TvShowWidgetEpisodeE.offsetsAndSizes,
    qt_meta_data_ZN19TvShowWidgetEpisodeE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN19TvShowWidgetEpisodeE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TvShowWidgetEpisode, std::true_type>,
        // method 'sigSetActionSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'sigSetActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'onSetEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onClear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSaveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onStartScraperSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPlayEpisode'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAddDirector'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveDirector'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAddWriter'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveWriter'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChooseThumbnail'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDeleteThumbnail'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImageDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QUrl, std::false_type>,
        // method 'onPosterDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<DownloadManagerElement, std::false_type>,
        // method 'onLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QSet<EpisodeScraperInfo>, std::false_type>,
        // method 'onRevertChanges'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onCaptureImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        // method 'onImdbIdChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTvdbIdChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTmdbIdChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTvmazeIdChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onNameChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onShowTitleChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onSeasonChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onEpisodeChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onDisplaySeasonChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onDisplayEpisodeChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onTop250Change'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onCertificationChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onFirstAiredChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QDate, std::false_type>,
        // method 'onPlayCountChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onLastPlayedChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QDateTime, std::false_type>,
        // method 'onStudioChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onEpBookmarkChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTime, std::false_type>,
        // method 'onOverviewChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDirectorEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onWriterEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onStreamDetailsEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onReloadStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'updateStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAddTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRemoveTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onAddActor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveActor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActorChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChangeActorImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onActorEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onImdbIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTvMazeIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void TvShowWidgetEpisode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TvShowWidgetEpisode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigSetActionSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 1: _t->sigSetActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 2: _t->onSetEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->onClear(); break;
        case 4: _t->onSaveInformation(); break;
        case 5: _t->onStartScraperSearch(); break;
        case 6: _t->onPlayEpisode(); break;
        case 7: _t->onAddDirector(); break;
        case 8: _t->onRemoveDirector(); break;
        case 9: _t->onAddWriter(); break;
        case 10: _t->onRemoveWriter(); break;
        case 11: _t->onChooseThumbnail(); break;
        case 12: _t->onDeleteThumbnail(); break;
        case 13: _t->onImageDropped((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 14: _t->onPosterDownloadFinished((*reinterpret_cast< std::add_pointer_t<DownloadManagerElement>>(_a[1]))); break;
        case 15: _t->onLoadDone((*reinterpret_cast< std::add_pointer_t<QSet<EpisodeScraperInfo>>>(_a[1]))); break;
        case 16: _t->onRevertChanges(); break;
        case 17: _t->onCaptureImage((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1]))); break;
        case 18: _t->onImdbIdChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->onTvdbIdChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->onTmdbIdChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->onTvmazeIdChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->onNameChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->onShowTitleChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->onSeasonChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->onEpisodeChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->onDisplaySeasonChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->onDisplayEpisodeChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->onTop250Change((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->onCertificationChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->onFirstAiredChange((*reinterpret_cast< std::add_pointer_t<QDate>>(_a[1]))); break;
        case 31: _t->onPlayCountChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->onLastPlayedChange((*reinterpret_cast< std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 33: _t->onStudioChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->onEpBookmarkChange((*reinterpret_cast< std::add_pointer_t<QTime>>(_a[1]))); break;
        case 35: _t->onOverviewChange(); break;
        case 36: _t->onDirectorEdited((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 37: _t->onWriterEdited((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 38: _t->onStreamDetailsEdited(); break;
        case 39: _t->onReloadStreamDetails(); break;
        case 40: _t->updateStreamDetails((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->updateStreamDetails(); break;
        case 42: _t->onAddTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->onRemoveTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 44: _t->onAddActor(); break;
        case 45: _t->onRemoveActor(); break;
        case 46: _t->onActorChanged(); break;
        case 47: _t->onChangeActorImage(); break;
        case 48: _t->onActorEdited((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 49: _t->onImdbIdOpen(); break;
        case 50: _t->onTvMazeIdOpen(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DownloadManagerElement >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (TvShowWidgetEpisode::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidgetEpisode::sigSetActionSearchEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (TvShowWidgetEpisode::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &TvShowWidgetEpisode::sigSetActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *TvShowWidgetEpisode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TvShowWidgetEpisode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN19TvShowWidgetEpisodeE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TvShowWidgetEpisode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 51)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 51)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    }
    return _id;
}

// SIGNAL 0
void TvShowWidgetEpisode::sigSetActionSearchEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TvShowWidgetEpisode::sigSetActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
