/********************************************************************************
** Form generated from reading UI file 'MusicMultiScrapeDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MUSICMULTISCRAPEDIALOG_H
#define UI_MUSICMULTISCRAPEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <ui/small_widgets/MyCheckBox.h>

QT_BEGIN_NAMESPACE

class Ui_MusicMultiScrapeDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_2;
    QGridLayout *gridLayout_2;
    MyCheckBox *chkBorn;
    MyCheckBox *chkName;
    MyCheckBox *chkYearsActive;
    MyCheckBox *chkDied;
    MyCheckBox *chkFormed;
    MyCheckBox *chkDisbanded;
    MyCheckBox *chkBiography;
    MyCheckBox *chkCdArt;
    MyCheckBox *chkCover;
    MyCheckBox *chkLogo;
    MyCheckBox *chkExtraFanarts;
    MyCheckBox *chkFanart;
    MyCheckBox *chkThumbnail;
    MyCheckBox *chkMoods;
    MyCheckBox *chkStyles;
    MyCheckBox *chkGenres;
    MyCheckBox *chkLabel;
    MyCheckBox *chkReleaseDate;
    MyCheckBox *chkRating;
    MyCheckBox *chkYear;
    MyCheckBox *chkReview;
    MyCheckBox *chkArtist;
    MyCheckBox *chkTitle;
    MyCheckBox *chkDiscography;
    QFrame *line;
    QCheckBox *chkUnCheckAll;
    QSpacerItem *verticalSpacer;
    QCheckBox *chkScrapeAllAlbums;
    QCheckBox *chkAutoSave;
    QHBoxLayout *horizontalLayout_2;
    QLabel *itemCounter;
    QLabel *itemName;
    QProgressBar *progressItem;
    QProgressBar *progressAll;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnCancel;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnClose;
    QPushButton *btnStartScraping;

    void setupUi(QDialog *MusicMultiScrapeDialog)
    {
        if (MusicMultiScrapeDialog->objectName().isEmpty())
            MusicMultiScrapeDialog->setObjectName("MusicMultiScrapeDialog");
        MusicMultiScrapeDialog->setWindowModality(Qt::WindowModal);
        MusicMultiScrapeDialog->resize(545, 492);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MusicMultiScrapeDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(MusicMultiScrapeDialog);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(MusicMultiScrapeDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");
        QFont font;
        font.setBold(true);
        label_2->setFont(font);

        verticalLayout_3->addWidget(label_2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        chkBorn = new MyCheckBox(groupBox);
        chkBorn->setObjectName("chkBorn");
        chkBorn->setChecked(true);
        chkBorn->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkBorn, 0, 1, 1, 1);

        chkName = new MyCheckBox(groupBox);
        chkName->setObjectName("chkName");
        chkName->setChecked(true);
        chkName->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkName, 0, 0, 1, 1);

        chkYearsActive = new MyCheckBox(groupBox);
        chkYearsActive->setObjectName("chkYearsActive");
        chkYearsActive->setChecked(true);
        chkYearsActive->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkYearsActive, 1, 0, 1, 1);

        chkDied = new MyCheckBox(groupBox);
        chkDied->setObjectName("chkDied");
        chkDied->setChecked(true);
        chkDied->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkDied, 1, 2, 1, 1);

        chkFormed = new MyCheckBox(groupBox);
        chkFormed->setObjectName("chkFormed");
        chkFormed->setChecked(true);
        chkFormed->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkFormed, 0, 2, 1, 1);

        chkDisbanded = new MyCheckBox(groupBox);
        chkDisbanded->setObjectName("chkDisbanded");
        chkDisbanded->setChecked(true);
        chkDisbanded->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkDisbanded, 1, 1, 1, 1);

        chkBiography = new MyCheckBox(groupBox);
        chkBiography->setObjectName("chkBiography");
        chkBiography->setChecked(true);
        chkBiography->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkBiography, 2, 0, 1, 1);

        chkCdArt = new MyCheckBox(groupBox);
        chkCdArt->setObjectName("chkCdArt");
        chkCdArt->setChecked(true);
        chkCdArt->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkCdArt, 7, 2, 1, 1);

        chkCover = new MyCheckBox(groupBox);
        chkCover->setObjectName("chkCover");
        chkCover->setChecked(true);
        chkCover->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkCover, 7, 1, 1, 1);

        chkLogo = new MyCheckBox(groupBox);
        chkLogo->setObjectName("chkLogo");
        chkLogo->setChecked(true);
        chkLogo->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkLogo, 7, 0, 1, 1);

        chkExtraFanarts = new MyCheckBox(groupBox);
        chkExtraFanarts->setObjectName("chkExtraFanarts");
        chkExtraFanarts->setChecked(true);
        chkExtraFanarts->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkExtraFanarts, 6, 2, 1, 1);

        chkFanart = new MyCheckBox(groupBox);
        chkFanart->setObjectName("chkFanart");
        chkFanart->setChecked(true);
        chkFanart->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkFanart, 6, 1, 1, 1);

        chkThumbnail = new MyCheckBox(groupBox);
        chkThumbnail->setObjectName("chkThumbnail");
        chkThumbnail->setChecked(true);
        chkThumbnail->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkThumbnail, 6, 0, 1, 1);

        chkMoods = new MyCheckBox(groupBox);
        chkMoods->setObjectName("chkMoods");
        chkMoods->setChecked(true);
        chkMoods->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkMoods, 5, 2, 1, 1);

        chkStyles = new MyCheckBox(groupBox);
        chkStyles->setObjectName("chkStyles");
        chkStyles->setChecked(true);
        chkStyles->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkStyles, 5, 1, 1, 1);

        chkGenres = new MyCheckBox(groupBox);
        chkGenres->setObjectName("chkGenres");
        chkGenres->setChecked(true);
        chkGenres->setProperty("type", QVariant(QString::fromUtf8("both")));

        gridLayout_2->addWidget(chkGenres, 5, 0, 1, 1);

        chkLabel = new MyCheckBox(groupBox);
        chkLabel->setObjectName("chkLabel");
        chkLabel->setChecked(true);
        chkLabel->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkLabel, 4, 2, 1, 1);

        chkReleaseDate = new MyCheckBox(groupBox);
        chkReleaseDate->setObjectName("chkReleaseDate");
        chkReleaseDate->setChecked(true);
        chkReleaseDate->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkReleaseDate, 4, 1, 1, 1);

        chkRating = new MyCheckBox(groupBox);
        chkRating->setObjectName("chkRating");
        chkRating->setChecked(true);
        chkRating->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkRating, 4, 0, 1, 1);

        chkYear = new MyCheckBox(groupBox);
        chkYear->setObjectName("chkYear");
        chkYear->setChecked(true);
        chkYear->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkYear, 3, 2, 1, 1);

        chkReview = new MyCheckBox(groupBox);
        chkReview->setObjectName("chkReview");
        chkReview->setChecked(true);
        chkReview->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkReview, 3, 1, 1, 1);

        chkArtist = new MyCheckBox(groupBox);
        chkArtist->setObjectName("chkArtist");
        chkArtist->setChecked(true);
        chkArtist->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkArtist, 3, 0, 1, 1);

        chkTitle = new MyCheckBox(groupBox);
        chkTitle->setObjectName("chkTitle");
        chkTitle->setChecked(true);
        chkTitle->setProperty("type", QVariant(QString::fromUtf8("album")));

        gridLayout_2->addWidget(chkTitle, 2, 2, 1, 1);

        chkDiscography = new MyCheckBox(groupBox);
        chkDiscography->setObjectName("chkDiscography");
        chkDiscography->setChecked(true);
        chkDiscography->setProperty("type", QVariant(QString::fromUtf8("artist")));

        gridLayout_2->addWidget(chkDiscography, 2, 1, 1, 1);


        verticalLayout_3->addLayout(gridLayout_2);

        line = new QFrame(groupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line);

        chkUnCheckAll = new QCheckBox(groupBox);
        chkUnCheckAll->setObjectName("chkUnCheckAll");
        chkUnCheckAll->setChecked(true);

        verticalLayout_3->addWidget(chkUnCheckAll);


        verticalLayout->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        chkScrapeAllAlbums = new QCheckBox(MusicMultiScrapeDialog);
        chkScrapeAllAlbums->setObjectName("chkScrapeAllAlbums");

        verticalLayout->addWidget(chkScrapeAllAlbums);

        chkAutoSave = new QCheckBox(MusicMultiScrapeDialog);
        chkAutoSave->setObjectName("chkAutoSave");

        verticalLayout->addWidget(chkAutoSave);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        itemCounter = new QLabel(MusicMultiScrapeDialog);
        itemCounter->setObjectName("itemCounter");
        itemCounter->setMinimumSize(QSize(30, 0));
        itemCounter->setText(QString::fromUtf8("1/20"));

        horizontalLayout_2->addWidget(itemCounter);

        itemName = new QLabel(MusicMultiScrapeDialog);
        itemName->setObjectName("itemName");
        itemName->setText(QString::fromUtf8("Item"));

        horizontalLayout_2->addWidget(itemName);

        horizontalLayout_2->setStretch(1, 1);

        verticalLayout->addLayout(horizontalLayout_2);

        progressItem = new QProgressBar(MusicMultiScrapeDialog);
        progressItem->setObjectName("progressItem");
        progressItem->setValue(24);

        verticalLayout->addWidget(progressItem);

        progressAll = new QProgressBar(MusicMultiScrapeDialog);
        progressAll->setObjectName("progressAll");
        progressAll->setValue(24);

        verticalLayout->addWidget(progressAll);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnCancel = new QPushButton(MusicMultiScrapeDialog);
        btnCancel->setObjectName("btnCancel");
        btnCancel->setAutoDefault(false);

        horizontalLayout->addWidget(btnCancel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnClose = new QPushButton(MusicMultiScrapeDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(true);

        horizontalLayout->addWidget(btnClose);

        btnStartScraping = new QPushButton(MusicMultiScrapeDialog);
        btnStartScraping->setObjectName("btnStartScraping");

        horizontalLayout->addWidget(btnStartScraping);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(MusicMultiScrapeDialog);
        QObject::connect(btnClose, &QPushButton::clicked, MusicMultiScrapeDialog, qOverload<>(&QDialog::accept));
        QObject::connect(btnCancel, &QPushButton::clicked, MusicMultiScrapeDialog, qOverload<>(&QDialog::reject));

        btnClose->setDefault(true);
        btnStartScraping->setDefault(true);


        QMetaObject::connectSlotsByName(MusicMultiScrapeDialog);
    } // setupUi

    void retranslateUi(QDialog *MusicMultiScrapeDialog)
    {
        MusicMultiScrapeDialog->setWindowTitle(QCoreApplication::translate("MusicMultiScrapeDialog", "Music Multi Scrape", nullptr));
        groupBox->setTitle(QString());
        label_2->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Infos to load", nullptr));
        chkBorn->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Born", nullptr));
        chkName->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Name", nullptr));
        chkYearsActive->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Years Active", nullptr));
        chkDied->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Died", nullptr));
        chkFormed->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Formed", nullptr));
        chkDisbanded->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Disbanded", nullptr));
        chkBiography->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Biography", nullptr));
        chkCdArt->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "CD Art", nullptr));
        chkCover->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Cover", nullptr));
        chkLogo->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Logo", nullptr));
        chkExtraFanarts->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Extra Fanarts", nullptr));
        chkFanart->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Fanart", nullptr));
        chkThumbnail->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Thumbnail", nullptr));
        chkMoods->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Moods", nullptr));
        chkStyles->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Styles", nullptr));
        chkGenres->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Genres", nullptr));
        chkLabel->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Label", nullptr));
        chkReleaseDate->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Release Date", nullptr));
        chkRating->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Rating", nullptr));
        chkYear->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Year", nullptr));
        chkReview->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Review", nullptr));
        chkArtist->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Artist", nullptr));
        chkTitle->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Title", nullptr));
        chkDiscography->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Discography", nullptr));
        chkUnCheckAll->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "(Un)Check all", nullptr));
        chkScrapeAllAlbums->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Scrape all albums of selected artists (and not only selected albums)", nullptr));
        chkAutoSave->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Automatically save each artist and album after scraping", nullptr));
        btnCancel->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Cancel", nullptr));
        btnClose->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Close", nullptr));
        btnStartScraping->setText(QCoreApplication::translate("MusicMultiScrapeDialog", "Start Scraping", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MusicMultiScrapeDialog: public Ui_MusicMultiScrapeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MUSICMULTISCRAPEDIALOG_H
