/****************************************************************************
** Meta object code from reading C++ file 'MovieWidget.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/movies/MovieWidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MovieWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MovieWidgetE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN11MovieWidgetE = QtMocHelpers::stringData(
    "MovieWidget",
    "actorDownloadStarted",
    "",
    "actorDownloadProgress",
    "actorDownloadFinished",
    "setActionSearchEnabled",
    "MainWidgets",
    "setActionSaveEnabled",
    "clear",
    "setMovie",
    "Movie*",
    "movie",
    "startScraperSearch",
    "saveInformation",
    "saveAll",
    "setEnabledTrue",
    "setDisabledTrue",
    "setBigWindow",
    "bigWindow",
    "updateMovieInfo",
    "onInfoLoadDone",
    "onLoadStarted",
    "onLoadDone",
    "onLoadImagesStarted",
    "onLoadingImages",
    "QSet<ImageType>",
    "imageTypes",
    "onDownloadProgress",
    "current",
    "maximum",
    "onSetImage",
    "ImageType",
    "type",
    "imageData",
    "onImageDropped",
    "imageType",
    "imageUrl",
    "onCaptureImage",
    "onExtraFanartsDropped",
    "QList<QUrl>",
    "imageUrls",
    "onChooseImage",
    "onDeleteImage",
    "movieNameChanged",
    "text",
    "addGenre",
    "genre",
    "removeGenre",
    "addTag",
    "tag",
    "removeTag",
    "addStudio",
    "studio",
    "removeStudio",
    "addCountry",
    "country",
    "removeCountry",
    "onRevertChanges",
    "onPlayMovie",
    "onArtPageOne",
    "onArtPageTwo",
    "onNameChange",
    "onOriginalNameChange",
    "onSortTitleChange",
    "onSetChange",
    "onTaglineChange",
    "onWriterChange",
    "onDirectorChange",
    "onUserRatingChange",
    "value",
    "onTop250Change",
    "onReleasedChange",
    "date",
    "onRuntimeChange",
    "onCertificationChange",
    "onTrailerChange",
    "onTvShowLinksChange",
    "onWatchedClicked",
    "onPlayCountChange",
    "onLastWatchedChange",
    "dateTime",
    "onOverviewChange",
    "onOutlineChange",
    "onImdbIdChange",
    "onTmdbIdChange",
    "onImdbIdOpen",
    "onTmdbIdOpen",
    "onSubtitleEdited",
    "QTableWidgetItem*",
    "item",
    "onStreamDetailsEdited",
    "onClickReloadStreamDetails",
    "updateStreamDetails",
    "reloadedFromFile",
    "onDownloadTrailer",
    "onInsertYoutubeLink",
    "onPlayLocalTrailer",
    "onRemoveExtraFanart",
    "file",
    "image",
    "onAddExtraFanart"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN11MovieWidgetE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      73,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,  452,    2, 0x06,    1 /* Public */,
       3,    3,  457,    2, 0x06,    4 /* Public */,
       4,    1,  464,    2, 0x06,    8 /* Public */,
       5,    2,  467,    2, 0x06,   10 /* Public */,
       7,    2,  472,    2, 0x06,   13 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       8,    0,  477,    2, 0x0a,   16 /* Public */,
       9,    1,  478,    2, 0x0a,   17 /* Public */,
      12,    0,  481,    2, 0x0a,   19 /* Public */,
      13,    0,  482,    2, 0x0a,   20 /* Public */,
      14,    0,  483,    2, 0x0a,   21 /* Public */,
      15,    0,  484,    2, 0x0a,   22 /* Public */,
      16,    0,  485,    2, 0x0a,   23 /* Public */,
      17,    1,  486,    2, 0x0a,   24 /* Public */,
      19,    0,  489,    2, 0x0a,   26 /* Public */,
      20,    1,  490,    2, 0x08,   27 /* Private */,
      21,    1,  493,    2, 0x08,   29 /* Private */,
      22,    1,  496,    2, 0x08,   31 /* Private */,
      23,    1,  499,    2, 0x08,   33 /* Private */,
      24,    2,  502,    2, 0x08,   35 /* Private */,
      27,    3,  507,    2, 0x08,   38 /* Private */,
      30,    3,  514,    2, 0x08,   42 /* Private */,
      34,    2,  521,    2, 0x08,   46 /* Private */,
      37,    1,  526,    2, 0x08,   49 /* Private */,
      38,    1,  529,    2, 0x08,   51 /* Private */,
      41,    0,  532,    2, 0x08,   53 /* Private */,
      42,    0,  533,    2, 0x08,   54 /* Private */,
      43,    1,  534,    2, 0x08,   55 /* Private */,
      45,    1,  537,    2, 0x08,   57 /* Private */,
      47,    1,  540,    2, 0x08,   59 /* Private */,
      48,    1,  543,    2, 0x08,   61 /* Private */,
      50,    1,  546,    2, 0x08,   63 /* Private */,
      51,    1,  549,    2, 0x08,   65 /* Private */,
      53,    1,  552,    2, 0x08,   67 /* Private */,
      54,    1,  555,    2, 0x08,   69 /* Private */,
      56,    1,  558,    2, 0x08,   71 /* Private */,
      57,    0,  561,    2, 0x08,   73 /* Private */,
      58,    0,  562,    2, 0x08,   74 /* Private */,
      59,    0,  563,    2, 0x08,   75 /* Private */,
      60,    0,  564,    2, 0x08,   76 /* Private */,
      61,    1,  565,    2, 0x08,   77 /* Private */,
      62,    1,  568,    2, 0x08,   79 /* Private */,
      63,    1,  571,    2, 0x08,   81 /* Private */,
      64,    1,  574,    2, 0x08,   83 /* Private */,
      65,    1,  577,    2, 0x08,   85 /* Private */,
      66,    1,  580,    2, 0x08,   87 /* Private */,
      67,    1,  583,    2, 0x08,   89 /* Private */,
      68,    1,  586,    2, 0x08,   91 /* Private */,
      70,    1,  589,    2, 0x08,   93 /* Private */,
      71,    1,  592,    2, 0x08,   95 /* Private */,
      73,    1,  595,    2, 0x08,   97 /* Private */,
      74,    1,  598,    2, 0x08,   99 /* Private */,
      75,    1,  601,    2, 0x08,  101 /* Private */,
      76,    0,  604,    2, 0x08,  103 /* Private */,
      77,    0,  605,    2, 0x08,  104 /* Private */,
      78,    1,  606,    2, 0x08,  105 /* Private */,
      79,    1,  609,    2, 0x08,  107 /* Private */,
      81,    0,  612,    2, 0x08,  109 /* Private */,
      82,    0,  613,    2, 0x08,  110 /* Private */,
      83,    1,  614,    2, 0x08,  111 /* Private */,
      84,    1,  617,    2, 0x08,  113 /* Private */,
      85,    0,  620,    2, 0x08,  115 /* Private */,
      86,    0,  621,    2, 0x08,  116 /* Private */,
      87,    1,  622,    2, 0x08,  117 /* Private */,
      90,    0,  625,    2, 0x08,  119 /* Private */,
      91,    0,  626,    2, 0x08,  120 /* Private */,
      92,    1,  627,    2, 0x08,  121 /* Private */,
      92,    0,  630,    2, 0x28,  123 /* Private | MethodCloned */,
      94,    0,  631,    2, 0x08,  124 /* Private */,
      95,    0,  632,    2, 0x08,  125 /* Private */,
      96,    0,  633,    2, 0x08,  126 /* Private */,
      97,    1,  634,    2, 0x08,  127 /* Private */,
      97,    1,  637,    2, 0x08,  129 /* Private */,
     100,    0,  640,    2, 0x08,  131 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 6,    2,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 6,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 25,   11,   26,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Int,   11,   28,   29,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 31, QMetaType::QByteArray,   11,   32,   33,
    QMetaType::Void, 0x80000000 | 31, QMetaType::QUrl,   35,   36,
    QMetaType::Void, 0x80000000 | 31,   32,
    QMetaType::Void, 0x80000000 | 39,   40,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   46,
    QMetaType::Void, QMetaType::QString,   46,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Void, QMetaType::QString,   52,
    QMetaType::Void, QMetaType::QString,   52,
    QMetaType::Void, QMetaType::QString,   55,
    QMetaType::Void, QMetaType::QString,   55,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::Double,   69,
    QMetaType::Void, QMetaType::Int,   69,
    QMetaType::Void, QMetaType::QDate,   72,
    QMetaType::Void, QMetaType::Int,   69,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   69,
    QMetaType::Void, QMetaType::QDateTime,   80,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void, QMetaType::QString,   44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 88,   89,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   93,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   98,
    QMetaType::Void, QMetaType::QByteArray,   99,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject MovieWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN11MovieWidgetE.offsetsAndSizes,
    qt_meta_data_ZN11MovieWidgetE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN11MovieWidgetE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MovieWidget, std::true_type>,
        // method 'actorDownloadStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'actorDownloadProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'actorDownloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'setActionSearchEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'setActionSaveEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<MainWidgets, std::false_type>,
        // method 'clear'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'startScraperSearch'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveInformation'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveAll'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setEnabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setDisabledTrue'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setBigWindow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'updateMovieInfo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onInfoLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onLoadStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onLoadDone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onLoadImagesStarted'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'onLoadingImages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QSet<ImageType>, std::false_type>,
        // method 'onDownloadProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onSetImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'onImageDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        QtPrivate::TypeAndForceComplete<QUrl, std::false_type>,
        // method 'onCaptureImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<ImageType, std::false_type>,
        // method 'onExtraFanartsDropped'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<QUrl>, std::false_type>,
        // method 'onChooseImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDeleteImage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'movieNameChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'addGenre'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'removeGenre'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'addTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'removeTag'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'addStudio'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'removeStudio'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'addCountry'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'removeCountry'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRevertChanges'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPlayMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtPageOne'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtPageTwo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onNameChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onOriginalNameChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onSortTitleChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onSetChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTaglineChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onWriterChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onDirectorChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onUserRatingChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<double, std::false_type>,
        // method 'onTop250Change'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onReleasedChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QDate, std::false_type>,
        // method 'onRuntimeChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onCertificationChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTrailerChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTvShowLinksChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onWatchedClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPlayCountChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onLastWatchedChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QDateTime, std::false_type>,
        // method 'onOverviewChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onOutlineChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onImdbIdChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onTmdbIdChange'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onImdbIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onTmdbIdOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onSubtitleEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'onStreamDetailsEdited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onClickReloadStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'updateStreamDetails'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onDownloadTrailer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onInsertYoutubeLink'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onPlayLocalTrailer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onRemoveExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onRemoveExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'onAddExtraFanart'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void MovieWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MovieWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->actorDownloadStarted((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->actorDownloadProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->actorDownloadFinished((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setActionSearchEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 4: _t->setActionSaveEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<MainWidgets>>(_a[2]))); break;
        case 5: _t->clear(); break;
        case 6: _t->setMovie((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 7: _t->startScraperSearch(); break;
        case 8: _t->saveInformation(); break;
        case 9: _t->saveAll(); break;
        case 10: _t->setEnabledTrue(); break;
        case 11: _t->setDisabledTrue(); break;
        case 12: _t->setBigWindow((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->updateMovieInfo(); break;
        case 14: _t->onInfoLoadDone((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 15: _t->onLoadStarted((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 16: _t->onLoadDone((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 17: _t->onLoadImagesStarted((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1]))); break;
        case 18: _t->onLoadingImages((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QSet<ImageType>>>(_a[2]))); break;
        case 19: _t->onDownloadProgress((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 20: _t->onSetImage((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 21: _t->onImageDropped((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 22: _t->onCaptureImage((*reinterpret_cast< std::add_pointer_t<ImageType>>(_a[1]))); break;
        case 23: _t->onExtraFanartsDropped((*reinterpret_cast< std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 24: _t->onChooseImage(); break;
        case 25: _t->onDeleteImage(); break;
        case 26: _t->movieNameChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->addGenre((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->removeGenre((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->addTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->removeTag((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->addStudio((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->removeStudio((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->addCountry((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->removeCountry((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->onRevertChanges(); break;
        case 36: _t->onPlayMovie(); break;
        case 37: _t->onArtPageOne(); break;
        case 38: _t->onArtPageTwo(); break;
        case 39: _t->onNameChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 40: _t->onOriginalNameChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->onSortTitleChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->onSetChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->onTaglineChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 44: _t->onWriterChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 45: _t->onDirectorChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 46: _t->onUserRatingChange((*reinterpret_cast< std::add_pointer_t<double>>(_a[1]))); break;
        case 47: _t->onTop250Change((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 48: _t->onReleasedChange((*reinterpret_cast< std::add_pointer_t<QDate>>(_a[1]))); break;
        case 49: _t->onRuntimeChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 50: _t->onCertificationChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 51: _t->onTrailerChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->onTvShowLinksChange(); break;
        case 53: _t->onWatchedClicked(); break;
        case 54: _t->onPlayCountChange((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 55: _t->onLastWatchedChange((*reinterpret_cast< std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 56: _t->onOverviewChange(); break;
        case 57: _t->onOutlineChange(); break;
        case 58: _t->onImdbIdChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 59: _t->onTmdbIdChange((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 60: _t->onImdbIdOpen(); break;
        case 61: _t->onTmdbIdOpen(); break;
        case 62: _t->onSubtitleEdited((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 63: _t->onStreamDetailsEdited(); break;
        case 64: _t->onClickReloadStreamDetails(); break;
        case 65: _t->updateStreamDetails((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 66: _t->updateStreamDetails(); break;
        case 67: _t->onDownloadTrailer(); break;
        case 68: _t->onInsertYoutubeLink(); break;
        case 69: _t->onPlayLocalTrailer(); break;
        case 70: _t->onRemoveExtraFanart((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 71: _t->onRemoveExtraFanart((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 72: _t->onAddExtraFanart(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<ImageType> >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QUrl> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (MovieWidget::*)(QString , int );
            if (_q_method_type _q_method = &MovieWidget::actorDownloadStarted; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (MovieWidget::*)(int , int , int );
            if (_q_method_type _q_method = &MovieWidget::actorDownloadProgress; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (MovieWidget::*)(int );
            if (_q_method_type _q_method = &MovieWidget::actorDownloadFinished; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (MovieWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &MovieWidget::setActionSearchEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (MovieWidget::*)(bool , MainWidgets );
            if (_q_method_type _q_method = &MovieWidget::setActionSaveEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
    }
}

const QMetaObject *MovieWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MovieWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN11MovieWidgetE.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MovieWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 73)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 73;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 73)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 73;
    }
    return _id;
}

// SIGNAL 0
void MovieWidget::actorDownloadStarted(QString _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void MovieWidget::actorDownloadProgress(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void MovieWidget::actorDownloadFinished(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void MovieWidget::setActionSearchEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void MovieWidget::setActionSaveEnabled(bool _t1, MainWidgets _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
