/********************************************************************************
** Form generated from reading UI file 'ScraperSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRAPERSETTINGSWIDGET_H
#define UI_SCRAPERSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/settings/CustomTvScraperSettingsWidget.h"
#include "ui/settings/ScraperSettingsTable.h"

QT_BEGIN_NAMESPACE

class Ui_ScraperSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_4;
    QTabWidget *tabWidget;
    QWidget *tabScrapers;
    QVBoxLayout *vLayout_MovieScrapers;
    ScraperSettingsTable *scraperSettings;
    QWidget *tabCustomMovieScraper;
    QVBoxLayout *verticalLayout_11;
    QLabel *lblCustomMovieScraperHelp;
    QTableWidget *customScraperTable;
    QWidget *tabCustomTvScraper;
    QVBoxLayout *verticalLayout;
    CustomTvScraperSettingsWidget *customTvScraperSettings;

    void setupUi(QWidget *ScraperSettingsWidget)
    {
        if (ScraperSettingsWidget->objectName().isEmpty())
            ScraperSettingsWidget->setObjectName("ScraperSettingsWidget");
        ScraperSettingsWidget->resize(970, 625);
        verticalLayout_4 = new QVBoxLayout(ScraperSettingsWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        tabWidget = new QTabWidget(ScraperSettingsWidget);
        tabWidget->setObjectName("tabWidget");
        tabScrapers = new QWidget();
        tabScrapers->setObjectName("tabScrapers");
        vLayout_MovieScrapers = new QVBoxLayout(tabScrapers);
        vLayout_MovieScrapers->setObjectName("vLayout_MovieScrapers");
        scraperSettings = new ScraperSettingsTable(tabScrapers);
        scraperSettings->setObjectName("scraperSettings");

        vLayout_MovieScrapers->addWidget(scraperSettings);

        tabWidget->addTab(tabScrapers, QString());
        tabCustomMovieScraper = new QWidget();
        tabCustomMovieScraper->setObjectName("tabCustomMovieScraper");
        verticalLayout_11 = new QVBoxLayout(tabCustomMovieScraper);
        verticalLayout_11->setObjectName("verticalLayout_11");
        lblCustomMovieScraperHelp = new QLabel(tabCustomMovieScraper);
        lblCustomMovieScraperHelp->setObjectName("lblCustomMovieScraperHelp");
        lblCustomMovieScraperHelp->setWordWrap(true);

        verticalLayout_11->addWidget(lblCustomMovieScraperHelp);

        customScraperTable = new QTableWidget(tabCustomMovieScraper);
        if (customScraperTable->columnCount() < 2)
            customScraperTable->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        customScraperTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        customScraperTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        customScraperTable->setObjectName("customScraperTable");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(customScraperTable->sizePolicy().hasHeightForWidth());
        customScraperTable->setSizePolicy(sizePolicy);
        customScraperTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        customScraperTable->setAlternatingRowColors(true);
        customScraperTable->setSelectionMode(QAbstractItemView::NoSelection);
        customScraperTable->setShowGrid(false);
        customScraperTable->verticalHeader()->setVisible(false);

        verticalLayout_11->addWidget(customScraperTable);

        tabWidget->addTab(tabCustomMovieScraper, QString());
        tabCustomTvScraper = new QWidget();
        tabCustomTvScraper->setObjectName("tabCustomTvScraper");
        verticalLayout = new QVBoxLayout(tabCustomTvScraper);
        verticalLayout->setObjectName("verticalLayout");
        customTvScraperSettings = new CustomTvScraperSettingsWidget(tabCustomTvScraper);
        customTvScraperSettings->setObjectName("customTvScraperSettings");

        verticalLayout->addWidget(customTvScraperSettings);

        tabWidget->addTab(tabCustomTvScraper, QString());

        verticalLayout_4->addWidget(tabWidget);


        retranslateUi(ScraperSettingsWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ScraperSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *ScraperSettingsWidget)
    {
        tabWidget->setTabText(tabWidget->indexOf(tabScrapers), QCoreApplication::translate("ScraperSettingsWidget", "Scrapers", nullptr));
        lblCustomMovieScraperHelp->setText(QCoreApplication::translate("ScraperSettingsWidget", "Combine multiple scrapers to your custom scraper. If you select other scrapers than IMDB, The Movie DB and Fanart.tv multiple searches may be necessary as only these three share an id.", nullptr));
        QTableWidgetItem *___qtablewidgetitem = customScraperTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("ScraperSettingsWidget", "Item", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = customScraperTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("ScraperSettingsWidget", "Scraper", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabCustomMovieScraper), QCoreApplication::translate("ScraperSettingsWidget", "Custom Movie Scraper", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabCustomTvScraper), QCoreApplication::translate("ScraperSettingsWidget", "Custom TV Scraper", nullptr));
        (void)ScraperSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class ScraperSettingsWidget: public Ui_ScraperSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCRAPERSETTINGSWIDGET_H
