/****************************************************************************
** Meta object code from reading C++ file 'TrailerDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/ui/trailer/TrailerDialog.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TrailerDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13TrailerDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN13TrailerDialogE = QtMocHelpers::stringData(
    "TrailerDialog",
    "exec",
    "",
    "Movie*",
    "movie",
    "reject",
    "search",
    "searchIndex",
    "comboIndex",
    "showResults",
    "QList<ScraperSearchResult>",
    "results",
    "showTrailers",
    "QList<TrailerResult>",
    "trailers",
    "resultClicked",
    "QTableWidgetItem*",
    "item",
    "trailerClicked",
    "backToResults",
    "backToTrailers",
    "startDownload",
    "cancelDownload",
    "downloadProgress",
    "received",
    "total",
    "downloadFinished",
    "downloadReadyRead",
    "onNewTotalTime",
    "totalTime",
    "onStateChanged",
    "QMediaPlayer::PlaybackState",
    "newState",
    "onPlayPause",
    "onAnimationFinished",
    "onUpdateTime",
    "currentTime",
    "onTrailerError",
    "QMediaPlayer::Error",
    "error",
    "errorString",
    "onSliderPositionChanged"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN13TrailerDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  152,    2, 0x0a,    1 /* Public */,
       1,    1,  153,    2, 0x0a,    2 /* Public */,
       5,    0,  156,    2, 0x0a,    4 /* Public */,
       6,    0,  157,    2, 0x08,    5 /* Private */,
       7,    1,  158,    2, 0x08,    6 /* Private */,
       9,    1,  161,    2, 0x08,    8 /* Private */,
      12,    1,  164,    2, 0x08,   10 /* Private */,
      15,    1,  167,    2, 0x08,   12 /* Private */,
      18,    1,  170,    2, 0x08,   14 /* Private */,
      19,    0,  173,    2, 0x08,   16 /* Private */,
      20,    0,  174,    2, 0x08,   17 /* Private */,
      21,    0,  175,    2, 0x08,   18 /* Private */,
      22,    0,  176,    2, 0x08,   19 /* Private */,
      23,    2,  177,    2, 0x08,   20 /* Private */,
      26,    0,  182,    2, 0x08,   23 /* Private */,
      27,    0,  183,    2, 0x08,   24 /* Private */,
      28,    1,  184,    2, 0x08,   25 /* Private */,
      30,    1,  187,    2, 0x08,   27 /* Private */,
      33,    0,  190,    2, 0x08,   29 /* Private */,
      34,    0,  191,    2, 0x08,   30 /* Private */,
      35,    1,  192,    2, 0x08,   31 /* Private */,
      37,    2,  195,    2, 0x08,   33 /* Private */,
      41,    0,  200,    2, 0x08,   36 /* Private */,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Int, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   24,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,   29,
    QMetaType::Void, 0x80000000 | 31,   32,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,   36,
    QMetaType::Void, 0x80000000 | 38, QMetaType::QString,   39,   40,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject TrailerDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN13TrailerDialogE.offsetsAndSizes,
    qt_meta_data_ZN13TrailerDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN13TrailerDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<TrailerDialog, std::true_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<Movie *, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'search'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'searchIndex'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'showResults'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<ScraperSearchResult>, std::false_type>,
        // method 'showTrailers'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVector<TrailerResult>, std::false_type>,
        // method 'resultClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'trailerClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QTableWidgetItem *, std::false_type>,
        // method 'backToResults'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'backToTrailers'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'startDownload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'cancelDownload'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'downloadProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'downloadFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'downloadReadyRead'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onNewTotalTime'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<qint64, std::false_type>,
        // method 'onStateChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMediaPlayer::PlaybackState, std::false_type>,
        // method 'onPlayPause'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAnimationFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onUpdateTime'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<qint64, std::false_type>,
        // method 'onTrailerError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QMediaPlayer::Error, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'onSliderPositionChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void TrailerDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TrailerDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: { int _r = _t->exec((*reinterpret_cast< std::add_pointer_t<Movie*>>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->reject(); break;
        case 3: _t->search(); break;
        case 4: _t->searchIndex((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->showResults((*reinterpret_cast< std::add_pointer_t<QList<ScraperSearchResult>>>(_a[1]))); break;
        case 6: _t->showTrailers((*reinterpret_cast< std::add_pointer_t<QList<TrailerResult>>>(_a[1]))); break;
        case 7: _t->resultClicked((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 8: _t->trailerClicked((*reinterpret_cast< std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 9: _t->backToResults(); break;
        case 10: _t->backToTrailers(); break;
        case 11: _t->startDownload(); break;
        case 12: _t->cancelDownload(); break;
        case 13: _t->downloadProgress((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 14: _t->downloadFinished(); break;
        case 15: _t->downloadReadyRead(); break;
        case 16: _t->onNewTotalTime((*reinterpret_cast< std::add_pointer_t<qint64>>(_a[1]))); break;
        case 17: _t->onStateChanged((*reinterpret_cast< std::add_pointer_t<QMediaPlayer::PlaybackState>>(_a[1]))); break;
        case 18: _t->onPlayPause(); break;
        case 19: _t->onAnimationFinished(); break;
        case 20: _t->onUpdateTime((*reinterpret_cast< std::add_pointer_t<qint64>>(_a[1]))); break;
        case 21: _t->onTrailerError((*reinterpret_cast< std::add_pointer_t<QMediaPlayer::Error>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->onSliderPositionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Movie* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *TrailerDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TrailerDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN13TrailerDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int TrailerDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
