/********************************************************************************
** Form generated from reading UI file 'MovieSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIESETTINGSWIDGET_H
#define UI_MOVIESETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/small_widgets/PlaceholderLineEdit.h>

QT_BEGIN_NAMESPACE

class Ui_MovieSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_6;
    QLabel *lblFilenameDescription;
    QLabel *lblPlaceholders;
    QFormLayout *formMovieFilenames;
    QLabel *lblNfo;
    QLabel *lblPoster;
    QLabel *lblBackdrop;
    QLabel *lblLogo;
    QLabel *lblClearArt;
    QLabel *lblDiscArt;
    PlaceholderLineEdit *movieNfo;
    PlaceholderLineEdit *moviePoster;
    PlaceholderLineEdit *movieBackdrop;
    PlaceholderLineEdit *movieLogo;
    PlaceholderLineEdit *movieClearArt;
    PlaceholderLineEdit *movieCdArt;
    QLabel *lblBanner;
    QLabel *lblThumb;
    PlaceholderLineEdit *movieBanner;
    PlaceholderLineEdit *movieThumb;
    QFormLayout *formMovieSettings;
    QLabel *lblMovieOutline;
    QCheckBox *usePlotForOutline;
    QLabel *lblMovieSetArtwork;
    QComboBox *comboMovieSetArtwork;
    QLabel *lblMovieSetPosterFilename;
    PlaceholderLineEdit *movieSetPosterFileName;
    QLabel *lblMovieSetFanartFilename;
    PlaceholderLineEdit *movieSetFanartFileName;
    QLabel *labelMovieSetArtworkDir;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *movieSetArtworkDir;
    QPushButton *btnMovieSetArtworkDir;
    QLabel *lblAlwaysStoreOriginalTitle;
    QCheckBox *ignoreDuplicateOriginalTitle;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *MovieSettingsWidget)
    {
        if (MovieSettingsWidget->objectName().isEmpty())
            MovieSettingsWidget->setObjectName("MovieSettingsWidget");
        MovieSettingsWidget->resize(970, 626);
        verticalLayout_6 = new QVBoxLayout(MovieSettingsWidget);
        verticalLayout_6->setObjectName("verticalLayout_6");
        lblFilenameDescription = new QLabel(MovieSettingsWidget);
        lblFilenameDescription->setObjectName("lblFilenameDescription");
        lblFilenameDescription->setWordWrap(true);

        verticalLayout_6->addWidget(lblFilenameDescription);

        lblPlaceholders = new QLabel(MovieSettingsWidget);
        lblPlaceholders->setObjectName("lblPlaceholders");
        lblPlaceholders->setWordWrap(true);

        verticalLayout_6->addWidget(lblPlaceholders);

        formMovieFilenames = new QFormLayout();
        formMovieFilenames->setObjectName("formMovieFilenames");
        formMovieFilenames->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        lblNfo = new QLabel(MovieSettingsWidget);
        lblNfo->setObjectName("lblNfo");

        formMovieFilenames->setWidget(0, QFormLayout::LabelRole, lblNfo);

        lblPoster = new QLabel(MovieSettingsWidget);
        lblPoster->setObjectName("lblPoster");

        formMovieFilenames->setWidget(1, QFormLayout::LabelRole, lblPoster);

        lblBackdrop = new QLabel(MovieSettingsWidget);
        lblBackdrop->setObjectName("lblBackdrop");

        formMovieFilenames->setWidget(2, QFormLayout::LabelRole, lblBackdrop);

        lblLogo = new QLabel(MovieSettingsWidget);
        lblLogo->setObjectName("lblLogo");

        formMovieFilenames->setWidget(3, QFormLayout::LabelRole, lblLogo);

        lblClearArt = new QLabel(MovieSettingsWidget);
        lblClearArt->setObjectName("lblClearArt");

        formMovieFilenames->setWidget(4, QFormLayout::LabelRole, lblClearArt);

        lblDiscArt = new QLabel(MovieSettingsWidget);
        lblDiscArt->setObjectName("lblDiscArt");

        formMovieFilenames->setWidget(5, QFormLayout::LabelRole, lblDiscArt);

        movieNfo = new PlaceholderLineEdit(MovieSettingsWidget);
        movieNfo->setObjectName("movieNfo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(movieNfo->sizePolicy().hasHeightForWidth());
        movieNfo->setSizePolicy(sizePolicy);
        movieNfo->setMinimumSize(QSize(200, 0));
        movieNfo->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(0, QFormLayout::FieldRole, movieNfo);

        moviePoster = new PlaceholderLineEdit(MovieSettingsWidget);
        moviePoster->setObjectName("moviePoster");
        sizePolicy.setHeightForWidth(moviePoster->sizePolicy().hasHeightForWidth());
        moviePoster->setSizePolicy(sizePolicy);
        moviePoster->setMinimumSize(QSize(200, 0));
        moviePoster->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(1, QFormLayout::FieldRole, moviePoster);

        movieBackdrop = new PlaceholderLineEdit(MovieSettingsWidget);
        movieBackdrop->setObjectName("movieBackdrop");
        sizePolicy.setHeightForWidth(movieBackdrop->sizePolicy().hasHeightForWidth());
        movieBackdrop->setSizePolicy(sizePolicy);
        movieBackdrop->setMinimumSize(QSize(200, 0));
        movieBackdrop->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(2, QFormLayout::FieldRole, movieBackdrop);

        movieLogo = new PlaceholderLineEdit(MovieSettingsWidget);
        movieLogo->setObjectName("movieLogo");
        sizePolicy.setHeightForWidth(movieLogo->sizePolicy().hasHeightForWidth());
        movieLogo->setSizePolicy(sizePolicy);
        movieLogo->setMinimumSize(QSize(200, 0));
        movieLogo->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(3, QFormLayout::FieldRole, movieLogo);

        movieClearArt = new PlaceholderLineEdit(MovieSettingsWidget);
        movieClearArt->setObjectName("movieClearArt");
        sizePolicy.setHeightForWidth(movieClearArt->sizePolicy().hasHeightForWidth());
        movieClearArt->setSizePolicy(sizePolicy);
        movieClearArt->setMinimumSize(QSize(200, 0));
        movieClearArt->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(4, QFormLayout::FieldRole, movieClearArt);

        movieCdArt = new PlaceholderLineEdit(MovieSettingsWidget);
        movieCdArt->setObjectName("movieCdArt");
        sizePolicy.setHeightForWidth(movieCdArt->sizePolicy().hasHeightForWidth());
        movieCdArt->setSizePolicy(sizePolicy);
        movieCdArt->setMinimumSize(QSize(200, 0));
        movieCdArt->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(5, QFormLayout::FieldRole, movieCdArt);

        lblBanner = new QLabel(MovieSettingsWidget);
        lblBanner->setObjectName("lblBanner");

        formMovieFilenames->setWidget(6, QFormLayout::LabelRole, lblBanner);

        lblThumb = new QLabel(MovieSettingsWidget);
        lblThumb->setObjectName("lblThumb");

        formMovieFilenames->setWidget(7, QFormLayout::LabelRole, lblThumb);

        movieBanner = new PlaceholderLineEdit(MovieSettingsWidget);
        movieBanner->setObjectName("movieBanner");
        sizePolicy.setHeightForWidth(movieBanner->sizePolicy().hasHeightForWidth());
        movieBanner->setSizePolicy(sizePolicy);
        movieBanner->setMinimumSize(QSize(200, 0));
        movieBanner->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(6, QFormLayout::FieldRole, movieBanner);

        movieThumb = new PlaceholderLineEdit(MovieSettingsWidget);
        movieThumb->setObjectName("movieThumb");
        sizePolicy.setHeightForWidth(movieThumb->sizePolicy().hasHeightForWidth());
        movieThumb->setSizePolicy(sizePolicy);
        movieThumb->setMinimumSize(QSize(200, 0));
        movieThumb->setMaximumSize(QSize(400, 16777215));

        formMovieFilenames->setWidget(7, QFormLayout::FieldRole, movieThumb);


        verticalLayout_6->addLayout(formMovieFilenames);

        formMovieSettings = new QFormLayout();
        formMovieSettings->setObjectName("formMovieSettings");
        formMovieSettings->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        lblMovieOutline = new QLabel(MovieSettingsWidget);
        lblMovieOutline->setObjectName("lblMovieOutline");

        formMovieSettings->setWidget(0, QFormLayout::LabelRole, lblMovieOutline);

        usePlotForOutline = new QCheckBox(MovieSettingsWidget);
        usePlotForOutline->setObjectName("usePlotForOutline");

        formMovieSettings->setWidget(0, QFormLayout::FieldRole, usePlotForOutline);

        lblMovieSetArtwork = new QLabel(MovieSettingsWidget);
        lblMovieSetArtwork->setObjectName("lblMovieSetArtwork");

        formMovieSettings->setWidget(2, QFormLayout::LabelRole, lblMovieSetArtwork);

        comboMovieSetArtwork = new QComboBox(MovieSettingsWidget);
        comboMovieSetArtwork->setObjectName("comboMovieSetArtwork");

        formMovieSettings->setWidget(2, QFormLayout::FieldRole, comboMovieSetArtwork);

        lblMovieSetPosterFilename = new QLabel(MovieSettingsWidget);
        lblMovieSetPosterFilename->setObjectName("lblMovieSetPosterFilename");

        formMovieSettings->setWidget(3, QFormLayout::LabelRole, lblMovieSetPosterFilename);

        movieSetPosterFileName = new PlaceholderLineEdit(MovieSettingsWidget);
        movieSetPosterFileName->setObjectName("movieSetPosterFileName");
        sizePolicy.setHeightForWidth(movieSetPosterFileName->sizePolicy().hasHeightForWidth());
        movieSetPosterFileName->setSizePolicy(sizePolicy);
        movieSetPosterFileName->setMinimumSize(QSize(200, 0));
        movieSetPosterFileName->setMaximumSize(QSize(400, 16777215));
        movieSetPosterFileName->setCurrentText(QString::fromUtf8(""));
        movieSetPosterFileName->setProperty("placeholders", QVariant(QStringList{QString::fromUtf8("setName")}));

        formMovieSettings->setWidget(3, QFormLayout::FieldRole, movieSetPosterFileName);

        lblMovieSetFanartFilename = new QLabel(MovieSettingsWidget);
        lblMovieSetFanartFilename->setObjectName("lblMovieSetFanartFilename");

        formMovieSettings->setWidget(4, QFormLayout::LabelRole, lblMovieSetFanartFilename);

        movieSetFanartFileName = new PlaceholderLineEdit(MovieSettingsWidget);
        movieSetFanartFileName->setObjectName("movieSetFanartFileName");
        sizePolicy.setHeightForWidth(movieSetFanartFileName->sizePolicy().hasHeightForWidth());
        movieSetFanartFileName->setSizePolicy(sizePolicy);
        movieSetFanartFileName->setMinimumSize(QSize(200, 0));
        movieSetFanartFileName->setMaximumSize(QSize(400, 16777215));
        movieSetFanartFileName->setProperty("placeholders", QVariant(QStringList{QString::fromUtf8("setName")}));

        formMovieSettings->setWidget(4, QFormLayout::FieldRole, movieSetFanartFileName);

        labelMovieSetArtworkDir = new QLabel(MovieSettingsWidget);
        labelMovieSetArtworkDir->setObjectName("labelMovieSetArtworkDir");

        formMovieSettings->setWidget(5, QFormLayout::LabelRole, labelMovieSetArtworkDir);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        movieSetArtworkDir = new QLineEdit(MovieSettingsWidget);
        movieSetArtworkDir->setObjectName("movieSetArtworkDir");

        horizontalLayout_2->addWidget(movieSetArtworkDir);

        btnMovieSetArtworkDir = new QPushButton(MovieSettingsWidget);
        btnMovieSetArtworkDir->setObjectName("btnMovieSetArtworkDir");

        horizontalLayout_2->addWidget(btnMovieSetArtworkDir);


        formMovieSettings->setLayout(5, QFormLayout::FieldRole, horizontalLayout_2);

        lblAlwaysStoreOriginalTitle = new QLabel(MovieSettingsWidget);
        lblAlwaysStoreOriginalTitle->setObjectName("lblAlwaysStoreOriginalTitle");

        formMovieSettings->setWidget(1, QFormLayout::LabelRole, lblAlwaysStoreOriginalTitle);

        ignoreDuplicateOriginalTitle = new QCheckBox(MovieSettingsWidget);
        ignoreDuplicateOriginalTitle->setObjectName("ignoreDuplicateOriginalTitle");

        formMovieSettings->setWidget(1, QFormLayout::FieldRole, ignoreDuplicateOriginalTitle);


        verticalLayout_6->addLayout(formMovieSettings);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        verticalLayout_6->setStretch(2, 1);

        retranslateUi(MovieSettingsWidget);

        QMetaObject::connectSlotsByName(MovieSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *MovieSettingsWidget)
    {
        lblFilenameDescription->setText(QCoreApplication::translate("MovieSettingsWidget", "Below you see the filenames which are used for loading and saving your movies. You can edit them as you like, if you want to use multiple files separate them by comma.", nullptr));
        lblPlaceholders->setText(QCoreApplication::translate("MovieSettingsWidget", "You can use the placeholder <baseFileName> which is the filename without extension.", nullptr));
        lblNfo->setText(QCoreApplication::translate("MovieSettingsWidget", "Nfo", nullptr));
        lblPoster->setText(QCoreApplication::translate("MovieSettingsWidget", "Poster", nullptr));
        lblBackdrop->setText(QCoreApplication::translate("MovieSettingsWidget", "Backdrop", nullptr));
        lblLogo->setText(QCoreApplication::translate("MovieSettingsWidget", "Logo", nullptr));
        lblClearArt->setText(QCoreApplication::translate("MovieSettingsWidget", "Clear Art", nullptr));
        lblDiscArt->setText(QCoreApplication::translate("MovieSettingsWidget", "Disc Art", nullptr));
        lblBanner->setText(QCoreApplication::translate("MovieSettingsWidget", "Banner", nullptr));
        lblThumb->setText(QCoreApplication::translate("MovieSettingsWidget", "Thumb", nullptr));
        lblMovieOutline->setText(QCoreApplication::translate("MovieSettingsWidget", "Movie outline", nullptr));
        usePlotForOutline->setText(QCoreApplication::translate("MovieSettingsWidget", "Use plot when outline is not available", nullptr));
        lblMovieSetArtwork->setText(QCoreApplication::translate("MovieSettingsWidget", "Movie Set Artwork", nullptr));
        lblMovieSetPosterFilename->setText(QCoreApplication::translate("MovieSettingsWidget", "Movie Set Poster Filename", nullptr));
        lblMovieSetFanartFilename->setText(QCoreApplication::translate("MovieSettingsWidget", "Movie Set Fanart Filname", nullptr));
        movieSetFanartFileName->setCurrentText(QString());
        labelMovieSetArtworkDir->setText(QCoreApplication::translate("MovieSettingsWidget", "Artwork directory", nullptr));
        btnMovieSetArtworkDir->setText(QCoreApplication::translate("MovieSettingsWidget", "Choose directory", nullptr));
        lblAlwaysStoreOriginalTitle->setText(QCoreApplication::translate("MovieSettingsWidget", "Movie Original Title", nullptr));
        ignoreDuplicateOriginalTitle->setText(QCoreApplication::translate("MovieSettingsWidget", "Don't store the original title if it is the same as the normal title", nullptr));
        (void)MovieSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class MovieSettingsWidget: public Ui_MovieSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIESETTINGSWIDGET_H
