/********************************************************************************
** Form generated from reading UI file 'ActorsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACTORSWIDGET_H
#define UI_ACTORSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MyLabel.h"

QT_BEGIN_NAMESPACE

class Ui_ActorsWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QTableView *actors;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout_9;
    QPushButton *buttonRemoveActor;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonAddActor;
    QHBoxLayout *horizontalLayout_8;
    MyLabel *actor;
    QLabel *actorResolution;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ActorsWidget)
    {
        if (ActorsWidget->objectName().isEmpty())
            ActorsWidget->setObjectName("ActorsWidget");
        ActorsWidget->resize(502, 286);
        horizontalLayout = new QHBoxLayout(ActorsWidget);
        horizontalLayout->setSpacing(9);
        horizontalLayout->setObjectName("horizontalLayout");
        actors = new QTableView(ActorsWidget);
        actors->setObjectName("actors");
        actors->setEditTriggers(QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed);
        actors->setSelectionBehavior(QAbstractItemView::SelectRows);
        actors->setShowGrid(false);
        actors->verticalHeader()->setVisible(false);

        horizontalLayout->addWidget(actors);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setSpacing(0);
        verticalLayout_9->setObjectName("verticalLayout_9");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        buttonRemoveActor = new QPushButton(ActorsWidget);
        buttonRemoveActor->setObjectName("buttonRemoveActor");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/minus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRemoveActor->setIcon(icon);

        horizontalLayout_9->addWidget(buttonRemoveActor);

        horizontalSpacer = new QSpacerItem(5, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer);

        buttonAddActor = new QPushButton(ActorsWidget);
        buttonAddActor->setObjectName("buttonAddActor");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/img/plus_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonAddActor->setIcon(icon1);

        horizontalLayout_9->addWidget(buttonAddActor);


        verticalLayout_9->addLayout(horizontalLayout_9);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");

        verticalLayout_9->addLayout(horizontalLayout_8);

        actor = new MyLabel(ActorsWidget);
        actor->setObjectName("actor");
        actor->setMinimumSize(QSize(120, 180));
        actor->setMaximumSize(QSize(120, 180));
        actor->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        actor->setPixmap(QPixmap(QString::fromUtf8(":/img/man.png")));
        actor->setAlignment(Qt::AlignCenter);

        verticalLayout_9->addWidget(actor);

        actorResolution = new QLabel(ActorsWidget);
        actorResolution->setObjectName("actorResolution");
        actorResolution->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        verticalLayout_9->addWidget(actorResolution);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_9);

        horizontalLayout->setStretch(0, 1);

        retranslateUi(ActorsWidget);

        QMetaObject::connectSlotsByName(ActorsWidget);
    } // setupUi

    void retranslateUi(QWidget *ActorsWidget)
    {
        ActorsWidget->setWindowTitle(QCoreApplication::translate("ActorsWidget", "Form", nullptr));
#if QT_CONFIG(tooltip)
        buttonRemoveActor->setToolTip(QCoreApplication::translate("ActorsWidget", "Remove Actor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemoveActor->setText(QString());
#if QT_CONFIG(tooltip)
        buttonAddActor->setToolTip(QCoreApplication::translate("ActorsWidget", "Add Actor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAddActor->setText(QString());
#if QT_CONFIG(tooltip)
        actor->setToolTip(QCoreApplication::translate("ActorsWidget", "Click to change", nullptr));
#endif // QT_CONFIG(tooltip)
        actor->setText(QString());
        actorResolution->setText(QCoreApplication::translate("ActorsWidget", "Resolution", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ActorsWidget: public Ui_ActorsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ACTORSWIDGET_H
