/********************************************************************************
** Form generated from reading UI file 'MovieWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVIEWIDGET_H
#define UI_MOVIEWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/ActorsWidget.h"
#include "ui/small_widgets/Badge.h"
#include "ui/small_widgets/ClosableImage.h"
#include "ui/small_widgets/ImageGallery.h"
#include "ui/small_widgets/MediaFlags.h"
#include "ui/small_widgets/MessageLabel.h"
#include "ui/small_widgets/RatingsWidget.h"
#include "ui/small_widgets/SlidingStackedWidget.h"
#include "ui/small_widgets/StereoModeComboBox.h"
#include "ui/small_widgets/TagCloud.h"

QT_BEGIN_NAMESPACE

class Ui_MovieWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer_4;
    QToolButton *buttonRevert;
    QLabel *movieName;
    QSpacerItem *titleSpacer;
    MediaFlags *mediaFlags;
    QToolButton *buttonPlay;
    QSpacerItem *hSpacerRight;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *movieGroupBox;
    QHBoxLayout *horizontalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_6;
    QFormLayout *formLayout;
    QLabel *lblFile;
    QLineEdit *files;
    QLabel *lblName;
    QLineEdit *name;
    QLabel *lblOriginalName;
    QLineEdit *originalName;
    QLabel *lblSortTitle;
    QLineEdit *sortTitle;
    QLabel *lblSet;
    QComboBox *set;
    QLabel *lblTagline;
    QLineEdit *tagline;
    QLabel *lblRatings;
    QLabel *lblReleased;
    QLabel *lblDirector;
    QLineEdit *director;
    QLabel *lblWriter;
    QLineEdit *writer;
    QLabel *lblCertification;
    QComboBox *certification;
    QLabel *lblTrailer;
    QLabel *lblPlayCount;
    QHBoxLayout *horizontalLayout_11;
    QSpinBox *playcount;
    QSpacerItem *horizontalSpacer_5;
    QLabel *lblLastPlayed;
    QDateTimeEdit *lastPlayed;
    QSpacerItem *horizontalSpacer_6;
    Badge *badgeWatched;
    QLabel *lblPlot;
    QHBoxLayout *horizontalLayout_18;
    QLineEdit *trailer;
    QToolButton *buttonYoutubeDummy;
    QToolButton *buttonDownloadTrailer;
    QToolButton *buttonPlayLocalTrailer;
    Badge *localTrailer;
    QVBoxLayout *vLayoutRating;
    QHBoxLayout *hLayoutUserRating;
    QLabel *lblTop250;
    QSpinBox *top250;
    QSpacerItem *horizontalSpacer_userRating;
    QLabel *lblUserRating;
    QDoubleSpinBox *userRating;
    QSpacerItem *horizontalSpacer_12;
    RatingsWidget *ratings;
    QHBoxLayout *horizontalLayout_4;
    QDateEdit *releaseDate;
    QSpacerItem *horizontalSpacer_2;
    QLabel *lblRuntime;
    QSpinBox *runtime;
    QLabel *lblOutline;
    QHBoxLayout *horizontalLayout;
    QLineEdit *imdbId;
    QPushButton *btnImdb;
    QSpacerItem *horizontalSpacer_14;
    QLabel *lblTmdbId;
    QLineEdit *tmdbId;
    QPushButton *btnTmdb;
    QSpacerItem *horizontalSpacer_15;
    QLabel *lblImdbId;
    QPlainTextEdit *overview;
    QPlainTextEdit *outline;
    QWidget *tabExtended;
    QVBoxLayout *verticalLayout_3;
    TagCloud *genreCloud;
    TagCloud *tagCloud;
    TagCloud *countryCloud;
    TagCloud *studioCloud;
    TagCloud *tvShowLinks;
    QSpacerItem *verticalSpacer_3;
    QWidget *tabActors;
    QHBoxLayout *horizontalLayout_3;
    ActorsWidget *actors;
    QWidget *tabExtraFanarts;
    QVBoxLayout *verticalLayout_7;
    MessageLabel *labelSepFoldersWarning;
    ImageGallery *fanarts;
    QHBoxLayout *horizontalLayout_5;
    QLabel *lblHintClosedImages;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *btnAddExtraFanart;
    QWidget *tabStreamdetails;
    QVBoxLayout *verticalLayout_12;
    QGridLayout *streamDetails;
    QLineEdit *videoCodec;
    QLabel *lblResolution;
    QLabel *lblScantype;
    QLabel *lblAspectRatio;
    QLineEdit *videoScantype;
    QDoubleSpinBox *videoAspectRatio;
    QLabel *labelStreamDetailsAudio;
    QLabel *lblVideo;
    QTimeEdit *videoDuration;
    QLabel *lblDuration;
    QHBoxLayout *horizontalLayout_16;
    QSpinBox *videoWidth;
    QLabel *lblResolutionTimes;
    QSpinBox *videoHeight;
    QSpacerItem *horizontalSpacer_10;
    QLabel *lblCodec;
    QLabel *lblStereoMode;
    StereoModeComboBox *stereoMode;
    QHBoxLayout *horizontalLayout_17;
    QPushButton *buttonReloadStreamDetails;
    QLabel *lblReloadStreamDetailsError;
    QSpacerItem *horizontalSpacer_9;
    QSpacerItem *verticalSpacer_4;
    QLabel *lblExternalSubtitles;
    QTableWidget *subtitles;
    QSpacerItem *verticalSpacer_8;
    QFrame *line;
    QVBoxLayout *verticalLayout_4;
    QWidget *artStackedWidgetButtons;
    QHBoxLayout *horizontalLayout_20;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *buttonArtPageOne;
    QPushButton *buttonArtPageTwo;
    QSpacerItem *horizontalSpacer_8;
    SlidingStackedWidget *artStackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_10;
    QLabel *labelPoster;
    ClosableImage *poster;
    QLabel *labelFanart;
    ClosableImage *backdrop;
    QLabel *labelThumb;
    ClosableImage *thumb;
    QSpacerItem *verticalSpacer;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_11;
    QLabel *labelLogo;
    ClosableImage *logo;
    QLabel *labelClearArt;
    ClosableImage *clearArt;
    QLabel *labelDiscArt;
    ClosableImage *cdArt;
    QLabel *labelBanner;
    ClosableImage *banner;
    QSpacerItem *verticalSpacer_7;

    void setupUi(QWidget *MovieWidget)
    {
        if (MovieWidget->objectName().isEmpty())
            MovieWidget->setObjectName("MovieWidget");
        MovieWidget->resize(934, 928);
        verticalLayout = new QVBoxLayout(MovieWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 20, 0, 0);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_4);

        buttonRevert = new QToolButton(MovieWidget);
        buttonRevert->setObjectName("buttonRevert");
        buttonRevert->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/arrow_circle_left.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonRevert->setIcon(icon);
        buttonRevert->setIconSize(QSize(16, 16));

        horizontalLayout_10->addWidget(buttonRevert);

        movieName = new QLabel(MovieWidget);
        movieName->setObjectName("movieName");
        QFont font;
        font.setPointSize(22);
        movieName->setFont(font);
        movieName->setIndent(4);
        movieName->setProperty("isHeader", QVariant(true));

        horizontalLayout_10->addWidget(movieName);

        titleSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(titleSpacer);

        mediaFlags = new MediaFlags(MovieWidget);
        mediaFlags->setObjectName("mediaFlags");

        horizontalLayout_10->addWidget(mediaFlags);

        buttonPlay = new QToolButton(MovieWidget);
        buttonPlay->setObjectName("buttonPlay");
        buttonPlay->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/actions/32/media-playback-start.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonPlay->setIcon(icon1);

        horizontalLayout_10->addWidget(buttonPlay);

        hSpacerRight = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(hSpacerRight);


        verticalLayout->addLayout(horizontalLayout_10);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(-1, 0, -1, -1);
        movieGroupBox = new QGroupBox(MovieWidget);
        movieGroupBox->setObjectName("movieGroupBox");
        horizontalLayout_2 = new QHBoxLayout(movieGroupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tabWidget = new QTabWidget(movieGroupBox);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_6 = new QVBoxLayout(tab);
        verticalLayout_6->setObjectName("verticalLayout_6");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setRowWrapPolicy(QFormLayout::RowWrapPolicy::DontWrapRows);
        formLayout->setLabelAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        formLayout->setHorizontalSpacing(20);
        lblFile = new QLabel(tab);
        lblFile->setObjectName("lblFile");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblFile);

        files = new QLineEdit(tab);
        files->setObjectName("files");
        files->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, files);

        lblName = new QLabel(tab);
        lblName->setObjectName("lblName");

        formLayout->setWidget(2, QFormLayout::LabelRole, lblName);

        name = new QLineEdit(tab);
        name->setObjectName("name");

        formLayout->setWidget(2, QFormLayout::FieldRole, name);

        lblOriginalName = new QLabel(tab);
        lblOriginalName->setObjectName("lblOriginalName");

        formLayout->setWidget(3, QFormLayout::LabelRole, lblOriginalName);

        originalName = new QLineEdit(tab);
        originalName->setObjectName("originalName");

        formLayout->setWidget(3, QFormLayout::FieldRole, originalName);

        lblSortTitle = new QLabel(tab);
        lblSortTitle->setObjectName("lblSortTitle");

        formLayout->setWidget(4, QFormLayout::LabelRole, lblSortTitle);

        sortTitle = new QLineEdit(tab);
        sortTitle->setObjectName("sortTitle");

        formLayout->setWidget(4, QFormLayout::FieldRole, sortTitle);

        lblSet = new QLabel(tab);
        lblSet->setObjectName("lblSet");

        formLayout->setWidget(5, QFormLayout::LabelRole, lblSet);

        set = new QComboBox(tab);
        set->setObjectName("set");
        set->setMinimumSize(QSize(200, 0));
        set->setEditable(true);
        set->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);
        set->setMinimumContentsLength(20);

        formLayout->setWidget(5, QFormLayout::FieldRole, set);

        lblTagline = new QLabel(tab);
        lblTagline->setObjectName("lblTagline");

        formLayout->setWidget(6, QFormLayout::LabelRole, lblTagline);

        tagline = new QLineEdit(tab);
        tagline->setObjectName("tagline");

        formLayout->setWidget(6, QFormLayout::FieldRole, tagline);

        lblRatings = new QLabel(tab);
        lblRatings->setObjectName("lblRatings");

        formLayout->setWidget(7, QFormLayout::LabelRole, lblRatings);

        lblReleased = new QLabel(tab);
        lblReleased->setObjectName("lblReleased");

        formLayout->setWidget(8, QFormLayout::LabelRole, lblReleased);

        lblDirector = new QLabel(tab);
        lblDirector->setObjectName("lblDirector");

        formLayout->setWidget(10, QFormLayout::LabelRole, lblDirector);

        director = new QLineEdit(tab);
        director->setObjectName("director");

        formLayout->setWidget(10, QFormLayout::FieldRole, director);

        lblWriter = new QLabel(tab);
        lblWriter->setObjectName("lblWriter");

        formLayout->setWidget(11, QFormLayout::LabelRole, lblWriter);

        writer = new QLineEdit(tab);
        writer->setObjectName("writer");

        formLayout->setWidget(11, QFormLayout::FieldRole, writer);

        lblCertification = new QLabel(tab);
        lblCertification->setObjectName("lblCertification");

        formLayout->setWidget(12, QFormLayout::LabelRole, lblCertification);

        certification = new QComboBox(tab);
        certification->setObjectName("certification");
        certification->setEditable(true);
        certification->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);
        certification->setMinimumContentsLength(10);

        formLayout->setWidget(12, QFormLayout::FieldRole, certification);

        lblTrailer = new QLabel(tab);
        lblTrailer->setObjectName("lblTrailer");

        formLayout->setWidget(13, QFormLayout::LabelRole, lblTrailer);

        lblPlayCount = new QLabel(tab);
        lblPlayCount->setObjectName("lblPlayCount");

        formLayout->setWidget(15, QFormLayout::LabelRole, lblPlayCount);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        playcount = new QSpinBox(tab);
        playcount->setObjectName("playcount");
        playcount->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        playcount->setMaximum(99);

        horizontalLayout_11->addWidget(playcount);

        horizontalSpacer_5 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_5);

        lblLastPlayed = new QLabel(tab);
        lblLastPlayed->setObjectName("lblLastPlayed");

        horizontalLayout_11->addWidget(lblLastPlayed);

        lastPlayed = new QDateTimeEdit(tab);
        lastPlayed->setObjectName("lastPlayed");
        lastPlayed->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        horizontalLayout_11->addWidget(lastPlayed);

        horizontalSpacer_6 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_6);

        badgeWatched = new Badge(tab);
        badgeWatched->setObjectName("badgeWatched");

        horizontalLayout_11->addWidget(badgeWatched);


        formLayout->setLayout(15, QFormLayout::FieldRole, horizontalLayout_11);

        lblPlot = new QLabel(tab);
        lblPlot->setObjectName("lblPlot");

        formLayout->setWidget(16, QFormLayout::LabelRole, lblPlot);

        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        trailer = new QLineEdit(tab);
        trailer->setObjectName("trailer");

        horizontalLayout_18->addWidget(trailer);

        buttonYoutubeDummy = new QToolButton(tab);
        buttonYoutubeDummy->setObjectName("buttonYoutubeDummy");
        buttonYoutubeDummy->setMinimumSize(QSize(22, 22));
        buttonYoutubeDummy->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout_18->addWidget(buttonYoutubeDummy);

        buttonDownloadTrailer = new QToolButton(tab);
        buttonDownloadTrailer->setObjectName("buttonDownloadTrailer");
        buttonDownloadTrailer->setMinimumSize(QSize(22, 22));
        buttonDownloadTrailer->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout_18->addWidget(buttonDownloadTrailer);

        buttonPlayLocalTrailer = new QToolButton(tab);
        buttonPlayLocalTrailer->setObjectName("buttonPlayLocalTrailer");
        buttonPlayLocalTrailer->setMinimumSize(QSize(22, 22));
        buttonPlayLocalTrailer->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonPlayLocalTrailer->setText(QString::fromUtf8(""));

        horizontalLayout_18->addWidget(buttonPlayLocalTrailer);

        localTrailer = new Badge(tab);
        localTrailer->setObjectName("localTrailer");

        horizontalLayout_18->addWidget(localTrailer);


        formLayout->setLayout(13, QFormLayout::FieldRole, horizontalLayout_18);

        vLayoutRating = new QVBoxLayout();
        vLayoutRating->setObjectName("vLayoutRating");
        hLayoutUserRating = new QHBoxLayout();
        hLayoutUserRating->setObjectName("hLayoutUserRating");
        lblTop250 = new QLabel(tab);
        lblTop250->setObjectName("lblTop250");

        hLayoutUserRating->addWidget(lblTop250);

        top250 = new QSpinBox(tab);
        top250->setObjectName("top250");
        top250->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        top250->setMaximum(250);

        hLayoutUserRating->addWidget(top250);

        horizontalSpacer_userRating = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hLayoutUserRating->addItem(horizontalSpacer_userRating);

        lblUserRating = new QLabel(tab);
        lblUserRating->setObjectName("lblUserRating");

        hLayoutUserRating->addWidget(lblUserRating);

        userRating = new QDoubleSpinBox(tab);
        userRating->setObjectName("userRating");
        userRating->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        userRating->setDecimals(2);
        userRating->setMaximum(10.000000000000000);

        hLayoutUserRating->addWidget(userRating);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hLayoutUserRating->addItem(horizontalSpacer_12);


        vLayoutRating->addLayout(hLayoutUserRating);

        ratings = new RatingsWidget(tab);
        ratings->setObjectName("ratings");

        vLayoutRating->addWidget(ratings);


        formLayout->setLayout(7, QFormLayout::FieldRole, vLayoutRating);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        releaseDate = new QDateEdit(tab);
        releaseDate->setObjectName("releaseDate");
        releaseDate->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        horizontalLayout_4->addWidget(releaseDate);

        horizontalSpacer_2 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);

        lblRuntime = new QLabel(tab);
        lblRuntime->setObjectName("lblRuntime");

        horizontalLayout_4->addWidget(lblRuntime);

        runtime = new QSpinBox(tab);
        runtime->setObjectName("runtime");
        runtime->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        runtime->setMaximum(999);

        horizontalLayout_4->addWidget(runtime);


        formLayout->setLayout(8, QFormLayout::FieldRole, horizontalLayout_4);

        lblOutline = new QLabel(tab);
        lblOutline->setObjectName("lblOutline");

        formLayout->setWidget(17, QFormLayout::LabelRole, lblOutline);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        imdbId = new QLineEdit(tab);
        imdbId->setObjectName("imdbId");
        imdbId->setMaximumSize(QSize(150, 16777215));

        horizontalLayout->addWidget(imdbId);

        btnImdb = new QPushButton(tab);
        btnImdb->setObjectName("btnImdb");

        horizontalLayout->addWidget(btnImdb);

        horizontalSpacer_14 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_14);

        lblTmdbId = new QLabel(tab);
        lblTmdbId->setObjectName("lblTmdbId");

        horizontalLayout->addWidget(lblTmdbId);

        tmdbId = new QLineEdit(tab);
        tmdbId->setObjectName("tmdbId");

        horizontalLayout->addWidget(tmdbId);

        btnTmdb = new QPushButton(tab);
        btnTmdb->setObjectName("btnTmdb");

        horizontalLayout->addWidget(btnTmdb);

        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_15);


        formLayout->setLayout(1, QFormLayout::FieldRole, horizontalLayout);

        lblImdbId = new QLabel(tab);
        lblImdbId->setObjectName("lblImdbId");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblImdbId);

        overview = new QPlainTextEdit(tab);
        overview->setObjectName("overview");

        formLayout->setWidget(16, QFormLayout::FieldRole, overview);

        outline = new QPlainTextEdit(tab);
        outline->setObjectName("outline");

        formLayout->setWidget(17, QFormLayout::FieldRole, outline);


        verticalLayout_6->addLayout(formLayout);

        tabWidget->addTab(tab, QString());
        tabExtended = new QWidget();
        tabExtended->setObjectName("tabExtended");
        verticalLayout_3 = new QVBoxLayout(tabExtended);
        verticalLayout_3->setObjectName("verticalLayout_3");
        genreCloud = new TagCloud(tabExtended);
        genreCloud->setObjectName("genreCloud");

        verticalLayout_3->addWidget(genreCloud);

        tagCloud = new TagCloud(tabExtended);
        tagCloud->setObjectName("tagCloud");

        verticalLayout_3->addWidget(tagCloud);

        countryCloud = new TagCloud(tabExtended);
        countryCloud->setObjectName("countryCloud");

        verticalLayout_3->addWidget(countryCloud);

        studioCloud = new TagCloud(tabExtended);
        studioCloud->setObjectName("studioCloud");

        verticalLayout_3->addWidget(studioCloud);

        tvShowLinks = new TagCloud(tabExtended);
        tvShowLinks->setObjectName("tvShowLinks");

        verticalLayout_3->addWidget(tvShowLinks);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        verticalLayout_3->setStretch(0, 2);
        verticalLayout_3->setStretch(1, 2);
        verticalLayout_3->setStretch(2, 2);
        verticalLayout_3->setStretch(3, 1);
        tabWidget->addTab(tabExtended, QString());
        tabActors = new QWidget();
        tabActors->setObjectName("tabActors");
        horizontalLayout_3 = new QHBoxLayout(tabActors);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        actors = new ActorsWidget(tabActors);
        actors->setObjectName("actors");

        horizontalLayout_3->addWidget(actors);

        tabWidget->addTab(tabActors, QString());
        tabExtraFanarts = new QWidget();
        tabExtraFanarts->setObjectName("tabExtraFanarts");
        verticalLayout_7 = new QVBoxLayout(tabExtraFanarts);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(12, -1, 12, -1);
        labelSepFoldersWarning = new MessageLabel(tabExtraFanarts);
        labelSepFoldersWarning->setObjectName("labelSepFoldersWarning");
        labelSepFoldersWarning->setWordWrap(true);

        verticalLayout_7->addWidget(labelSepFoldersWarning);

        fanarts = new ImageGallery(tabExtraFanarts);
        fanarts->setObjectName("fanarts");

        verticalLayout_7->addWidget(fanarts);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        lblHintClosedImages = new QLabel(tabExtraFanarts);
        lblHintClosedImages->setObjectName("lblHintClosedImages");
        QFont font1;
        lblHintClosedImages->setFont(font1);
        lblHintClosedImages->setWordWrap(true);

        horizontalLayout_5->addWidget(lblHintClosedImages);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        btnAddExtraFanart = new QPushButton(tabExtraFanarts);
        btnAddExtraFanart->setObjectName("btnAddExtraFanart");

        horizontalLayout_5->addWidget(btnAddExtraFanart);

        horizontalLayout_5->setStretch(0, 1);

        verticalLayout_7->addLayout(horizontalLayout_5);

        verticalLayout_7->setStretch(1, 1);
        tabWidget->addTab(tabExtraFanarts, QString());
        tabStreamdetails = new QWidget();
        tabStreamdetails->setObjectName("tabStreamdetails");
        verticalLayout_12 = new QVBoxLayout(tabStreamdetails);
        verticalLayout_12->setObjectName("verticalLayout_12");
        streamDetails = new QGridLayout();
        streamDetails->setObjectName("streamDetails");
        videoCodec = new QLineEdit(tabStreamdetails);
        videoCodec->setObjectName("videoCodec");
        videoCodec->setMaximumSize(QSize(100, 16777215));

        streamDetails->addWidget(videoCodec, 1, 1, 1, 1);

        lblResolution = new QLabel(tabStreamdetails);
        lblResolution->setObjectName("lblResolution");
        lblResolution->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblResolution, 2, 0, 1, 1);

        lblScantype = new QLabel(tabStreamdetails);
        lblScantype->setObjectName("lblScantype");
        lblScantype->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblScantype, 4, 0, 1, 1);

        lblAspectRatio = new QLabel(tabStreamdetails);
        lblAspectRatio->setObjectName("lblAspectRatio");
        lblAspectRatio->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblAspectRatio, 3, 0, 1, 1);

        videoScantype = new QLineEdit(tabStreamdetails);
        videoScantype->setObjectName("videoScantype");
        videoScantype->setMaximumSize(QSize(200, 16777215));

        streamDetails->addWidget(videoScantype, 4, 1, 1, 1);

        videoAspectRatio = new QDoubleSpinBox(tabStreamdetails);
        videoAspectRatio->setObjectName("videoAspectRatio");
        videoAspectRatio->setMaximumSize(QSize(100, 16777215));
        videoAspectRatio->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        videoAspectRatio->setDecimals(3);

        streamDetails->addWidget(videoAspectRatio, 3, 1, 1, 1);

        labelStreamDetailsAudio = new QLabel(tabStreamdetails);
        labelStreamDetailsAudio->setObjectName("labelStreamDetailsAudio");
        QFont font2;
        font2.setBold(true);
        labelStreamDetailsAudio->setFont(font2);
        labelStreamDetailsAudio->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(labelStreamDetailsAudio, 7, 0, 1, 1);

        lblVideo = new QLabel(tabStreamdetails);
        lblVideo->setObjectName("lblVideo");
        lblVideo->setFont(font2);
        lblVideo->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblVideo, 0, 0, 1, 1);

        videoDuration = new QTimeEdit(tabStreamdetails);
        videoDuration->setObjectName("videoDuration");
        videoDuration->setMaximumSize(QSize(200, 16777215));
        videoDuration->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);

        streamDetails->addWidget(videoDuration, 6, 1, 1, 1);

        lblDuration = new QLabel(tabStreamdetails);
        lblDuration->setObjectName("lblDuration");
        lblDuration->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblDuration, 6, 0, 1, 1);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        videoWidth = new QSpinBox(tabStreamdetails);
        videoWidth->setObjectName("videoWidth");
        videoWidth->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        videoWidth->setMaximum(9999);

        horizontalLayout_16->addWidget(videoWidth);

        lblResolutionTimes = new QLabel(tabStreamdetails);
        lblResolutionTimes->setObjectName("lblResolutionTimes");
        lblResolutionTimes->setText(QString::fromUtf8("x"));

        horizontalLayout_16->addWidget(lblResolutionTimes);

        videoHeight = new QSpinBox(tabStreamdetails);
        videoHeight->setObjectName("videoHeight");
        videoHeight->setButtonSymbols(QAbstractSpinBox::ButtonSymbols::NoButtons);
        videoHeight->setMaximum(99999);

        horizontalLayout_16->addWidget(videoHeight);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_10);


        streamDetails->addLayout(horizontalLayout_16, 2, 1, 1, 1);

        lblCodec = new QLabel(tabStreamdetails);
        lblCodec->setObjectName("lblCodec");
        lblCodec->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblCodec, 1, 0, 1, 1);

        lblStereoMode = new QLabel(tabStreamdetails);
        lblStereoMode->setObjectName("lblStereoMode");
        lblStereoMode->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        streamDetails->addWidget(lblStereoMode, 5, 0, 1, 1);

        stereoMode = new StereoModeComboBox(tabStreamdetails);
        stereoMode->setObjectName("stereoMode");
        stereoMode->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);

        streamDetails->addWidget(stereoMode, 5, 1, 1, 1);


        verticalLayout_12->addLayout(streamDetails);

        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        buttonReloadStreamDetails = new QPushButton(tabStreamdetails);
        buttonReloadStreamDetails->setObjectName("buttonReloadStreamDetails");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/img/arrow_circle_right_white.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        buttonReloadStreamDetails->setIcon(icon2);

        horizontalLayout_17->addWidget(buttonReloadStreamDetails);

        lblReloadStreamDetailsError = new QLabel(tabStreamdetails);
        lblReloadStreamDetailsError->setObjectName("lblReloadStreamDetailsError");
        lblReloadStreamDetailsError->setText(QString::fromUtf8("Error"));
        lblReloadStreamDetailsError->setTextFormat(Qt::TextFormat::PlainText);

        horizontalLayout_17->addWidget(lblReloadStreamDetailsError);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_9);


        verticalLayout_12->addLayout(horizontalLayout_17);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_12->addItem(verticalSpacer_4);

        lblExternalSubtitles = new QLabel(tabStreamdetails);
        lblExternalSubtitles->setObjectName("lblExternalSubtitles");
        lblExternalSubtitles->setFont(font2);

        verticalLayout_12->addWidget(lblExternalSubtitles);

        subtitles = new QTableWidget(tabStreamdetails);
        if (subtitles->columnCount() < 3)
            subtitles->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        subtitles->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        subtitles->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        subtitles->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        subtitles->setObjectName("subtitles");
        subtitles->setSelectionMode(QAbstractItemView::SelectionMode::NoSelection);
        subtitles->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        subtitles->setShowGrid(false);
        subtitles->verticalHeader()->setVisible(false);

        verticalLayout_12->addWidget(subtitles);

        verticalSpacer_8 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_12->addItem(verticalSpacer_8);

        tabWidget->addTab(tabStreamdetails, QString());

        horizontalLayout_2->addWidget(tabWidget);

        line = new QFrame(movieGroupBox);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Shadow::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(0);
        line->setFrameShape(QFrame::Shape::VLine);

        horizontalLayout_2->addWidget(line);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(10, 0, -1, -1);
        artStackedWidgetButtons = new QWidget(movieGroupBox);
        artStackedWidgetButtons->setObjectName("artStackedWidgetButtons");
        horizontalLayout_20 = new QHBoxLayout(artStackedWidgetButtons);
        horizontalLayout_20->setObjectName("horizontalLayout_20");
        horizontalLayout_20->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_7 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_7);

        buttonArtPageOne = new QPushButton(artStackedWidgetButtons);
        buttonArtPageOne->setObjectName("buttonArtPageOne");
        buttonArtPageOne->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonArtPageOne->setCheckable(true);
        buttonArtPageOne->setChecked(true);

        horizontalLayout_20->addWidget(buttonArtPageOne);

        buttonArtPageTwo = new QPushButton(artStackedWidgetButtons);
        buttonArtPageTwo->setObjectName("buttonArtPageTwo");
        buttonArtPageTwo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonArtPageTwo->setCheckable(true);

        horizontalLayout_20->addWidget(buttonArtPageTwo);

        horizontalSpacer_8 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_8);


        verticalLayout_4->addWidget(artStackedWidgetButtons);

        artStackedWidget = new SlidingStackedWidget(movieGroupBox);
        artStackedWidget->setObjectName("artStackedWidget");
        artStackedWidget->setMinimumSize(QSize(180, 0));
        artStackedWidget->setMaximumSize(QSize(180, 16777215));
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_10 = new QVBoxLayout(page);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        labelPoster = new QLabel(page);
        labelPoster->setObjectName("labelPoster");

        verticalLayout_10->addWidget(labelPoster);

        poster = new ClosableImage(page);
        poster->setObjectName("poster");
        poster->setMinimumSize(QSize(180, 290));
        poster->setMaximumSize(QSize(180, 290));
        poster->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        poster->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/poster.png")));
        poster->setAlignment(Qt::AlignmentFlag::AlignCenter);
        poster->setProperty("clickable", QVariant(true));
        poster->setProperty("myFixedHeight", QVariant(290));

        verticalLayout_10->addWidget(poster);

        labelFanart = new QLabel(page);
        labelFanart->setObjectName("labelFanart");

        verticalLayout_10->addWidget(labelFanart);

        backdrop = new ClosableImage(page);
        backdrop->setObjectName("backdrop");
        backdrop->setMinimumSize(QSize(180, 120));
        backdrop->setMaximumSize(QSize(180, 120));
        backdrop->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        backdrop->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/fanart.png")));
        backdrop->setAlignment(Qt::AlignmentFlag::AlignCenter);
        backdrop->setProperty("clickable", QVariant(true));
        backdrop->setProperty("myFixedHeight", QVariant(120));

        verticalLayout_10->addWidget(backdrop);

        labelThumb = new QLabel(page);
        labelThumb->setObjectName("labelThumb");

        verticalLayout_10->addWidget(labelThumb);

        thumb = new ClosableImage(page);
        thumb->setObjectName("thumb");
        thumb->setMinimumSize(QSize(180, 102));
        thumb->setMaximumSize(QSize(180, 102));
        thumb->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        thumb->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/thumb.png")));
        thumb->setAlignment(Qt::AlignmentFlag::AlignCenter);
        thumb->setProperty("clickable", QVariant(true));
        thumb->setProperty("myFixedHeight", QVariant(102));

        verticalLayout_10->addWidget(thumb);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_10->addItem(verticalSpacer);

        artStackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_11 = new QVBoxLayout(page_2);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_11->setContentsMargins(0, 0, 0, 0);
        labelLogo = new QLabel(page_2);
        labelLogo->setObjectName("labelLogo");

        verticalLayout_11->addWidget(labelLogo);

        logo = new ClosableImage(page_2);
        logo->setObjectName("logo");
        logo->setMinimumSize(QSize(180, 110));
        logo->setMaximumSize(QSize(180, 110));
        logo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        logo->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/logo.png")));
        logo->setAlignment(Qt::AlignmentFlag::AlignCenter);
        logo->setProperty("clickable", QVariant(true));
        logo->setProperty("myFixedHeight", QVariant(110));

        verticalLayout_11->addWidget(logo);

        labelClearArt = new QLabel(page_2);
        labelClearArt->setObjectName("labelClearArt");

        verticalLayout_11->addWidget(labelClearArt);

        clearArt = new ClosableImage(page_2);
        clearArt->setObjectName("clearArt");
        clearArt->setMinimumSize(QSize(180, 155));
        clearArt->setMaximumSize(QSize(180, 155));
        clearArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        clearArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/clear_art.png")));
        clearArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        clearArt->setProperty("clickable", QVariant(true));
        clearArt->setProperty("myFixedHeight", QVariant(155));

        verticalLayout_11->addWidget(clearArt);

        labelDiscArt = new QLabel(page_2);
        labelDiscArt->setObjectName("labelDiscArt");

        verticalLayout_11->addWidget(labelDiscArt);

        cdArt = new ClosableImage(page_2);
        cdArt->setObjectName("cdArt");
        cdArt->setMinimumSize(QSize(180, 200));
        cdArt->setMaximumSize(QSize(180, 200));
        cdArt->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        cdArt->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/cd_art.png")));
        cdArt->setAlignment(Qt::AlignmentFlag::AlignCenter);
        cdArt->setProperty("clickable", QVariant(true));
        cdArt->setProperty("myFixedHeight", QVariant(200));

        verticalLayout_11->addWidget(cdArt);

        labelBanner = new QLabel(page_2);
        labelBanner->setObjectName("labelBanner");

        verticalLayout_11->addWidget(labelBanner);

        banner = new ClosableImage(page_2);
        banner->setObjectName("banner");
        banner->setMinimumSize(QSize(180, 53));
        banner->setMaximumSize(QSize(180, 53));
        banner->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        banner->setPixmap(QPixmap(QString::fromUtf8(":/img/placeholders/banner.png")));
        banner->setAlignment(Qt::AlignmentFlag::AlignCenter);
        banner->setProperty("myFixedHeight", QVariant(53));
        banner->setProperty("clickable", QVariant(true));

        verticalLayout_11->addWidget(banner);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_11->addItem(verticalSpacer_7);

        artStackedWidget->addWidget(page_2);

        verticalLayout_4->addWidget(artStackedWidget);

        verticalLayout_4->setStretch(1, 1);

        horizontalLayout_2->addLayout(verticalLayout_4);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout_5->addWidget(movieGroupBox);


        verticalLayout->addLayout(verticalLayout_5);

        verticalLayout->setStretch(1, 1);
        QWidget::setTabOrder(buttonRevert, tabWidget);
        QWidget::setTabOrder(tabWidget, files);
        QWidget::setTabOrder(files, name);
        QWidget::setTabOrder(name, originalName);
        QWidget::setTabOrder(originalName, sortTitle);
        QWidget::setTabOrder(sortTitle, set);
        QWidget::setTabOrder(set, tagline);
        QWidget::setTabOrder(tagline, director);
        QWidget::setTabOrder(director, writer);
        QWidget::setTabOrder(writer, certification);
        QWidget::setTabOrder(certification, playcount);
        QWidget::setTabOrder(playcount, lastPlayed);
        QWidget::setTabOrder(lastPlayed, videoCodec);
        QWidget::setTabOrder(videoCodec, videoWidth);
        QWidget::setTabOrder(videoWidth, videoHeight);
        QWidget::setTabOrder(videoHeight, videoAspectRatio);
        QWidget::setTabOrder(videoAspectRatio, videoScantype);

        retranslateUi(MovieWidget);
        QObject::connect(buttonArtPageOne, SIGNAL(clicked()), MovieWidget, SLOT(onArtPageOne()));
        QObject::connect(buttonArtPageTwo, SIGNAL(clicked()), MovieWidget, SLOT(onArtPageTwo()));

        tabWidget->setCurrentIndex(0);
        artStackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MovieWidget);
    } // setupUi

    void retranslateUi(QWidget *MovieWidget)
    {
#if QT_CONFIG(tooltip)
        buttonRevert->setToolTip(QCoreApplication::translate("MovieWidget", "Movie has changed. Click to revert changes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRevert->setText(QString());
        movieName->setText(QCoreApplication::translate("MovieWidget", "Movie Name", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlay->setToolTip(QCoreApplication::translate("MovieWidget", "Play movie", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlay->setText(QCoreApplication::translate("MovieWidget", "...", nullptr));
        movieGroupBox->setTitle(QString());
        lblFile->setText(QCoreApplication::translate("MovieWidget", "Files", nullptr));
        lblName->setText(QCoreApplication::translate("MovieWidget", "Name", nullptr));
        lblOriginalName->setText(QCoreApplication::translate("MovieWidget", "Original Name", nullptr));
        lblSortTitle->setText(QCoreApplication::translate("MovieWidget", "Sort Title", nullptr));
        lblSet->setText(QCoreApplication::translate("MovieWidget", "Set", nullptr));
        lblTagline->setText(QCoreApplication::translate("MovieWidget", "Tagline", nullptr));
        lblRatings->setText(QCoreApplication::translate("MovieWidget", "Ratings", nullptr));
        lblReleased->setText(QCoreApplication::translate("MovieWidget", "Released", nullptr));
        lblDirector->setText(QCoreApplication::translate("MovieWidget", "Director", nullptr));
        lblWriter->setText(QCoreApplication::translate("MovieWidget", "Writer", nullptr));
        lblCertification->setText(QCoreApplication::translate("MovieWidget", "Certification", nullptr));
        lblTrailer->setText(QCoreApplication::translate("MovieWidget", "Trailer", nullptr));
        lblPlayCount->setText(QCoreApplication::translate("MovieWidget", "Play Count", nullptr));
        lblLastPlayed->setText(QCoreApplication::translate("MovieWidget", "Last Played", nullptr));
        badgeWatched->setText(QCoreApplication::translate("MovieWidget", "Watched", nullptr));
        badgeWatched->setProperty("activeText", QVariant(QCoreApplication::translate("MovieWidget", "Watched", nullptr)));
        badgeWatched->setProperty("inactiveText", QVariant(QCoreApplication::translate("MovieWidget", "Not watched", nullptr)));
        lblPlot->setText(QCoreApplication::translate("MovieWidget", "Plot", nullptr));
#if QT_CONFIG(tooltip)
        buttonYoutubeDummy->setToolTip(QCoreApplication::translate("MovieWidget", "Insert YouTube Dummy Link", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonYoutubeDummy->setText(QString());
#if QT_CONFIG(tooltip)
        buttonDownloadTrailer->setToolTip(QCoreApplication::translate("MovieWidget", "Download Trailer", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonPlayLocalTrailer->setToolTip(QCoreApplication::translate("MovieWidget", "Play local trailer", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        localTrailer->setToolTip(QCoreApplication::translate("MovieWidget", "Local trailer is available", nullptr));
#endif // QT_CONFIG(tooltip)
        localTrailer->setText(QCoreApplication::translate("MovieWidget", "Local Trailer", nullptr));
        lblTop250->setText(QCoreApplication::translate("MovieWidget", "Top 250", nullptr));
        lblUserRating->setText(QCoreApplication::translate("MovieWidget", "User Rating", nullptr));
        releaseDate->setDisplayFormat(QCoreApplication::translate("MovieWidget", "yyyy-MM-dd", nullptr));
        lblRuntime->setText(QCoreApplication::translate("MovieWidget", "Runtime", nullptr));
        runtime->setSuffix(QCoreApplication::translate("MovieWidget", " Minutes", nullptr));
        lblOutline->setText(QCoreApplication::translate("MovieWidget", "Outline", nullptr));
#if QT_CONFIG(tooltip)
        btnImdb->setToolTip(QCoreApplication::translate("MovieWidget", "Open movie on IMDb.com", nullptr));
#endif // QT_CONFIG(tooltip)
        btnImdb->setText(QCoreApplication::translate("MovieWidget", "Go", nullptr));
        lblTmdbId->setText(QCoreApplication::translate("MovieWidget", "TMDb ID", nullptr));
#if QT_CONFIG(tooltip)
        btnTmdb->setToolTip(QCoreApplication::translate("MovieWidget", "Open movie on TheMovieDB.org", nullptr));
#endif // QT_CONFIG(tooltip)
        btnTmdb->setText(QCoreApplication::translate("MovieWidget", "Go", nullptr));
        lblImdbId->setText(QCoreApplication::translate("MovieWidget", "IMDb ID", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("MovieWidget", "Information", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtended), QCoreApplication::translate("MovieWidget", "Extended", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabActors), QCoreApplication::translate("MovieWidget", "Actors", nullptr));
        labelSepFoldersWarning->setText(QCoreApplication::translate("MovieWidget", "Support for extra fanarts is only available when your movies are stored in separate folders. Check the settings if you've stored your movies in separate folders already.", nullptr));
        lblHintClosedImages->setText(QCoreApplication::translate("MovieWidget", "Hint: Closed images will be deleted on save.", nullptr));
        btnAddExtraFanart->setText(QCoreApplication::translate("MovieWidget", "Add Images", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtraFanarts), QCoreApplication::translate("MovieWidget", "Extra Fanarts", nullptr));
        lblResolution->setText(QCoreApplication::translate("MovieWidget", "Resolution", nullptr));
        lblScantype->setText(QCoreApplication::translate("MovieWidget", "Scantype", nullptr));
        lblAspectRatio->setText(QCoreApplication::translate("MovieWidget", "Aspect Ratio", nullptr));
        labelStreamDetailsAudio->setText(QCoreApplication::translate("MovieWidget", "Audio", nullptr));
        lblVideo->setText(QCoreApplication::translate("MovieWidget", "Video", nullptr));
        videoDuration->setDisplayFormat(QCoreApplication::translate("MovieWidget", "HH:mm:ss", nullptr));
        lblDuration->setText(QCoreApplication::translate("MovieWidget", "Duration", nullptr));
        lblCodec->setText(QCoreApplication::translate("MovieWidget", "Codec", nullptr));
        lblStereoMode->setText(QCoreApplication::translate("MovieWidget", "Stereo Mode", nullptr));
        buttonReloadStreamDetails->setText(QCoreApplication::translate("MovieWidget", "Reload from File", nullptr));
        lblExternalSubtitles->setText(QCoreApplication::translate("MovieWidget", "External Subtitles", nullptr));
        QTableWidgetItem *___qtablewidgetitem = subtitles->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("MovieWidget", "Files", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = subtitles->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("MovieWidget", "Language", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = subtitles->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("MovieWidget", "Forced", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabStreamdetails), QCoreApplication::translate("MovieWidget", "Streamdetails", nullptr));
        buttonArtPageOne->setText(QString());
        buttonArtPageTwo->setText(QString());
        labelPoster->setText(QCoreApplication::translate("MovieWidget", "Poster", nullptr));
#if QT_CONFIG(tooltip)
        poster->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        poster->setText(QString());
        poster->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "Poster", nullptr)));
        labelFanart->setText(QCoreApplication::translate("MovieWidget", "Fanart", nullptr));
#if QT_CONFIG(tooltip)
        backdrop->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        backdrop->setText(QString());
        backdrop->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "Fanart", nullptr)));
        labelThumb->setText(QCoreApplication::translate("MovieWidget", "Thumb", nullptr));
#if QT_CONFIG(tooltip)
        thumb->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        thumb->setText(QString());
        thumb->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "Thumb", nullptr)));
        labelLogo->setText(QCoreApplication::translate("MovieWidget", "Logo", nullptr));
#if QT_CONFIG(tooltip)
        logo->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        logo->setText(QString());
        logo->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "Logo", nullptr)));
        labelClearArt->setText(QCoreApplication::translate("MovieWidget", "Clear Art", nullptr));
#if QT_CONFIG(tooltip)
        clearArt->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        clearArt->setText(QString());
        clearArt->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "Clear Art", nullptr)));
        labelDiscArt->setText(QCoreApplication::translate("MovieWidget", "Disc Art", nullptr));
#if QT_CONFIG(tooltip)
        cdArt->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        cdArt->setText(QString());
        cdArt->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "CD Art", nullptr)));
        labelBanner->setText(QCoreApplication::translate("MovieWidget", "Banner", nullptr));
#if QT_CONFIG(tooltip)
        banner->setToolTip(QCoreApplication::translate("MovieWidget", "Click to Change", nullptr));
#endif // QT_CONFIG(tooltip)
        banner->setText(QString());
        banner->setProperty("title", QVariant(QCoreApplication::translate("MovieWidget", "Banner", nullptr)));
        (void)MovieWidget;
    } // retranslateUi

};

namespace Ui {
    class MovieWidget: public Ui_MovieWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVIEWIDGET_H
