/********************************************************************************
** Form generated from reading UI file 'LoadingStreamDetails.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOADINGSTREAMDETAILS_H
#define UI_LOADINGSTREAMDETAILS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LoadingStreamDetails
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QProgressBar *progressBar;
    QLabel *currentFile;

    void setupUi(QDialog *LoadingStreamDetails)
    {
        if (LoadingStreamDetails->objectName().isEmpty())
            LoadingStreamDetails->setObjectName("LoadingStreamDetails");
        LoadingStreamDetails->setWindowModality(Qt::WindowModal);
        LoadingStreamDetails->resize(400, 300);
        LoadingStreamDetails->setMinimumSize(QSize(300, 0));
        LoadingStreamDetails->setModal(true);
        verticalLayout = new QVBoxLayout(LoadingStreamDetails);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(LoadingStreamDetails);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        progressBar = new QProgressBar(LoadingStreamDetails);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        verticalLayout->addWidget(progressBar);

        currentFile = new QLabel(LoadingStreamDetails);
        currentFile->setObjectName("currentFile");

        verticalLayout->addWidget(currentFile);

        verticalLayout->setStretch(0, 1);

        retranslateUi(LoadingStreamDetails);

        QMetaObject::connectSlotsByName(LoadingStreamDetails);
    } // setupUi

    void retranslateUi(QDialog *LoadingStreamDetails)
    {
        LoadingStreamDetails->setWindowTitle(QCoreApplication::translate("LoadingStreamDetails", "Loading Stream Details", nullptr));
        label->setText(QCoreApplication::translate("LoadingStreamDetails", "Loading Stream Details...", nullptr));
        currentFile->setText(QCoreApplication::translate("LoadingStreamDetails", "File", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LoadingStreamDetails: public Ui_LoadingStreamDetails {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LOADINGSTREAMDETAILS_H
