/****************************************************************************
** Meta object code from reading C++ file 'MusicMultiScrapeDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/ui/music/MusicMultiScrapeDialog.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MusicMultiScrapeDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22MusicMultiScrapeDialogE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN22MusicMultiScrapeDialogE = QtMocHelpers::stringData(
    "MusicMultiScrapeDialog",
    "exec",
    "",
    "reject",
    "accept",
    "onChkToggled",
    "onChkAllToggled",
    "toggled",
    "onStartScraping",
    "onScrapingFinished",
    "onAlbumSearchFinished",
    "mediaelch::scraper::AlbumSearchJob*",
    "searchJob",
    "onArtistSearchFinished",
    "mediaelch::scraper::ArtistSearchJob*",
    "scrapeNext",
    "onArtistProgress",
    "Artist*",
    "artist",
    "current",
    "maximum",
    "onAlbumProgress",
    "Album*",
    "album"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN22MusicMultiScrapeDialogE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   86,    2, 0x0a,    1 /* Public */,
       3,    0,   87,    2, 0x0a,    2 /* Public */,
       4,    0,   88,    2, 0x0a,    3 /* Public */,
       5,    0,   89,    2, 0x08,    4 /* Private */,
       6,    1,   90,    2, 0x08,    5 /* Private */,
       8,    0,   93,    2, 0x08,    7 /* Private */,
       9,    0,   94,    2, 0x08,    8 /* Private */,
      10,    1,   95,    2, 0x08,    9 /* Private */,
      13,    1,   98,    2, 0x08,   11 /* Private */,
      15,    0,  101,    2, 0x08,   13 /* Private */,
      16,    3,  102,    2, 0x08,   14 /* Private */,
      21,    3,  109,    2, 0x08,   18 /* Private */,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 14,   12,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Int, QMetaType::Int,   18,   19,   20,
    QMetaType::Void, 0x80000000 | 22, QMetaType::Int, QMetaType::Int,   23,   19,   20,

       0        // eod
};

Q_CONSTINIT const QMetaObject MusicMultiScrapeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ZN22MusicMultiScrapeDialogE.offsetsAndSizes,
    qt_meta_data_ZN22MusicMultiScrapeDialogE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN22MusicMultiScrapeDialogE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MusicMultiScrapeDialog, std::true_type>,
        // method 'exec'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'reject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'accept'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChkToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onChkAllToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'onStartScraping'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onScrapingFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onAlbumSearchFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::AlbumSearchJob *, std::false_type>,
        // method 'onArtistSearchFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::scraper::ArtistSearchJob *, std::false_type>,
        // method 'scrapeNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onArtistProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Artist *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'onAlbumProgress'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Album *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>
    >,
    nullptr
} };

void MusicMultiScrapeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MusicMultiScrapeDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->reject(); break;
        case 2: _t->accept(); break;
        case 3: _t->onChkToggled(); break;
        case 4: _t->onChkAllToggled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->onStartScraping(); break;
        case 6: _t->onScrapingFinished(); break;
        case 7: _t->onAlbumSearchFinished((*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::AlbumSearchJob*>>(_a[1]))); break;
        case 8: _t->onArtistSearchFinished((*reinterpret_cast< std::add_pointer_t<mediaelch::scraper::ArtistSearchJob*>>(_a[1]))); break;
        case 9: _t->scrapeNext(); break;
        case 10: _t->onArtistProgress((*reinterpret_cast< std::add_pointer_t<Artist*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 11: _t->onAlbumProgress((*reinterpret_cast< std::add_pointer_t<Album*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::AlbumSearchJob* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mediaelch::scraper::ArtistSearchJob* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MusicMultiScrapeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MusicMultiScrapeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN22MusicMultiScrapeDialogE.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int MusicMultiScrapeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
