/********************************************************************************
** Form generated from reading UI file 'FileScannerDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILESCANNERDIALOG_H
#define UI_FILESCANNERDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_FileScannerDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *status;
    QProgressBar *progressBar;
    QLabel *currentDir;

    void setupUi(QDialog *FileScannerDialog)
    {
        if (FileScannerDialog->objectName().isEmpty())
            FileScannerDialog->setObjectName("FileScannerDialog");
        FileScannerDialog->setWindowModality(Qt::WindowModal);
        FileScannerDialog->resize(400, 300);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/MediaElch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        FileScannerDialog->setWindowIcon(icon);
        FileScannerDialog->setModal(true);
        verticalLayout = new QVBoxLayout(FileScannerDialog);
        verticalLayout->setObjectName("verticalLayout");
        status = new QLabel(FileScannerDialog);
        status->setObjectName("status");
        status->setText(QString::fromUtf8("<status>"));

        verticalLayout->addWidget(status);

        progressBar = new QProgressBar(FileScannerDialog);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        verticalLayout->addWidget(progressBar);

        currentDir = new QLabel(FileScannerDialog);
        currentDir->setObjectName("currentDir");
        currentDir->setMinimumSize(QSize(300, 0));
        currentDir->setMaximumSize(QSize(300, 16777215));
        currentDir->setText(QString::fromUtf8("<current directory>"));

        verticalLayout->addWidget(currentDir);


        retranslateUi(FileScannerDialog);

        QMetaObject::connectSlotsByName(FileScannerDialog);
    } // setupUi

    void retranslateUi(QDialog *FileScannerDialog)
    {
        FileScannerDialog->setWindowTitle(QCoreApplication::translate("FileScannerDialog", "File Scanner", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FileScannerDialog: public Ui_FileScannerDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILESCANNERDIALOG_H
