/****************************************************************************
** Meta object code from reading C++ file 'FanartTvMusicArtists.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/scrapers/image/FanartTvMusicArtists.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FanartTvMusicArtists.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9mediaelch7scraper20FanartTvMusicArtistsE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN9mediaelch7scraper20FanartTvMusicArtistsE = QtMocHelpers::stringData(
    "mediaelch::scraper::FanartTvMusicArtists",
    "searchMovie",
    "",
    "searchStr",
    "limit",
    "searchConcert",
    "searchTvShow",
    "mediaelch::Locale",
    "locale",
    "searchArtist",
    "searchAlbum",
    "artistName",
    "onSearchArtistFinished",
    "onLoadConcertFinished"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN9mediaelch7scraper20FanartTvMusicArtistsE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2,   86,    2, 0x0a,    1 /* Public */,
       1,    1,   91,    2, 0x2a,    4 /* Public | MethodCloned */,
       5,    2,   94,    2, 0x0a,    6 /* Public */,
       5,    1,   99,    2, 0x2a,    9 /* Public | MethodCloned */,
       6,    3,  102,    2, 0x0a,   11 /* Public */,
       6,    2,  109,    2, 0x2a,   15 /* Public | MethodCloned */,
       9,    2,  114,    2, 0x0a,   18 /* Public */,
       9,    1,  119,    2, 0x2a,   21 /* Public | MethodCloned */,
      10,    3,  122,    2, 0x0a,   23 /* Public */,
      10,    2,  129,    2, 0x2a,   27 /* Public | MethodCloned */,
      12,    0,  134,    2, 0x08,   30 /* Private */,
      13,    0,  135,    2, 0x08,   31 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 7, QMetaType::Int,    3,    8,    4,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 7,    3,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   11,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   11,    3,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject mediaelch::scraper::FanartTvMusicArtists::staticMetaObject = { {
    QMetaObject::SuperData::link<ImageProvider::staticMetaObject>(),
    qt_meta_stringdata_ZN9mediaelch7scraper20FanartTvMusicArtistsE.offsetsAndSizes,
    qt_meta_data_ZN9mediaelch7scraper20FanartTvMusicArtistsE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN9mediaelch7scraper20FanartTvMusicArtistsE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<FanartTvMusicArtists, std::true_type>,
        // method 'searchMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchMovie'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'searchConcert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchConcert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'searchTvShow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::Locale, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchTvShow'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<mediaelch::Locale, std::false_type>,
        // method 'searchArtist'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchArtist'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'searchAlbum'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'searchAlbum'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        QtPrivate::TypeAndForceComplete<QString, std::false_type>,
        // method 'onSearchArtistFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onLoadConcertFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void mediaelch::scraper::FanartTvMusicArtists::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FanartTvMusicArtists *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchMovie((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->searchMovie((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->searchConcert((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->searchConcert((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->searchTvShow((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::Locale>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->searchTvShow((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<mediaelch::Locale>>(_a[2]))); break;
        case 6: _t->searchArtist((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->searchArtist((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->searchAlbum((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->searchAlbum((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->onSearchArtistFinished(); break;
        case 11: _t->onLoadConcertFinished(); break;
        default: ;
        }
    }
}

const QMetaObject *mediaelch::scraper::FanartTvMusicArtists::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mediaelch::scraper::FanartTvMusicArtists::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN9mediaelch7scraper20FanartTvMusicArtistsE.stringdata0))
        return static_cast<void*>(this);
    return ImageProvider::qt_metacast(_clname);
}

int mediaelch::scraper::FanartTvMusicArtists::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ImageProvider::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
