/********************************************************************************
** Form generated from reading UI file 'ExportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTDIALOG_H
#define UI_EXPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "ui/small_widgets/MessageLabel.h"

QT_BEGIN_NAMESPACE

class Ui_ExportDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *lblExportIntro;
    MessageLabel *message;
    QFormLayout *formLayout;
    QLabel *lblTheme;
    QComboBox *comboTheme;
    QLabel *lblItemsToExport;
    QCheckBox *chkMovies;
    QCheckBox *chkTvShows;
    QCheckBox *chkConcerts;
    QProgressBar *progressBar;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnClose;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *btnExport;

    void setupUi(QDialog *ExportDialog)
    {
        if (ExportDialog->objectName().isEmpty())
            ExportDialog->setObjectName("ExportDialog");
        ExportDialog->setWindowModality(Qt::WindowModal);
        ExportDialog->resize(400, 300);
        ExportDialog->setMinimumSize(QSize(400, 300));
        ExportDialog->setModal(true);
        verticalLayout = new QVBoxLayout(ExportDialog);
        verticalLayout->setObjectName("verticalLayout");
        lblExportIntro = new QLabel(ExportDialog);
        lblExportIntro->setObjectName("lblExportIntro");

        verticalLayout->addWidget(lblExportIntro);

        message = new MessageLabel(ExportDialog);
        message->setObjectName("message");
        message->setWordWrap(true);

        verticalLayout->addWidget(message);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        lblTheme = new QLabel(ExportDialog);
        lblTheme->setObjectName("lblTheme");

        formLayout->setWidget(0, QFormLayout::LabelRole, lblTheme);

        comboTheme = new QComboBox(ExportDialog);
        comboTheme->setObjectName("comboTheme");

        formLayout->setWidget(0, QFormLayout::FieldRole, comboTheme);

        lblItemsToExport = new QLabel(ExportDialog);
        lblItemsToExport->setObjectName("lblItemsToExport");

        formLayout->setWidget(1, QFormLayout::LabelRole, lblItemsToExport);

        chkMovies = new QCheckBox(ExportDialog);
        chkMovies->setObjectName("chkMovies");
        chkMovies->setChecked(true);

        formLayout->setWidget(1, QFormLayout::FieldRole, chkMovies);

        chkTvShows = new QCheckBox(ExportDialog);
        chkTvShows->setObjectName("chkTvShows");
        chkTvShows->setChecked(true);

        formLayout->setWidget(2, QFormLayout::FieldRole, chkTvShows);

        chkConcerts = new QCheckBox(ExportDialog);
        chkConcerts->setObjectName("chkConcerts");
        chkConcerts->setChecked(true);

        formLayout->setWidget(3, QFormLayout::FieldRole, chkConcerts);


        verticalLayout->addLayout(formLayout);

        progressBar = new QProgressBar(ExportDialog);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        verticalLayout->addWidget(progressBar);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnClose = new QPushButton(ExportDialog);
        btnClose->setObjectName("btnClose");
        btnClose->setAutoDefault(false);

        horizontalLayout->addWidget(btnClose);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        btnExport = new QPushButton(ExportDialog);
        btnExport->setObjectName("btnExport");
        btnExport->setAutoDefault(false);

        horizontalLayout->addWidget(btnExport);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(2, 1);

        retranslateUi(ExportDialog);
        QObject::connect(btnExport, SIGNAL(clicked()), ExportDialog, SLOT(onBtnExport()));
        QObject::connect(comboTheme, SIGNAL(currentIndexChanged(int)), ExportDialog, SLOT(onThemeChanged()));
        QObject::connect(btnClose, SIGNAL(clicked()), ExportDialog, SLOT(onBtnClose()));

        QMetaObject::connectSlotsByName(ExportDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportDialog)
    {
        ExportDialog->setWindowTitle(QCoreApplication::translate("ExportDialog", "Export", nullptr));
        lblExportIntro->setText(QCoreApplication::translate("ExportDialog", "Export your complete collection.", nullptr));
        message->setText(QCoreApplication::translate("ExportDialog", "Message", nullptr));
        lblTheme->setText(QCoreApplication::translate("ExportDialog", "Theme", nullptr));
        lblItemsToExport->setText(QCoreApplication::translate("ExportDialog", "Items to export", nullptr));
        chkMovies->setText(QCoreApplication::translate("ExportDialog", "Movies", nullptr));
        chkTvShows->setText(QCoreApplication::translate("ExportDialog", "TV Shows", nullptr));
        chkConcerts->setText(QCoreApplication::translate("ExportDialog", "Concerts", nullptr));
        btnClose->setText(QCoreApplication::translate("ExportDialog", "Close", nullptr));
        btnExport->setText(QCoreApplication::translate("ExportDialog", "Export", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportDialog: public Ui_ExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTDIALOG_H
