/********************************************************************************
** Form generated from reading UI file 'ExportSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTSETTINGSWIDGET_H
#define UI_EXPORTSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/small_widgets/MessageLabel.h"

QT_BEGIN_NAMESPACE

class Ui_ExportSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_9;
    MessageLabel *themesErrorMessage;
    QTableWidget *exportTemplates;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ExportSettingsWidget)
    {
        if (ExportSettingsWidget->objectName().isEmpty())
            ExportSettingsWidget->setObjectName("ExportSettingsWidget");
        ExportSettingsWidget->resize(970, 625);
        verticalLayout_9 = new QVBoxLayout(ExportSettingsWidget);
        verticalLayout_9->setObjectName("verticalLayout_9");
        themesErrorMessage = new MessageLabel(ExportSettingsWidget);
        themesErrorMessage->setObjectName("themesErrorMessage");

        verticalLayout_9->addWidget(themesErrorMessage);

        exportTemplates = new QTableWidget(ExportSettingsWidget);
        if (exportTemplates->columnCount() < 1)
            exportTemplates->setColumnCount(1);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        exportTemplates->setHorizontalHeaderItem(0, __qtablewidgetitem);
        exportTemplates->setObjectName("exportTemplates");
        exportTemplates->setEditTriggers(QAbstractItemView::NoEditTriggers);
        exportTemplates->setSelectionMode(QAbstractItemView::NoSelection);
        exportTemplates->setShowGrid(false);
        exportTemplates->horizontalHeader()->setVisible(false);
        exportTemplates->horizontalHeader()->setStretchLastSection(true);
        exportTemplates->verticalHeader()->setVisible(false);

        verticalLayout_9->addWidget(exportTemplates);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer);

        verticalLayout_9->setStretch(1, 1);

        retranslateUi(ExportSettingsWidget);

        QMetaObject::connectSlotsByName(ExportSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *ExportSettingsWidget)
    {
        themesErrorMessage->setText(QCoreApplication::translate("ExportSettingsWidget", "Message", nullptr));
        QTableWidgetItem *___qtablewidgetitem = exportTemplates->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("ExportSettingsWidget", "Theme", nullptr));
        (void)ExportSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class ExportSettingsWidget: public Ui_ExportSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTSETTINGSWIDGET_H
