/********************************************************************************
** Form generated from reading UI file 'ImportSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.8.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTSETTINGSWIDGET_H
#define UI_IMPORTSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ImportSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_13;
    QFormLayout *formLayout_8;
    QLabel *lblAutomaticArchiveDeletion;
    QCheckBox *chkDeleteArchives;
    QLabel *lblPathToUnrar;
    QHBoxLayout *horizontalLayout_5;
    QLineEdit *unrarPath;
    QPushButton *btnChooseUnrar;
    QLabel *lblPathToMakeMkvCon;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *makemkvconPath;
    QPushButton *btnChooseMakemkvcon;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ImportSettingsWidget)
    {
        if (ImportSettingsWidget->objectName().isEmpty())
            ImportSettingsWidget->setObjectName("ImportSettingsWidget");
        ImportSettingsWidget->resize(970, 625);
        verticalLayout_13 = new QVBoxLayout(ImportSettingsWidget);
        verticalLayout_13->setObjectName("verticalLayout_13");
        formLayout_8 = new QFormLayout();
        formLayout_8->setObjectName("formLayout_8");
        lblAutomaticArchiveDeletion = new QLabel(ImportSettingsWidget);
        lblAutomaticArchiveDeletion->setObjectName("lblAutomaticArchiveDeletion");

        formLayout_8->setWidget(0, QFormLayout::LabelRole, lblAutomaticArchiveDeletion);

        chkDeleteArchives = new QCheckBox(ImportSettingsWidget);
        chkDeleteArchives->setObjectName("chkDeleteArchives");

        formLayout_8->setWidget(0, QFormLayout::FieldRole, chkDeleteArchives);

        lblPathToUnrar = new QLabel(ImportSettingsWidget);
        lblPathToUnrar->setObjectName("lblPathToUnrar");

        formLayout_8->setWidget(1, QFormLayout::LabelRole, lblPathToUnrar);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        unrarPath = new QLineEdit(ImportSettingsWidget);
        unrarPath->setObjectName("unrarPath");

        horizontalLayout_5->addWidget(unrarPath);

        btnChooseUnrar = new QPushButton(ImportSettingsWidget);
        btnChooseUnrar->setObjectName("btnChooseUnrar");

        horizontalLayout_5->addWidget(btnChooseUnrar);


        formLayout_8->setLayout(1, QFormLayout::FieldRole, horizontalLayout_5);

        lblPathToMakeMkvCon = new QLabel(ImportSettingsWidget);
        lblPathToMakeMkvCon->setObjectName("lblPathToMakeMkvCon");

        formLayout_8->setWidget(2, QFormLayout::LabelRole, lblPathToMakeMkvCon);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        makemkvconPath = new QLineEdit(ImportSettingsWidget);
        makemkvconPath->setObjectName("makemkvconPath");

        horizontalLayout_3->addWidget(makemkvconPath);

        btnChooseMakemkvcon = new QPushButton(ImportSettingsWidget);
        btnChooseMakemkvcon->setObjectName("btnChooseMakemkvcon");

        horizontalLayout_3->addWidget(btnChooseMakemkvcon);


        formLayout_8->setLayout(2, QFormLayout::FieldRole, horizontalLayout_3);


        verticalLayout_13->addLayout(formLayout_8);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_13->addItem(verticalSpacer);


        retranslateUi(ImportSettingsWidget);

        QMetaObject::connectSlotsByName(ImportSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *ImportSettingsWidget)
    {
        lblAutomaticArchiveDeletion->setText(QCoreApplication::translate("ImportSettingsWidget", "Automatically delete archives after extraction", nullptr));
        chkDeleteArchives->setText(QString());
        lblPathToUnrar->setText(QCoreApplication::translate("ImportSettingsWidget", "Path to unrar", nullptr));
        btnChooseUnrar->setText(QCoreApplication::translate("ImportSettingsWidget", "Choose", nullptr));
        lblPathToMakeMkvCon->setText(QCoreApplication::translate("ImportSettingsWidget", "Path to makemkvcon", nullptr));
        btnChooseMakemkvcon->setText(QCoreApplication::translate("ImportSettingsWidget", "Choose", nullptr));
        (void)ImportSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class ImportSettingsWidget: public Ui_ImportSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMPORTSETTINGSWIDGET_H
